@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 // This program and the accompanying materials
@@ -14,7 +14,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id$
 ///
 // Parent object for all activities. Derived classes generate trips for each
 // household.
@@ -27,8 +26,9 @@
 #include <config.h>
 
 #include <utils/common/RandHelper.h>
+#include <activitygen/city/AGHousehold.h>
+#include <activitygen/city/AGTime.h>
 #include "AGActivity.h"
-#include "../city/AGTime.h"
 
 
 // ===========================================================================
@@ -39,6 +39,7 @@ AGActivity::isGenerated() {
     return genDone;
 }
 
+
 bool
 AGActivity::generateTrips() {
     return true;
@@ -98,12 +99,14 @@ AGActivity::availableTranspMeans(AGPosition from, AGPosition to) {
     return available;
 }
 
+
 int
 AGActivity::timeToDrive(AGPosition from, AGPosition to) {
     double dist = from.distanceTo(to);
     return (int)(timePerKm * dist / 1000.0);
 }
 
+
 int
 AGActivity::depHour(AGPosition from, AGPosition to, int arrival) {
     // ?? departure.addDays(1); // in case of negative time: arrival < timeToDrive
@@ -111,11 +114,13 @@ AGActivity::depHour(AGPosition from, AGPosition to, int arrival) {
     return (arrival - timeToDrive(from, to));
 }
 
+
 int
 AGActivity::arrHour(AGPosition from, AGPosition to, int departure) {
     return (departure + timeToDrive(from, to));
 }
 
+
 int
 AGActivity::randomTimeBetween(int begin, int end) {
     if (0 > begin || begin > end) {
@@ -128,9 +133,11 @@ AGActivity::randomTimeBetween(int begin, int end) {
     return (begin + tAlea);
 }
 
+
 std::list<AGTrip>&
 AGActivity::getPartialActivityTrips() {
     return myPartialActivityTrips;
 }
 
+
 /****************************************************************************/
