@@ -1,15 +1,13 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    AGWorkAndSchool.cpp
 /// @author  Piotr Woznica
@@ -17,7 +15,6 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id$
 ///
 // Generates trips to work and to school
 /****************************************************************************/
@@ -26,15 +23,16 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
-#include "AGWorkAndSchool.h"
 #include <list>
-#include "../city/AGCar.h"
+#include <utils/common/SUMOVehicleClass.h>
+#include <activitygen/city/AGCar.h>
+#include <activitygen/city/AGChild.h>
+#include <activitygen/city/AGHousehold.h>
+#include <activitygen/city/AGStreet.h>
+#include <activitygen/city/AGWorkPosition.h>
+#include "AGWorkAndSchool.h"
 
 
 // ===========================================================================
