@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 // This program and the accompanying materials
@@ -15,7 +15,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id$
 ///
 // References a street of the city and defines a position in this street
 /****************************************************************************/
@@ -28,9 +27,9 @@
 // ===========================================================================
 #include <config.h>
 
-#include "../../utils/geom/Position.h"
 #include <list>
 #include <map>
+#include <utils/geom/Position.h>
 
 
 // ===========================================================================
