@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 // This program and the accompanying materials
@@ -15,7 +15,6 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    July 2010
-/// @version $Id$
 ///
 // Time manager: able to manipulate the time using Sumo's format (seconds)
 /****************************************************************************/
@@ -36,15 +35,14 @@
 // ===========================================================================
 class AGTime {
 public:
-    AGTime() {};
     AGTime(int seconds) :
-        sec(seconds) {};
+        mySeconds(seconds) {};
     AGTime(int hour, int minutes) :
-        sec(convert(0, hour, minutes, 0)) {};
+        mySeconds(convert(0, hour, minutes, 0)) {};
     AGTime(int day, int hour, int min) :
-        sec(convert(day, hour, min, 0)) {};
+        mySeconds(convert(day, hour, min, 0)) {};
     AGTime(int day, int hour, int min, int sec) :
-        sec(convert(day, hour, min, sec)) {};
+        mySeconds(convert(day, hour, min, sec)) {};
     AGTime(const AGTime& time);
     bool operator==(const AGTime& time);
     bool operator<(const AGTime& time);
@@ -127,7 +125,7 @@ private:
 
     // @brief: the seconds representing this date (day, hour, minute)
     // @brief: used for in/out
-    int sec;
+    int mySeconds;
 };
 
 #endif
