@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2006-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2006-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -16,7 +16,6 @@
 /// @author  Laura Bieker
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id$
 ///
 // Class representing a detector within the DFROUTER
 /****************************************************************************/
@@ -205,7 +204,7 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
                     }
                 }
                 if (splitEdge != nullptr) {
-                    j = find(j, (*ri).edges2Pass.end(), splitEdge);
+                    j = std::find(j, (*ri).edges2Pass.end(), splitEdge);
                 } else {
                     ++j;
                 }
@@ -308,6 +307,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
         return false;
     }
     // insertions
+    int vehicleIndex = 0;
     if (insertionsOnly || flows.knows(myID)) {
         // get the flows for this detector
         const std::vector<FlowDef>& mflows = flows.getFlowDefs(myID);
@@ -367,11 +367,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
 
                 // write
                 out.openTag(SUMO_TAG_VEHICLE);
-                if (getType() == SOURCE_DETECTOR) {
-                    out.writeAttr(SUMO_ATTR_ID, "emitter_" + myID + "_" + toString(ctime));
-                } else {
-                    out.writeAttr(SUMO_ATTR_ID, "calibrator_" + myID + "_" + toString(ctime));
-                }
+                out.writeAttr(SUMO_ATTR_ID, myID + "." + toString(vehicleIndex));
                 if (oc.getBool("vtype")) {
                     out.writeAttr(SUMO_ATTR_TYPE, vtype);
                 }
@@ -422,6 +418,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 }
                 out.closeTag();
                 srcFD.isLKW += srcFD.fLKW;
+                vehicleIndex++;
             }
         }
     }
@@ -720,6 +717,8 @@ RODFDetectorCon::setSpeedFactorAndDev(SUMOVTypeParameter& type, double maxFactor
         // individual speedFactors to match departSpeed (MSEdge::insertVehicle())
         type.speedFactor.getParameter()[1] = dev;
         type.parametersSet |= VTYPEPARS_SPEEDFACTOR_SET;
+    } else {
+        type.speedFactor.getParameter()[1] = -1; // do not write speedDev, only simple speedFactor
     }
 }
 
@@ -877,7 +876,7 @@ RODFDetectorCon::removeDetector(const std::string& id) {
     myDetectorMap.erase(ri1);
     //
     std::vector<RODFDetector*>::iterator ri2 =
-        find(myDetectors.begin(), myDetectors.end(), oldDet);
+        std::find(myDetectors.begin(), myDetectors.end(), oldDet);
     myDetectors.erase(ri2);
     //
     bool found = false;
@@ -955,7 +954,7 @@ RODFDetectorCon::getAnyDetectorForEdge(const RODFEdge* const edge) const {
 void
 RODFDetectorCon::clearDists(std::map<SUMOTime, RandomDistributor<int>* >& dists) const {
     for (std::map<SUMOTime, RandomDistributor<int>* >::iterator i = dists.begin(); i != dists.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
