@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2006-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2006-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id$
 ///
 // Sets and checks options for df-routing
 /****************************************************************************/
@@ -99,7 +98,7 @@ RODFFrame::fillOptions() {
     oc.doRegister("vtype", new Option_Bool(false));
     oc.addDescription("vtype", "Output", "Add vehicle types to the emitters file (PKW, LKW)");
 
-    oc.doRegister("vtype-output", new Option_FileName(""));
+    oc.doRegister("vtype-output", new Option_FileName(StringVector({ "" })));
     oc.addDescription("vtype-output", "Output", "Write generated vehicle types into separate FILE instead of including them into the emitters-output");
 
     oc.doRegister("emitters-poi-output", new Option_FileName()); // !!! describe
@@ -154,7 +153,7 @@ RODFFrame::fillOptions() {
     oc.doRegister("emissions-only", new Option_Bool(false));
     oc.addDescription("emissions-only", "Processing", "Writes only emission times");
 
-    oc.doRegister("disallowed-edges", new Option_String(""));
+    oc.doRegister("disallowed-edges", new Option_StringVector(StringVector({ "" })));
     oc.addDescription("disallowed-edges", "Processing", "Do not route on these edges");
 
     oc.doRegister("keep-turnarounds", new Option_Bool(false));
