@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Sets and checks options for dua-routing
 /****************************************************************************/
@@ -79,6 +78,12 @@ RODUAFrame::addImportOptions() {
     oc.doRegister("intermodal-weight-output", new Option_FileName());
     oc.addDescription("intermodal-weight-output", "Output", "Write intermodal edges with lengths and travel times to FILE");
 
+    oc.doRegister("write-trips", new Option_Bool(false));
+    oc.addDescription("write-trips", "Output", "Write trips instead of vehicles (for validating trip input)");
+
+    oc.doRegister("write-trips.geo", new Option_Bool(false));
+    oc.addDescription("write-trips.geo", "Output", "Write trips with geo-coordinates");
+
     // register import options
     oc.doRegister("weight-files", 'w', new Option_FileName());
     oc.addSynonyme("weight-files", "weights");
@@ -91,15 +96,15 @@ RODUAFrame::addImportOptions() {
     oc.addSynonyme("weight-attribute", "measure", true);
     oc.addDescription("weight-attribute", "Input", "Name of the xml attribute which gives the edge weight");
 
-    oc.doRegister("phemlight-path", new Option_FileName("./PHEMlight/"));
-    oc.addDescription("phemlight-path", "Input", "Determines where to load PHEMlight definitions from.");
-
     // register further processing options
     // ! The subtopic "Processing" must be initialised earlier !
     oc.doRegister("weights.expand", new Option_Bool(false));
     oc.addSynonyme("weights.expand", "expand-weights", true);
     oc.addDescription("weights.expand", "Processing", "Expand weights behind the simulation's end");
 
+    oc.doRegister("weights.random-factor", new Option_Float(1.));
+    oc.addDescription("weights.random-factor", "Processing", "Edge weights for routing are dynamically disturbed by a random factor drawn uniformly from [1,FLOAT)");
+
     oc.doRegister("routing-algorithm", new Option_String("dijkstra"));
     oc.addDescription("routing-algorithm", "Processing", "Select among routing algorithms ['dijkstra', 'astar', 'CH', 'CHWrapper']");
 
@@ -162,8 +167,9 @@ RODUAFrame::addDUAOptions() {
     oc.doRegister("persontrip.walkfactor", new Option_Float(double(0.75)));
     oc.addDescription("persontrip.walkfactor", "Processing", "Use FLOAT as a factor on pedestrian maximum speed during intermodal routing");
 
-    oc.doRegister("persontrip.transfer.car-walk", new Option_String("parkingAreas"));
-    oc.addDescription("persontrip.transfer.car-walk", "Processing", "Where are mode changes from car to walking allowed (possible values: 'parkingAreas', 'ptStops', 'allJunctions' and combinations)");
+    oc.doRegister("persontrip.transfer.car-walk", new Option_StringVector(StringVector({ "parkingAreas" })));
+    oc.addDescription("persontrip.transfer.car-walk", "Processing",
+                      "Where are mode changes from car to walking allowed (possible values: 'parkingAreas', 'ptStops', 'allJunctions' and combinations)");
 
 }
 
