@@ -37,11 +37,6 @@
      extern "C" void init_tcpip( shawn::SimulationController& );
 #endif
 
-// Disable exception handling warnings
-#ifdef _MSC_VER
-	#pragma warning( disable : 4290 )
-#endif
-
 #include <string>
 #include <map>
 #include <vector>
@@ -56,23 +51,10 @@ struct sockaddr_in;
 namespace tcpip
 {
 
-	class SocketException: public std::exception
+	class SocketException: public std::runtime_error
 	{
-	private:
-		std::string what_;
 	public:
-		SocketException( std::string what ) throw() 
-		{
-			what_ = what;
-			//std::cerr << "tcpip::SocketException: " << what << std::endl << std::flush;
-		}
-
-		virtual const char* what() const throw()
-		{
-			return what_.c_str();
-		}
-
-		~SocketException() throw() {}
+        SocketException(std::string what) : std::runtime_error(what.c_str()) {}
 	};
 
 	class Socket
@@ -88,22 +70,26 @@ namespace tcpip
 		/// Destructor
 		~Socket();
 
+		/// @brief Returns an free port on the system
+		/// @note This is done by binding a socket with port=0, getting the assigned port, and closing the socket again
+		static int getFreeSocketPort();
+
 		/// Connects to host_:port_
-		void connect() throw( SocketException );
+		void connect();
 
 		/// Wait for a incoming connection to port_
-        Socket* accept(const bool create = false) throw(SocketException);
+        Socket* accept(const bool create = false);
 
-		void send( const std::vector<unsigned char> &buffer) throw( SocketException );
-		void sendExact( const Storage & ) throw( SocketException );
+		void send( const std::vector<unsigned char> &buffer);
+		void sendExact( const Storage & );
 		/// Receive up to \p bufSize available bytes from Socket::socket_
-		std::vector<unsigned char> receive( int bufSize = 2048 ) throw( SocketException );
+		std::vector<unsigned char> receive( int bufSize = 2048 );
 		/// Receive a complete TraCI message from Socket::socket_
-		bool receiveExact( Storage &) throw( SocketException );
+		bool receiveExact( Storage &);
 		void close();
 		int port();
-		void set_blocking(bool) throw( SocketException );
-		bool is_blocking() throw();
+		void set_blocking(bool);
+		bool is_blocking();
 		bool has_client_connection() const;
 
 		// If verbose, each send and received data is written to stderr
@@ -123,12 +109,12 @@ namespace tcpip
 
 	private:
 		void init();
-		void BailOnSocketError( std::string ) const throw( SocketException );
+		static void BailOnSocketError(std::string context);
 #ifdef WIN32
-		std::string GetWinsockErrorString(int err) const;
+		static std::string GetWinsockErrorString(int err);
 #endif
 		bool atoaddr(std::string, struct sockaddr_in& addr);
-		bool datawaiting(int sock) const throw();
+		bool datawaiting(int sock) const;
 
 		std::string host_;
 		int port_;
