@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // The main window of the SUMO-gui.
 /****************************************************************************/
@@ -30,7 +29,7 @@
 #include <vector>
 #include <iostream>
 #include <fx.h>
-#include <utils/foxtools/MFXEventQue.h>
+#include <utils/foxtools/FXSynchQue.h>
 #include <utils/foxtools/FXThreadEvent.h>
 #include <utils/foxtools/MFXInterThreadEventClient.h>
 #include <utils/foxtools/FXLCDLabel.h>
@@ -132,12 +131,18 @@ public:
     /// @brief Called on menu File->Load Shapes
     long onCmdOpenShapes(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu File->Load EdgeData
+    long onCmdOpenEdgeData(FXObject*, FXSelector, void*);
+
     /// @brief Called on reload
     long onCmdReload(FXObject*, FXSelector, void*);
 
     /// @brief Called on opening a recent file
     long onCmdOpenRecent(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu File->Close
+    long onCmdSaveConfig(FXObject*, FXSelector, void*);
+
     /// @brief Called on menu File->Close
     long onCmdClose(FXObject*, FXSelector, void*);
 
@@ -238,6 +243,9 @@ public:
     /// @brief Determines whether some buttons which require an active simulation may be shown
     long onUpdNeedsSimulation(FXObject*, FXSelector, void*);
 
+    /// @brief Determines whether traci is active
+    long onUpdTraCIStatus(FXObject*, FXSelector, void*);
+
     /// @brief Called if the message window shall be cleared
     long onCmdClearMsgWindow(FXObject*, FXSelector, void*);
 
@@ -276,6 +284,10 @@ public:
         mySimDelay = delay;
     }
 
+    /** @brief Sets the breakpoints of the parent application
+     */
+    virtual void setBreakpoints(const std::vector<SUMOTime>& breakpoints);
+
     /** @brief Sends an event from the application thread to the GUI and waits until it is handled
      * @param event the event to send
      */
@@ -301,10 +313,10 @@ private:
 
     /// @brief handles additional game-related events
     void checkGamingEvents();
+    void checkGamingEventsDRT();
 
 protected:
-    /// FOX needs this for static members
-    GUIApplicationWindow() { }
+    FOX_CONSTRUCTOR(GUIApplicationWindow)
 
 protected:
     /// Builds the menu bar
@@ -324,7 +336,7 @@ protected:
     GUIRunThread* myRunThread;
 
     /// @brief  the information whether the simulation was started before
-    bool myWasStarted;
+    bool myWasStarted = false;
 
     /// @brief The current view number
     int myViewNumber;
@@ -333,53 +345,53 @@ protected:
     bool myAmLoading;
 
     /// @brief the submenus
-    FXMenuPane* myFileMenu, *myEditMenu, *mySelectByPermissions, *mySettingsMenu,
-                *myLocatorMenu, *myControlMenu,
-                *myWindowsMenu, *myHelpMenu;
+    FXMenuPane* myFileMenu = nullptr, *myEditMenu = nullptr, *mySelectByPermissions = nullptr, *mySettingsMenu = nullptr,
+                *myLocatorMenu, *myControlMenu = nullptr,
+                                 *myWindowsMenu, *myHelpMenu = nullptr;
 
     /// @brief the menu cascades
-    FXMenuCascade* mySelectLanesMenuCascade;
+    FXMenuCascade* mySelectLanesMenuCascade = nullptr;
 
     /// @brief Buttons showing and running values and triggering statistic windows
     std::vector<FXButton*> myStatButtons;
 
     /// @brief A window to display messages, warnings and error in
-    GUIMessageWindow* myMessageWindow;
+    GUIMessageWindow* myMessageWindow = nullptr;
 
-    /// @brief The splitter that divides the main window into vies and the log window
-    FXSplitter* myMainSplitter;
+    /// @brief The splitter that divides the main window into views and the log window
+    FXSplitter* myMainSplitter = nullptr;
 
     /// @brief for some menu detaching fun
-    FXToolBarShell* myToolBarDrag1, *myToolBarDrag2, *myToolBarDrag3,
-                    *myToolBarDrag4, *myToolBarDrag5, *myMenuBarDrag,
-                    *myToolBarDrag8;
+    FXToolBarShell* myToolBarDrag1 = nullptr, *myToolBarDrag2 = nullptr, *myToolBarDrag3 = nullptr,
+                    *myToolBarDrag4 = nullptr, *myToolBarDrag5 = nullptr, *myMenuBarDrag = nullptr,
+                     *myToolBarDrag8 = nullptr;
 
     /// @brief the simulation delay in milliseconds
-    double mySimDelay;
-    FXDataTarget* mySimDelayTarget;
-    FXRealSpinner* mySimDelaySpinner;
-    FXSlider* mySimDelaySlider;
+    double mySimDelay = 0.;
+    FXDataTarget* mySimDelayTarget = nullptr;
+    FXRealSpinner* mySimDelaySpinner = nullptr;
+    FXSlider* mySimDelaySlider = nullptr;
 
     /// @brief the demand scale
-    FXRealSpinner* myDemandScaleSpinner;
+    FXRealSpinner* myDemandScaleSpinner = nullptr;
 
     /// @brief The alternate simulation delay in milliseconds for toggling
     double myAlternateSimDelay;
 
     /// @brief List of got requests
-    MFXEventQue<GUIEvent*> myEvents;
+    FXSynchQue<GUIEvent*> myEvents;
 
     /// @brief The menu used for the MDI-windows
-    FXMDIMenu* myMDIMenu;
+    FXMDIMenu* myMDIMenu = nullptr;
 
     /// @brief The application menu bar
-    FXMenuBar* myMenuBar;
+    FXMenuBar* myMenuBar = nullptr;
 
     /// @brief The application tool bar
-    FXToolBar* myToolBar1, *myToolBar2, *myToolBar3, *myToolBar4, *myToolBar5, *myToolBar8;
+    FXToolBar* myToolBar1 = nullptr, *myToolBar2 = nullptr, *myToolBar3 = nullptr, *myToolBar4 = nullptr, *myToolBar5 = nullptr, *myToolBar8 = nullptr;
 
     /// @brief the simulation step display
-    FXEX::FXLCDLabel* myLCDLabel;
+    FXEX::FXLCDLabel* myLCDLabel = nullptr;
 
     /// @brief io-event with the load-thread
     FXEX::FXThreadEvent myLoadThreadEvent;
@@ -419,14 +431,20 @@ protected:
     /// @brief A random number generator used to choose a gaming sound
     static std::mt19937 myGamingRNG;
     int myPreviousCollisionNumber;
+    /// @brief current game mode
+    bool myTLSGame;
 
     /// @brief performance indicators
-    FXEX::FXLCDLabel* myWaitingTimeLabel;
-    FXEX::FXLCDLabel* myTimeLossLabel;
+    FXEX::FXLCDLabel* myWaitingTimeLabel = nullptr;
+    FXEX::FXLCDLabel* myTimeLossLabel = nullptr;
+    FXEX::FXLCDLabel* myTotalDistanceLabel = nullptr;
+    FXEX::FXLCDLabel* myEmergencyVehicleLabel = nullptr;
     SUMOTime myWaitingTime;
     SUMOTime myTimeLoss;
-    FXToolBar* myToolBar6, *myToolBar7;
-    FXToolBarShell* myToolBarDrag6, *myToolBarDrag7;
+    SUMOTime myEmergencyVehicleCount;
+    double myTotalDistance;
+    FXToolBar* myToolBar6 = nullptr, *myToolBar7 = nullptr, *myToolBar9 = nullptr, *myToolBar10 = nullptr;
+    FXToolBarShell* myToolBarDrag6 = nullptr, *myToolBarDrag7 = nullptr, *myToolBarDrag9 = nullptr, *myToolBarDrag10 = nullptr;
     ////}
 
 };
