@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // The thread that runs the simulation
 /****************************************************************************/
@@ -32,8 +31,7 @@
 #include <fx.h>
 #include <utils/foxtools/FXSingleEventThread.h>
 #include <utils/foxtools/FXThreadEvent.h>
-#include <utils/foxtools/MFXEventQue.h>
-#include <utils/common/MsgHandler.h>
+#include <utils/foxtools/FXSynchQue.h>
 #include <utils/common/SUMOTime.h>
 
 
@@ -59,7 +57,7 @@ class GUIRunThread : public FXSingleEventThread {
 public:
     /// constructor
     GUIRunThread(FXApp* app, MFXInterThreadEventClient* mw,
-                 double& simDelay, MFXEventQue<GUIEvent*>& eq, FXEX::FXThreadEvent& ev);
+                 double& simDelay, FXSynchQue<GUIEvent*>& eq, FXEX::FXThreadEvent& ev);
 
     /// destructor
     virtual ~GUIRunThread();
@@ -153,11 +151,11 @@ protected:
 
     double& mySimDelay;
 
-    MFXEventQue<GUIEvent*>& myEventQue;
+    FXSynchQue<GUIEvent*>& myEventQue;
 
     FXEX::FXThreadEvent& myEventThrow;
 
-    MFXMutex mySimulationLock;
+    FXMutex mySimulationLock;
 
     /// @brief List of breakpoints
     std::vector<SUMOTime> myBreakpoints;
