@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A single child window which contains a view of the simulation area
 /****************************************************************************/
@@ -109,6 +108,10 @@ public:
     /// @brief  locator-callback
     long onCmdLocate(FXObject*, FXSelector, void*);
 
+    /// @brief  speedFactor-callback
+    long onCmdSpeedFactor(FXObject*, FXSelector, void*);
+    long onUpdSpeedFactor(FXObject*, FXSelector, void*);
+
     /// @brief handle keys
     long onKeyPress(FXObject* o, FXSelector sel, void* data);
     long onKeyRelease(FXObject* o, FXSelector sel, void* data);
@@ -120,8 +123,15 @@ public:
     void setToolBarVisibility(const bool value);
 
 protected:
-    /// @brief FOX needs this
-    GUISUMOViewParent() { }
+    void buildSpeedControlToolbar();
+
+    FXToolBarShell* myToolBarDragSpeed = nullptr;
+    FXToolBar* myToolBarSpeed = nullptr;
+    FXSlider* mySpeedFactorSlider = nullptr;
+
+
+protected:
+    FOX_CONSTRUCTOR(GUISUMOViewParent)
 
 };
 
