@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Oct/Nov 2003
-/// @version $Id$
 ///
 // A window displaying the phase diagram of a tl-logic
 /****************************************************************************/
@@ -52,7 +51,6 @@
 class GUIMainWindow;
 class MSTrafficLightLogic;
 class GUITrafficLightLogicWrapper;
-class MFXMutex;
 
 
 // ===========================================================================
@@ -201,7 +199,7 @@ private:
     GUITLLogicPhasesTrackerPanel* myPanel;
 
     /// @brief A lock to avoid addition of new values while drawing
-    MFXMutex myLock;
+    FXMutex myLock;
 
     /** @brief The names of links
      *
@@ -250,4 +248,3 @@ protected:
 #endif
 
 /****************************************************************************/
-
