@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A view on the simulation; this view is a microscopic one
 /****************************************************************************/
@@ -84,11 +83,22 @@ public:
 
     bool setColorScheme(const std::string& name);
 
-    ///@brief recalibrate color scheme according to the current value range
-    void buildColorRainbow(const GUIVisualizationSettings& s, GUIColorScheme& scheme, int active, GUIGlObjectType objectType);
+    /// @brief recalibrate color scheme according to the current value range
+    void buildColorRainbow(const GUIVisualizationSettings& s, GUIColorScheme& scheme, int active, GUIGlObjectType objectType,
+                           bool hide = false, double hideThreshold = 0);
+
+    /// @brief return list of loaded edgeData attributes
+    std::vector<std::string> getEdgeDataAttrs() const;
+
+    /// @brief return list of available edge parameters
+    std::vector<std::string> getEdgeLaneParamKeys(bool edgeKeys) const;
+
+    /// @brief return list of available vehicle parameters
+    std::vector<std::string> getVehicleParamKeys(bool vTypeKeys) const;
 
     /// @brief handle mouse click in gaming mode
     void onGamingClick(Position pos);
+    void onGamingRightClick(Position pos);
 
     /// @brief get the current simulation time
     SUMOTime getCurrentTimeStep() const;
@@ -98,6 +108,9 @@ public:
     long onCmdCloseEdge(FXObject*, FXSelector, void*);
     long onCmdAddRerouter(FXObject*, FXSelector, void*);
 
+    /// @brief highlight edges according to reachability
+    long onCmdShowReachability(FXObject*, FXSelector, void*);
+
     long onDoubleClicked(FXObject*, FXSelector, void*);
 
     /** @brief Adds a frame to a video snapshot which will be initialized if neccessary
@@ -123,6 +136,9 @@ protected:
 private:
     GUIGlID myTrackedID;
 
+    /// @brief whether game mode was set to 'tls'
+    bool myTLSGame;
+
 #ifdef HAVE_FFMPEG
     GUIVideoEncoder* myCurrentVideo;
 #endif
