@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Thu, 17 Jun 2004
-/// @version $Id$
 ///
 // Editor for simulation breakpoints
 /****************************************************************************/
@@ -75,9 +74,10 @@ public:
     long onCmdEditTable(FXObject*, FXSelector, void*);
     /// @}
 
+    virtual void layout();
+
 protected:
-    /// FOX needs this
-    GUIDialog_Breakpoints() {}
+    FOX_CONSTRUCTOR(GUIDialog_Breakpoints)
 
 private:
     /// @brief Rebuilds the entire list
