@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Derivation of NLEdgeControlBuilder which build gui-edges
 /****************************************************************************/
@@ -52,8 +51,9 @@ GUIEdgeControlBuilder::addLane(const std::string& id,
                                const PositionVector& shape,
                                double width,
                                SVCPermissions permissions,
-                               int index, bool isRampAccel) {
-    MSLane* lane = new GUILane(id, maxSpeed, length, myActiveEdge, myCurrentNumericalLaneID++, shape, width, permissions, index, isRampAccel);
+                               int index, bool isRampAccel,
+                               const std::string& type) {
+    MSLane* lane = new GUILane(id, maxSpeed, length, myActiveEdge, myCurrentNumericalLaneID++, shape, width, permissions, index, isRampAccel, type);
     myLaneStorage->push_back(lane);
     myCurrentLaneIndex = index;
     return lane;
@@ -63,8 +63,8 @@ GUIEdgeControlBuilder::addLane(const std::string& id,
 
 MSEdge*
 GUIEdgeControlBuilder::buildEdge(const std::string& id, const SumoXMLEdgeFunc function,
-                                 const std::string& streetName, const std::string& edgeType, const int priority) {
-    return new GUIEdge(id, myCurrentNumericalEdgeID++, function, streetName, edgeType, priority);
+                                 const std::string& streetName, const std::string& edgeType, const int priority, const double distance) {
+    return new GUIEdge(id, myCurrentNumericalEdgeID++, function, streetName, edgeType, priority, distance);
 }
 
 /****************************************************************************/
