@@ -1,13 +1,11 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    GUIEdgeControlBuilder.h
 /// @author  Daniel Krajzewicz
@@ -15,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Derivation of NLEdgeControlBuilder which builds gui-edges
 /****************************************************************************/
@@ -26,11 +23,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <vector>
@@ -84,7 +77,8 @@ public:
                             double maxSpeed, double length,
                             const PositionVector& shape, double width,
                             SVCPermissions permissions,
-                            int index, bool isRampAccel);
+                            int index, bool isRampAccel,
+                            const std::string& type);
 
 
 
@@ -97,7 +91,7 @@ public:
      * @param[in] id The id of the edge to build
      */
     MSEdge* buildEdge(const std::string& id, const SumoXMLEdgeFunc function,
-                      const std::string& streetName, const std::string& edgeType, const int priority);
+                      const std::string& streetName, const std::string& edgeType, const int priority, const double distance);
 
 
 private:
