@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 07.12.2005
-/// @version $Id$
 ///
 // A lane area vehicles can halt at (gui-version)
 /****************************************************************************/
@@ -72,7 +71,7 @@ public:
      */
     GUIBusStop(const std::string& id,
                const std::vector<std::string>& lines, MSLane& lane,
-               double frompos, double topos, const std::string name);
+               double frompos, double topos, const std::string name, int personCapacity);
 
 
     /// @brief Destructor
@@ -116,12 +115,19 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
+    /// @brief Returns the street name
+    const std::string getOptionalName() const;
 
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
      * @see GUIGlObject::drawGL
      */
     void drawGL(const GUIVisualizationSettings& s) const;
+
+    /** @brief Returns the next free waiting place for pedestrians / containers
+     * @return The next free waiting place for pedestrians / containers
+     */
+    Position getWaitPosition(MSTransportable* person) const;
     //@}
 
 
@@ -141,9 +147,15 @@ private:
     /// @brief The rotation of the sign
     double myFGSignRot;
 
+    /// @brief The visual width of the stoppling place
+    double myWidth;
+
     /// @brief The coordinates of access points
     PositionVector myAccessCoords;
 
+    /// @brief The current person exaggeration
+    mutable double myPersonExaggeration;
+
 
 };
 
