@@ -1,20 +1,17 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    GUICalibrator.h
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id$
 ///
 // Changes flow and speed on a set of lanes (gui version)
 /****************************************************************************/
@@ -25,18 +22,13 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <vector>
 #include <string>
 #include <microsim/trigger/MSCalibrator.h>
 #include <utils/gui/globjects/GUIGlObject_AbstractAdd.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
-#include <utils/foxtools/FXRealSpinDial.h>
 #include <gui/GUIManipulator.h>
 
 
@@ -64,7 +56,9 @@ public:
                   const std::string& aXMLFilename,
                   const std::string& outputFilename,
                   const SUMOTime freq,
-                  const MSRouteProbe* probe);
+                  const MSRouteProbe* probe,
+                  const std::string& vTypes,
+                  bool addLaneMeanData = true);
 
 
     /** destructor */
@@ -172,7 +166,7 @@ public:
 
         FXDataTarget mySpeedTarget;
 
-        FXRealSpinDial* myUserDefinedSpeed;
+        FXRealSpinner* myUserDefinedSpeed;
 
         FXComboBox* myPredefinedValues;
 
