@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Wed, 01.08.2014
-/// @version $Id$
 ///
 // A MSContainer extended by some values for usage within the gui
 /****************************************************************************/
@@ -34,7 +33,6 @@
 #include <microsim/devices/MSDevice_Vehroutes.h>
 #include <utils/common/StringUtils.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
-#include <utils/common/AbstractMutex.h>
 #include <utils/geom/GeomHelper.h>
 #include <utils/gui/images/GUITexturesHelper.h>
 #include <utils/gui/windows/GUISUMOAbstractView.h>
@@ -56,23 +54,14 @@
 // ===========================================================================
 // FOX callback mapping
 // ===========================================================================
-/*
 FXDEFMAP(GUIContainer::GUIContainerPopupMenu) GUIContainerPopupMenuMap[] = {
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_ALLROUTES, GUIContainer::GUIContainerPopupMenu::onCmdShowAllRoutes),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_ALLROUTES, GUIContainer::GUIContainerPopupMenu::onCmdHideAllRoutes),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_CURRENTROUTE, GUIContainer::GUIContainerPopupMenu::onCmdShowCurrentRoute),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_CURRENTROUTE, GUIContainer::GUIContainerPopupMenu::onCmdHideCurrentRoute),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_BEST_LANES, GUIContainer::GUIContainerPopupMenu::onCmdShowBestLanes),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_BEST_LANES, GUIContainer::GUIContainerPopupMenu::onCmdHideBestLanes),
     FXMAPFUNC(SEL_COMMAND, MID_START_TRACK, GUIContainer::GUIContainerPopupMenu::onCmdStartTrack),
-    FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK, GUIContainer::GUIContainerPopupMenu::onCmdStopTrack),
-    FXMAPFUNC(SEL_COMMAND, MID_SHOW_LFLINKITEMS, GUIContainer::GUIContainerPopupMenu::onCmdShowLFLinkItems),
-    FXMAPFUNC(SEL_COMMAND, MID_HIDE_LFLINKITEMS, GUIContainer::GUIContainerPopupMenu::onCmdHideLFLinkItems),
+    FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK,  GUIContainer::GUIContainerPopupMenu::onCmdStopTrack),
+    FXMAPFUNC(SEL_COMMAND, MID_SHOWPLAN,    GUIContainer::GUIContainerPopupMenu::onCmdShowPlan),
 };
 
 // Object implementation
 FXIMPLEMENT(GUIContainer::GUIContainerPopupMenu, GUIGLObjectPopupMenu, GUIContainerPopupMenuMap, ARRAYNUMBER(GUIContainerPopupMenuMap))
-*/
 
 #define WATER_WAY_OFFSET 6.0
 
@@ -83,15 +72,50 @@ FXIMPLEMENT(GUIContainer::GUIContainerPopupMenu, GUIGLObjectPopupMenu, GUIContai
  * GUIContainer::GUIContainerPopupMenu - methods
  * ----------------------------------------------------------------------- */
 GUIContainer::GUIContainerPopupMenu::GUIContainerPopupMenu(
-    GUIMainWindow& app, GUISUMOAbstractView& parent,
-    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations)
-    : GUIGLObjectPopupMenu(app, parent, o), myVehiclesAdditionalVisualizations(additionalVisualizations) {
+    GUIMainWindow& app, GUISUMOAbstractView& parent, GUIGlObject& o)
+    : GUIGLObjectPopupMenu(app, parent, o) {
 }
 
 
 GUIContainer::GUIContainerPopupMenu::~GUIContainerPopupMenu() {}
 
 
+long
+GUIContainer::GUIContainerPopupMenu::onCmdShowPlan(FXObject*, FXSelector, void*) {
+    GUIContainer* p = dynamic_cast<GUIContainer*>(myObject);
+    if (p == nullptr) {
+        return 1;
+    }
+    GUIParameterTableWindow* ret = new GUIParameterTableWindow(*myApplication, *p, p->getNumStages());
+    // add items
+    for (int stage = 1; stage < p->getNumStages(); stage++) {
+        ret->mkItem(toString(stage).c_str(), false, p->getStageSummary(stage));
+    }
+    // close building (use an object that is not Parameterised as argument)
+    Parameterised dummy;
+    ret->closeBuilding(&dummy);
+    return 1;
+}
+
+
+long
+GUIContainer::GUIContainerPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    if (myParent->getTrackedID() != static_cast<GUIContainer*>(myObject)->getGlID()) {
+        myParent->startTrack(static_cast<GUIContainer*>(myObject)->getGlID());
+    }
+    return 1;
+}
+
+long
+GUIContainer::GUIContainerPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_PERSON);
+    myParent->stopTrack();
+    return 1;
+}
+
+
+
 
 /* -------------------------------------------------------------------------
  * GUIContainer - methods
@@ -109,13 +133,23 @@ GUIContainer::~GUIContainer() {
 GUIGLObjectPopupMenu*
 GUIContainer::getPopUpMenu(GUIMainWindow& app,
                            GUISUMOAbstractView& parent) {
-    GUIGLObjectPopupMenu* ret = new GUIContainerPopupMenu(app, parent, *this, myAdditionalVisualizations);
+    GUIGLObjectPopupMenu* ret = new GUIContainerPopupMenu(app, parent, *this);
     buildPopupHeader(ret, app);
     buildCenterPopupEntry(ret);
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
+    new FXMenuSeparator(ret);
+    if (parent.getTrackedID() != getGlID()) {
+        new FXMenuCommand(ret, "Start Tracking", nullptr, ret, MID_START_TRACK);
+    } else {
+        new FXMenuCommand(ret, "Stop Tracking", nullptr, ret, MID_STOP_TRACK);
+    }
     //
+
     buildShowParamsPopupEntry(ret);
+    buildShowTypeParamsPopupEntry(ret);
+    new FXMenuCommand(ret, "Show Plan", GUIIconSubSys::getIcon(ICON_APP_TABLE), ret, MID_SHOWPLAN);
+    new FXMenuSeparator(ret);
     buildPositionCopyEntry(ret, false);
     return ret;
 }
@@ -128,8 +162,11 @@ GUIContainer::getParameterWindow(GUIMainWindow& app,
         new GUIParameterTableWindow(app, *this, 12 + (int)getParameter().getParametersMap().size());
     // add items
     ret->mkItem("stage", false, getCurrentStageDescription());
+    // there is always the "start" stage which we do not count here because it is not strictly part of the plan
+    ret->mkItem("stage index", false, toString(getNumStages() - getNumRemainingStages()) + " of " + toString(getNumStages() - 1));
     ret->mkItem("start edge [id]", false, getFromEdge()->getID());
     ret->mkItem("dest edge [id]", false, getDestination()->getID());
+    ret->mkItem("arrivalPos [m]", false, toString(getCurrentStage()->getArrivalPos()));
     ret->mkItem("edge [id]", false, getEdge()->getID());
     ret->mkItem("position [m]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getEdgePos));
     ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIContainer, double>(this, &GUIContainer::getSpeed));
@@ -182,7 +219,6 @@ GUIContainer::drawGL(const GUIVisualizationSettings& s) const {
     }
     glTranslated(p1.x(), p1.y(), getType());
     glRotated(90, 0, 0, 1);
-    // XXX use container specific gui settings
     // set container color
     setColor(s);
     // scale
@@ -264,7 +300,7 @@ void
 GUIContainer::setColor(const GUIVisualizationSettings& s) const {
     const GUIColorer& c = s.containerColorer;
     if (!setFunctionalColor(c.getActive())) {
-        GLHelper::setColor(c.getScheme().getColor(getColorValue(c.getActive())));
+        GLHelper::setColor(c.getScheme().getColor(getColorValue(s, c.getActive())));
     }
 }
 
@@ -309,7 +345,7 @@ GUIContainer::setFunctionalColor(int activeScheme) const {
 
 
 double
-GUIContainer::getColorValue(int activeScheme) const {
+GUIContainer::getColorValue(const GUIVisualizationSettings& /* s */, int activeScheme) const {
     switch (activeScheme) {
         case 4:
             return getSpeed();
@@ -330,14 +366,14 @@ GUIContainer::getColorValue(int activeScheme) const {
 
 double
 GUIContainer::getEdgePos() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSContainer::getEdgePos();
 }
 
 
 Position
 GUIContainer::getPosition() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     if (getCurrentStageType() == WAITING && getEdge()->getPermissions() == SVC_SHIP) {
         MSLane* lane = getEdge()->getLanes().front();   //the most right lane of the water way
         PositionVector laneShape = lane->getShape();
@@ -349,21 +385,21 @@ GUIContainer::getPosition() const {
 
 double
 GUIContainer::getAngle() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSContainer::getAngle();
 }
 
 
 double
 GUIContainer::getWaitingSeconds() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSContainer::getWaitingSeconds();
 }
 
 
 double
 GUIContainer::getSpeed() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSContainer::getSpeed();
 }
 
