@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Wed, 01.08.2014
-/// @version $Id$
 ///
 // A lane area vehicles can halt at (gui-version)
 /****************************************************************************/
@@ -52,9 +51,9 @@
 // method definitions
 // ===========================================================================
 GUIContainerStop::GUIContainerStop(const std::string& id, const std::vector<std::string>& lines, MSLane& lane,
-                                   double frompos, double topos)
-    : MSStoppingPlace(id, lines, lane, frompos, topos),
-      GUIGlObject_AbstractAdd(GLO_CONTAINER_STOP, id) {
+                                   double frompos, double topos, const std::string& name, int containerCapacity) :
+    MSStoppingPlace(id, lines, lane, frompos, topos, name, containerCapacity),
+    GUIGlObject_AbstractAdd(GLO_CONTAINER_STOP, id) {
     const double offsetSign = MSNet::getInstance()->lefthand() ? -1 : 1;
     myFGShape = lane.getShape();
     myFGShape.move2side(1.65 * offsetSign);
@@ -122,11 +121,11 @@ GUIContainerStop::drawGL(const GUIVisualizationSettings& s) const {
     glPushMatrix();
     // draw the area
     glTranslated(0, 0, getType());
-    GLHelper::setColor(s.SUMO_color_containerStop);
+    GLHelper::setColor(s.colorSettings.containerStop);
     const double exaggeration = s.addSize.getExaggeration(s, this);
     GLHelper::drawBoxLines(myFGShape, myFGShapeRotations, myFGShapeLengths, 1.0);
     // draw details unless zoomed out to far
-    if (s.scale * exaggeration >= 10) {
+    if (s.drawDetail(s.detailSettings.stoppingPlaceDetails, exaggeration)) {
         glPushMatrix();
         // draw the lines
         const double rotSign = MSNet::getInstance()->lefthand() ? -1 : 1;
@@ -138,7 +137,7 @@ GUIContainerStop::drawGL(const GUIVisualizationSettings& s) const {
             glTranslated(myFGSignPos.x(), myFGSignPos.y(), 0);
             glRotated(rotSign * myFGSignRot, 0, 0, 1);
             // draw line
-            GLHelper::drawText(myLines[i].c_str(), Position(1.2, (double)i), .1, 1.f, s.SUMO_color_containerStop, 0, FONS_ALIGN_LEFT);
+            GLHelper::drawText(myLines[i].c_str(), Position(1.2, (double)i), .1, 1.f, s.colorSettings.containerStop, 0, FONS_ALIGN_LEFT);
             // pop matrix for every line
             glPopMatrix();
         }
@@ -151,20 +150,20 @@ GUIContainerStop::drawGL(const GUIVisualizationSettings& s) const {
         glScaled(exaggeration, exaggeration, 1);
         GLHelper::drawFilledCircle((double) 1.1, noPoints);
         glTranslated(0, 0, .1);
-        GLHelper::setColor(s.SUMO_color_containerStop_sign);
+        GLHelper::setColor(s.colorSettings.containerStop_sign);
         GLHelper::drawFilledCircle((double) 0.9, noPoints);
-        if (s.scale * exaggeration >= 4.5) {
-            GLHelper::drawText("C", Position(), .1, 1.6, s.SUMO_color_containerStop, myFGSignRot);
+        if (s.drawDetail(s.detailSettings.stoppingPlaceText, exaggeration)) {
+            GLHelper::drawText("C", Position(), .1, 1.6, s.colorSettings.containerStop, myFGSignRot);
         }
         glPopMatrix();
     }
     glPopMatrix();
     glPopName();
     drawName(getCenteringBoundary().getCenter(), s.scale, s.addName);
-    for (std::vector<MSTransportable*>::const_iterator i = myWaitingTransportables.begin(); i != myWaitingTransportables.end(); ++i) {
-        glTranslated(0, 1, 0); // make multiple containers viewable
-        static_cast<GUIContainer*>(*i)->drawGL(s);
-    }
+    //for (std::vector<MSTransportable*>::const_iterator i = myWaitingTransportables.begin(); i != myWaitingTransportables.end(); ++i) {
+    //    glTranslated(0, 1, 0); // make multiple containers viewable
+    //    static_cast<GUIContainer*>(*i)->drawGL(s);
+    //}
 }
 
 
@@ -175,6 +174,11 @@ GUIContainerStop::getCenteringBoundary() const {
     return b;
 }
 
+const std::string
+GUIContainerStop::getOptionalName() const {
+    return myName;
+}
+
 
 
 /****************************************************************************/
