@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Jan 2004
-/// @version $Id$
 ///
 // The gui-version of a MSE3Collector
 /****************************************************************************/
@@ -97,12 +96,12 @@ GUIE3Collector::MyWrapper::drawGL(const GUIVisualizationSettings& s) const {
     glTranslated(0, 0, GLO_JUNCTION + 0.4); // do not draw on top of linkRules
     typedef std::vector<SingleCrossingDefinition> CrossingDefinitions;
     CrossingDefinitions::const_iterator i;
-    GLHelper::setColor(s.SUMO_color_E3Entry);
+    GLHelper::setColor(s.colorSettings.E3Entry);
     const double exaggeration = s.addSize.getExaggeration(s, this);
     for (i = myEntryDefinitions.begin(); i != myEntryDefinitions.end(); ++i) {
         drawSingleCrossing((*i).myFGPosition, (*i).myFGRotation, exaggeration);
     }
-    GLHelper::setColor(s.SUMO_color_E3Exit);
+    GLHelper::setColor(s.colorSettings.E3Exit);
     for (i = myExitDefinitions.begin(); i != myExitDefinitions.end(); ++i) {
         drawSingleCrossing((*i).myFGPosition, (*i).myFGRotation, exaggeration);
     }
@@ -117,9 +116,9 @@ GUIE3Collector::MyWrapper::drawSingleCrossing(const Position& pos,
         double rot, double upscale) const {
     glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     glPushMatrix();
-    glScaled(upscale, upscale, 1);
     glTranslated(pos.x(), pos.y(), 0);
     glRotated(rot, 0, 0, 1);
+    glScaled(upscale, upscale, 1);
     glBegin(GL_LINES);
     glVertex2d(1.7, 0);
     glVertex2d(-1.7, 0);
