@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
@@ -28,10 +27,10 @@
 
 #include <vector>
 #include <string>
+#include <fx.h>
 #include <microsim/MSEdge.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/gui/settings/GUIPropertySchemeStorage.h>
-#include <utils/foxtools/MFXMutex.h>
 
 
 // ===========================================================================
@@ -59,7 +58,8 @@ public:
      */
     GUIEdge(const std::string& id, int numericalID,
             const SumoXMLEdgeFunc function,
-            const std::string& streetName, const std::string& edgeType, int priority);
+            const std::string& streetName, const std::string& edgeType, int priority,
+            double distance);
 
 
     /// @brief Destructor.
@@ -124,6 +124,8 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
+    /// @brief Returns the street name
+    const std::string getOptionalName() const;
 
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
@@ -134,23 +136,23 @@ public:
 
 
     void addPerson(MSTransportable* p) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::addPerson(p);
     }
 
     void removePerson(MSTransportable* p) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::removePerson(p);
     }
 
 
     void addContainer(MSTransportable* c) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::addContainer(c);
     }
 
     void removeContainer(MSTransportable* c) const {
-        AbstractMutex::ScopedLocker locker(myLock);
+        FXMutexLock locker(myLock);
         MSEdge::removeContainer(c);
     }
 
@@ -226,7 +228,7 @@ private:
 
 private:
     /// The mutex used to avoid concurrent updates of myPersons/ myContainers
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
 
     mutable RGBColor myMesoColor;
 
