@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 1 Jul 2003
-/// @version $Id$
 ///
 // Holds geometrical values for a junction
 /****************************************************************************/
@@ -63,7 +62,7 @@ public:
     /** @brief Constructor
      * @param[in] junction The represented junction
      */
-    GUIJunctionWrapper(MSJunction& junction);
+    GUIJunctionWrapper(MSJunction& junction, const std::string& tllID);
 
 
     /// @brief Destructor
@@ -103,6 +102,8 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
+    /// @brief Returns the value for generic parameter 'name' or ''
+    const std::string getOptionalName() const;
 
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
@@ -140,7 +141,7 @@ public:
 #endif
 
 private:
-    double getColorValue(const GUIVisualizationSettings& s) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
 private:
     /// @brief A reference to the represented junction
@@ -157,6 +158,11 @@ private:
 
     /// @brief whether this junction has only waterways as incoming and outgoing edges
     bool myAmWaterway;
+    /// @brief whether this junction has only railways as incoming and outgoing edges
+    bool myAmRailway;
+
+    /// @brief the associated traffic light or ""
+    const std::string myTLLID;
 
 #ifdef HAVE_OSG
     osg::Geometry* myGeom;
