@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GUIParkingArea.h
 /// @author  Mirco Sturari
 /// @date    Tue, 19.01.2016
-/// @version $Id$
 ///
 // A area where vehicles can park next to the road (gui version)
 /****************************************************************************/
@@ -119,9 +118,12 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
+    /// @brief Returns the stopping place name
+    const std::string getOptionalName() const;
+
     /// @brief extend boundary
     void addLotEntry(double x, double y, double z,
-            double width, double length, double angle);
+                     double width, double length, double angle);
 
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
