@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A MSPerson extended by some values for usage within the gui
 /****************************************************************************/
@@ -23,34 +22,17 @@
 // ===========================================================================
 #include <config.h>
 
-#include <cmath>
-#include <vector>
-#include <string>
-#include <microsim/MSVehicleControl.h>
-#include <microsim/MSVehicleType.h>
-#include <microsim/pedestrians/MSPerson.h>
-#include <microsim/pedestrians/MSPModel_Striping.h>
-#include <microsim/logging/CastingFunctionBinding.h>
-#include <microsim/logging/FunctionBinding.h>
-#include <microsim/devices/MSDevice_Vehroutes.h>
-#include <utils/common/StringUtils.h>
-#include <utils/vehicle/SUMOVehicleParameter.h>
-#include <utils/common/AbstractMutex.h>
-#include <utils/geom/GeomHelper.h>
-#include <utils/gui/images/GUITexturesHelper.h>
-#include <utils/gui/windows/GUISUMOAbstractView.h>
-#include <utils/gui/windows/GUIAppEnum.h>
-#include <utils/gui/div/GUIParameterTableWindow.h>
-#include <utils/gui/div/GUIGlobalSelection.h>
-#include <utils/gui/div/GLHelper.h>
-#include <utils/gui/div/GLObjectValuePassConnector.h>
-#include <utils/gui/globjects/GLIncludes.h>
-#include <utils/gui/images/GUIIconSubSys.h>
 #include <gui/GUIApplicationWindow.h>
-#include <gui/GUIGlobals.h>
+#include <microsim/MSTransportableControl.h>
+#include <microsim/logging/FunctionBinding.h>
+#include <microsim/pedestrians/MSPModel_Striping.h>
+#include <utils/gui/div/GLHelper.h>
+#include <utils/gui/div/GUIGlobalSelection.h>
+#include <utils/gui/div/GUIParameterTableWindow.h>
+#include <utils/gui/globjects/GLIncludes.h>
+#include <utils/gui/div/GUIBasePersonHelper.h>
+
 #include "GUILane.h"
-#include "GUINet.h"
-#include "GUIEdge.h"
 #include "GUIPerson.h"
 
 //#define GUIPerson_DEBUG_DRAW_WALKINGAREA_PATHS 1
@@ -58,6 +40,7 @@
 // ===========================================================================
 // FOX callback mapping
 // ===========================================================================
+
 FXDEFMAP(GUIPerson::GUIPersonPopupMenu) GUIPersonPopupMenuMap[] = {
     FXMAPFUNC(SEL_COMMAND, MID_SHOW_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute),
     FXMAPFUNC(SEL_COMMAND, MID_HIDE_CURRENTROUTE,     GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute),
@@ -66,29 +49,29 @@ FXDEFMAP(GUIPerson::GUIPersonPopupMenu) GUIPersonPopupMenuMap[] = {
     FXMAPFUNC(SEL_COMMAND, MID_SHOWPLAN,              GUIPerson::GUIPersonPopupMenu::onCmdShowPlan),
     FXMAPFUNC(SEL_COMMAND, MID_START_TRACK,           GUIPerson::GUIPersonPopupMenu::onCmdStartTrack),
     FXMAPFUNC(SEL_COMMAND, MID_STOP_TRACK,            GUIPerson::GUIPersonPopupMenu::onCmdStopTrack),
+    FXMAPFUNC(SEL_COMMAND, MID_REMOVE_OBJECT,         GUIPerson::GUIPersonPopupMenu::onCmdRemoveObject),
 };
 
 // Object implementation
 FXIMPLEMENT(GUIPerson::GUIPersonPopupMenu, GUIGLObjectPopupMenu, GUIPersonPopupMenuMap, ARRAYNUMBER(GUIPersonPopupMenuMap))
 
-
-
 // ===========================================================================
 // method definitions
 // ===========================================================================
-/* -------------------------------------------------------------------------
- * GUIPerson::GUIPersonPopupMenu - methods
- * ----------------------------------------------------------------------- */
+
+// -------------------------------------------------------------------------
+// GUIPerson::GUIPersonPopupMenu - methods
+// -------------------------------------------------------------------------
+
 GUIPerson::GUIPersonPopupMenu::GUIPersonPopupMenu(
-    GUIMainWindow& app, GUISUMOAbstractView& parent,
-    GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations) :
-    GUIGLObjectPopupMenu(app, parent, o),
-    myVehiclesAdditionalVisualizations(additionalVisualizations) {
+    GUIMainWindow& app, GUISUMOAbstractView& parent, GUIGlObject& o) :
+    GUIGLObjectPopupMenu(app, parent, o) {
 }
 
 
 GUIPerson::GUIPersonPopupMenu::~GUIPersonPopupMenu() {}
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -98,6 +81,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdShowCurrentRoute(FXObject*, FXSelector, void
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -106,7 +90,6 @@ GUIPerson::GUIPersonPopupMenu::onCmdHideCurrentRoute(FXObject*, FXSelector, void
 }
 
 
-
 long
 GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -116,6 +99,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdShowWalkingareaPath(FXObject*, FXSelector, v
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdHideWalkingareaPath(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -136,8 +120,8 @@ GUIPerson::GUIPersonPopupMenu::onCmdShowPlan(FXObject*, FXSelector, void*) {
         ret->mkItem(toString(stage).c_str(), false, p->getStageSummary(stage));
     }
     // close building (use an object that is not Parameterised as argument)
-    Parameterised dummy;
-    ret->closeBuilding(&dummy);
+    Parameterised dummyParameterised;
+    ret->closeBuilding(&dummyParameterised);
     return 1;
 }
 
@@ -151,6 +135,7 @@ GUIPerson::GUIPersonPopupMenu::onCmdStartTrack(FXObject*, FXSelector, void*) {
     return 1;
 }
 
+
 long
 GUIPerson::GUIPersonPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
     assert(myObject->getType() == GLO_PERSON);
@@ -159,11 +144,24 @@ GUIPerson::GUIPersonPopupMenu::onCmdStopTrack(FXObject*, FXSelector, void*) {
 }
 
 
+long
+GUIPerson::GUIPersonPopupMenu::onCmdRemoveObject(FXObject*, FXSelector, void*) {
+    GUIPerson* person = static_cast<GUIPerson*>(myObject);
+    MSTransportable::Stage* stage = person->getCurrentStage();
+    stage->abort(person);
+    stage->getEdge()->removePerson(person);
+    if (stage->getDestinationStop() != nullptr) {
+        stage->getDestinationStop()->removeTransportable(person);
+    }
+    MSNet::getInstance()->getPersonControl().erase(person);
+    myParent->update();
+    return 1;
+}
 
+// -------------------------------------------------------------------------
+// GUIPerson - methods
+// -------------------------------------------------------------------------
 
-/* -------------------------------------------------------------------------
- * GUIPerson - methods
- * ----------------------------------------------------------------------- */
 GUIPerson::GUIPerson(const SUMOVehicleParameter* pars, MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan, const double speedFactor) :
     MSPerson(pars, vtype, plan, speedFactor),
     GUIGlObject(GLO_PERSON, pars->id),
@@ -184,9 +182,8 @@ GUIPerson::~GUIPerson() {
 
 
 GUIGLObjectPopupMenu*
-GUIPerson::getPopUpMenu(GUIMainWindow& app,
-                        GUISUMOAbstractView& parent) {
-    GUIGLObjectPopupMenu* ret = new GUIPersonPopupMenu(app, parent, *this, myAdditionalVisualizations);
+GUIPerson::getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) {
+    GUIGLObjectPopupMenu* ret = new GUIPersonPopupMenu(app, parent, *this);
     buildPopupHeader(ret, app);
     buildCenterPopupEntry(ret);
     buildNameCopyPopupEntry(ret);
@@ -207,6 +204,7 @@ GUIPerson::getPopUpMenu(GUIMainWindow& app,
     } else {
         new FXMenuCommand(ret, "Stop Tracking", nullptr, ret, MID_STOP_TRACK);
     }
+    new FXMenuCommand(ret, "Remove", nullptr, ret, MID_REMOVE_OBJECT);
     new FXMenuSeparator(ret);
     //
     buildShowParamsPopupEntry(ret);
@@ -224,13 +222,13 @@ GUIPerson::getParameterWindow(GUIMainWindow& app,
     GUIParameterTableWindow* ret =
         new GUIParameterTableWindow(app, *this, 12 + (int)getParameter().getParametersMap().size());
     // add items
-    ret->mkItem("stage", false, getCurrentStageDescription());
+    ret->mkItem("stage", true, new FunctionBindingString<GUIPerson>(this, &MSTransportable::getCurrentStageDescription));
     // there is always the "start" stage which we do not count here because it is not strictly part of the plan
-    ret->mkItem("stage index", false, toString(getNumStages() - getNumRemainingStages()) + " of " + toString(getNumStages() - 1));
-    ret->mkItem("start edge [id]", false, getFromEdge()->getID());
-    ret->mkItem("dest edge [id]", false, getDestination()->getID());
-    ret->mkItem("arrivalPos [m]", false, toString(getCurrentStage()->getArrivalPos()));
-    ret->mkItem("edge [id]", false, getEdge()->getID());
+    ret->mkItem("stage index", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getStageIndexDescription));
+    ret->mkItem("start edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getFromEdgeID));
+    ret->mkItem("dest edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getDestinationEdgeID));
+    ret->mkItem("arrivalPos [m]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getStageArrivalPos));
+    ret->mkItem("edge [id]", true, new FunctionBindingString<GUIPerson>(this, &GUIPerson::getEdgeID));
     ret->mkItem("position [m]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getEdgePos));
     ret->mkItem("speed [m/s]", true, new FunctionBinding<GUIPerson, double>(this, &GUIPerson::getSpeed));
     ret->mkItem("speed factor", false, getSpeedFactor());
@@ -282,19 +280,22 @@ GUIPerson::drawGL(const GUIVisualizationSettings& s) const {
     // set person color
     setColor(s);
     // scale
-    const double upscale = s.personSize.getExaggeration(s, this, 80);
-    glScaled(upscale, upscale, 1);
+    const double exaggeration = s.personSize.getExaggeration(s, this, 80);
+    glScaled(exaggeration, exaggeration, 1);
     switch (s.personQuality) {
         case 0:
+            GUIBasePersonHelper::drawAction_drawAsTriangle(getAngle(), getVehicleType().getLength(), getVehicleType().getWidth());
+            break;
         case 1:
-            drawAction_drawAsTriangle(s);
+            GUIBasePersonHelper::drawAction_drawAsCircle(getVehicleType().getLength(), getVehicleType().getWidth());
             break;
         case 2:
-            drawAction_drawAsPoly(s);
+            GUIBasePersonHelper::drawAction_drawAsPoly(getAngle(), getVehicleType().getLength(), getVehicleType().getWidth());
             break;
         case 3:
         default:
-            drawAction_drawAsImage(s);
+            GUIBasePersonHelper::drawAction_drawAsImage(getAngle(), getVehicleType().getLength(), getVehicleType().getWidth(),
+                    getVehicleType().getImgFile(), getVehicleType().getGuiShape(), exaggeration);
             break;
     }
     glPopMatrix();
@@ -304,12 +305,13 @@ GUIPerson::drawGL(const GUIVisualizationSettings& s) const {
     drawName(p1, s.scale, s.personName, s.angle);
     if (s.personValue.show) {
         Position p2 = p1 + Position(0, 0.6 * s.personName.scaledSize(s.scale));
-        const double value = getColorValue(s.personColorer.getActive());
+        const double value = getColorValue(s, s.personColorer.getActive());
         GLHelper::drawTextSettings(s.personValue, toString(value), p2, s.scale, s.angle, GLO_MAX - getType());
     }
     glPopName();
 }
 
+
 void
 GUIPerson::drawAction_drawWalkingareaPath(const GUIVisualizationSettings& s) const {
     MSPersonStage_Walking* stage = dynamic_cast<MSPersonStage_Walking*>(getCurrentStage());
@@ -328,6 +330,7 @@ GUIPerson::drawAction_drawWalkingareaPath(const GUIVisualizationSettings& s) con
     }
 }
 
+
 void
 GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
@@ -357,13 +360,17 @@ GUIPerson::drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualiz
 }
 
 
+void
+GUIPerson::setPositionInVehicle(const Position& pos) {
+    myPositionInVehicle = pos;
+}
 
 
 void
 GUIPerson::setColor(const GUIVisualizationSettings& s) const {
     const GUIColorer& c = s.personColorer;
     if (!setFunctionalColor(c.getActive())) {
-        GLHelper::setColor(c.getScheme().getColor(getColorValue(c.getActive())));
+        GLHelper::setColor(c.getScheme().getColor(getColorValue(s, c.getActive())));
     }
 }
 
@@ -414,7 +421,7 @@ GUIPerson::setFunctionalColor(int activeScheme) const {
 
 
 double
-GUIPerson::getColorValue(int activeScheme) const {
+GUIPerson::getColorValue(const GUIVisualizationSettings& /* s */, int activeScheme) const {
     switch (activeScheme) {
         case 4:
             return getSpeed();
@@ -435,21 +442,21 @@ GUIPerson::getColorValue(int activeScheme) const {
 
 double
 GUIPerson::getEdgePos() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSPerson::getEdgePos();
 }
 
 
 Position
 GUIPerson::getPosition() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSPerson::getPosition();
 }
 
 
 Position
 GUIPerson::getGUIPosition() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     if (getCurrentStageType() == DRIVING && !isWaiting4Vehicle() && myPositionInVehicle != Position::INVALID) {
         return myPositionInVehicle;
     } else {
@@ -460,95 +467,63 @@ GUIPerson::getGUIPosition() const {
 
 double
 GUIPerson::getNaviDegree() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return GeomHelper::naviDegree(MSPerson::getAngle());
 }
 
 
 double
 GUIPerson::getWaitingSeconds() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSPerson::getWaitingSeconds();
 }
 
 
 double
 GUIPerson::getSpeed() const {
-    AbstractMutex::ScopedLocker locker(myLock);
+    FXMutexLock locker(myLock);
     return MSPerson::getSpeed();
 }
 
 
-void
-GUIPerson::drawAction_drawAsTriangle(const GUIVisualizationSettings& /* s */) const {
-    // draw triangle pointing forward
-    glRotated(RAD2DEG(getAngle() + M_PI / 2.), 0, 0, 1);
-    glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0., 0.);
-    glVertex2d(1, -0.5);
-    glVertex2d(1, 0.5);
-    glEnd();
-    // draw a smaller triangle to indicate facing
-    GLHelper::setColor(GLHelper::getColor().changedBrightness(-64));
-    glTranslated(0, 0, .045);
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0., 0.);
-    glVertex2d(0.5, -0.25);
-    glVertex2d(0.5, 0.25);
-    glEnd();
-    glTranslated(0, 0, -.045);
+std::string
+GUIPerson::getStageIndexDescription() const {
+    FXMutexLock locker(myLock);
+    return toString(getNumStages() - getNumRemainingStages()) + " of " + toString(getNumStages() - 1);
 }
 
 
-void
-GUIPerson::drawAction_drawAsPoly(const GUIVisualizationSettings& /* s */) const {
-    // draw pedestrian shape
-    glRotated(GeomHelper::naviDegree(getAngle()) - 180, 0, 0, -1);
-    glScaled(getVehicleType().getLength(), getVehicleType().getWidth(), 1);
-    RGBColor lighter = GLHelper::getColor().changedBrightness(51);
-    glTranslated(0, 0, .045);
-    // head
-    glScaled(1, 0.5, 1.);
-    GLHelper::drawFilledCircle(0.5);
-    // nose
-    glBegin(GL_TRIANGLES);
-    glVertex2d(0.0, -0.2);
-    glVertex2d(0.0, 0.2);
-    glVertex2d(-0.6, 0.0);
-    glEnd();
-    glTranslated(0, 0, -.045);
-    // body
-    glScaled(0.9, 2.0, 1);
-    glTranslated(0, 0, .04);
-    GLHelper::setColor(lighter);
-    GLHelper::drawFilledCircle(0.5);
-    glTranslated(0, 0, -.04);
+std::string
+GUIPerson::getEdgeID() const {
+    FXMutexLock locker(myLock);
+    return  getEdge()->getID();
 }
 
 
-void
-GUIPerson::drawAction_drawAsImage(const GUIVisualizationSettings& s) const {
-    const std::string& file = getVehicleType().getImgFile();
-    if (file != "") {
-        if (getVehicleType().getGuiShape() == SVS_PEDESTRIAN) {
-            glRotated(RAD2DEG(getAngle() + M_PI / 2.), 0, 0, 1);
-        }
-        int textureID = GUITexturesHelper::getTextureID(file);
-        if (textureID > 0) {
-            const double exaggeration = s.personSize.getExaggeration(s, this);
-            const double halfLength = getVehicleType().getLength() / 2.0 * exaggeration;
-            const double halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
-            GUITexturesHelper::drawTexturedBox(textureID, -halfWidth, -halfLength, halfWidth, halfLength);
-        }
-    } else {
-        // fallback if no image is defined
-        drawAction_drawAsPoly(s);
-    }
+std::string
+GUIPerson::getFromEdgeID() const {
+    FXMutexLock locker(myLock);
+    return getFromEdge()->getID();
 }
 
 
-// ------------ Additional visualisations
+std::string
+GUIPerson::getDestinationEdgeID() const {
+    FXMutexLock locker(myLock);
+    return getDestination()->getID();
+}
+
+
+double
+GUIPerson::getStageArrivalPos() const {
+    FXMutexLock locker(myLock);
+    return getCurrentStage()->getArrivalPos();
+}
+
+// -------------------------------------------------------------------------
+// GUIPerson - Additional Visualsation methods
+// -------------------------------------------------------------------------
+
 bool
 GUIPerson::hasActiveAddVisualisation(GUISUMOAbstractView* const parent, int which) const {
     return myAdditionalVisualizations.find(parent) != myAdditionalVisualizations.end() && (myAdditionalVisualizations.find(parent)->second & which) != 0;
