@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
@@ -29,11 +28,11 @@
 #include <vector>
 #include <set>
 #include <string>
+#include <fx.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <utils/common/RGBColor.h>
 #include <microsim/pedestrians/MSPerson.h>
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
-#include <utils/foxtools/MFXMutex.h>
 #include <utils/gui/settings/GUIPropertySchemeStorage.h>
 
 
@@ -52,19 +51,16 @@ class MSDevice_Vehroutes;
  * @class GUIPerson
  */
 class GUIPerson : public MSPerson, public GUIGlObject {
-public:
-    /** @brief Constructor
-     */
-    GUIPerson(const SUMOVehicleParameter* pars, MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan, const double speedFactor);
 
+public:
+    /// @brief Constructor
+    GUIPerson(const SUMOVehicleParameter* pars, MSVehicleType* vtype, MSTransportable::MSTransportablePlan* plan, const double speedFactor);
 
     /// @brief destructor
     ~GUIPerson();
 
-
     /// @name inherited from GUIGlObject
-    //@{
-
+    /// @{
     /** @brief Returns an own popup-menu
      *
      * @param[in] app The application needed to build the popup-menu
@@ -74,7 +70,6 @@ public:
      */
     GUIGLObjectPopupMenu* getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent);
 
-
     /** @brief Returns an own parameter window
      *
      * @param[in] app The application needed to build the parameter window
@@ -99,30 +94,25 @@ public:
      */
     Boundary getCenteringBoundary() const;
 
-
     /** @brief Draws the object
      * @param[in] s The settings for the current view (may influence drawing)
      * @see GUIGlObject::drawGL
      */
     void drawGL(const GUIVisualizationSettings& s) const;
 
-
     /** @brief Draws additionally triggered visualisations
      * @param[in] parent The view
      * @param[in] s The settings for the current view (may influence drawing)
      */
     virtual void drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualizationSettings& s) const;
-    //@}
+    //* @}
 
     /* @brief set the position of a person while riding in a vehicle
      * @note This must be called by the vehicle before the call to drawGl */
-    void setPositionInVehicle(const Position& pos) {
-        myPositionInVehicle = pos;
-    }
+    void setPositionInVehicle(const Position& pos);
 
     /// @name inherited from MSPerson with added locking
-    //@{
-
+    /// @{
     /// @brief return the offset from the start of the current edge
     double getEdgePos() const;
 
@@ -142,6 +132,21 @@ public:
     /// @brief the current speed of the person
     double getSpeed() const;
 
+    /// @brief get stage index description
+    std::string getStageIndexDescription() const;
+
+    /// @brief get edge ID
+    std::string getEdgeID() const;
+
+    /// @brief ger from edge ID
+    std::string getFromEdgeID() const;
+
+    /// @brief get destination edge ID
+    std::string getDestinationEdgeID() const;
+
+    /// @brief get stage arrival position
+    double getStageArrivalPos() const;
+
     //@}
 
     /// @brief whether this person is selected in the GUI
@@ -155,50 +160,48 @@ public:
      */
     class GUIPersonPopupMenu : public GUIGLObjectPopupMenu {
         FXDECLARE(GUIPersonPopupMenu)
+
     public:
         /** @brief Constructor
          * @param[in] app The main window for instantiation of other windows
          * @param[in] parent The parent view for changing it
          * @param[in] o The object of interest
-         * @param[in, out] additionalVisualizations Information which additional visualisations are enabled (per view)
          */
-        GUIPersonPopupMenu(GUIMainWindow& app,
-                           GUISUMOAbstractView& parent, GUIGlObject& o, std::map<GUISUMOAbstractView*, int>& additionalVisualizations);
+        GUIPersonPopupMenu(GUIMainWindow& app, GUISUMOAbstractView& parent, GUIGlObject& o);
 
         /// @brief Destructor
         ~GUIPersonPopupMenu();
 
         /// @brief Called if the current route of the person shall be shown
         long onCmdShowCurrentRoute(FXObject*, FXSelector, void*);
+
         /// @brief Called if the current route of the person shall be hidden
         long onCmdHideCurrentRoute(FXObject*, FXSelector, void*);
+
         /// @brief Called if the walkingarea path of the person shall be shown
         long onCmdShowWalkingareaPath(FXObject*, FXSelector, void*);
+
         /// @brief Called if the walkingarea path of the person shall be hidden
         long onCmdHideWalkingareaPath(FXObject*, FXSelector, void*);
+
         /// @brief Called if the plan shall be shown
         long onCmdShowPlan(FXObject*, FXSelector, void*);
+
         /// @brief Called if the person shall be tracked
         long onCmdStartTrack(FXObject*, FXSelector, void*);
+
         /// @brief Called if the person shall not be tracked any longer
         long onCmdStopTrack(FXObject*, FXSelector, void*);
 
-
-    protected:
-        /// @brief Information which additional visualisations are enabled (per view)
-        std::map<GUISUMOAbstractView*, int>& myVehiclesAdditionalVisualizations;
-        /// @brief Needed for parameterless instantiation
-        std::map<GUISUMOAbstractView*, int> dummy;
+        /// @brief Called when removing the person
+        long onCmdRemoveObject(FXObject*, FXSelector, void*);
 
     protected:
         /// @brief default constructor needed by FOX
-        GUIPersonPopupMenu() : myVehiclesAdditionalVisualizations(dummy) { }
-
+        FOX_CONSTRUCTOR(GUIPersonPopupMenu)
     };
 
-
-    /** @brief Additional visualisation feature ids
-     */
+    /// @brief Additional visualisation feature ids
     enum VisualisationFeatures {
         /// @brief show the current walkingarea path
         VO_SHOW_WALKINGAREA_PATH = 1,
@@ -208,13 +211,8 @@ public:
         VO_TRACKED = 8
     };
 
-    /// @brief Enabled visualisations, per view
-    std::map<GUISUMOAbstractView*, int> myAdditionalVisualizations;
-
-
     /// @name Additional visualisations
     /// @{
-
     /** @brief Returns whether the named feature is enabled in the given view
      * @param[in] parent The view for which the feature may be enabled
      * @param[in] which The visualisation feature
@@ -222,7 +220,6 @@ public:
      */
     bool hasActiveAddVisualisation(GUISUMOAbstractView* const parent, int which) const;
 
-
     /** @brief Adds the named visualisation feature to the given view
      * @param[in] parent The view for which the feature shall be enabled
      * @param[in] which The visualisation feature to enable
@@ -230,40 +227,36 @@ public:
      */
     void addActiveAddVisualisation(GUISUMOAbstractView* const parent, int which);
 
-
     /** @brief Adds the named visualisation feature to the given view
      * @param[in] parent The view for which the feature shall be enabled
      * @param[in] which The visualisation feature to enable
      * @see GUISUMOAbstractView::removeAdditionalGLVisualisation
      */
     void removeActiveAddVisualisation(GUISUMOAbstractView* const parent, int which);
+
     /// @}
 
-
 private:
-    /// The mutex used to avoid concurrent updates of the vehicle buffer
-    mutable MFXMutex myLock;
-
-    /// The position of a person while riding a vehicle
-    Position myPositionInVehicle;
-
     /// @brief sets the color according to the currente settings
     void setColor(const GUIVisualizationSettings& s) const;
 
     /// @brief gets the color value according to the current scheme index
-    double getColorValue(int activeScheme) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
     /// @brief sets the color according to the current scheme index and some vehicle function
     bool setFunctionalColor(int activeScheme) const;
 
-    /// @name drawing helper methods
-    /// @{
-    void drawAction_drawAsTriangle(const GUIVisualizationSettings& s) const;
-    void drawAction_drawAsPoly(const GUIVisualizationSettings& s) const;
-    void drawAction_drawAsImage(const GUIVisualizationSettings& s) const;
-
+    /// @brief draw walking area path
     void drawAction_drawWalkingareaPath(const GUIVisualizationSettings& s) const;
-    /// @}
+
+    /// The mutex used to avoid concurrent updates of the vehicle buffer
+    mutable FXMutex myLock;
+
+    /// The position of a person while riding a vehicle
+    Position myPositionInVehicle;
+
+    /// @brief Enabled visualisations, per view
+    std::map<GUISUMOAbstractView*, int> myAdditionalVisualizations;
 };
 
 
