@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Oct/Nov 2003
-/// @version $Id$
 ///
 // A wrapper for tl-logics to allow their visualisation and interaction
 /****************************************************************************/
@@ -37,6 +36,7 @@
 #include <microsim/MSLane.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
+#include <microsim/traffic_lights/MSActuatedTrafficLightLogic.h>
 #include <microsim/logging/FunctionBinding.h>
 #include <microsim/logging/FuncBinding_StringParam.h>
 #include <gui/GUIApplicationWindow.h>
@@ -54,6 +54,7 @@ FXDEFMAP(GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu)
 GUITrafficLightLogicWrapperPopupMenuMap[] = {
     FXMAPFUNC(SEL_COMMAND,  MID_SHOWPHASES,             GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowPhases),
     FXMAPFUNC(SEL_COMMAND,  MID_TRACKPHASES,            GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdBegin2TrackPhases),
+    FXMAPFUNC(SEL_COMMAND,  MID_SHOW_DETECTORS,         GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowDetectors),
     FXMAPFUNC(SEL_COMMAND,  MID_SWITCH_OFF,             GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLS2Off),
     FXMAPFUNCS(SEL_COMMAND, MID_SWITCH, MID_SWITCH + 20, GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLSLogic),
 };
@@ -95,6 +96,16 @@ GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowPhas
     return 1;
 }
 
+long
+GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdShowDetectors(
+    FXObject*, FXSelector, void*) {
+    assert(myObject->getType() == GLO_TLLOGIC);
+    GUITrafficLightLogicWrapper* w = static_cast<GUITrafficLightLogicWrapper*>(myObject);
+    MSActuatedTrafficLightLogic* act = dynamic_cast<MSActuatedTrafficLightLogic*>(&w->getTLLogic());
+    assert(act != 0);
+    act->setShowDetectors(!act->showDetectors());
+    return 1;
+}
 
 long
 GUITrafficLightLogicWrapper::GUITrafficLightLogicWrapperPopupMenu::onCmdSwitchTLS2Off(
@@ -151,11 +162,19 @@ GUITrafficLightLogicWrapper::getPopUpMenu(GUIMainWindow& app,
     new FXMenuCommand(ret, "Switch off", GUIIconSubSys::getIcon(ICON_FLAG_MINUS), ret, MID_SWITCH_OFF);
     new FXMenuCommand(ret, "Track Phases", nullptr, ret, MID_TRACKPHASES);
     new FXMenuCommand(ret, "Show Phases", nullptr, ret, MID_SHOWPHASES);
+    MSActuatedTrafficLightLogic* act = dynamic_cast<MSActuatedTrafficLightLogic*>(&myTLLogic);
+    if (act != nullptr) {
+        new FXMenuCommand(ret, act->showDetectors() ? "Hide Detectors" : "Show Detectors", nullptr, ret, MID_SHOW_DETECTORS);
+    }
     new FXMenuSeparator(ret);
-    MSTrafficLightLogic* tll = myTLLogicControl.getActive(myTLLogic.getID());
+    MSTrafficLightLogic* tll = getActiveTLLogic();
     buildNameCopyPopupEntry(ret);
     buildSelectionPopupEntry(ret);
     new FXMenuCommand(ret, ("phase: " + toString(tll->getCurrentPhaseIndex())).c_str(), nullptr, nullptr, 0);
+    const std::string& name =  tll->getCurrentPhaseDef().getName();
+    if (name != "") {
+        new FXMenuCommand(ret, ("phase name: " + name).c_str(), nullptr, nullptr, 0);
+    }
     new FXMenuSeparator(ret);
     buildShowParamsPopupEntry(ret, false);
     buildPositionCopyEntry(ret, false);
@@ -189,9 +208,15 @@ GUIParameterTableWindow*
 GUITrafficLightLogicWrapper::getParameterWindow(GUIMainWindow& app,
         GUISUMOAbstractView&) {
     GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 3 + (int)myTLLogic.getParametersMap().size());
+        new GUIParameterTableWindow(app, *this, 9 + (int)myTLLogic.getParametersMap().size());
     ret->mkItem("tlLogic [id]", false, myTLLogic.getID());
     ret->mkItem("program", false, myTLLogic.getProgramID());
+    ret->mkItem("phase", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentPhase));
+    ret->mkItem("phase name", true, new FunctionBindingString<GUITrafficLightLogicWrapper>(this, &GUITrafficLightLogicWrapper::getCurrentPhaseName));
+    ret->mkItem("duration", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentDuration));
+    ret->mkItem("minDur", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentMinDur));
+    ret->mkItem("maxDur", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getCurrentMaxDur));
+    ret->mkItem("running duration", true, new FunctionBinding<GUITrafficLightLogicWrapper, int>(this, &GUITrafficLightLogicWrapper::getRunningDuration));
     // close building
     ret->closeBuilding(&myTLLogic);
     return ret;
@@ -217,8 +242,7 @@ void
 GUITrafficLightLogicWrapper::switchTLSLogic(int to) {
     if (to == -1) {
         myTLLogicControl.switchTo(myTLLogic.getID(), "off");
-        MSTrafficLightLogic* tll = myTLLogicControl.getActive(myTLLogic.getID());
-        GUINet::getGUIInstance()->createTLWrapper(tll);
+        GUINet::getGUIInstance()->createTLWrapper(getActiveTLLogic());
     } else {
         const MSTLLogicControl::TLSLogicVariants& vars = myTLLogicControl.get(myTLLogic.getID());
         std::vector<MSTrafficLightLogic*> logics = vars.getAllLogics();
@@ -280,6 +304,41 @@ GUITrafficLightLogicWrapper::drawGL(const GUIVisualizationSettings& s) const {
     }
 }
 
+MSTrafficLightLogic*
+GUITrafficLightLogicWrapper::getActiveTLLogic() const {
+    return myTLLogicControl.getActive(myTLLogic.getID());
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentPhase() const {
+    return getActiveTLLogic()->getCurrentPhaseIndex();
+}
+
+std::string
+GUITrafficLightLogicWrapper::getCurrentPhaseName() const {
+    return getActiveTLLogic()->getCurrentPhaseDef().getName();
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentDuration() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().duration);
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentMinDur() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().minDuration);
+}
+
+int
+GUITrafficLightLogicWrapper::getCurrentMaxDur() const {
+    return (int)STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().maxDuration);
+}
+
+int
+GUITrafficLightLogicWrapper::getRunningDuration() const {
+    return (int)(SIMTIME - STEPS2TIME(getActiveTLLogic()->getCurrentPhaseDef().myLastSwitch));
+}
+
 
 /****************************************************************************/
 
