@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 25.07.2005
-/// @version $Id$
 ///
 // Reroutes vehicles passing an edge (gui version)
 /****************************************************************************/
@@ -33,6 +32,8 @@
 #include <microsim/MSNet.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSEdge.h>
+#include <microsim/MSRoute.h>
+#include <microsim/MSVehicle.h>
 #include <guisim/GUINet.h>
 #include <guisim/GUIEdge.h>
 #include "GUITriggeredRerouter.h"
@@ -69,6 +70,7 @@ FXDEFMAP(GUITriggeredRerouter::GUIManip_TriggeredRerouter) GUIManip_TriggeredRer
     FXMAPFUNC(SEL_COMMAND,  GUITriggeredRerouter::GUIManip_TriggeredRerouter::MID_USER_DEF, GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdUserDef),
     FXMAPFUNC(SEL_UPDATE,   GUITriggeredRerouter::GUIManip_TriggeredRerouter::MID_USER_DEF, GUITriggeredRerouter::GUIManip_TriggeredRerouter::onUpdUserDef),
     FXMAPFUNC(SEL_COMMAND,  GUITriggeredRerouter::GUIManip_TriggeredRerouter::MID_OPTION,   GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdChangeOption),
+    FXMAPFUNC(SEL_COMMAND,  GUITriggeredRerouter::GUIManip_TriggeredRerouter::MID_SHIFT_PROBS,  GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdShiftProbs),
     FXMAPFUNC(SEL_COMMAND,  GUITriggeredRerouter::GUIManip_TriggeredRerouter::MID_CLOSE,    GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdClose),
 };
 
@@ -93,7 +95,7 @@ GUITriggeredRerouter::GUIManip_TriggeredRerouter::GUIManip_TriggeredRerouter(
     FXVerticalFrame* f1 =
         new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0);
 
-    FXGroupBox* gp = new FXGroupBox(f1, "Change Probability",
+    FXGroupBox* gp = new FXGroupBox(f1, "Change Trigger Probability",
                                     GROUPBOX_TITLE_LEFT | FRAME_SUNKEN | FRAME_RIDGE,
                                     0, 0, 0, 0,  4, 4, 1, 1, 2, 0);
     {
@@ -132,8 +134,16 @@ GUITriggeredRerouter::GUIManip_TriggeredRerouter::GUIManip_TriggeredRerouter(
                     ? myObject->getUserProbability() > 0
                     ? 1 : 2
                     : 0;
+
+    FXGroupBox* gp2 = new FXGroupBox(f1, "Change Route Probability",
+                                     GROUPBOX_TITLE_LEFT | FRAME_SUNKEN | FRAME_RIDGE,
+                                     0, 0, 0, 0,  4, 4, 1, 1, 2, 0);
+    new FXButton(gp2, "Shift", nullptr, this, MID_SHIFT_PROBS,
+                 BUTTON_INITIAL | BUTTON_DEFAULT | FRAME_RAISED | FRAME_THICK | LAYOUT_TOP | LAYOUT_LEFT | LAYOUT_CENTER_X, 0, 0, 0, 0, 30, 30, 4, 4);
+
     new FXButton(f1, "Close", nullptr, this, MID_CLOSE,
                  BUTTON_INITIAL | BUTTON_DEFAULT | FRAME_RAISED | FRAME_THICK | LAYOUT_TOP | LAYOUT_LEFT | LAYOUT_CENTER_X, 0, 0, 0, 0, 30, 30, 4, 4);
+
 }
 
 
@@ -188,6 +198,14 @@ GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdChangeOption(FXObject*, F
     return 1;
 }
 
+long
+GUITriggeredRerouter::GUIManip_TriggeredRerouter::onCmdShiftProbs(FXObject*, FXSelector, void*) {
+    static_cast<GUITriggeredRerouter*>(myObject)->shiftProbs();
+    myParent->updateChildren();
+    return 1;
+}
+
+
 
 /* -------------------------------------------------------------------------
  * GUITriggeredRerouter::GUITriggeredRerouterPopupMenu - methods
@@ -217,10 +235,11 @@ GUITriggeredRerouter::GUITriggeredRerouterPopupMenu::onCmdOpenManip(FXObject*,
 GUITriggeredRerouter::GUITriggeredRerouter(const std::string& id, const MSEdgeVector& edges, double prob,
         const std::string& aXMLFilename, bool off, SUMOTime timeThreshold, const std::string& vTypes, SUMORTree& rtree) :
     MSTriggeredRerouter(id, edges, prob, aXMLFilename, off, timeThreshold, vTypes),
-    GUIGlObject_AbstractAdd(GLO_REROUTER, id) {
+    GUIGlObject_AbstractAdd(GLO_REROUTER, id),
+    myShiftProbDistIndex(0) {
     // add visualisation objects for edges which trigger the rerouter
     for (MSEdgeVector::const_iterator it = edges.begin(); it != edges.end(); ++it) {
-        myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, false));
+        myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, REROUTER_TRIGGER_EDGE));
         rtree.addAdditionalGLObject(myEdgeVisualizations.back());
         myBoundary.add(myEdgeVisualizations.back()->getCenteringBoundary());
     }
@@ -242,10 +261,43 @@ GUITriggeredRerouter::myEndElement(int element) {
         // add visualisation objects for closed edges
         const RerouteInterval& ri = myIntervals.back();
         for (MSEdgeVector::const_iterator it = ri.closed.begin(); it != ri.closed.end(); ++it) {
-            myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, true));
+            myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, REROUTER_CLOSED_EDGE));
             dynamic_cast<GUINet*>(GUINet::getInstance())->getVisualisationSpeedUp().addAdditionalGLObject(myEdgeVisualizations.back());
             myBoundary.add(myEdgeVisualizations.back()->getCenteringBoundary());
         }
+        // add visualisation objects for switches
+        if (ri.routeProbs.getProbs().size() > 1) {
+            // find last common edge of all routes
+            const MSRoute* route0 = ri.routeProbs.getVals()[0];
+            const MSEdge* lastEdge = nullptr;
+            int nextIndex = 0;
+            for (int i = 0; i < (int)route0->getEdges().size(); i++) {
+                const MSEdge* cand = route0->getEdges()[i];
+                for (const MSRoute* route : ri.routeProbs.getVals()) {
+                    const MSEdge* nextEdge = i < (int)route->getEdges().size() ? route->getEdges()[i] : nullptr;
+                    if (nextEdge != cand) {
+                        cand = nullptr;
+                    }
+                }
+                if (cand != nullptr) {
+                    lastEdge = cand;
+                } else {
+                    nextIndex = i;
+                    break;
+                }
+            }
+            if (lastEdge != nullptr) {
+                for (int i = 0; i < (int)ri.routeProbs.getVals().size(); i++) {
+                    const ConstMSEdgeVector& edges = ri.routeProbs.getVals()[i]->getEdges();
+                    if (nextIndex < (int)edges.size()) {
+                        GUIEdge* edge = dynamic_cast<GUIEdge*>(const_cast<MSEdge*>(edges[nextIndex]));
+                        myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(edge, this, REROUTER_SWITCH_EDGE, i));
+                        dynamic_cast<GUINet*>(GUINet::getInstance())->getVisualisationSpeedUp().addAdditionalGLObject(myEdgeVisualizations.back());
+                        myBoundary.add(myEdgeVisualizations.back()->getCenteringBoundary());
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -297,20 +349,48 @@ GUITriggeredRerouter::openManipulator(GUIMainWindow& app,
 }
 
 
+void
+GUITriggeredRerouter::shiftProbs() {
+    const RerouteInterval* const ri = getCurrentReroute(MSNet::getInstance()->getCurrentTimeStep());
+    if (ri != nullptr && ri->routeProbs.getProbs().size() > 1) {
+        auto& rp = const_cast<RandomDistributor<const MSRoute*>&>(ri->routeProbs);
+        myShiftProbDistIndex = myShiftProbDistIndex % rp.getProbs().size();
+        double prob = rp.getProbs()[myShiftProbDistIndex];
+        rp.add(rp.getVals()[myShiftProbDistIndex], -prob);
+        myShiftProbDistIndex = (myShiftProbDistIndex + 1) % rp.getProbs().size();
+        rp.add(rp.getVals()[myShiftProbDistIndex], prob);
+        // notify vehicles currently on a trigger edge
+        for (auto rrEdge : myEdgeVisualizations) {
+            if (rrEdge->getRerouterEdgeType() == REROUTER_TRIGGER_EDGE) {
+                if (!MSGlobals::gUseMesoSim) {
+                    for (MSLane* lane : rrEdge->getEdge()->getLanes()) {
+                        for (const MSVehicle* veh : lane->getVehiclesSecure()) {
+                            const_cast<MSVehicle*>(veh)->addReminder(this);
+                        }
+                        lane->releaseVehicles();
+                    }
+                }
+            }
+        }
+    }
+}
+
+
 /* -------------------------------------------------------------------------
  * GUITriggeredRerouterEdge - methods
  * ----------------------------------------------------------------------- */
-GUITriggeredRerouter::GUITriggeredRerouterEdge::GUITriggeredRerouterEdge(GUIEdge* edge, GUITriggeredRerouter* parent, bool closed) :
+GUITriggeredRerouter::GUITriggeredRerouterEdge::GUITriggeredRerouterEdge(GUIEdge* edge, GUITriggeredRerouter* parent, RerouterEdgeType edgeType, int distIndex) :
     GUIGlObject(GLO_REROUTER_EDGE, parent->getID() + ":" + edge->getID()),
     myParent(parent),
     myEdge(edge),
-    myAmClosedEdge(closed) {
+    myEdgeType(edgeType),
+    myDistIndex(distIndex) {
     const std::vector<MSLane*>& lanes = edge->getLanes();
     myFGPositions.reserve(lanes.size());
     myFGRotations.reserve(lanes.size());
     for (std::vector<MSLane*>::const_iterator i = lanes.begin(); i != lanes.end(); ++i) {
         const PositionVector& v = (*i)->getShape();
-        const double pos = closed ? 3 : v.length() - (double) 6.;
+        const double pos = edgeType == REROUTER_TRIGGER_EDGE ? v.length() - (double) 6. : 3;
         myFGPositions.push_back((*i)->geometryPositionAtOffset(pos));
         myFGRotations.push_back(-v.rotationDegreeAtOffset(pos));
         myBoundary.add(myFGPositions.back());
@@ -341,7 +421,7 @@ GUITriggeredRerouter::GUITriggeredRerouterEdge::drawGL(const GUIVisualizationSet
     if (s.scale * exaggeration >= 3) {
         glPushName(getGlID());
         const double prob = myParent->getProbability();
-        if (myAmClosedEdge) {
+        if (myEdgeType == REROUTER_CLOSED_EDGE) {
             // draw closing symbol onto all lanes
             const RerouteInterval* const ri =
                 myParent->getCurrentReroute(MSNet::getInstance()->getCurrentTimeStep());
@@ -386,7 +466,7 @@ GUITriggeredRerouter::GUITriggeredRerouterEdge::drawGL(const GUIVisualizationSet
                 }
             }
 
-        } else {
+        } else if (myEdgeType == REROUTER_TRIGGER_EDGE) {
             // draw rerouter symbol onto all lanes
             for (int i = 0; i < (int)myFGPositions.size(); ++i) {
                 const Position& pos = myFGPositions[i];
@@ -417,6 +497,40 @@ GUITriggeredRerouter::GUITriggeredRerouterEdge::drawGL(const GUIVisualizationSet
 
                 glPopMatrix();
             }
+        } else if (myEdgeType == REROUTER_SWITCH_EDGE) {
+            const RerouteInterval* const ri =
+                myParent->getCurrentReroute(MSNet::getInstance()->getCurrentTimeStep());
+            const double routeProb = ri != nullptr && prob > 0 ?  ri->routeProbs.getProbs()[myDistIndex] / ri->routeProbs.getOverallProb() : 0;
+            if (routeProb > 0) {
+                for (int i = 0; i < (int)myFGPositions.size(); ++i) {
+                    const Position& pos = myFGPositions[i];
+                    double rot = myFGRotations[i];
+                    glPushMatrix();
+                    glTranslated(pos.x(), pos.y(), 0);
+                    glRotated(rot, 0, 0, 1);
+                    glTranslated(0, 0, getType());
+                    glScaled(exaggeration, exaggeration, 1);
+                    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+
+                    glBegin(GL_TRIANGLES);
+                    glColor3d(0, 1, 1);
+                    // base
+                    glVertex2d(0 - 0.0, 0);
+                    glVertex2d(0 - 1.4, 6);
+                    glVertex2d(0 + 1.4, 6);
+                    glVertex2d(0 + 0.0, 0);
+                    glVertex2d(0 + 1.4, 6);
+                    glEnd();
+
+                    // draw "P"
+                    GLHelper::drawText("P", Position(0, 3.5), .1, 2, RGBColor::BLACK, 180);
+
+                    // draw Probability for this target edge
+                    GLHelper::drawText((toString((int)(routeProb * 100)) + "%").c_str(), Position(0, 5), .1, 0.7, RGBColor::BLACK, 180);
+
+                    glPopMatrix();
+                }
+            }
         }
         glPopName();
     }
@@ -431,5 +545,11 @@ GUITriggeredRerouter::GUITriggeredRerouterEdge::getCenteringBoundary() const {
 }
 
 
+void
+GUITriggeredRerouter::GUITriggeredRerouterEdge::onLeftBtnPress(void* /*data*/) {
+    myParent->shiftProbs();
+}
+
+
 /****************************************************************************/
 
