@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
@@ -39,6 +38,7 @@
 #include <utils/gui/div/GLHelper.h>
 #include <utils/gui/div/GLObjectValuePassConnector.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
+#include <utils/gui/div/GUIBaseVehicleHelper.h>
 #include <microsim/MSGlobals.h>
 #include <microsim/MSVehicle.h>
 #include <microsim/MSJunction.h>
@@ -47,10 +47,13 @@
 #include <microsim/logging/FunctionBinding.h>
 #include <microsim/lcmodels/MSAbstractLaneChangeModel.h>
 #include <microsim/devices/MSDevice_Vehroutes.h>
+#include <microsim/devices/MSDevice_Routing.h>
+#include <microsim/devices/MSRoutingEngine.h>
 #include <microsim/devices/MSDevice_Transportable.h>
 #include <microsim/devices/MSDevice_BTreceiver.h>
 #include <gui/GUIApplicationWindow.h>
 #include <gui/GUIGlobals.h>
+
 #include "GUIVehicle.h"
 #include "GUIPerson.h"
 #include "GUIContainer.h"
@@ -93,16 +96,14 @@ GUIVehicle::getParameterWindow(GUIMainWindow& app,
                                GUISUMOAbstractView&) {
     const int sublaneParams = MSGlobals::gLateralResolution > 0 ? 4 : 0;
     GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 37 + sublaneParams + (int)getParameter().getParametersMap().size());
+        new GUIParameterTableWindow(app, *this, 39 + sublaneParams + (int)getParameter().getParametersMap().size());
     // add items
-    ret->mkItem("lane [id]", false, Named::getIDSecure(myLane, "n/a"));
+    ret->mkItem("lane [id]", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getLaneID));
     if (MSAbstractLaneChangeModel::haveLateralDynamics()) {
-        const MSLane* shadowLane = getLaneChangeModel().getShadowLane();
-        ret->mkItem("shadow lane [id]", false, shadowLane == nullptr ? "" : shadowLane->getID());
+        ret->mkItem("shadow lane [id]", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getShadowLaneID));
     }
     if (MSGlobals::gLateralResolution > 0) {
-        const MSLane* targetLane = getLaneChangeModel().getTargetLane();
-        ret->mkItem("target lane [id]", false, targetLane == nullptr ? "" : targetLane->getID());
+        ret->mkItem("target lane [id]", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getTargetLaneID));
     }
     ret->mkItem("position [m]", true,
                 new FunctionBinding<GUIVehicle, double>(this, &MSVehicle::getPositionOnLane));
@@ -142,7 +143,7 @@ GUIVehicle::getParameterWindow(GUIMainWindow& app,
     if (getParameter().repetitionProbability > 0) {
         ret->mkItem("insertion probability", false, getParameter().repetitionProbability);
     }
-    ret->mkItem("stop info", false, getStopInfo());
+    ret->mkItem("stop info", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getStopInfo));
     ret->mkItem("line", false, myParameter->line);
     ret->mkItem("CO2 [mg/s]", true,
                 new FunctionBinding<GUIVehicle, double>(this, &MSVehicle::getCO2Emissions));
@@ -165,6 +166,8 @@ GUIVehicle::getParameterWindow(GUIMainWindow& app,
                 new FunctionBinding<GUIVehicle, int>(this, &MSVehicle::getPersonNumber));
     ret->mkItem("containers", true,
                 new FunctionBinding<GUIVehicle, int>(this, &MSVehicle::getContainerNumber));
+    ret->mkItem("lcState right", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getLCStateRight));
+    ret->mkItem("lcState left", true, new FunctionBindingString<GUIVehicle>(this, &GUIVehicle::getLCStateLeft));
     // close building
     if (MSGlobals::gLateralResolution > 0) {
         ret->mkItem("right side on edge [m]", true, new FunctionBinding<GUIVehicle, double>(this, &GUIVehicle::getRightSideOnEdge2));
@@ -182,7 +185,7 @@ GUIParameterTableWindow*
 GUIVehicle::getTypeParameterWindow(GUIMainWindow& app,
                                    GUISUMOAbstractView&) {
     GUIParameterTableWindow* ret =
-        new GUIParameterTableWindow(app, *this, 25
+        new GUIParameterTableWindow(app, *this, 27
                                     + (int)myType->getParameter().getParametersMap().size()
                                     + (int)myType->getParameter().lcParameter.size()
                                     + (int)myType->getParameter().jmParameter.size());
@@ -197,6 +200,7 @@ GUIVehicle::getTypeParameterWindow(GUIMainWindow& app,
     ret->mkItem("emission class", false, PollutantsInterface::getName(myType->getEmissionClass()));
     ret->mkItem("carFollowModel", false, SUMOXMLDefinitions::CarFollowModels.getString((SumoXMLTag)getCarFollowModel().getModelID()));
     ret->mkItem("LaneChangeModel", false, SUMOXMLDefinitions::LaneChangeModels.getString(getLaneChangeModel().getModelID()));
+    ret->mkItem("guiShape", false, getVehicleShapeName(myType->getGuiShape()));
     ret->mkItem("maximum speed [m/s]", false, getMaxSpeed());
     ret->mkItem("maximum acceleration [m/s^2]", false, getCarFollowModel().getMaxAccel());
     ret->mkItem("maximum deceleration [m/s^2]", false, getCarFollowModel().getMaxDecel());
@@ -204,6 +208,9 @@ GUIVehicle::getTypeParameterWindow(GUIMainWindow& app,
     ret->mkItem("apparent deceleration [m/s^2]", false, getCarFollowModel().getApparentDecel());
     ret->mkItem("imperfection (sigma)", false, getCarFollowModel().getImperfection());
     ret->mkItem("desired headway (tau)", false, getCarFollowModel().getHeadwayTime());
+    if (myType->getParameter().wasSet(VTYPEPARS_ACTIONSTEPLENGTH_SET)) {
+        ret->mkItem("action step length [s]", false, myType->getActionStepLengthSecs());
+    }
     ret->mkItem("person capacity", false, myType->getPersonCapacity());
     ret->mkItem("boarding time", false, STEPS2TIME(myType->getBoardingDuration()));
     ret->mkItem("container capacity", false, myType->getContainerCapacity());
@@ -221,6 +228,9 @@ GUIVehicle::getTypeParameterWindow(GUIMainWindow& app,
     for (auto item : myType->getParameter().jmParameter) {
         ret->mkItem(toString(item.first).c_str(), false, toString(item.second));
     }
+    if (MSGlobals::gModelParkingManoeuver) {
+        ret->mkItem("manoeuver Angle vs Times", false, myType->getParameter().getManoeuverAngleTimesS());
+    }
 
     // close building
     ret->closeBuilding(&(myType->getParameter()));
@@ -259,28 +269,152 @@ GUIVehicle::drawAction_drawLinkItems(const GUIVisualizationSettings& s) const {
 }
 
 
-bool
-GUIVehicle::drawAction_drawCarriageClass(const GUIVisualizationSettings& s, SUMOVehicleShape guiShape, bool asImage) const {
-    switch (guiShape) {
-        case SVS_BUS_FLEXIBLE:
-            drawAction_drawRailCarriages(s, 8.25, 0, 0, asImage); // 16.5 overall, 2 modules http://de.wikipedia.org/wiki/Ikarus_180
-            break;
-        case SVS_RAIL:
-            drawAction_drawRailCarriages(s, 24.5, 1, 1, asImage); // http://de.wikipedia.org/wiki/UIC-Y-Wagen_%28DR%29
-            break;
-        case SVS_RAIL_CAR:
-            drawAction_drawRailCarriages(s, 16.85, 1, 0, asImage); // 67.4m overall, 4 carriages http://de.wikipedia.org/wiki/DB-Baureihe_423
-//            drawAction_drawRailCarriages(s, 5.71, 0, 0, asImage); // 40.0m overall, 7 modules http://de.wikipedia.org/wiki/Bombardier_Flexity_Berlin
-//            drawAction_drawRailCarriages(s, 9.44, 1, 1, asImage); // actually length of the locomotive http://de.wikipedia.org/wiki/KJI_Nr._20_und_21
-//            drawAction_drawRailCarriages(s, 24.775, 0, 0, asImage); // http://de.wikipedia.org/wiki/ICE_3
-            break;
-        case SVS_RAIL_CARGO:
-            drawAction_drawRailCarriages(s, 13.86, 1, 0, asImage); // UIC 571-1 http://de.wikipedia.org/wiki/Flachwagen
-            break;
-        default:
-            return false;
+void
+GUIVehicle::drawAction_drawCarriageClass(const GUIVisualizationSettings& s, bool asImage) const {
+    RGBColor current = GLHelper::getColor();
+    RGBColor darker = current.changedBrightness(-51);
+    const double exaggeration = s.vehicleSize.getExaggeration(s, this);
+    const double totalLength = getVType().getLength();
+    double upscaleLength = exaggeration;
+    if (exaggeration > 1 && totalLength > 5) {
+        // reduce the length/width ratio because this is not usefull at high zoom
+        upscaleLength = MAX2(1.0, upscaleLength * (5 + sqrt(totalLength - 5)) / totalLength);
+    }
+    const double locomotiveLength = getVehicleType().getParameter().locomotiveLength * upscaleLength;
+    if (exaggeration == 0) {
+        return;
+    }
+    const double defaultLength = getVehicleType().getParameter().carriageLength * upscaleLength;
+    const double carriageGap = getVehicleType().getParameter().carriageGap * upscaleLength;
+    const double length = totalLength * upscaleLength;
+    const double halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
+    glPopMatrix(); // undo initial translation and rotation
+    const double xCornerCut = 0.3 * exaggeration;
+    const double yCornerCut = 0.4 * exaggeration;
+    // round to closest integer
+    const int numCarriages = MAX2(1, 1 + (int)((length - locomotiveLength) / (defaultLength + carriageGap) + 0.5));
+    assert(numCarriages > 0);
+    double carriageLengthWithGap = length / numCarriages;
+    double carriageLength = carriageLengthWithGap - carriageGap;
+    double firstCarriageLength = carriageLength;
+    if (defaultLength != locomotiveLength && numCarriages > 1) {
+        firstCarriageLength = locomotiveLength;
+        carriageLengthWithGap = (length - locomotiveLength) / (numCarriages - 1);
+        carriageLength = carriageLengthWithGap - carriageGap;
+    }
+    const int firstPassengerCarriage = defaultLength == locomotiveLength || numCarriages == 1 ? 0 : 1;
+    const int totalSeats = getVType().getPersonCapacity() + getVType().getContainerCapacity();
+    const int seatsPerCarriage = (int)ceil(totalSeats / (numCarriages - firstPassengerCarriage));
+    // lane on which the carriage front is situated
+    MSLane* lane = myLane;
+    int furtherIndex = 0;
+    // lane on which the carriage back is situated
+    MSLane* backLane = myLane;
+    int backFurtherIndex = furtherIndex;
+    // offsets of front and back
+    double carriageOffset = myState.pos();
+    double carriageBackOffset = myState.pos() - firstCarriageLength;
+    // handle seats
+    int requiredSeats = getNumPassengers() + getNumContainers();
+    if (requiredSeats > 0) {
+        mySeatPositions.clear();
+    }
+    Position front, back;
+    double angle = 0.;
+    // draw individual carriages
+    double curCLength = firstCarriageLength;
+    //std::cout << SIMTIME << " veh=" << getID() << " curCLength=" << curCLength << " loc=" << locomotiveLength << " car=" << carriageLength << " tlen=" << totalLength << " len=" << length << "\n";
+    for (int i = 0; i < numCarriages; ++i) {
+        if (i > 0) {
+            curCLength = carriageLength;
+        }
+        while (carriageOffset < 0) {
+            MSLane* prev = getPreviousLane(lane, furtherIndex);
+            if (prev != lane) {
+                carriageOffset += prev->getLength();
+            } else {
+                // no lane available for drawing.
+                carriageOffset = 0;
+            }
+            lane = prev;
+        }
+        while (carriageBackOffset < 0) {
+            MSLane* prev = getPreviousLane(backLane, backFurtherIndex);
+            if (prev != backLane) {
+                carriageBackOffset += prev->getLength();
+            } else {
+                // no lane available for drawing.
+                carriageBackOffset = 0;
+            }
+            backLane = prev;
+        }
+        front = lane->geometryPositionAtOffset(carriageOffset);
+        back = backLane->geometryPositionAtOffset(carriageBackOffset);
+        if (front == back) {
+            // no place for drawing available
+            continue;
+        }
+        const double drawnCarriageLength = front.distanceTo2D(back);
+        angle = atan2((front.x() - back.x()), (back.y() - front.y())) * (double) 180.0 / (double) M_PI;
+        if (i >= firstPassengerCarriage) {
+            computeSeats(front, back, seatsPerCarriage, exaggeration, requiredSeats);
+        }
+        glPushMatrix();
+        glTranslated(front.x(), front.y(), getType());
+        glRotated(angle, 0, 0, 1);
+        if (!asImage || !GUIBaseVehicleHelper::drawAction_drawVehicleAsImage(s, getVType().getImgFile(), this, getVType().getWidth(), curCLength / exaggeration)) {
+            switch (getVType().getGuiShape()) {
+                case SVS_TRUCK_SEMITRAILER:
+                case SVS_TRUCK_1TRAILER:
+                    if (i == 0) {
+                        GUIBaseVehicleHelper::drawAction_drawVehicleAsPoly(s, getVType().getGuiShape(), getVType().getWidth() * exaggeration, curCLength, i);
+                    } else {
+                        GLHelper::setColor(current);
+                        GLHelper::drawBoxLine(Position(0, 0), 180, curCLength, halfWidth);
+                    }
+                    break;
+                default: {
+                    if (i == 0) {
+                        GLHelper::setColor(darker);
+                    } else {
+                        GLHelper::setColor(current);
+                    }
+                    // generic rail carriage
+                    glBegin(GL_TRIANGLE_FAN);
+                    glVertex2d(-halfWidth + xCornerCut, 0);
+                    glVertex2d(-halfWidth, yCornerCut);
+                    glVertex2d(-halfWidth, drawnCarriageLength - yCornerCut);
+                    glVertex2d(-halfWidth + xCornerCut, drawnCarriageLength);
+                    glVertex2d(halfWidth - xCornerCut, drawnCarriageLength);
+                    glVertex2d(halfWidth, drawnCarriageLength - yCornerCut);
+                    glVertex2d(halfWidth, yCornerCut);
+                    glVertex2d(halfWidth - xCornerCut, 0);
+                    glEnd();
+                }
+            }
+        }
+        glPopMatrix();
+        carriageOffset -= (curCLength + carriageGap);
+        carriageBackOffset -= carriageLengthWithGap;
+    }
+    if (getVType().getGuiShape() == SVS_RAIL_CAR) {
+        glPushMatrix();
+        glTranslated(front.x(), front.y(), getType());
+        glRotated(angle, 0, 0, 1);
+        drawAction_drawVehicleBlinker(curCLength);
+        drawAction_drawVehicleBrakeLight(curCLength);
+        glPopMatrix();
+    }
+    // restore matrix
+    glPushMatrix();
+    front = getPosition();
+    glTranslated(front.x(), front.y(), getType());
+    const double degAngle = RAD2DEG(getAngle() + M_PI / 2.);
+    glRotated(degAngle, 0, 0, 1);
+    glScaled(exaggeration, upscaleLength, 1);
+    if (mySeatPositions.size() == 0) {
+        mySeatPositions.push_back(back);
     }
-    return true;
 }
 
 #define BLINKER_POS_FRONT .5
@@ -310,7 +444,7 @@ GUIVehicle::drawAction_drawVehicleBlinker(double length) const {
         drawAction_drawBlinker(-offset, length);
     }
     if (signalSet(MSVehicle::VEH_SIGNAL_BLINKER_LEFT)) {
-        drawAction_drawBlinker(offset, length);;
+        drawAction_drawBlinker(offset, length);
     }
     if (signalSet(MSVehicle::VEH_SIGNAL_BLINKER_EMERGENCY)) {
         drawAction_drawBlinker(-offset, length);
@@ -353,7 +487,7 @@ GUIVehicle::drawAction_drawVehicleBlueLight() const {
 
 
 double
-GUIVehicle::getColorValue(int activeScheme) const {
+GUIVehicle::getColorValue(const GUIVisualizationSettings& s, int activeScheme) const {
     switch (activeScheme) {
         case 8:
             return getSpeed();
@@ -410,6 +544,19 @@ GUIVehicle::getColorValue(int activeScheme) const {
             return getElectricityConsumption();
         case 28:
             return getTimeLossSeconds();
+        case 29:
+            return getLaneChangeModel().getSpeedLat();
+        case 31: // by numerical param value
+            try {
+                return StringUtils::toDouble(myParameter->getParameter(s.vehicleParam, "0"));
+            } catch (NumberFormatException&) {
+                try {
+                    return StringUtils::toBool(myParameter->getParameter(s.vehicleParam, "0"));
+                } catch (BoolFormatException&) {
+                    WRITE_WARNING("Vehicle parameter '" + myParameter->getParameter(s.vehicleParam, "0") + "' key '" + s.vehicleParam + "' is not a number for vehicle '" + getID() + "'");
+                    return -1;
+                }
+            }
     }
     return 0;
 }
@@ -451,12 +598,13 @@ GUIVehicle::drawBestLanes() const {
 
 
 void
-GUIVehicle::drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r) const {
-    const double exaggeration = s.vehicleSize.getExaggeration(s, this);
-    MSRouteIterator i = r.begin();
+GUIVehicle::drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r, bool future, const RGBColor& col) const {
+    const double exaggeration = s.vehicleSize.getExaggeration(s, this) * (s.gaming ? 0.5 : 1);
+    MSRouteIterator i = future ? myCurrEdge : r.begin();
     const std::vector<MSLane*>& bestLaneConts = getBestLanesContinuation();
     // draw continuation lanes when drawing the current route where available
     int bestLaneIndex = (&r == myRoute ? 0 : (int)bestLaneConts.size());
+    std::map<const MSLane*, int> repeatLane; // count repeated occurrences of the same edge
     for (; i != r.end(); ++i) {
         const GUILane* lane;
         if (bestLaneIndex < (int)bestLaneConts.size() && bestLaneConts[bestLaneIndex] != 0 && (*i) == &(bestLaneConts[bestLaneIndex]->getEdge())) {
@@ -471,19 +619,47 @@ GUIVehicle::drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r)
             }
         }
         GLHelper::drawBoxLines(lane->getShape(), lane->getShapeRotations(), lane->getShapeLengths(), exaggeration);
+        if (s.showRouteIndex) {
+            std::string label = toString((int)(i - myCurrEdge));
+            const double textSize = s.vehicleName.size / s.scale;
+            Position pos = lane->getShape().front() - Position(0, textSize * repeatLane[lane]);
+            //GLHelper::drawText(label, pos, 1.0, textSize, s.vehicleName.color);
+            GLHelper::drawTextSettings(s.vehicleName, label, pos, s.scale, s.angle, 1.0);
+            repeatLane[lane]++;
+            GLHelper::setColor(col);
+        }
     }
+    // draw stop labels
+    // (vertical shift for repeated stops at the same position
+    std::map<std::pair<const MSLane*, double>, int> repeat; // count repeated occurrences of the same position
     int stopIndex = 0;
     for (const Stop& stop : myStops) {
-        Position pos = stop.lane->geometryPositionAtOffset(stop.reached ? getPositionOnLane() : stop.getEndPos(*this));
+        double stopLanePos;
+        if (stop.pars.speed > 0) {
+            stopLanePos = stop.reached ? stop.pars.endPos : stop.pars.startPos;
+        } else {
+            stopLanePos = stop.reached ? getPositionOnLane() : stop.getEndPos(*this);
+        }
+        Position pos = stop.lane->geometryPositionAtOffset(stopLanePos);
         GLHelper::drawBoxLines(stop.lane->getShape().getOrthogonal(pos, 10, true, stop.lane->getWidth()), 0.1);
         std::string label = stop.reached ? "stopped" : "stop " + toString(stopIndex);
+#ifdef _DEBUG
+        label += " (" + toString(stop.edge - myCurrEdge) + "e)";
+#endif
         if (stop.pars.until >= 0) {
             label += " until:" + time2string(stop.pars.until);
         }
-        if (stop.duration >= 0) {
-            label += " duration:" + time2string(stop.duration);
+        if (stop.duration >= 0 || stop.pars.duration > 0) {
+            if (STEPS2TIME(stop.duration) > 3600 * 24) {
+                label += " duration:1day+";
+            } else {
+                label += " duration:" + time2string(stop.duration);
+            }
         }
-        GLHelper::drawText(label, pos, 1.0, s.vehicleName.size / s.scale, s.vehicleName.color);
+        std::pair<const MSLane*, double> stopPos = std::make_pair(stop.lane, stop.getEndPos(*this));
+        const double textSize = s.vehicleName.size / s.scale;
+        GLHelper::drawText(label, pos - Position(0, textSize * repeat[stopPos]), 1.0, textSize, s.vehicleName.color, s.angle);
+        repeat[stopPos]++;
         stopIndex++;
     }
 }
@@ -495,165 +671,52 @@ GUIVehicle::getPreviousLane(MSLane* current, int& furtherIndex) const {
     if (furtherIndex < (int)myFurtherLanes.size()) {
         return myFurtherLanes[furtherIndex++];
     } else {
-        const int routeIndex = (int)(getCurrentRouteEdge() - myRoute->begin());
-        int backIndex = furtherIndex + 1;
-        for (MSLane* l : myFurtherLanes) {
-            if (l->isInternal()) {
-                backIndex--;
-            }
-        }
-        if (routeIndex >= backIndex) {
-            furtherIndex++;
-            // could also look for the first lane that allows this vehicle class
-            // but this is probably not an issue for trains
-            return (*(getCurrentRouteEdge() - backIndex))->getLanes()[0];
-        }
-        return current;
-    }
-}
-
-
-void
-GUIVehicle::drawAction_drawRailCarriages(const GUIVisualizationSettings& s, double defaultLength, double carriageGap, int firstPassengerCarriage, bool asImage) const {
-    RGBColor current = GLHelper::getColor();
-    RGBColor darker = current.changedBrightness(-51);
-    const double exaggeration = s.vehicleSize.getExaggeration(s, this);
-    const double totalLength = getVType().getLength();
-    double upscaleLength = exaggeration;
-    if (exaggeration > 1 && totalLength > 5) {
-        // reduce the length/width ratio because this is not usefull at high zoom
-        upscaleLength = MAX2(1.0, upscaleLength * (5 + sqrt(totalLength - 5)) / totalLength);
-    }
-    defaultLength *= upscaleLength;
-    if (exaggeration == 0) {
-        return;
-    }
-    carriageGap *= upscaleLength;
-    const double length = totalLength * upscaleLength;
-    const double halfWidth = getVehicleType().getWidth() / 2.0 * exaggeration;
-    glPopMatrix(); // undo scaling and 90 degree rotation
-    glPopMatrix(); // undo initial translation and rotation
-    GLHelper::setColor(darker);
-    const double xCornerCut = 0.3 * exaggeration;
-    const double yCornerCut = 0.4 * exaggeration;
-    // round to closest integer
-    const int numCarriages = MAX2(1, (int)(length / (defaultLength + carriageGap) + 0.5));
-    assert(numCarriages > 0);
-    const double carriageLengthWithGap = length / numCarriages;
-    const double carriageLength = carriageLengthWithGap - carriageGap;
-    // lane on which the carriage front is situated
-    MSLane* lane = myLane;
-    int furtherIndex = 0;
-    // lane on which the carriage back is situated
-    MSLane* backLane = myLane;
-    int backFurtherIndex = furtherIndex;
-    // offsets of front and back
-    double carriageOffset = myState.pos();
-    double carriageBackOffset = myState.pos() - carriageLength;
-    // handle seats
-    int requiredSeats = getNumPassengers();
-    if (requiredSeats > 0) {
-        mySeatPositions.clear();
-    }
-    Position front, back;
-    double angle = 0.;
-    // draw individual carriages
-    for (int i = 0; i < numCarriages; ++i) {
-        while (carriageOffset < 0) {
-            MSLane* prev = getPreviousLane(lane, furtherIndex);
-            if (prev != lane) {
-                carriageOffset += prev->getLength();
+        // try to use route information
+        int routeIndex = getRoutePosition();
+        bool resultInternal;
+        if (MSGlobals::gUsingInternalLanes && MSNet::getInstance()->hasInternalLinks()) {
+            if (myLane->isInternal()) {
+                if (furtherIndex % 2 == 0) {
+                    routeIndex -= (furtherIndex + 0) / 2;
+                    resultInternal = false;
+                } else {
+                    routeIndex -= (furtherIndex + 1) / 2;
+                    resultInternal = false;
+                }
             } else {
-                // no lane available for drawing.
-                carriageOffset = 0;
+                if (furtherIndex % 2 != 0) {
+                    routeIndex -= (furtherIndex + 1) / 2;
+                    resultInternal = false;
+                } else {
+                    routeIndex -= (furtherIndex + 2) / 2;
+                    resultInternal = true;
+                }
             }
-            lane = prev;
+        } else {
+            routeIndex -= furtherIndex;
+            resultInternal = false;
         }
-        while (carriageBackOffset < 0) {
-            MSLane* prev = getPreviousLane(backLane, backFurtherIndex);
-            if (prev != backLane) {
-                carriageBackOffset += prev->getLength();
+        furtherIndex++;
+        if (routeIndex >= 0) {
+            if (resultInternal) {
+                const MSEdge* prevNormal = myRoute->getEdges()[routeIndex];
+                for (MSLane* cand : prevNormal->getLanes()) {
+                    for (MSLink* link : cand->getLinkCont()) {
+                        if (link->getLane() == current) {
+                            if (link->getViaLane() != nullptr) {
+                                return link->getViaLane();
+                            } else {
+                                return const_cast<MSLane*>(link->getLaneBefore());
+                            }
+                        }
+                    }
+                }
             } else {
-                // no lane available for drawing.
-                carriageBackOffset = 0;
+                return myRoute->getEdges()[routeIndex]->getLanes()[0];
             }
-            backLane = prev;
-        }
-        front = lane->geometryPositionAtOffset(carriageOffset);
-        back = backLane->geometryPositionAtOffset(carriageBackOffset);
-        if (front == back) {
-            // no place for drawing available
-            continue;
-        }
-        const double drawnCarriageLength = front.distanceTo2D(back);
-        angle = atan2((front.x() - back.x()), (back.y() - front.y())) * (double) 180.0 / (double) M_PI;
-        if (i >= firstPassengerCarriage) {
-            computeSeats(front, back, requiredSeats);
-        }
-        glPushMatrix();
-        glTranslated(front.x(), front.y(), getType());
-        glRotated(angle, 0, 0, 1);
-        if (!asImage || !drawAction_drawVehicleAsImage(s, carriageLength)) {
-            glBegin(GL_TRIANGLE_FAN);
-            glVertex2d(-halfWidth + xCornerCut, 0);
-            glVertex2d(-halfWidth, yCornerCut);
-            glVertex2d(-halfWidth, drawnCarriageLength - yCornerCut);
-            glVertex2d(-halfWidth + xCornerCut, drawnCarriageLength);
-            glVertex2d(halfWidth - xCornerCut, drawnCarriageLength);
-            glVertex2d(halfWidth, drawnCarriageLength - yCornerCut);
-            glVertex2d(halfWidth, yCornerCut);
-            glVertex2d(halfWidth - xCornerCut, 0);
-            glEnd();
-        }
-        glPopMatrix();
-        carriageOffset -= carriageLengthWithGap;
-        carriageBackOffset -= carriageLengthWithGap;
-        GLHelper::setColor(current);
-    }
-    if (getVType().getGuiShape() == SVS_RAIL_CAR) {
-        glPushMatrix();
-        glTranslated(front.x(), front.y(), getType());
-        glRotated(angle, 0, 0, 1);
-        drawAction_drawVehicleBlinker(carriageLength);
-        drawAction_drawVehicleBrakeLight(carriageLength);
-        glPopMatrix();
-    }
-    // restore matrices
-    glPushMatrix();
-    front = getPosition();
-    glTranslated(front.x(), front.y(), getType());
-    const double degAngle = RAD2DEG(getAngle() + M_PI / 2.);
-    glRotated(degAngle, 0, 0, 1);
-    glScaled(exaggeration, upscaleLength, 1);
-    glPushMatrix();
-}
-
-
-int
-GUIVehicle::getNumPassengers() const {
-    if (myPersonDevice != nullptr) {
-        return (int)myPersonDevice->size();
-    }
-    return 0;
-}
-
-
-void
-GUIVehicle::computeSeats(const Position& front, const Position& back, int& requiredSeats) const {
-    if (requiredSeats <= 0) {
-        return; // save some work
-    }
-    const double length = front.distanceTo2D(back);
-    if (length < 4) {
-        // small vehicle, sit at the center
-        mySeatPositions.push_back(PositionVector::positionAtOffset2D(front, back, length / 2));
-        requiredSeats--;
-    } else {
-        for (double p = 2; p <= length - 1; p += 1) {
-            mySeatPositions.push_back(PositionVector::positionAtOffset2D(front, back, p));
-            requiredSeats--;
         }
     }
+    return current;
 }
 
 
@@ -702,7 +765,7 @@ GUIVehicle::selectBlockingFoes() const {
             /// XXX if the vehicle intends to stop on an intersection, there could be a relevant exitLink (see #4299)
             continue;
         }
-        std::vector<const SUMOVehicle*> blockingFoes;
+        MSLink::BlockingFoes blockingFoes;
         std::vector<const MSPerson*> blockingPersons;
 #ifdef DEBUG_FOES
         std::cout << "   foeLink=" << dpi.myLink->getViaLaneOrLane()->getID() << "\n";
@@ -713,8 +776,8 @@ GUIVehicle::selectBlockingFoes() const {
 #ifdef DEBUG_FOES
         if (!isOpen) {
             std::cout << "     closed due to:\n";
-            for (std::vector<const SUMOVehicle*>::const_iterator it = blockingFoes.begin(); it != blockingFoes.end(); ++it) {
-                std::cout << "   " << (*it)->getID() << "\n";
+            for (const auto& item : blockingFoes) {
+                std::cout << "   " << item->getID() << "\n";
             }
         }
 #endif
@@ -733,15 +796,15 @@ GUIVehicle::selectBlockingFoes() const {
 #ifdef DEBUG_FOES
                 if (!isShadowOpen) {
                     std::cout <<  "    foes at shadow link=" << parallelLink->getViaLaneOrLane()->getID() << ":\n";
-                    for (std::vector<const SUMOVehicle*>::const_iterator it = blockingFoes.begin(); it != blockingFoes.end(); ++it) {
-                        std::cout << "   " << (*it)->getID() << "\n";
+                    for (const auto& item : blockingFoes) {
+                        std::cout << "   " << item->getID() << "\n";
                     }
                 }
 #endif
             }
         }
-        for (std::vector<const SUMOVehicle*>::const_iterator it = blockingFoes.begin(); it != blockingFoes.end(); ++it) {
-            gSelected.select(static_cast<const GUIVehicle*>(*it)->getGlID());
+        for (const auto& item : blockingFoes) {
+            gSelected.select(static_cast<const GUIVehicle*>(item)->getGlID());
         }
 #ifdef DEBUG_FOES
         gDebugFlag1 = true;
@@ -821,10 +884,100 @@ GUIVehicle::getLeftSublaneOnEdge() const {
     return -1;
 }
 
+
+std::string
+GUIVehicle::getLCStateRight() const {
+    return toString((LaneChangeAction)getLaneChangeModel().getSavedState(-1).second);
+}
+
+std::string
+GUIVehicle::getLCStateLeft() const {
+    return toString((LaneChangeAction)getLaneChangeModel().getSavedState(1).second);
+}
+
+std::string
+GUIVehicle::getLaneID() const {
+    return Named::getIDSecure(myLane, "n/a");
+}
+
+std::string
+GUIVehicle::getShadowLaneID() const {
+    return Named::getIDSecure(getLaneChangeModel().getShadowLane(), "");
+}
+
+std::string
+GUIVehicle::getTargetLaneID() const {
+    return Named::getIDSecure(getLaneChangeModel().getTargetLane(), "");
+}
+
 double
 GUIVehicle::getManeuverDist() const {
     return getLaneChangeModel().getManeuverDist();
 }
 
-/****************************************************************************/
+void
+GUIVehicle::rerouteDRTStop(MSStoppingPlace* busStop) {
+    SUMOTime intermediateDuration = TIME2STEPS(20);
+    SUMOTime finalDuration = SUMOTime_MAX;
+    if (myParameter->stops.size() >= 2) {
+        // copy durations from the original stops
+        intermediateDuration = myParameter->stops.front().duration;
+        finalDuration = myParameter->stops.back().duration;
+    }
+    // if the stop is already in the list of stops, cancel all stops that come
+    // after it and set the stop duration
+    std::string line = "";
+    int destinations = 0;
+    bool add = true;
+    for (auto it = myStops.begin(); it != myStops.end(); it++) {
+        if (!it->reached && destinations < 2 && it->busstop != nullptr) {
+            line += it->busstop->getID();
+            destinations++;
+        }
+        if (it->busstop == busStop) {
+            it->duration = finalDuration;
+            myStops.erase(++it, myStops.end());
+            add = false;
+            break;
+        } else {
+            it->duration = MIN2(it->duration, intermediateDuration);
+        }
+    }
+    if (destinations < 2) {
+        line += busStop->getID();
+    }
+    if (add) {
+        // create new stop
+        SUMOVehicleParameter::Stop stopPar;
+        stopPar.busstop = busStop->getID();
+        stopPar.lane = busStop->getLane().getID();
+        stopPar.startPos = busStop->getBeginLanePosition();
+        stopPar.endPos = busStop->getEndLanePosition();
+        stopPar.duration = finalDuration;
+        stopPar.until = -1;
+        stopPar.triggered = false;
+        stopPar.containerTriggered = false;
+        stopPar.parking = false;
+        stopPar.index = STOP_INDEX_FIT;
+        stopPar.parametersSet = STOP_START_SET | STOP_END_SET;
+        // clean up prior route to improve visualisation, ensure that the stop can be added immediately
+        ConstMSEdgeVector edges = myRoute->getEdges();
+        edges.erase(edges.begin(), edges.begin() + getRoutePosition());
+        edges.push_back(&busStop->getLane().getEdge());
+        replaceRouteEdges(edges, -1, 0, "DRT.tmp", false, false, false);
+        std::string errorMsg;
+        // add stop
+        addStop(stopPar, errorMsg);
+    }
+    const bool hasReroutingDevice = getDevice(typeid(MSDevice_Routing)) != nullptr;
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& router = hasReroutingDevice
+            ? MSRoutingEngine::getRouterTT(getRNGIndex())
+            : MSNet::getInstance()->getRouterTT(getRNGIndex());
+    // reroute to ensure the new stop is reached
+    reroute(MSNet::getInstance()->getCurrentTimeStep(), "DRT", router);
+    myParameter->line = line;
+    assert(haveValidStopEdges());
+}
 
+
+/****************************************************************************/
