@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A MSVehicle extended by some values for usage within the gui
 /****************************************************************************/
@@ -86,7 +85,7 @@ public:
     /** @brief Draws the route
      * @param[in] r The route to draw
      */
-    void drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r) const;
+    void drawRouteHelper(const GUIVisualizationSettings& s, const MSRoute& r, bool future, const RGBColor& col) const;
 
     void drawAction_drawVehicleBlinker(double length) const;
     void drawAction_drawVehicleBrakeLight(double length, bool onlyOne = false) const;
@@ -109,7 +108,7 @@ public:
     void selectBlockingFoes() const;
 
     /// @brief gets the color value according to the current scheme index
-    double getColorValue(int activeScheme) const;
+    double getColorValue(const GUIVisualizationSettings& s, int activeScheme) const;
 
     /** @brief Returns an own parameter window
      *
@@ -144,10 +143,20 @@ public:
     int getRightSublaneOnEdge() const;
     int getLeftSublaneOnEdge() const;
 
+    /// @brief return the lanechange state
+    std::string getLCStateRight() const;
+    std::string getLCStateLeft() const;
+
+    /// @brief return vehicle lane id
+    std::string getLaneID() const;
+    std::string getShadowLaneID() const;
+    std::string getTargetLaneID() const;
 
     /// @brief return the lane-change maneuver distance
     double getManeuverDist() const;
 
+    /// @brief handle route to accomodate to given stop
+    void rerouteDRTStop(MSStoppingPlace* busStop);
 
 protected:
     /// @brief register vehicle for drawing while outside the network
@@ -158,12 +167,7 @@ private:
     /* @brief draw train with individual carriages. The number of carriages is
      * determined from defaultLength of carriages and vehicle length
      * passengerSeats are computed beginning at firstPassengerCarriage */
-    void drawAction_drawRailCarriages(const GUIVisualizationSettings& s, double defaultLength, double carriageGap,
-                                      int firstPassengerCarriage, bool asImage) const;
-    /// @}
-
-    /// @brief draws the given guiShape if it has distinct carriages/modules and returns true if so
-    bool drawAction_drawCarriageClass(const GUIVisualizationSettings& s, SUMOVehicleShape guiShape, bool asImage) const;
+    void drawAction_drawCarriageClass(const GUIVisualizationSettings& s, bool asImage) const;
 
     /* @brief return the previous lane in this vehicles route including internal lanes
      * @param[in] current The lane of which the predecessor should be returned
@@ -171,12 +175,6 @@ private:
      */
     MSLane* getPreviousLane(MSLane* current, int& furtherIndex) const;
 
-    /// @brief return the number of passengers
-    int getNumPassengers() const;
-
-    /// @brief add seats to mySeatPositions and update requiredSeats
-    void computeSeats(const Position& front, const Position& back, int& requiredSeats) const;
-
     /// @brief retrieve information about the current stop state
     std::string getStopInfo() const;
 
