@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Sets and checks options for jtr-routing
 /****************************************************************************/
@@ -65,14 +64,15 @@ ROJTRFrame::fillOptions() {
     oc.addDescription("exit-times", "Output", "Write exit times (weights) for each edge");
 
     oc.doRegister("max-edges-factor", new Option_Float(2.0));
-    oc.addDescription("max-edges-factor", "Processing", "Routes are cut off when the route edges to net edges ratio is larger than FLOAT");
+    oc.addDescription("max-edges-factor", "Processing",
+                      "Routes are cut off when the route edges to net edges ratio is larger than FLOAT");
 
-    oc.doRegister("turn-defaults", 'T', new Option_String("30,50,20"));
-    oc.addDescription("turn-defaults", "Processing", "Use STR as default turn definition");
+    oc.doRegister("turn-defaults", 'T', new Option_StringVector(StringVector({ "30", "50", "20" })));
+    oc.addDescription("turn-defaults", "Processing", "Use STR[] as default turn definition");
 
-    oc.doRegister("sink-edges", new Option_String());
+    oc.doRegister("sink-edges", new Option_StringVector());
     oc.addSynonyme("sink-edges", "sinks");
-    oc.addDescription("sink-edges", "Processing", "Use STR as list of sink edges");
+    oc.addDescription("sink-edges", "Processing", "Use STR[] as list of sink edges");
 
     oc.doRegister("accept-all-destinations", 'A', new Option_Bool(false));
     oc.addDescription("accept-all-destinations", "Processing", "Whether all edges are allowed as sink edges");
