@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2017-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2017-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Laura Bieker-Walz
 /// @author  Robert Hilbrich
 /// @date    15.09.2017
-/// @version $Id$
 ///
 // C++ TraCI client API implementation
 /****************************************************************************/
@@ -28,8 +27,8 @@
 #include <utils/common/SystemFrame.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/options/OptionsIO.h>
-#include <utils/vehicle/IntermodalRouter.h>
-#include <utils/vehicle/PedestrianRouter.h>
+#include <utils/router/IntermodalRouter.h>
+#include <utils/router/PedestrianRouter.h>
 #include <utils/xml/XMLSubSys.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSEdgeControl.h>
@@ -44,7 +43,7 @@
 #include <microsim/MSParkingArea.h>
 #include <microsim/devices/MSRoutingEngine.h>
 #include <netload/NLBuilder.h>
-#include <traci-server/TraCIConstants.h>
+#include <libsumo/TraCIConstants.h>
 #include "Simulation.h"
 #include <libsumo/TraCIDefs.h>
 
@@ -93,6 +92,7 @@ Simulation::step(const double time) {
 
 void
 Simulation::close() {
+    Helper::clearSubscriptions();
     if (MSNet::hasInstance()) {
         MSNet::getInstance()->closeSimulation(0);
         delete MSNet::getInstance();
@@ -102,12 +102,9 @@ Simulation::close() {
 }
 
 
-LIBSUMO_SUBSCRIPTION_IMPLEMENTATION(Simulation, SIM)
-
-
 void
-Simulation::subscribe(const std::vector<int>& vars, double beginTime, double endTime) {
-    libsumo::Helper::subscribe(CMD_SUBSCRIBE_SIM_VARIABLE, "", vars, beginTime, endTime);
+Simulation::subscribe(const std::vector<int>& varIDs, double begin, double end) {
+    libsumo::Helper::subscribe(CMD_SUBSCRIBE_SIM_VARIABLE, "", varIDs, begin, end);
 }
 
 
@@ -260,6 +257,14 @@ Simulation::getEndingTeleportIDList() {
     return Helper::getVehicleStateChanges(MSNet::VEHICLE_STATE_ENDING_TELEPORT);
 }
 
+std::vector<std::string>
+Simulation::getBusStopIDList() {
+    std::vector<std::string> result;
+    for (const auto pair : MSNet::getInstance()->getStoppingPlaces(SUMO_TAG_BUS_STOP)) {
+        result.push_back(pair.first);
+    }
+    return result;
+}
 
 int
 Simulation::getBusStopWaiting(const std::string& id) {
@@ -270,6 +275,17 @@ Simulation::getBusStopWaiting(const std::string& id) {
     return s->getTransportableNumber();
 }
 
+std::vector<std::string>
+Simulation::getBusStopWaitingIDList(const std::string& id) {
+    MSStoppingPlace* s = MSNet::getInstance()->getStoppingPlace(id, SUMO_TAG_BUS_STOP);
+    std::vector<MSTransportable*> transportables = s->getTransportables();
+    std::vector<std::string> result;
+    for (std::vector<MSTransportable*>::iterator it = transportables.begin(); it != transportables.end(); it++) {
+        result.push_back((*it)->getID());
+    }
+    return result;
+}
+
 
 double
 Simulation::getDeltaT() {
@@ -303,7 +319,7 @@ Simulation::getMinExpectedNumber() {
 
 TraCIPosition
 Simulation::convert2D(const std::string& edgeID, double pos, int laneIndex, bool toGeo) {
-    Position result = Helper::getLaneChecking(edgeID, laneIndex, pos)->getShape().positionAtOffset(pos);
+    Position result = Helper::getLaneChecking(edgeID, laneIndex, pos)->geometryPositionAtOffset(pos);
     if (toGeo) {
         GeoConvHelper::getFinal().cartesian2geo(result);
     }
@@ -314,7 +330,7 @@ Simulation::convert2D(const std::string& edgeID, double pos, int laneIndex, bool
 
 TraCIPosition
 Simulation::convert3D(const std::string& edgeID, double pos, int laneIndex, bool toGeo) {
-    Position result = Helper::getLaneChecking(edgeID, laneIndex, pos)->getShape().positionAtOffset(pos);
+    Position result = Helper::getLaneChecking(edgeID, laneIndex, pos)->geometryPositionAtOffset(pos);
     if (toGeo) {
         GeoConvHelper::getFinal().cartesian2geo(result);
     }
@@ -323,12 +339,16 @@ Simulation::convert3D(const std::string& edgeID, double pos, int laneIndex, bool
 
 
 TraCIRoadPosition
-Simulation::convertRoad(double x, double y, bool isGeo) {
+Simulation::convertRoad(double x, double y, bool isGeo, const std::string& vClass) {
     Position pos(x, y);
     if (isGeo) {
         GeoConvHelper::getFinal().x2cartesian_const(pos);
     }
-    std::pair<MSLane*, double> roadPos = libsumo::Helper::convertCartesianToRoadMap(pos);
+    if (!SumoVehicleClassStrings.hasString(vClass)) {
+        throw TraCIException("Unknown vehicle class '" + vClass + "'.");
+    }
+    const SUMOVehicleClass vc = SumoVehicleClassStrings.get(vClass);
+    std::pair<MSLane*, double> roadPos = libsumo::Helper::convertCartesianToRoadMap(pos, vc);
     if (roadPos.first == nullptr) {
         throw TraCIException("Cannot convert position to road.");
     }
@@ -361,8 +381,8 @@ Simulation::getDistance2D(double x1, double y1, double x2, double y2, bool isGeo
         GeoConvHelper::getFinal().x2cartesian_const(pos2);
     }
     if (isDriving) {
-        std::pair<const MSLane*, double> roadPos1 = libsumo::Helper::convertCartesianToRoadMap(pos1);
-        std::pair<const MSLane*, double> roadPos2 = libsumo::Helper::convertCartesianToRoadMap(pos2);
+        std::pair<const MSLane*, double> roadPos1 = libsumo::Helper::convertCartesianToRoadMap(pos1, SVC_IGNORING);
+        std::pair<const MSLane*, double> roadPos2 = libsumo::Helper::convertCartesianToRoadMap(pos2, SVC_IGNORING);
         if ((roadPos1.first == roadPos2.first) && (roadPos1.second <= roadPos2.second)) {
             // same edge
             return roadPos2.second - roadPos1.second;
@@ -374,7 +394,7 @@ Simulation::getDistance2D(double x1, double y1, double x2, double y2, bool isGeo
                 roadPos2.first = roadPos2.first->getLogicalPredecessorLane();
                 roadPos2.second = roadPos2.first->getLength();
             }
-            MSNet::getInstance()->getRouterTT().compute(
+            MSNet::getInstance()->getRouterTT(0).compute(
                 &roadPos1.first->getEdge(), &roadPos2.first->getEdge(), nullptr, MSNet::getInstance()->getCurrentTimeStep(), newRoute);
             MSRoute route("", newRoute, false, nullptr, std::vector<SUMOVehicleParameter::Stop>());
             return distance + route.getDistanceBetween(roadPos1.second, roadPos2.second, &roadPos1.first->getEdge(), &roadPos2.first->getEdge());
@@ -401,15 +421,15 @@ Simulation::getDistanceRoad(const std::string& edgeID1, double pos1, const std::
                 roadPos2.first = roadPos2.first->getLogicalPredecessorLane();
                 roadPos2.second = roadPos2.first->getLength();
             }
-            MSNet::getInstance()->getRouterTT().compute(
+            MSNet::getInstance()->getRouterTT(0).compute(
                 &roadPos1.first->getEdge(), &roadPos2.first->getEdge(), nullptr, MSNet::getInstance()->getCurrentTimeStep(), newRoute);
             MSRoute route("", newRoute, false, nullptr, std::vector<SUMOVehicleParameter::Stop>());
             return distance + route.getDistanceBetween(roadPos1.second, roadPos2.second, &roadPos1.first->getEdge(), &roadPos2.first->getEdge());
         }
     } else {
-        Position pos1 = roadPos1.first->getShape().positionAtOffset(roadPos1.second);
-        Position pos2 = roadPos2.first->getShape().positionAtOffset(roadPos2.second);
-        return pos1.distanceTo(pos2);
+        const Position p1 = roadPos1.first->geometryPositionAtOffset(roadPos1.second);
+        const Position p2 = roadPos2.first->geometryPositionAtOffset(roadPos2.second);
+        return p1.distanceTo(p2);
     }
 }
 
@@ -443,12 +463,12 @@ Simulation::findRoute(const std::string& from, const std::string& to, const std:
     }
     ConstMSEdgeVector edges;
     const SUMOTime dep = depart < 0 ? MSNet::getInstance()->getCurrentTimeStep() : TIME2STEPS(depart);
-    SUMOAbstractRouter<MSEdge, SUMOVehicle>& router = routingMode == ROUTING_MODE_AGGREGATED ? MSRoutingEngine::getRouterTT() : MSNet::getInstance()->getRouterTT();
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& router = routingMode == ROUTING_MODE_AGGREGATED ? MSRoutingEngine::getRouterTT(0) : MSNet::getInstance()->getRouterTT(0);
     router.compute(fromEdge, toEdge, vehicle, dep, edges);
     for (const MSEdge* e : edges) {
         result.edges.push_back(e->getID());
     }
-    result.travelTime = result.cost = router.recomputeCosts(edges, vehicle, dep);
+    result.travelTime = result.cost = router.recomputeCosts(edges, vehicle, dep, &result.length);
     if (vehicle != nullptr) {
         MSNet::getInstance()->getVehicleControl().deleteVehicle(vehicle, true);
     }
@@ -482,19 +502,21 @@ Simulation::findIntermodalRoute(const std::string& from, const std::string& to,
     }
     for (StringTokenizer st(modes); st.hasNext();) {
         const std::string mode = st.next();
-        if (mode == "car") {
+        if (mode == toString(PERSONMODE_CAR)) {
             pars.push_back(new SUMOVehicleParameter());
             pars.back()->vtypeid = DEFAULT_VTYPE_ID;
             pars.back()->id = mode;
             modeSet |= SVC_PASSENGER;
-        } else if (mode == "bicycle") {
+        } else if (mode == toString(PERSONMODE_BICYCLE)) {
             pars.push_back(new SUMOVehicleParameter());
             pars.back()->vtypeid = DEFAULT_BIKETYPE_ID;
             pars.back()->id = mode;
             modeSet |= SVC_BICYCLE;
-        } else if (mode == "public") {
+        } else if (mode == toString(PERSONMODE_PUBLIC)) {
             pars.push_back(nullptr);
             modeSet |= SVC_BUS;
+        } else if (mode == toString(PERSONMODE_WALK)) {
+            // do nothing
         } else {
             throw TraCIException("Unknown person mode '" + mode + "'.");
         }
@@ -514,7 +536,7 @@ Simulation::findIntermodalRoute(const std::string& from, const std::string& to,
     if (walkFactor < 0) {
         walkFactor = OptionsCont::getOptions().getFloat("persontrip.walkfactor");
     }
-    const double externalFactor = StringUtils::toDouble(pedType->getParameter().getParameter("externalEffortFactor", "1.0"));
+    const double externalFactor = StringUtils::toDouble(pedType->getParameter().getParameter("externalEffortFactor", "100"));
     if (departPos < 0) {
         departPos += fromEdge->getLength();
     }
@@ -530,7 +552,7 @@ Simulation::findIntermodalRoute(const std::string& from, const std::string& to,
         throw TraCIException("Invalid arrival position " + toString(arrivalPos) + " for edge '" + to + "'.");
     }
     double minCost = std::numeric_limits<double>::max();
-    MSNet::MSIntermodalRouter& router = MSNet::getInstance()->getIntermodalRouter(routingMode);
+    MSNet::MSIntermodalRouter& router = MSNet::getInstance()->getIntermodalRouter(0, routingMode);
     for (SUMOVehicleParameter* vehPar : pars) {
         std::vector<TraCIStage> resultCand;
         SUMOVehicle* vehicle = nullptr;
@@ -599,6 +621,10 @@ Simulation::getParameter(const std::string& objectID, const std::string& key) {
             return toString(cs->getTotalCharged());
         } else if (attrName == toString(SUMO_ATTR_NAME)) {
             return toString(cs->getMyName());
+        } else if (attrName == "lane") {
+            return cs->getLane().getID();
+        } else if (cs->knowsParameter(attrName)) {
+            return cs->getParameter(attrName);
         } else {
             throw TraCIException("Invalid chargingStation parameter '" + attrName + "'");
         }
@@ -611,9 +637,13 @@ Simulation::getParameter(const std::string& objectID, const std::string& key) {
         if (attrName == "capacity") {
             return toString(pa->getCapacity());
         } else if (attrName == "occupancy") {
-            return toString(pa->getOccupancy());
+            return toString(pa->getOccupancyIncludingBlocked());
         } else if (attrName == toString(SUMO_ATTR_NAME)) {
             return toString(pa->getMyName());
+        } else if (attrName == "lane") {
+            return pa->getLane().getID();
+        } else if (pa->knowsParameter(attrName)) {
+            return pa->getParameter(attrName);
         } else {
             throw TraCIException("Invalid parkingArea parameter '" + attrName + "'");
         }
@@ -625,6 +655,10 @@ Simulation::getParameter(const std::string& objectID, const std::string& key) {
         }
         if (attrName == toString(SUMO_ATTR_NAME)) {
             return toString(bs->getMyName());
+        } else if (attrName == "lane") {
+            return bs->getLane().getID();
+        } else if (bs->knowsParameter(attrName)) {
+            return bs->getParameter(attrName);
         } else {
             throw TraCIException("Invalid busStop parameter '" + attrName + "'");
         }
@@ -645,6 +679,11 @@ Simulation::saveState(const std::string& fileName) {
     MSStateHandler::saveState(fileName, MSNet::getInstance()->getCurrentTimeStep());
 }
 
+void
+Simulation::writeMessage(const std::string& msg) {
+    WRITE_MESSAGE(msg);
+}
+
 
 std::shared_ptr<VariableWrapper>
 Simulation::makeWrapper() {
