@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Sets and checks options for ma-routing
 /****************************************************************************/
@@ -124,7 +123,7 @@ ROMAFrame::addImportOptions() {
     oc.doRegister("weight-adaption", new Option_Float(0.));
     oc.addDescription("weight-adaption", "Input", "The travel time influence of prior intervals");
 
-    oc.doRegister("taz-param", new Option_String());
+    oc.doRegister("taz-param", new Option_StringVector());
     oc.addDescription("taz-param", "Input", "Parameter key(s) defining source (and sink) taz");
 
     // register the time settings
@@ -207,8 +206,8 @@ ROMAFrame::addAssignmentOptions() {
     oc.doRegister("prefix", new Option_String(""));
     oc.addDescription("prefix", "Processing", "Defines the prefix for vehicle flow names");
 
-    oc.doRegister("timeline", new Option_String());
-    oc.addDescription("timeline", "Processing", "Uses STR as a timeline definition");
+    oc.doRegister("timeline", new Option_StringVector());
+    oc.addDescription("timeline", "Processing", "Uses STR[] as a timeline definition");
 
     oc.doRegister("timeline.day-in-hours", new Option_Bool(false));
     oc.addDescription("timeline.day-in-hours", "Processing", "Uses STR as a 24h-timeline definition");
