@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GUIMEVehicleControl.h
 /// @author  Jakob Erdmann
 /// @date    Okt 2012
-/// @version $Id$
 ///
 // The class responsible for building and deletion of meso vehicles (gui-version)
 /****************************************************************************/
@@ -24,15 +23,11 @@
 #include <config.h>
 
 #include <vector>
-#include <utils/foxtools/MFXMutex.h>
+#include <fx.h>
 #include <utils/gui/globjects/GUIGlObject.h>
 #include <mesosim/MEVehicleControl.h>
 
 
-// ===========================================================================
-// class declarations
-// ===========================================================================
-
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -105,7 +100,7 @@ public:
 
 private:
     /// The mutex used to avoid concurrent updates of the vehicle buffer
-    mutable MFXMutex myLock;
+    mutable FXMutex myLock;
 
 
 private:
