@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MELoop.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id$
 ///
 // The main mesocopic simulation loop
 /****************************************************************************/
@@ -220,6 +219,16 @@ MELoop::removeLeaderCar(MEVehicle* v) {
     cands.erase(find(cands.begin(), cands.end(), v));
 }
 
+void
+MELoop::vaporizeCar(MEVehicle* v) {
+    v->getSegment()->send(v, nullptr, MSNet::getInstance()->getCurrentTimeStep(), MSMoveReminder::NOTIFICATION_VAPORIZED);
+    // try removeLeaderCar
+    std::vector<MEVehicle*>& cands = myLeaderCars[v->getEventTime()];
+    auto it = find(cands.begin(), cands.end(), v);
+    if (it != cands.end()) {
+        cands.erase(it);
+    }
+}
 
 MESegment*
 MELoop::nextSegment(MESegment* s, MEVehicle* v) {
@@ -281,6 +290,9 @@ MELoop::buildSegmentsFor(const MSEdge& e, const OptionsCont& oc) {
 
 MESegment*
 MELoop::getSegmentForEdge(const MSEdge& e, double pos) {
+    if (e.getNumericalID() >= (int)myEdges2FirstSegments.size()) {
+        return nullptr;
+    }
     MESegment* s = myEdges2FirstSegments[e.getNumericalID()];
     if (pos > 0) {
         double cpos = 0;
