@@ -1,32 +1,24 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
+/****************************************************************************/
 /// @file    MESegment.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id: MESegment.cpp 23529 2017-03-18 10:22:36Z behrisch $
 ///
 // A single mesoscopic segment (cell)
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2017 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
 
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 #include <limits>
@@ -50,14 +42,21 @@
 #include "MELoop.h"
 #include "MESegment.h"
 
-#define DEFAULT_VEH_LENGHT_WITH_GAP (SUMOVTypeParameter::getDefault().length + SUMOVTypeParameter::getDefault().minGap)
+#define DEFAULT_VEH_LENGTH_WITH_GAP (SUMOVTypeParameter::getDefault().length + SUMOVTypeParameter::getDefault().minGap)
 // avoid division by zero when driving very slowly
 #define MESO_MIN_SPEED (0.05)
 
+//#define DEBUG_OPENED
+//#define DEBUG_JAMTHRESHOLD
+//#define DEBUG_COND (getID() == "blocker")
+//#define DEBUG_COND (true)
+#define DEBUG_COND (myEdge.isSelected())
+#define DEBUG_COND2(obj) ((obj != 0 && (obj)->isSelected()))
+
 // ===========================================================================
 // static member defintion
 // ===========================================================================
-MSEdge MESegment::myDummyParent("MESegmentDummyParent", -1, MSEdge::EDGEFUNCTION_UNKNOWN, "", "", -1);
+MSEdge MESegment::myDummyParent("MESegmentDummyParent", -1, EDGEFUNC_UNKNOWN, "", "", -1, 0);
 MESegment MESegment::myVaporizationTarget("vaporizationTarget");
 const double MESegment::DO_NOT_PATCH_JAM_THRESHOLD(std::numeric_limits<double>::max());
 
@@ -78,23 +77,24 @@ MESegment::MESegment(const std::string& id,
     myTau_jf((SUMOTime)(taujf / parent.getLanes().size())),
     myTau_jj((SUMOTime)(taujj / parent.getLanes().size())),
     myTau_length(MAX2(MESO_MIN_SPEED, speed) * parent.getLanes().size() / TIME2STEPS(1)),
-    myHeadwayCapacity(length / DEFAULT_VEH_LENGHT_WITH_GAP * parent.getLanes().size())/* Eissfeldt p. 69 */,
+    myHeadwayCapacity(length / DEFAULT_VEH_LENGTH_WITH_GAP * parent.getLanes().size())/* Eissfeldt p. 69 */,
     myCapacity(length * parent.getLanes().size()),
     myOccupancy(0.f),
     myJunctionControl(junctionControl),
     myTLSPenalty(MSGlobals::gMesoTLSPenalty > 0 &&
                  // only apply to the last segment of a tls-controlled edge
-                 myNextSegment == 0 && (
+                 myNextSegment == nullptr && (
                      parent.getToJunction()->getType() == NODETYPE_TRAFFIC_LIGHT ||
                      parent.getToJunction()->getType() == NODETYPE_TRAFFIC_LIGHT_NOJUNCTION ||
                      parent.getToJunction()->getType() == NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED)),
     myMinorPenalty(MSGlobals::gMesoMinorPenalty > 0 &&
                    // only apply to the last segment of an uncontrolled edge that has at least 1 minor link
-                   myNextSegment == 0 &&
+                   myNextSegment == nullptr &&
                    parent.getToJunction()->getType() != NODETYPE_TRAFFIC_LIGHT &&
                    parent.getToJunction()->getType() != NODETYPE_TRAFFIC_LIGHT_NOJUNCTION &&
                    parent.getToJunction()->getType() != NODETYPE_TRAFFIC_LIGHT_RIGHT_ON_RED &&
                    parent.hasMinorLink()),
+    myNumCars(0),
     myEntryBlockTime(SUMOTime_MIN),
     myLastHeadway(TIME2STEPS(-1)),
     myMeanSpeed(speed),
@@ -113,7 +113,7 @@ MESegment::MESegment(const std::string& id,
             assert(allowed != 0);
             assert(allowed->size() > 0);
             for (std::vector<MSLane*>::const_iterator j = allowed->begin(); j != allowed->end(); ++j) {
-                std::vector<MSLane*>::const_iterator it = find(lanes.begin(), lanes.end(), *j);
+                std::vector<MSLane*>::const_iterator it = std::find(lanes.begin(), lanes.end(), *j);
                 myFollowerMap[edge].push_back((int)distance(lanes.begin(), it));
             }
         }
@@ -125,7 +125,7 @@ MESegment::MESegment(const std::string& id,
 MESegment::MESegment(const std::string& id):
     Named(id),
     myEdge(myDummyParent), // arbitrary edge needed to supply the needed reference
-    myNextSegment(0), myLength(0), myIndex(0),
+    myNextSegment(nullptr), myLength(0), myIndex(0),
     myTau_ff(0), myTau_fj(0), myTau_jf(0), myTau_jj(0), myTau_length(1),
     myHeadwayCapacity(0), myCapacity(0), myJunctionControl(false),
     myTLSPenalty(false),
@@ -167,7 +167,7 @@ MESegment::recomputeJamThreshold(double jamThresh) {
     // f(n_jam_threshold) = tau_jf_withLength (for continuity)
     // f(myHeadwayCapacity) = myTau_jj * myHeadwayCapacity
 
-    const SUMOTime tau_jf_withLength = tauWithVehLength(myTau_jf, DEFAULT_VEH_LENGHT_WITH_GAP);
+    const SUMOTime tau_jf_withLength = tauWithVehLength(myTau_jf, DEFAULT_VEH_LENGTH_WITH_GAP);
     if (myJamThreshold < myCapacity) {
         // jamming is possible
         const double n_jam_threshold = myHeadwayCapacity * myJamThreshold / myCapacity; // number of vehicles above which the segment is jammed
@@ -197,7 +197,13 @@ MESegment::jamThresholdForSpeed(double speed, double jamThresh) const {
     if (speed == 0) {
         return std::numeric_limits<double>::max();  // never jam. Irrelevant at speed 0 anyway
     }
-    return std::ceil(myLength / (-jamThresh * speed * STEPS2TIME(tauWithVehLength(myTau_ff, DEFAULT_VEH_LENGHT_WITH_GAP)))) * DEFAULT_VEH_LENGHT_WITH_GAP;
+#ifdef DEBUG_JAMTHRESHOLD
+    if (true || DEBUG_COND) {
+        std::cout << "jamThresholdForSpeed seg=" << getID() << " speed=" << speed << " jamThresh=" << jamThresh << " ffVehs=" << std::ceil(myLength / (-jamThresh * speed * STEPS2TIME(tauWithVehLength(myTau_ff, DEFAULT_VEH_LENGTH_WITH_GAP)))) << " thresh=" << std::ceil(myLength / (-jamThresh * speed * STEPS2TIME(tauWithVehLength(myTau_ff, DEFAULT_VEH_LENGTH_WITH_GAP)))) * DEFAULT_VEH_LENGTH_WITH_GAP
+                  << "\n";
+    }
+#endif
+    return std::ceil(myLength / (-jamThresh * speed * STEPS2TIME(tauWithVehLength(myTau_ff, DEFAULT_VEH_LENGTH_WITH_GAP)))) * DEFAULT_VEH_LENGTH_WITH_GAP;
 }
 
 
@@ -214,7 +220,7 @@ MESegment::addDetector(MSMoveReminder* data) {
 
 void
 MESegment::removeDetector(MSMoveReminder* data) {
-    std::vector<MSMoveReminder*>::iterator it = find(
+    std::vector<MSMoveReminder*>::iterator it = std::find(
                 myDetectorData.begin(), myDetectorData.end(), data);
     if (it != myDetectorData.end()) {
         myDetectorData.erase(it);
@@ -227,22 +233,6 @@ MESegment::removeDetector(MSMoveReminder* data) {
 }
 
 
-void
-MESegment::updateDetectorsOnLeave(MEVehicle* v, SUMOTime currentTime, MESegment* next) {
-    MSMoveReminder::Notification reason;
-    if (next == 0) {
-        reason = MSMoveReminder::NOTIFICATION_ARRIVED;
-    } else if (next == &myVaporizationTarget) {
-        reason = MSMoveReminder::NOTIFICATION_VAPORIZED;
-    } else if (myNextSegment == 0) {
-        reason = MSMoveReminder::NOTIFICATION_JUNCTION;
-    } else {
-        reason = MSMoveReminder::NOTIFICATION_SEGMENT;
-    }
-    v->updateDetectors(currentTime, true, reason);
-}
-
-
 void
 MESegment::prepareDetectorForWriting(MSMoveReminder& data) {
     const SUMOTime currentTime = MSNet::getInstance()->getCurrentTimeStep();
@@ -299,16 +289,6 @@ MESegment::initialise(MEVehicle* veh, SUMOTime time) {
 }
 
 
-int
-MESegment::getCarNumber() const {
-    int total = 0;
-    for (Queues::const_iterator k = myCarQues.begin(); k != myCarQues.end(); ++k) {
-        total += (int)k->size();
-    }
-    return total;
-}
-
-
 double
 MESegment::getMeanSpeed(bool useCached) const {
     const SUMOTime currentTime = MSNet::getInstance()->getCurrentTimeStep();
@@ -346,12 +326,13 @@ MESegment::writeVehicles(OutputDevice& of) const {
 
 
 MEVehicle*
-MESegment::removeCar(MEVehicle* v, SUMOTime leaveTime, MESegment* next) {
+MESegment::removeCar(MEVehicle* v, SUMOTime leaveTime, const MSMoveReminder::Notification reason) {
     myOccupancy = MAX2(0., myOccupancy - v->getVehicleType().getLengthWithGap());
     std::vector<MEVehicle*>& cars = myCarQues[v->getQueIndex()];
     assert(std::find(cars.begin(), cars.end(), v) != cars.end());
     // One could be tempted to do  v->setSegment(next); here but position on lane will be invalid if next == 0
-    updateDetectorsOnLeave(v, leaveTime, next);
+    v->updateDetectors(leaveTime, true, reason);
+    myNumCars--;
     myEdge.lock();
     if (v == cars.back()) {
         cars.pop_back();
@@ -363,7 +344,7 @@ MESegment::removeCar(MEVehicle* v, SUMOTime leaveTime, MESegment* next) {
         cars.erase(std::find(cars.begin(), cars.end(), v));
     }
     myEdge.unlock();
-    return 0;
+    return nullptr;
 }
 
 
@@ -395,41 +376,61 @@ MSLink*
 MESegment::getLink(const MEVehicle* veh, bool penalty) const {
     if (myJunctionControl || penalty) {
         const MSEdge* const nextEdge = veh->succEdge(1);
-        if (nextEdge == 0) {
-            return 0;
+        if (nextEdge == nullptr) {
+            return nullptr;
         }
         // try to find any link leading to our next edge, start with the lane pointed to by the que index
         const MSLane* const bestLane = myEdge.getLanes()[veh->getQueIndex()];
-        const MSLinkCont& links = bestLane->getLinkCont();
-        for (std::vector<MSLink*>::const_iterator j = links.begin(); j != links.end(); ++j) {
-            if (&(*j)->getLane()->getEdge() == nextEdge) {
-                return *j;
+        for (MSLink* const link : bestLane->getLinkCont()) {
+            if (&link->getLane()->getEdge() == nextEdge) {
+                return link;
             }
         }
         // this is for the non-multique case, maybe we should use caching here !!!
-        for (std::vector<MSLane*>::const_iterator l = myEdge.getLanes().begin(); l != myEdge.getLanes().end(); ++l) {
-            if ((*l) != bestLane) {
-                const MSLinkCont& links = (*l)->getLinkCont();
-                for (std::vector<MSLink*>::const_iterator j = links.begin(); j != links.end(); ++j) {
-                    if (&(*j)->getLane()->getEdge() == nextEdge) {
-                        return *j;
+        for (const MSLane* const lane : myEdge.getLanes()) {
+            if (lane != bestLane) {
+                for (MSLink* const link : lane->getLinkCont()) {
+                    if (&link->getLane()->getEdge() == nextEdge) {
+                        return link;
                     }
                 }
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 
 bool
 MESegment::isOpen(const MEVehicle* veh) const {
+#ifdef DEBUG_OPENED
+    if (DEBUG_COND || DEBUG_COND2(veh)) {
+        std::cout << SIMTIME << " opened seg=" << getID() << " veh=" << Named::getIDSecure(veh)
+                  << " tlsPenalty=" << myTLSPenalty;
+        const MSLink* link = getLink(veh);
+        if (link == 0) {
+            std::cout << " link=0";
+        } else {
+            std::cout << " prio=" << link->havePriority()
+                      << " override=" << limitedControlOverride(link)
+                      << " isOpen=" << link->opened(veh->getEventTime(), veh->getSpeed(), veh->estimateLeaveSpeed(link),
+                                                    veh->getVehicleType().getLengthWithGap(), veh->getImpatience(),
+                                                    veh->getVehicleType().getCarFollowModel().getMaxDecel(), veh->getWaitingTime())
+                      << " et=" << veh->getEventTime()
+                      << " v=" << veh->getSpeed()
+                      << " vLeave=" << veh->estimateLeaveSpeed(link)
+                      << " impatience=" << veh->getImpatience()
+                      << " tWait=" << veh->getWaitingTime();
+        }
+        std::cout << "\n";
+    }
+#endif
     if (myTLSPenalty) {
         // XXX should limited control take precedence over tls penalty?
         return true;
     }
     const MSLink* link = getLink(veh);
-    return (link == 0
+    return (link == nullptr
             || link->havePriority()
             || limitedControlOverride(link)
             || link->opened(veh->getEventTime(), veh->getSpeed(), veh->estimateLeaveSpeed(link),
@@ -447,27 +448,30 @@ MESegment::limitedControlOverride(const MSLink* link) const {
     // if the target segment of this link is not saturated junction control is disabled
     const MSEdge& targetEdge = link->getLane()->getEdge();
     const MESegment* target = MSGlobals::gMesoNet->getSegmentForEdge(targetEdge);
-    return target->myOccupancy * 2 < target->myJamThreshold;
+    return (target->myOccupancy * 2 < target->myJamThreshold) && !targetEdge.isRoundabout();
 }
 
 
 void
-MESegment::send(MEVehicle* veh, MESegment* next, SUMOTime time) {
+MESegment::send(MEVehicle* veh, MESegment* next, SUMOTime time, const MSMoveReminder::Notification reason) {
     assert(isInvalid(next) || time >= myBlockTimes[veh->getQueIndex()]);
     MSLink* link = getLink(veh);
-    if (link != 0) {
+    if (link != nullptr) {
         link->removeApproaching(veh);
     }
-    MEVehicle* lc = removeCar(veh, time, next); // new leaderCar
+    MEVehicle* lc = removeCar(veh, time, reason); // new leaderCar
     myBlockTimes[veh->getQueIndex()] = time;
     if (!isInvalid(next)) {
         myLastHeadway = next->getTimeHeadway(this, veh);
         myBlockTimes[veh->getQueIndex()] += myLastHeadway;
     }
-    if (lc != 0) {
+    if (lc != nullptr) {
         lc->setEventTime(MAX2(lc->getEventTime(), myBlockTimes[veh->getQueIndex()]));
         MSGlobals::gMesoNet->addLeaderCar(lc, getLink(lc));
     }
+    if (veh->isStopped()) {
+        veh->processStop();
+    }
 }
 
 bool
@@ -498,7 +502,7 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
         veh->setEventTime(time + TIME2STEPS(myLength / speed)); // for correct arrival speed
         addReminders(veh);
         veh->activateReminders(MSMoveReminder::NOTIFICATION_JUNCTION);
-        updateDetectorsOnLeave(veh, time, 0);
+        veh->updateDetectors(time, true, MSMoveReminder::NOTIFICATION_ARRIVED);
         MSNet::getInstance()->getVehicleControl().scheduleVehicleRemoval(veh);
         return;
     }
@@ -509,7 +513,7 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
     if (myCarQues.size() > 1) {
         const MSEdge* succ = veh->succEdge(1);
         // succ may be invalid if called from initialise() with an invalid route
-        if (succ != 0 && myFollowerMap.count(succ) > 0) {
+        if (succ != nullptr && myFollowerMap.count(succ) > 0) {
             const std::vector<int>& indices = myFollowerMap[succ];
             nextQueIndex = indices[0];
             for (std::vector<int>::const_iterator i = indices.begin() + 1; i != indices.end(); ++i) {
@@ -520,8 +524,11 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
         }
     }
     std::vector<MEVehicle*>& cars = myCarQues[nextQueIndex];
-    MEVehicle* newLeader = 0; // first vehicle in the current queue
-    SUMOTime tleave = MAX2(time + TIME2STEPS(myLength / uspeed) + veh->getStoptime(this) + getLinkPenalty(veh), myBlockTimes[nextQueIndex]);
+    MEVehicle* newLeader = nullptr; // first vehicle in the current queue
+    SUMOTime tleave = MAX2(veh->getStoptime(this, time) + TIME2STEPS(myLength / uspeed) + getLinkPenalty(veh), myBlockTimes[nextQueIndex]);
+    if (veh->isStopped()) {
+        myEdge.addWaiting(veh);
+    }
     myEdge.lock();
     if (cars.empty()) {
         cars.push_back(veh);
@@ -540,6 +547,7 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
         }
     }
     myEdge.unlock();
+    myNumCars++;
     if (!isDepart) {
         // regular departs could take place anywhere on the edge so they should not block regular flow
         // the -1 facilitates interleaving of multiple streams
@@ -557,7 +565,7 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
     } else {
         veh->activateReminders(MSMoveReminder::NOTIFICATION_SEGMENT);
     }
-    if (newLeader != 0) {
+    if (newLeader != nullptr) {
         MSGlobals::gMesoNet->addLeaderCar(newLeader, getLink(newLeader));
     }
 }
@@ -565,7 +573,7 @@ MESegment::receive(MEVehicle* veh, SUMOTime time, bool isDepart, bool afterTelep
 
 bool
 MESegment::vaporizeAnyCar(SUMOTime currentTime) {
-    MEVehicle* remove = 0;
+    MEVehicle* remove = nullptr;
     for (Queues::const_iterator k = myCarQues.begin(); k != myCarQues.end(); ++k) {
         if (!k->empty()) {
             // remove last in queue
@@ -649,13 +657,16 @@ MESegment::saveState(OutputDevice& out) {
 
 
 void
-MESegment::loadState(std::vector<std::string>& vehIds, MSVehicleControl& vc, const SUMOTime block, const int queIdx) {
-    for (std::vector<std::string>::const_iterator it = vehIds.begin(); it != vehIds.end(); ++it) {
-        MEVehicle* v = static_cast<MEVehicle*>(vc.getVehicle(*it));
-        assert(v != 0);
-        assert(v->getSegment() == this);
-        myCarQues[queIdx].push_back(v);
-        myOccupancy += v->getVehicleType().getLengthWithGap();
+MESegment::loadState(const std::vector<std::string>& vehIds, MSVehicleControl& vc, const SUMOTime block, const int queIdx) {
+    for (const std::string& id : vehIds) {
+        MEVehicle* v = static_cast<MEVehicle*>(vc.getVehicle(id));
+        // vehicle could be removed due to options
+        if (v != nullptr) {
+            assert(v->getSegment() == this);
+            myCarQues[queIdx].push_back(v);
+            myNumCars++;
+            myOccupancy += v->getVehicleType().getLengthWithGap();
+        }
     }
     if (myCarQues[queIdx].size() != 0) {
         // add the last vehicle of this queue
@@ -698,7 +709,7 @@ MESegment::getFlow() const {
 SUMOTime
 MESegment::getLinkPenalty(const MEVehicle* veh) const {
     const MSLink* link = getLink(veh, myTLSPenalty || myMinorPenalty);
-    if (link != 0) {
+    if (link != nullptr) {
         SUMOTime result = 0;
         if (link->isTLSControlled()) {
             result += link->getMesoTLSPenalty();
@@ -720,7 +731,7 @@ double
 MESegment::getTLSCapacity(const MEVehicle* veh) const {
     if (myTLSPenalty) {
         const MSLink* link = getLink(veh, true);
-        if (link != 0) {
+        if (link != nullptr) {
             assert(link->isTLSControlled());
             assert(link->getGreenFraction() > 0);
             return link->getGreenFraction();
