@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    METriggeredCalibrator.cpp
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id$
 ///
 // Calibrates the flow on a segment to a specified one
 /****************************************************************************/
@@ -37,7 +36,7 @@
 #include <utils/xml/XMLSubSys.h>
 #include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
-#include <utils/xml/SUMOVehicleParserHelper.h>
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 #include <utils/distribution/RandomDistributor.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include "MELoop.h"
@@ -54,8 +53,9 @@ METriggeredCalibrator::METriggeredCalibrator(const std::string& id,
         const std::string& aXMLFilename,
         const std::string& outputFilename,
         const SUMOTime freq, const double length,
-        const MSRouteProbe* probe) :
-    MSCalibrator(id, edge, (MSLane*)nullptr, pos, aXMLFilename, outputFilename, freq, length, probe, false),
+        const MSRouteProbe* probe,
+        const std::string& vTypes) :
+    MSCalibrator(id, edge, (MSLane*)nullptr, pos, aXMLFilename, outputFilename, freq, length, probe, vTypes, false),
     mySegment(MSGlobals::gMesoNet->getSegmentForEdge(*edge, pos)) {
     myEdgeMeanData.setDescription("meandata_calibrator_" + getID());
     mySegment->addDetector(&myEdgeMeanData);
@@ -65,7 +65,7 @@ METriggeredCalibrator::METriggeredCalibrator(const std::string& id,
 METriggeredCalibrator::~METriggeredCalibrator() {
     if (myCurrentStateInterval != myIntervals.end()) {
         // need to do it here and not in MSCalibrator because otherwise meandata is gone
-        writeXMLOutput();
+        intervalEnd();
         // but avoid to call it again in MSCalibrator
         myCurrentStateInterval = myIntervals.end();
     }
@@ -129,7 +129,7 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
     while (invalidJam()) {
         hadInvalidJam = true;
         if (!myHaveWarnedAboutClearingJam) {
-            WRITE_WARNING("Clearing jam at calibrator '" + myID + "' at time " + time2string(currentTime));
+            WRITE_WARNING("Clearing jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
         }
         // remove one vehicle currently on the segment
         if (mySegment->vaporizeAnyCar(currentTime)) {
@@ -137,7 +137,7 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         } else {
             if (!myHaveWarnedAboutClearingJam) {
                 // this frequenly happens for very short edges
-                WRITE_WARNING("Could not clear jam at calibrator '" + myID + "' at time " + time2string(currentTime));
+                WRITE_WARNING("Could not clear jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
             }
             break;
         }
@@ -168,11 +168,11 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
                     route = MSRoute::dictionary(pars->routeid);
                 }
                 if (route == nullptr) {
-                    WRITE_WARNING("No valid routes in calibrator '" + myID + "'.");
+                    WRITE_WARNING("No valid routes in calibrator '" + getID() + "'.");
                     break;
                 }
                 if (!route->contains(myEdge)) {
-                    WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + myID + "' does not contain edge '" + myEdge->getID() + "'.");
+                    WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + getID() + "' does not contain edge '" + myEdge->getID() + "'.");
                     break;
                 }
                 MSVehicleType* vtype = MSNet::getInstance()->getVehicleControl().getVType(pars->vtypeid);
@@ -180,7 +180,7 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
                 // build the vehicle
                 const SUMOTime depart = mySegment->getNextInsertionTime(currentTime);
                 SUMOVehicleParameter* newPars = new SUMOVehicleParameter(*pars);
-                newPars->id = myID + "." + toString(depart) + "." + toString(myInserted);
+                newPars->id = getNewVehicleID();
                 newPars->depart = depart;
                 newPars->routeid = route->getID();
                 MEVehicle* vehicle;
@@ -221,6 +221,7 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         while (totalWishedNum < adaptedNum) {
             if (!mySegment->vaporizeAnyCar(currentTime)) {
                 // @bug: short edges may be jumped in a single step, giving us no chance to remove a vehicle
+                // @bug2: vehicleApplies() must be considered (#3082)
                 break;
             }
             myRemoved++;
@@ -228,9 +229,12 @@ METriggeredCalibrator::execute(SUMOTime currentTime) {
         }
     }
     if (myCurrentStateInterval->end <= currentTime + myFrequency) {
-        writeXMLOutput();
+        intervalEnd();
+    }
+    //assert(!invalidJam());
+    if (invalidJam()) {
+        WRITE_WARNING("DEBUG: Could not clear jam at calibrator '" + getID() + "' at time " + time2string(currentTime));
     }
-    assert(!invalidJam());
     return myFrequency;
 }
 
