@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    METriggeredCalibrator.h
 /// @author  Daniel Krajzewicz
 /// @date    Tue, May 2005
-/// @version $Id$
 ///
 // Calibrates the flow on a segment to a specified one
 /****************************************************************************/
@@ -44,7 +43,8 @@ public:
                           const std::string& aXMLFilename,
                           const std::string& outputFilename,
                           const SUMOTime freq, const double length,
-                          const MSRouteProbe* probe);
+                          const MSRouteProbe* probe,
+                          const std::string& vTypes);
 
     /** destructor */
     virtual ~METriggeredCalibrator();
