@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Mon, 8 Nov 2010
-/// @version $Id$
 ///
 // A base class for vehicle implementations
 /****************************************************************************/
@@ -41,9 +40,15 @@
 #include "MSNet.h"
 #include "devices/MSDevice.h"
 #include "devices/MSDevice_Routing.h"
+#include "devices/MSDevice_Battery.h"
 #include <microsim/devices/MSDevice_Transportable.h>
 #include "MSInsertionControl.h"
 
+//#define DEBUG_REROUTE
+//#define DEBUG_COND (getID() == "follower")
+//#define DEBUG_COND (true)
+#define DEBUG_COND (isSelected())
+
 // ===========================================================================
 // static members
 // ===========================================================================
@@ -71,7 +76,7 @@ MSBaseVehicle::MSBaseVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
     myRoute(route),
     myType(type),
     myCurrEdge(route->begin()),
-    myChosenSpeedFactor(speedFactor),
+    myChosenSpeedFactor(pars->speedFactor < 0 ? speedFactor : pars->speedFactor),
     myMoveReminders(0),
     myPersonDevice(nullptr),
     myContainerDevice(nullptr),
@@ -80,6 +85,7 @@ MSBaseVehicle::MSBaseVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
     myArrivalPos(-1),
     myArrivalLane(-1),
     myNumberReroutes(0),
+    myOdometer(0.),
     myNumericalID(myCurrentNumericalIndex++)
 #ifdef _DEBUG
     , myTraceMoveReminders(myShallTraceMoveReminders.count(pars->id) > 0)
@@ -88,22 +94,31 @@ MSBaseVehicle::MSBaseVehicle(SUMOVehicleParameter* pars, const MSRoute* route,
     if ((*myRoute->begin())->isTazConnector() || myRoute->getLastEdge()->isTazConnector()) {
         pars->parametersSet |= VEHPARS_FORCE_REROUTE;
     }
-    // init devices
-    MSDevice::buildVehicleDevices(*this, myDevices);
-    //
-    for (MSVehicleDevice* dev : myDevices) {
-        myMoveReminders.push_back(std::make_pair(dev, 0.));
-    }
-    myRoute->addReference();
     if (!pars->wasSet(VEHPARS_FORCE_REROUTE)) {
         calculateArrivalParams();
         if (MSGlobals::gCheckRoutes) {
             std::string msg;
             if (!hasValidRoute(msg)) {
-                throw ProcessError("Vehicle '" + pars->id + "' has no valid route. " + msg);
+                msg = "Vehicle '" + pars->id + "' has no valid route. " + msg;
+                delete myParameter;
+                throw ProcessError(msg);
             }
         }
     }
+    // init devices
+    try {
+        MSDevice::buildVehicleDevices(*this, myDevices);
+    } catch (ProcessError&) {
+        for (MSVehicleDevice* dev : myDevices) {
+            delete dev;
+        }
+        delete myParameter;
+        throw;
+    }
+    myRoute->addReference();
+    for (MSVehicleDevice* dev : myDevices) {
+        myMoveReminders.push_back(std::make_pair(dev, 0.));
+    }
 }
 
 
@@ -130,6 +145,16 @@ MSBaseVehicle::getParameter() const {
     return *myParameter;
 }
 
+const std::map<int, double>*
+MSBaseVehicle::getEmissionParameters() const {
+    MSDevice_Battery* batteryDevice = static_cast<MSDevice_Battery*>(getDevice(typeid(MSDevice_Battery)));
+    if (batteryDevice != nullptr) {
+        return &batteryDevice->getEnergyParams();
+    } else {
+        return nullptr;
+    }
+}
+
 void
 MSBaseVehicle::replaceParameter(const SUMOVehicleParameter* newParameter) {
     delete myParameter;
@@ -159,7 +184,7 @@ MSBaseVehicle::getEdge() const {
 
 
 void
-MSBaseVehicle::reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, const bool onInit, const bool withTaz) {
+MSBaseVehicle::reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, const bool onInit, const bool withTaz, const bool silent) {
     // check whether to reroute
     const MSEdge* source = withTaz && onInit ? MSEdge::dictionary(myParameter->fromTaz + "-source") : getRerouteOrigin();
     if (source == nullptr) {
@@ -173,12 +198,40 @@ MSBaseVehicle::reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<M
     ConstMSEdgeVector edges;
     ConstMSEdgeVector stops;
     if (myParameter->via.size() == 0) {
-        stops = getStopEdges();
+        double firstPos = -1;
+        double lastPos = -1;
+        stops = getStopEdges(firstPos, lastPos);
+        if (stops.size() > 0) {
+            const double sourcePos = onInit ? 0 : getPositionOnLane();
+            // avoid superfluous waypoints for first and last edge
+            const bool skipFirst = stops.front() == source && sourcePos < firstPos;
+            const bool skipLast = stops.back() == sink && myArrivalPos > lastPos;
+#ifdef DEBUG_REROUTE
+            if (DEBUG_COND) {
+                std::cout << SIMTIME << " reroute " << info << " veh=" << getID() << " lane=" << getLane()->getID()
+                          << " source=" << source->getID() << " sourcePos=" << sourcePos << " firstPos=" << firstPos << " arrivalPos=" << myArrivalPos << " lastPos=" << lastPos
+                          << " route=" << toString(myRoute->getEdges()) << " stopEdges=" << toString(stops) << " skipFirst=" << skipFirst << " skipLast=" << skipLast << "\n";
+            }
+#endif
+            if (stops.size() == 1 && (skipFirst || skipLast)) {
+                stops.clear();
+            } else {
+                if (skipFirst) {
+                    stops.erase(stops.begin());
+                }
+                if (skipLast) {
+                    stops.erase(stops.end() - 1);
+                }
+            }
+        }
     } else {
         // via takes precedence over stop edges
         // XXX check for inconsistencies #2275
         for (std::vector<std::string>::const_iterator it = myParameter->via.begin(); it != myParameter->via.end(); ++it) {
             MSEdge* viaEdge = MSEdge::dictionary(*it);
+            if (viaEdge == source || viaEdge == sink) {
+                continue;
+            }
             assert(viaEdge != 0);
             if (!viaEdge->isTazConnector() && viaEdge->allowedLanes(getVClass()) == nullptr) {
                 throw ProcessError("Vehicle '" + getID() + "' is not allowed on any lane of via edge '" + viaEdge->getID() + "'.");
@@ -188,32 +241,33 @@ MSBaseVehicle::reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<M
     }
 
     for (MSRouteIterator s = stops.begin(); s != stops.end(); ++s) {
-        if (*s != source) {
-            // !!! need to adapt t here
-            ConstMSEdgeVector into;
-            router.compute(source, *s, this, t, into);
-            if (into.size() > 0) {
-                into.pop_back();
-                edges.insert(edges.end(), into.begin(), into.end());
-                if ((*s)->isTazConnector()) {
-                    source = into.back();
-                    edges.pop_back();
-                } else {
-                    source = *s;
-                }
+        // !!! need to adapt t here
+        ConstMSEdgeVector into;
+        router.computeLooped(source, *s, this, t, into, silent);
+        if (into.size() > 0) {
+            into.pop_back();
+            edges.insert(edges.end(), into.begin(), into.end());
+            if ((*s)->isTazConnector()) {
+                source = into.back();
+                edges.pop_back();
             } else {
-                std::string error = "Vehicle '" + getID() + "' has no valid route from edge '" + source->getID() + "' to stop edge '" + (*s)->getID() + "'.";
-                if (MSGlobals::gCheckRoutes) {
-                    throw ProcessError(error);
-                } else {
-                    WRITE_WARNING(error);
-                    edges.push_back(source);
-                }
                 source = *s;
             }
+        } else {
+            std::string error = "Vehicle '" + getID() + "' has no valid route from edge '" + source->getID() + "' to stop edge '" + (*s)->getID() + "'.";
+            if (MSGlobals::gCheckRoutes || silent) {
+                throw ProcessError(error);
+            } else {
+                WRITE_WARNING(error);
+                edges.push_back(source);
+            }
+            source = *s;
         }
     }
-    router.compute(source, sink, this, t, edges);
+    router.compute(source, sink, this, t, edges, silent);
+    if (edges.empty() && silent) {
+        return;
+    }
     if (!edges.empty() && edges.front()->isTazConnector()) {
         edges.erase(edges.begin());
     }
@@ -270,7 +324,8 @@ MSBaseVehicle::replaceRouteEdges(ConstMSEdgeVector& edges, double cost, double s
         }
         edges.insert(edges.begin(), myRoute->begin(), myCurrEdge);
     }
-    if (edges == myRoute->getEdges()) {
+    if (edges == myRoute->getEdges() && !StringUtils::endsWith(info, toString(SUMO_TAG_PARKING_ZONE_REROUTE))) {
+        // re-assign stop iterators when rerouting to a new parkingArea
         return true;
     }
     const RGBColor& c = myRoute->getColor();
@@ -331,6 +386,27 @@ MSBaseVehicle::hasArrived() const {
     return succEdge(1) == nullptr;
 }
 
+
+int
+MSBaseVehicle::getRoutePosition() const {
+    return (int) std::distance(myRoute->begin(), myCurrEdge);
+}
+
+
+void
+MSBaseVehicle::resetRoutePosition(int index, DepartLaneDefinition departLaneProcedure) {
+    myCurrEdge = myRoute->begin() + index;
+    const_cast<SUMOVehicleParameter*>(myParameter)->departLaneProcedure = departLaneProcedure;
+    // !!! hack
+    myArrivalPos = (*(myRoute->end() - 1))->getLanes()[0]->getLength();
+}
+
+double
+MSBaseVehicle::getOdometer() const {
+    return -myDepartPos + myOdometer + (hasArrived() ? myArrivalPos : getPositionOnLane());
+}
+
+
 void
 MSBaseVehicle::addPerson(MSTransportable* person) {
     if (myPersonDevice == nullptr) {
@@ -502,7 +578,9 @@ MSBaseVehicle::saveState(OutputDevice& out) {
     myParameter->write(out, OptionsCont::getOptions(), SUMO_TAG_VEHICLE, getVehicleType().getID());
     // params and stops must be written in child classes since they may wish to add additional attributes first
     out.writeAttr(SUMO_ATTR_ROUTE, myRoute->getID());
-    out.writeAttr(SUMO_ATTR_SPEEDFACTOR, myChosenSpeedFactor);
+    if (!myParameter->wasSet(VEHPARS_SPEEDFACTOR_SET)) {
+        out.writeAttr(SUMO_ATTR_SPEEDFACTOR, myChosenSpeedFactor);
+    }
     if (myParameter->wasSet(VEHPARS_FORCE_REROUTE) && !hasDeparted()) {
         out.writeAttr(SUMO_ATTR_REROUTE, true);
     }
@@ -560,10 +638,12 @@ MSBaseVehicle::getContainerNumber() const {
 
 void
 MSBaseVehicle::removeTransportable(MSTransportable* t) {
-    const bool isPerson = dynamic_cast<MSPerson*>(t) != nullptr;
-    MSDevice_Transportable* device = isPerson ? myPersonDevice : myContainerDevice;
-    if (device != nullptr) {
-        device->removeTransportable(t);
+    // this might be called from the MSTransportable destructor so we cannot do a dynamic cast to determine the type
+    if (myPersonDevice != nullptr) {
+        myPersonDevice->removeTransportable(t);
+    }
+    if (myContainerDevice != nullptr) {
+        myContainerDevice->removeTransportable(t);
     }
 }
 
@@ -663,6 +743,28 @@ MSBaseVehicle::getSingularType() {
 }
 
 
+int
+MSBaseVehicle::getRNGIndex() const {
+    const MSLane* const lane = getLane();
+    if (lane == nullptr) {
+        return getEdge()->getLanes()[0]->getRNGIndex();
+    } else {
+        return lane->getRNGIndex();
+    }
+}
+
+
+std::mt19937*
+MSBaseVehicle::getRNG() const {
+    const MSLane* lane = getLane();
+    if (lane == nullptr) {
+        return getEdge()->getLanes()[0]->getRNG();
+    } else {
+        return lane->getRNG();
+    }
+}
+
+
 #ifdef _DEBUG
 void
 MSBaseVehicle::initMoveReminderOutput(const OptionsCont& oc) {
