@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2010-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2010-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 8 Nov 2010
-/// @version $Id$
 ///
 // A base class for vehicle implementations
 /****************************************************************************/
@@ -76,6 +75,9 @@ public:
     /// @brief Destructor
     virtual ~MSBaseVehicle();
 
+    bool isVehicle() const {
+        return true;
+    }
 
     /// Returns the name of the vehicle
     const std::string& getID() const;
@@ -86,6 +88,12 @@ public:
      */
     const SUMOVehicleParameter& getParameter() const;
 
+    /** @brief Returns the vehicle's emission model parameter
+     *
+     * @return The vehicle's emission parameters
+     */
+    const std::map<int, double>* getEmissionParameters() const;
+
     /// @brief replace the vehicle parameter (deleting the old one)
     void replaceParameter(const SUMOVehicleParameter* newParameter);
 
@@ -208,7 +216,7 @@ public:
      * @param[in] router The router to use
      * @see replaceRoute
      */
-    void reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, const bool onInit = false, const bool withTaz = false);
+    void reroute(SUMOTime t, const std::string& info, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, const bool onInit = false, const bool withTaz = false, const bool silent = false);
 
 
     /** @brief Replaces the current route by the given edges
@@ -290,6 +298,17 @@ public:
      */
     virtual bool hasArrived() const;
 
+    /// @brief return index of edge within route
+    int getRoutePosition() const;
+
+    /// @brief reset index of edge within route
+    void resetRoutePosition(int index, DepartLaneDefinition departLaneProcedure);
+
+    /** @brief Returns the distance that was already driven by this vehicle
+     * @return the distance driven [m]
+     */
+    double getOdometer() const;
+
     /** @brief Returns the number of new routes this vehicle got
      * @return the number of new routes this vehicle got
      */
@@ -461,10 +480,24 @@ public:
         return false;
     }
 
+    /// @brief @return The index of the vehicle's associated RNG
+    int getRNGIndex() const;
+
+    /// @brief @return The vehicle's associated RNG
+    std::mt19937* getRNG() const;
+
     inline NumericalID getNumericalID() const {
         return myNumericalID;
     }
 
+    const MSDevice_Transportable* getPersonDevice() const {
+        return myPersonDevice;
+    }
+
+    const MSDevice_Transportable* getContainerDevice() const {
+        return myContainerDevice;
+    }
+
 protected:
     /** @brief (Re-)Calculates the arrival position and lane from the vehicle parameters
      */
@@ -472,7 +505,7 @@ protected:
 
     /** @brief Returns the list of still pending stop edges
      */
-    virtual const ConstMSEdgeVector getStopEdges() const = 0;
+    virtual const ConstMSEdgeVector getStopEdges(double& firstPos, double& lastPos) const = 0;
 
 protected:
     /// @brief This vehicle's parameter.
@@ -529,6 +562,9 @@ protected:
     /// @brief The number of reroutings
     int myNumberReroutes;
 
+    /// @brief A simple odometer to keep track of the length of the route already driven
+    double myOdometer;
+
     /* @brief magic value for undeparted vehicles
      * @note: in previous versions this was -1
      */
