@@ -1,19 +1,16 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2014-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2014-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSCModel_NonInteracting.cpp
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Tue, 29 July 2014
-/// @version $Id$
 ///
 // The container following model for tranship (prototype)
 /****************************************************************************/
@@ -21,11 +18,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 //
 #include <cmath>
 #include <algorithm>
@@ -42,7 +35,7 @@
 // ===========================================================================
 // static members
 // ===========================================================================
-MSCModel_NonInteracting* MSCModel_NonInteracting::myModel(0);
+MSCModel_NonInteracting* MSCModel_NonInteracting::myModel(nullptr);
 
 
 // named constants
@@ -66,7 +59,7 @@ MSCModel_NonInteracting::~MSCModel_NonInteracting() {
 
 MSCModel_NonInteracting*
 MSCModel_NonInteracting::getModel() {
-    if (myModel == 0) {
+    if (myModel == nullptr) {
         MSNet* net = MSNet::getInstance();
         myModel = new MSCModel_NonInteracting(net);
     }
@@ -76,7 +69,7 @@ MSCModel_NonInteracting::getModel() {
 CState*
 MSCModel_NonInteracting::add(MSTransportable* container, MSContainer::MSContainerStage_Tranship* stage, SUMOTime now) {
     CState* state = new CState();
-    const SUMOTime firstEdgeDuration = state->computeTranshipTime(0, *stage, now);
+    const SUMOTime firstEdgeDuration = state->computeTranshipTime(nullptr, *stage, now);
     myNet->getBeginOfTimestepEvents()->addEvent(new MoveToNextEdge(container, *stage), now + firstEdgeDuration);
     return state;
 }
@@ -84,9 +77,9 @@ MSCModel_NonInteracting::add(MSTransportable* container, MSContainer::MSContaine
 
 void
 MSCModel_NonInteracting::cleanup() {
-    if (myModel != 0) {
+    if (myModel != nullptr) {
         delete myModel;
-        myModel = 0;
+        myModel = nullptr;
     }
 }
 
