@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Thu, 12 Jun 2014
-/// @version $Id$
 ///
 // The class for modelling container-movements
 /****************************************************************************/
@@ -76,6 +75,8 @@ public:
         /// destructor
         ~MSContainerStage_Driving();
 
+        Stage* clone() const;
+
         /// proceeds to the next step
         virtual void proceed(MSNet* net, MSTransportable* container, SUMOTime now, Stage* previous);
 
@@ -113,6 +114,8 @@ public:
         /// destructor
         ~MSContainerStage_Tranship();
 
+        Stage* clone() const;
+
         /// proceeds to the next step
         virtual void proceed(MSNet* net, MSTransportable* container, SUMOTime now, Stage* previous);
 
@@ -134,6 +137,9 @@ public:
         /// Returns the angle of the container
         double getAngle(SUMOTime now) const;
 
+        /// @brief get travel distance in this stage
+        double getDistance() const;
+
         /// Returns the time the container spent waiting
         SUMOTime getWaitingTime(SUMOTime now) const;
 
