@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Andreas Kendziorra
 /// @author  Michael Behrisch
 /// @date    Thu, 12 Jun 2014
-/// @version $Id$
 ///
 // The common superclass for modelling transportable objects like persons and containers
 /****************************************************************************/
@@ -24,6 +23,7 @@
 #include <config.h>
 
 #include <math.h>
+#include <cmath>
 #include <utils/common/RandHelper.h>
 #include <utils/common/SUMOTime.h>
 //#include <microsim/MSVehicle.h>
@@ -85,6 +85,7 @@ double DriverStateDefaults::speedDifferenceErrorCoefficient = 0.15;
 double DriverStateDefaults::headwayErrorCoefficient = 0.75;
 double DriverStateDefaults::speedDifferenceChangePerceptionThreshold = 0.1;
 double DriverStateDefaults::headwayChangePerceptionThreshold = 0.1;
+double DriverStateDefaults::maximalReactionTimeFactor = 1.0;
 
 
 // ===========================================================================
@@ -111,6 +112,11 @@ OUProcess::step(double dt) {
 #endif
 }
 
+double
+OUProcess::step(double state, double dt, double timeScale, double noiseIntensity) {
+    /// see above
+    return exp(-dt / timeScale) * state + noiseIntensity * sqrt(2 * dt / timeScale) * RandHelper::randNorm(0, 1, &myRNG);
+}
 
 double
 OUProcess::getState() const {
@@ -129,7 +135,9 @@ MSSimpleDriverState::MSSimpleDriverState(MSVehicle* veh) :
     myHeadwayErrorCoefficient(DriverStateDefaults::headwayErrorCoefficient),
     myHeadwayChangePerceptionThreshold(DriverStateDefaults::headwayChangePerceptionThreshold),
     mySpeedDifferenceChangePerceptionThreshold(DriverStateDefaults::speedDifferenceChangePerceptionThreshold),
-    myActionStepLength(TS),
+    myOriginalReactionTime(veh->getActionStepLengthSecs()),
+    myMaximalReactionTime(DriverStateDefaults::maximalReactionTimeFactor * myOriginalReactionTime),
+//    myActionStepLength(TS),
     myStepDuration(TS),
     myLastUpdateTime(SIMTIME - TS),
     myDebugLock(false) {
@@ -137,25 +145,28 @@ MSSimpleDriverState::MSSimpleDriverState(MSVehicle* veh) :
     std::cout << "Constructing driver state for veh '" << veh->getID() << "'." << std::endl;
 #endif
     updateError();
+    updateReactionTime();
 }
 
 
 void
 MSSimpleDriverState::update() {
 #ifdef DEBUG_AWARENESS
-    if DEBUG_COND {
-    std::cout << SIMTIME << " veh=" << myVehicle->getID() << ", DriverState::update()" << std::endl;
+    if (DEBUG_COND) {
+        std::cout << SIMTIME << " veh=" << myVehicle->getID() << ", DriverState::update()" << std::endl;
     }
 #endif
     // Adapt step duration
     updateStepDuration();
     // Update error
     updateError();
+    // Update actionStepLength, aka reaction time
+    updateReactionTime();
     // Update assumed gaps
     updateAssumedGaps();
 #ifdef DEBUG_AWARENESS
-    if DEBUG_COND {
-    std::cout << SIMTIME << " stepDuration=" << myStepDuration << ", error=" << myError.getState() << std::endl;
+    if (DEBUG_COND) {
+        std::cout << SIMTIME << " stepDuration=" << myStepDuration << ", error=" << myError.getState() << std::endl;
     }
 #endif
 }
@@ -177,29 +188,44 @@ MSSimpleDriverState::updateError() {
     }
 }
 
+void
+MSSimpleDriverState::updateReactionTime() {
+    if (myAwareness == 1.0 || myAwareness == 0.0) {
+        myActionStepLength = myOriginalReactionTime;
+    } else {
+        const double theta = (myAwareness - myMinAwareness) / (1.0 - myMinAwareness);
+        myActionStepLength = myOriginalReactionTime + theta * (myMaximalReactionTime - myOriginalReactionTime);
+        // Round to multiple of simstep length
+        int quotient;
+        remquo(myActionStepLength, TS, &quotient);
+        myActionStepLength = TS * MAX2(quotient, 1);
+    }
+}
+
 void
 MSSimpleDriverState::setAwareness(const double value) {
     assert(value >= 0.);
     assert(value <= 1.);
 #ifdef DEBUG_AWARENESS
-    if DEBUG_COND {
-    std::cout << SIMTIME << " veh=" << myVehicle->getID() << ", setAwareness(" << MAX2(value, minAwareness) << ")" << std::endl;
+    if (DEBUG_COND) {
+        std::cout << SIMTIME << " veh=" << myVehicle->getID() << ", setAwareness(" << MAX2(value, myMinAwareness) << ")" << std::endl;
     }
 #endif
     myAwareness = MAX2(value, myMinAwareness);
     if (myAwareness == 1.) {
         myError.setState(0.);
     }
+    updateReactionTime();
 }
 
 
 double
 MSSimpleDriverState::getPerceivedHeadway(const double trueGap, const void* objID) {
 #ifdef DEBUG_PERCEPTION_ERRORS
-    if DEBUG_COND {
-    if (!debugLocked()) {
+    if (DEBUG_COND) {
+        if (!debugLocked()) {
             std::cout << SIMTIME << " getPerceivedHeadway() for veh '" << myVehicle->getID() << "'\n"
-            << "    trueGap=" << trueGap << " objID=" << objID << std::endl;
+                      << "    trueGap=" << trueGap << " objID=" << objID << std::endl;
         }
     }
 #endif
@@ -222,10 +248,10 @@ MSSimpleDriverState::getPerceivedHeadway(const double trueGap, const void* objID
     } else {
 
 #ifdef DEBUG_PERCEPTION_ERRORS
-        if DEBUG_COND {
-        if (!debugLocked()) {
+        if (DEBUG_COND) {
+            if (!debugLocked()) {
                 std::cout << "    new perceived gap (=" << perceivedGap << ") does *not* differ significantly from the assumed (="
-                << (assumedGap->second) << ")" << std::endl;
+                          << (assumedGap->second) << ")" << std::endl;
             }
         }
 #endif
@@ -254,10 +280,10 @@ MSSimpleDriverState::updateAssumedGaps() {
 double
 MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifference, const double trueGap, const void* objID) {
 #ifdef DEBUG_PERCEPTION_ERRORS
-    if DEBUG_COND {
-    if (!debugLocked()) {
+    if (DEBUG_COND) {
+        if (!debugLocked()) {
             std::cout << SIMTIME << " getPerceivedSpeedDifference() for veh '" << myVehicle->getID() << "'\n"
-            << "    trueGap=" << trueGap << " trueSpeedDifference=" << trueSpeedDifference << " objID=" << objID << std::endl;
+                      << "    trueGap=" << trueGap << " trueSpeedDifference=" << trueSpeedDifference << " objID=" << objID << std::endl;
         }
     }
 #endif
@@ -267,11 +293,11 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
             || fabs(perceivedSpeedDifference - lastPerceivedSpeedDifference->second) > mySpeedDifferenceChangePerceptionThreshold * trueGap * (1.0 - myAwareness)) {
 
 #ifdef DEBUG_PERCEPTION_ERRORS
-        if DEBUG_COND {
-        if (!debugLocked()) {
+        if (DEBUG_COND) {
+            if (!debugLocked()) {
                 std::cout << "    new perceived speed difference (=" << perceivedSpeedDifference << ") differs significantly from the last perceived (="
-                << (lastPerceivedSpeedDifference == myLastPerceivedSpeedDifference.end() ? "NA" : toString(lastPerceivedSpeedDifference->second)) << ")"
-                << std::endl;
+                          << (lastPerceivedSpeedDifference == myLastPerceivedSpeedDifference.end() ? "NA" : toString(lastPerceivedSpeedDifference->second)) << ")"
+                          << std::endl;
             }
         }
 #endif
@@ -376,7 +402,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //void
 //MSDriverState::updateAccelerationError() {
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << SIMTIME << " Updating acceleration error (for " << myStepDuration << " s.):\n  "
 //                << myAccelerationError.getState() << " -> ";
 //    }
@@ -385,7 +411,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //    updateErrorProcess(myAccelerationError, myAccelerationErrorTimeScaleCoefficient, myAccelerationErrorNoiseIntensityCoefficient);
 //
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << myAccelerationError.getState() << std::endl;
 //    }
 //#endif
@@ -394,7 +420,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //void
 //MSDriverState::updateSpeedPerceptionError() {
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << SIMTIME << " Updating speed perception error (for " << myStepDuration << " s.):\n  "
 //        << mySpeedPerceptionError.getState() << " -> ";
 //    }
@@ -403,7 +429,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //    updateErrorProcess(mySpeedPerceptionError, mySpeedPerceptionErrorTimeScaleCoefficient, mySpeedPerceptionErrorNoiseIntensityCoefficient);
 //
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << mySpeedPerceptionError.getState() << std::endl;
 //    }
 //#endif
@@ -412,7 +438,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //void
 //MSDriverState::updateHeadwayPerceptionError() {
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << SIMTIME << " Updating headway perception error (for " << myStepDuration << " s.):\n  "
 //        << myHeadwayPerceptionError.getState() << " -> ";
 //    }
@@ -421,7 +447,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //    updateErrorProcess(myHeadwayPerceptionError, myHeadwayPerceptionErrorTimeScaleCoefficient, myHeadwayPerceptionErrorNoiseIntensityCoefficient);
 //
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << myHeadwayPerceptionError.getState() << std::endl;
 //    }
 //#endif
@@ -430,7 +456,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //void
 //MSDriverState::updateActionStepLength() {
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << SIMTIME << " Updating action step length (for " << myStepDuration << " s.): \n" << myActionStepLength;
 //    }
 //#endif
@@ -440,7 +466,7 @@ MSSimpleDriverState::getPerceivedSpeedDifference(const double trueSpeedDifferenc
 //        myActionStepLength = MIN2(myActionStepLengthCoefficient*myCurrentDrivingDifficulty - myMinActionStepLength, myMaxActionStepLength);
 //    }
 //#ifdef DEBUG_OUPROCESS
-//    if DEBUG_COND {
+//    if (DEBUG_COND) {
 //        std::cout << " -> " << myActionStepLength << std::endl;
 //    }
 //#endif
