@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -16,7 +16,6 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 06 Mar 2001
-/// @version $Id$
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
@@ -65,7 +64,8 @@ MSEdge::MSEdge(const std::string& id, int numericalID,
                const SumoXMLEdgeFunc function,
                const std::string& streetName,
                const std::string& edgeType,
-               int priority) :
+               int priority,
+               double distance) :
     Named(id), myNumericalID(numericalID), myLanes(nullptr),
     myLaneChanger(nullptr), myFunction(function), myVaporizationRequests(0),
     myLastFailedInsertionTime(-1),
@@ -73,52 +73,37 @@ MSEdge::MSEdge(const std::string& id, int numericalID,
     myStreetName(streetName),
     myEdgeType(edgeType),
     myPriority(priority),
+    myDistance(distance),
     myWidth(0.),
     myLength(0.),
     myEmptyTraveltime(0.),
+    myTimePenalty(0.),
     myAmDelayed(false),
     myAmRoundabout(false),
     myAmFringe(true),
-    myBidiEdge(nullptr) 
+    myBidiEdge(nullptr)
 { }
 
 
 MSEdge::~MSEdge() {
     delete myLaneChanger;
-    for (auto i : myAllowed) {
-        if (i.second != myLanes) {
-            delete i.second;
-        }
-    }
-    for (auto i2 : myAllowedTargets) {
-        for (auto i1 : i2.second) {
-            if (i1.second != myLanes) {
-                delete i1.second;
-            }
-        }
-    }
-    delete myLanes;
 }
 
 
 void
 MSEdge::initialize(const std::vector<MSLane*>* lanes) {
     assert(lanes != 0);
-    myLanes = lanes;
+    myLanes = std::shared_ptr<const std::vector<MSLane*> >(lanes);
     if (myFunction == EDGEFUNC_CONNECTOR) {
         myCombinedPermissions = SVCAll;
     }
-    for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-        myWidth += (*i)->getWidth();
-    }
-    double widthBefore = 0;
-    for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-        (*i)->setRightSideOnEdge(widthBefore, (int)mySublaneSides.size());
-        MSLeaderInfo ahead(*i);
+    for (MSLane* const lane : *lanes) {
+        lane->setRightSideOnEdge(myWidth, (int)mySublaneSides.size());
+        MSLeaderInfo ahead(lane);
         for (int j = 0; j < ahead.numSublanes(); ++j) {
-            mySublaneSides.push_back(widthBefore + j * MSGlobals::gLateralResolution);
+            mySublaneSides.push_back(myWidth + j * MSGlobals::gLateralResolution);
         }
-        widthBefore += (*i)->getWidth();
+        myWidth += lane->getWidth();
     }
 }
 
@@ -149,6 +134,12 @@ void MSEdge::recalcCache() {
         if (minPenalty > 0) {
             myEmptyTraveltime += STEPS2TIME(minPenalty);
         }
+    } else if (isInternal()) {
+        const MSLink* link = myLanes->front()->getIncomingLanes()[0].viaLink;
+        if (!link->isTLSControlled() && !link->havePriority()) {
+            myEmptyTraveltime += MSGlobals::gMinorPenalty;
+            myTimePenalty = MSGlobals::gMinorPenalty;
+        }
     }
 }
 
@@ -180,6 +171,7 @@ MSEdge::closeBuilding() {
                 }
             }
         }
+        lane->checkBufferType();
     }
     std::sort(mySuccessors.begin(), mySuccessors.end(), by_id_sorter());
     rebuildAllowedLanes();
@@ -197,12 +189,12 @@ MSEdge::buildLaneChanger() {
         const bool allowChanging = allowsLaneChanging();
         if (MSGlobals::gLateralResolution > 0) {
             // may always initiate sublane-change
-            myLaneChanger = new MSLaneChangerSublane(myLanes, allowChanging);
+            myLaneChanger = new MSLaneChangerSublane(myLanes.get(), allowChanging);
         } else {
             if (MSGlobals::gLaneChangeDuration > 0) {
-                myLaneChanger = new MSLaneChanger(myLanes, allowChanging);
+                myLaneChanger = new MSLaneChanger(myLanes.get(), allowChanging);
             } else if (myLanes->size() > 1 || canChangeToOpposite()) {
-                myLaneChanger = new MSLaneChanger(myLanes, allowChanging);
+                myLaneChanger = new MSLaneChanger(myLanes.get(), allowChanging);
             }
         }
     }
@@ -232,23 +224,16 @@ MSEdge::allowsLaneChanging() const {
 }
 
 
-void 
-MSEdge::addToAllowed(const SVCPermissions permissions, const std::vector<MSLane*>* allowedLanes, AllowedLanesCont& laneCont) const {
-    // recheck whether we had this list to save memory
-    if (allowedLanes->empty()) {
-        delete allowedLanes;
-        allowedLanes = nullptr;
-    } else {
+void
+MSEdge::addToAllowed(const SVCPermissions permissions, std::shared_ptr<const std::vector<MSLane*> > allowedLanes, AllowedLanesCont& laneCont) const {
+    if (!allowedLanes->empty()) {
+        // recheck whether we had this list to save memory
         for (auto& allowed : laneCont) {
             if (*allowed.second == *allowedLanes) {
-                delete allowedLanes;
-                allowedLanes = nullptr;
                 allowed.first |= permissions;
-                break;
+                return;
             }
         }
-    }
-    if (allowedLanes != nullptr) {
         laneCont.push_back(std::make_pair(permissions, allowedLanes));
     }
 }
@@ -264,19 +249,14 @@ MSEdge::rebuildAllowedLanes() {
         myCombinedPermissions |= lane->getPermissions();
     }
     // rebuild myAllowed
-    for (const auto i : myAllowed) {
-        if (i.second != myLanes) {
-            delete i.second;
-        }
-    }
     myAllowed.clear();
     if (myCombinedPermissions != myMinimumPermissions) {
         myAllowed.push_back(std::make_pair(SVC_IGNORING, myLanes));
-        for (SUMOVehicleClass vclass = SVC_PRIVATE; vclass <= SUMOVehicleClass_MAX; vclass = (SUMOVehicleClass)(2 * (int)vclass)) {
+        for (int vclass = SVC_PRIVATE; vclass <= SUMOVehicleClass_MAX; vclass *= 2) {
             if ((myCombinedPermissions & vclass) == vclass) {
-                std::vector<MSLane*>* allowedLanes = new std::vector<MSLane*>();
+                std::shared_ptr<std::vector<MSLane*> > allowedLanes = std::make_shared<std::vector<MSLane*> >();
                 for (MSLane* const lane : *myLanes) {
-                    if (lane->allowsVehicleClass(vclass)) {
+                    if (lane->allowsVehicleClass((SUMOVehicleClass)vclass)) {
                         allowedLanes->push_back(lane);
                     }
                 }
@@ -290,18 +270,10 @@ MSEdge::rebuildAllowedLanes() {
 
 void
 MSEdge::rebuildAllowedTargets(const bool updateVehicles) {
-    for (const auto i2 : myAllowedTargets) {
-        for (const auto i1 : i2.second) {
-            if (i1.second != myLanes) {
-                delete i1.second;
-            }
-        }
-    }
     myAllowedTargets.clear();
-
     for (const MSEdge* target : mySuccessors) {
         bool universalMap = true; // whether the mapping for SVC_IGNORING is also valid for all vehicle classes
-        std::vector<MSLane*>* allLanes = new std::vector<MSLane*>();
+        std::shared_ptr<std::vector<MSLane*> > allLanes = std::make_shared<std::vector<MSLane*> >();
         // compute the mapping for SVC_IGNORING
         for (MSLane* const lane : *myLanes) {
             SVCPermissions combinedTargetPermissions = 0;
@@ -320,21 +292,20 @@ MSEdge::rebuildAllowedTargets(const bool updateVehicles) {
                 // we have no lane specific permissions
                 myAllowedTargets[target].push_back(std::make_pair(myMinimumPermissions, myLanes));
             } else {
-                for (const auto i : myAllowed) {
-                    // we cannot add the lane vectors directly because they are deleted separately (shared_ptr anyone?)
-                    addToAllowed(i.first, new std::vector<MSLane*>(*i.second), myAllowedTargets[target]);
+                for (const auto& i : myAllowed) {
+                    addToAllowed(i.first, i.second, myAllowedTargets[target]);
                 }
             }
         } else {
             addToAllowed(SVC_IGNORING, allLanes, myAllowedTargets[target]);
             // compute the vclass specific mapping
-            for (SUMOVehicleClass vclass = SVC_PRIVATE; vclass <= SUMOVehicleClass_MAX; vclass = (SUMOVehicleClass)(2 * (int)vclass)) {
+            for (int vclass = SVC_PRIVATE; vclass <= SUMOVehicleClass_MAX; vclass *= 2) {
                 if ((myCombinedPermissions & vclass) == vclass) {
-                    std::vector<MSLane*>* allowedLanes = new std::vector<MSLane*>();
+                    std::shared_ptr<std::vector<MSLane*> > allowedLanes = std::make_shared<std::vector<MSLane*> >();
                     for (MSLane* const lane : *myLanes) {
-                        if (lane->allowsVehicleClass(vclass)) {
+                        if (lane->allowsVehicleClass((SUMOVehicleClass)vclass)) {
                             for (const MSLink* const link : lane->getLinkCont()) {
-                                if (link->getLane()->allowsVehicleClass(vclass) && &link->getLane()->getEdge() == target) {
+                                if (link->getLane()->allowsVehicleClass((SUMOVehicleClass)vclass) && &link->getLane()->getEdge() == target && (link->getViaLane() == nullptr || link->getViaLane()->allowsVehicleClass((SUMOVehicleClass)vclass))) {
                                     allowedLanes->push_back(lane);
                                 }
                             }
@@ -372,13 +343,11 @@ MSEdge::rightLane(const MSLane* const lane) const {
 
 
 MSLane*
-MSEdge::parallelLane(const MSLane* const lane, int offset) const {
-    const int index = (int)(find(myLanes->begin(), myLanes->end(), lane) - myLanes->begin());
-    if (index == (int)myLanes->size()) {
-        return nullptr;
-    }
-    const int resultIndex = index + offset;
-    if (resultIndex >= (int)myLanes->size() || resultIndex < 0) {
+MSEdge::parallelLane(const MSLane* const lane, int offset, bool includeOpposite) const {
+    const int resultIndex = lane->getIndex() + offset;
+    if (resultIndex == (int)myLanes->size() && includeOpposite) {
+        return lane->getOpposite();
+    } else if (resultIndex >= (int)myLanes->size() || resultIndex < 0) {
         return nullptr;
     } else {
         return (*myLanes)[resultIndex];
@@ -392,7 +361,7 @@ MSEdge::allowedLanes(const MSEdge& destination, SUMOVehicleClass vclass) const {
     if (i != myAllowedTargets.end()) {
         for (const auto& allowed : i->second) {
             if ((allowed.first & vclass) == vclass) {
-                return allowed.second;
+                return allowed.second.get();
             }
         }
     }
@@ -403,12 +372,12 @@ MSEdge::allowedLanes(const MSEdge& destination, SUMOVehicleClass vclass) const {
 const std::vector<MSLane*>*
 MSEdge::allowedLanes(SUMOVehicleClass vclass) const {
     if ((myMinimumPermissions & vclass) == vclass) {
-        return myLanes;
+        return myLanes.get();
     } else {
         if ((myCombinedPermissions & vclass) == vclass) {
             for (const auto& allowed : myAllowed) {
                 if ((allowed.first & vclass) == vclass) {
-                    return allowed.second;
+                    return allowed.second.get();
                 }
             }
         }
@@ -441,7 +410,7 @@ MSEdge::getFreeLane(const std::vector<MSLane*>* allowed, const SUMOVehicleClass
     if (allowed != nullptr) {
         double largestGap = 0;
         MSLane* resByGap = nullptr;
-        double leastOccupancy = std::numeric_limits<double>::max();;
+        double leastOccupancy = std::numeric_limits<double>::max();
         for (std::vector<MSLane*>::const_iterator i = allowed->begin(); i != allowed->end(); ++i) {
             const double occupancy = (*i)->getBruttoOccupancy();
             if (occupancy < leastOccupancy) {
@@ -696,7 +665,6 @@ MSEdge::changeLanes(SUMOTime t) {
 }
 
 
-
 const MSEdge*
 MSEdge::getInternalFollowingEdge(const MSEdge* followerAfterInternal) const {
     //@todo to be optimized
@@ -714,6 +682,7 @@ MSEdge::getInternalFollowingEdge(const MSEdge* followerAfterInternal) const {
     return nullptr;
 }
 
+
 double
 MSEdge::getInternalFollowingLengthTo(const MSEdge* followerAfterInternal) const {
     assert(followerAfterInternal != 0);
@@ -728,6 +697,7 @@ MSEdge::getInternalFollowingLengthTo(const MSEdge* followerAfterInternal) const
     return dist;
 }
 
+
 const MSEdge*
 MSEdge::getNormalBefore() const {
     const MSEdge* result = this;
@@ -759,6 +729,14 @@ MSEdge::getMeanSpeed() const {
             v += vehNo * (*i)->getMeanSpeed();
             no += vehNo;
         }
+        if (myBidiEdge != nullptr) {
+            for (const MSLane* lane : myBidiEdge->getLanes()) {
+                if (lane->getVehicleNumber() > 0) {
+                    // do not route across edges which are already occupied in reverse direction
+                    return 0;
+                }
+            }
+        }
         if (no == 0) {
             return getSpeedLimit();
         }
@@ -783,7 +761,6 @@ MSEdge::getRoutingSpeed() const {
 }
 
 
-
 bool
 MSEdge::dictionary(const std::string& id, MSEdge* ptr) {
     DictType::iterator it = myDict.find(id);
@@ -826,7 +803,7 @@ MSEdge::getAllEdges() {
 void
 MSEdge::clear() {
     for (DictType::iterator i = myDict.begin(); i != myDict.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
     myDict.clear();
     myEdges.clear();
@@ -873,6 +850,7 @@ MSEdge::parseEdgesList(const std::vector<std::string>& desc, ConstMSEdgeVector&
 
 double
 MSEdge::getDistanceTo(const MSEdge* other, const bool doBoundaryEstimate) const {
+    assert(this != other);
     if (doBoundaryEstimate) {
         return myBoundary.distanceTo2D(other->myBoundary);
     }
@@ -908,7 +886,7 @@ MSEdge::getLengthGeometryFactor() const {
 }
 
 double
-MSEdge::getVehicleMaxSpeed(const SUMOVehicle* const veh) const {
+MSEdge::getVehicleMaxSpeed(const SUMOTrafficObject* const veh) const {
     // @note lanes might have different maximum speeds in theory
     return myLanes->empty() ? 1 : getLanes()[0]->getVehicleMaxSpeed(veh);
 }
@@ -983,9 +961,7 @@ MSEdge::getSuccessors(SUMOVehicleClass vClass) const {
         return mySuccessors;
     }
 #ifdef HAVE_FOX
-    if (MSRoutingEngine::isParallel()) {
-        MSRoutingEngine::lock();
-    }
+    FXConditionalLock lock(mySuccessorMutex, MSGlobals::gNumThreads > 1);
 #endif
     std::map<SUMOVehicleClass, MSEdgeVector>::iterator i = myClassesSuccessorMap.find(vClass);
     if (i == myClassesSuccessorMap.end()) {
@@ -1005,11 +981,6 @@ MSEdge::getSuccessors(SUMOVehicleClass vClass) const {
         }
     }
     // can use cached value
-#ifdef HAVE_FOX
-    if (MSRoutingEngine::isParallel()) {
-        MSRoutingEngine::unlock();
-    }
-#endif
     return i->second;
 }
 
@@ -1020,18 +991,11 @@ MSEdge::getViaSuccessors(SUMOVehicleClass vClass) const {
         return myViaSuccessors;
     }
 #ifdef HAVE_FOX
-    if (MSRoutingEngine::isParallel()) {
-        MSRoutingEngine::lock();
-    }
+    FXConditionalLock lock(mySuccessorMutex, MSGlobals::gNumThreads > 1);
 #endif
     auto i = myClassesViaSuccessorMap.find(vClass);
     if (i != myClassesViaSuccessorMap.end()) {
         // can use cached value
-#ifdef HAVE_FOX
-        if (MSRoutingEngine::isParallel()) {
-            MSRoutingEngine::unlock();
-        }
-#endif
         return i->second;
     }
     // instantiate vector
@@ -1047,11 +1011,6 @@ MSEdge::getViaSuccessors(SUMOVehicleClass vClass) const {
             }
         }
     }
-#ifdef HAVE_FOX
-    if (MSRoutingEngine::isParallel()) {
-        MSRoutingEngine::unlock();
-    }
-#endif
     return result;
 }
 
@@ -1099,14 +1058,15 @@ MSEdge::hasMinorLink() const {
 }
 
 
-void MSEdge::checkAndRegisterBiDirEdge(const std::string& bidiID) {
+void
+MSEdge::checkAndRegisterBiDirEdge(const std::string& bidiID) {
     if (bidiID != "") {
         myBidiEdge = dictionary(bidiID);
         if (myBidiEdge == nullptr) {
             WRITE_ERROR("Bidi-edge '" + bidiID + "' does not exist");
         }
         return;
-    } 
+    }
     if (getFunction() != EDGEFUNC_NORMAL) {
         return;
     }
@@ -1123,7 +1083,8 @@ void MSEdge::checkAndRegisterBiDirEdge(const std::string& bidiID) {
 }
 
 
-bool MSEdge::isSuperposable(const MSEdge* other) {
+bool
+MSEdge::isSuperposable(const MSEdge* other) {
     if (other == nullptr || other->getLanes().size() != myLanes->size()) {
         return false;
     }
@@ -1141,4 +1102,47 @@ bool MSEdge::isSuperposable(const MSEdge* other) {
 }
 
 
+void
+MSEdge::addWaiting(SUMOVehicle* vehicle) const {
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myWaitingMutex, MSGlobals::gNumSimThreads > 1);
+#endif
+    myWaiting.push_back(vehicle);
+}
+
+
+void
+MSEdge::removeWaiting(const SUMOVehicle* vehicle) const {
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myWaitingMutex, MSGlobals::gNumSimThreads > 1);
+#endif
+    std::vector<SUMOVehicle*>::iterator it = std::find(myWaiting.begin(), myWaiting.end(), vehicle);
+    if (it != myWaiting.end()) {
+        myWaiting.erase(it);
+    }
+}
+
+
+SUMOVehicle*
+MSEdge::getWaitingVehicle(MSTransportable* transportable, const double position) const {
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myWaitingMutex, MSGlobals::gNumSimThreads > 1);
+#endif
+    for (SUMOVehicle* const vehicle : myWaiting) {
+        if (transportable->isWaitingFor(vehicle)) {
+            if (vehicle->isStoppedInRange(position, MSGlobals::gStopTolerance) ||
+                    (!vehicle->hasDeparted() &&
+                     (vehicle->getParameter().departProcedure == DEPART_TRIGGERED ||
+                      vehicle->getParameter().departProcedure == DEPART_CONTAINER_TRIGGERED))) {
+                return vehicle;
+            }
+            // !!! this gives false warnings when there are two stops on the same edge
+            WRITE_WARNING(transportable->getID() + " at edge '" + getID() + "' position " + toString(position) + " cannot use waiting vehicle '"
+                          + vehicle->getID() + "' at position " + toString(vehicle->getPositionOnLane()) + " because it is too far away.");
+        }
+    }
+    return nullptr;
+}
+
+
 /****************************************************************************/
