@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id$
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
@@ -31,12 +30,16 @@
 #include <map>
 #include <string>
 #include <iostream>
+#ifdef HAVE_FOX
+#include <fx.h>
+#endif
 #include <utils/common/Named.h>
 #include <utils/common/Parameterised.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/geom/Boundary.h>
 #include <utils/vehicle/SUMOVehicle.h>
+#include <utils/vehicle/SUMOTrafficObject.h>
 #include "MSNet.h"
 
 
@@ -75,7 +78,7 @@ typedef std::vector<std::pair<const MSEdge*, const MSEdge*> > MSConstEdgePairVec
 class MSEdge : public Named, public Parameterised {
 private:
     /** @brief "Map" from vehicle class to allowed lanes */
-    typedef std::vector<std::pair<SVCPermissions, const std::vector<MSLane*>* > > AllowedLanesCont;
+    typedef std::vector<std::pair<SVCPermissions, std::shared_ptr<const std::vector<MSLane*> > > > AllowedLanesCont;
 
     /** @brief Succeeding edges (keys) and allowed lanes to reach these edges (values). */
     typedef std::map<const MSEdge*, AllowedLanesCont> AllowedLanesByTarget;
@@ -94,7 +97,8 @@ public:
      * @param[in] streetName The street name for that edge
      */
     MSEdge(const std::string& id, int numericalID, const SumoXMLEdgeFunc function,
-           const std::string& streetName, const std::string& edgeType, int priority);
+           const std::string& streetName, const std::string& edgeType, int priority,
+           double distance);
 
 
     /// @brief Destructor.
@@ -150,9 +154,10 @@ public:
      *
      * @param[in] lane The base lane
      * @param[in] offset The offset of the result lane
+     * @param[in] includeOpposte Whether an opposite direction lane may be returned
      * @todo This method searches for the given in the container; probably, this could be done faster
      */
-    MSLane* parallelLane(const MSLane* const lane, int offset) const;
+    MSLane* parallelLane(const MSLane* const lane, int offset, bool includeOpposite = true) const;
 
 
     /** @brief Returns this edge's lanes
@@ -221,6 +226,11 @@ public:
         return myFunction;
     }
 
+    /// @brief return whether this edge is an internal edge
+    inline bool isNormal() const {
+        return myFunction == EDGEFUNC_NORMAL;
+    }
+
     /// @brief return whether this edge is an internal edge
     inline bool isInternal() const {
         return myFunction == EDGEFUNC_INTERNAL;
@@ -233,7 +243,7 @@ public:
 
 
     /// @brief check and register the opposite superposable edge if any
-    void checkAndRegisterBiDirEdge(const std::string& bidiID="");
+    void checkAndRegisterBiDirEdge(const std::string& bidiID = "");
 
     /// @brief return opposite superposable/congruent edge, if it exist and 0 else
     inline const MSEdge* getBidiEdge() const {
@@ -274,6 +284,12 @@ public:
     int getPriority() const {
         return myPriority;
     }
+
+    /** @brief Returns the kilometrage/mileage at the start of the edge
+     */
+    double getDistance() const {
+        return myDistance;
+    }
     /// @}
 
     /**@brief Sets the crossed edge ids for a crossing edge
@@ -406,9 +422,9 @@ public:
         if (myFunction == EDGEFUNC_CONNECTOR) {
             return 0;
         } else if (veh != 0) {
-            return getLength() / getVehicleMaxSpeed(veh);
+            return getLength() / getVehicleMaxSpeed(veh) + myTimePenalty;
         } else {
-            return getLength() / getSpeedLimit();
+            return myEmptyTraveltime;
         }
     }
 
@@ -529,6 +545,11 @@ public:
         return (myCombinedPermissions & svc) != svc;
     }
 
+    /// @brief Returns whether the vehicle (class) is not allowed on the edge
+    inline bool restricts(const SUMOVehicle* const /* vehicle */) const {
+        return false;
+    }
+
     inline SVCPermissions getPermissions() const {
         return myCombinedPermissions;
     }
@@ -547,7 +568,7 @@ public:
 
     void rebuildAllowedLanes();
 
-    void rebuildAllowedTargets(const bool updateVehicles=true);
+    void rebuildAllowedTargets(const bool updateVehicles = true);
 
 
     /** @brief optimistic air distance heuristic for use in routing
@@ -555,7 +576,7 @@ public:
      * @param[in] doBoundaryEstimate whether the distance should be estimated by looking at the distance of the bounding boxes
      * @return The distance to the other edge
      */
-    double getDistanceTo(const MSEdge* other, const bool doBoundaryEstimate=false) const;
+    double getDistanceTo(const MSEdge* other, const bool doBoundaryEstimate = false) const;
 
 
     /// @brief return the coordinates of the center of the given stop
@@ -589,7 +610,7 @@ public:
      * @caution Only the first lane is considered
      * @return The maximum velocity on this edge for the given vehicle
      */
-    double getVehicleMaxSpeed(const SUMOVehicle* const veh) const;
+    double getVehicleMaxSpeed(const SUMOTrafficObject* const veh) const;
 
 
     virtual void addPerson(MSTransportable* p) const {
@@ -630,7 +651,7 @@ public:
 
     // return whether there have been vehicles on this edge at least once
     inline bool isDelayed() const {
-        return myAmDelayed;
+        return myAmDelayed || myBidiEdge == nullptr || myBidiEdge->myAmDelayed;
     }
 
     bool hasLaneChanger() const {
@@ -665,6 +686,18 @@ public:
     /// @brief release exclusive access to the mesoscopic state
     virtual void unlock() const {};
 
+    /// @brief Adds a vehicle to the list of waiting vehicles
+    void addWaiting(SUMOVehicle* vehicle) const;
+
+    /// @brief Removes a vehicle from the list of waiting vehicles
+    void removeWaiting(const SUMOVehicle* vehicle) const;
+
+    /* @brief returns a vehicle that is waiting for a for a person or a container at this edge at the given position
+     * @param[in] transportable The person or container that wants to ride
+     * @param[in] position The vehicle shall be positioned in the interval [position - t, position + t], where t is some tolerance
+     */
+    SUMOVehicle* getWaitingVehicle(MSTransportable* transportable, const double position) const;
+
     /** @brief Inserts edge into the static dictionary
         Returns true if the key id isn't already in the dictionary. Otherwise
         returns false. */
@@ -752,7 +785,7 @@ protected:
     const int myNumericalID;
 
     /// @brief Container for the edge's lane; should be sorted: (right-hand-traffic) the more left the lane, the higher the container-index
-    const std::vector<MSLane*>* myLanes;
+    std::shared_ptr<const std::vector<MSLane*> > myLanes;
 
     /// @brief This member will do the lane-change
     MSLaneChanger* myLaneChanger;
@@ -802,9 +835,9 @@ protected:
     AllowedLanesByTarget myAllowedTargets;
 
     /// @brief The intersection of lane permissions for this edge
-    SVCPermissions myMinimumPermissions;
+    SVCPermissions myMinimumPermissions = SVCAll;
     /// @brief The union of lane permissions for this edge
-    SVCPermissions myCombinedPermissions;
+    SVCPermissions myCombinedPermissions = 0;
     /// @}
 
     /// @brief the real-world name of this edge (need not be unique)
@@ -816,6 +849,9 @@ protected:
     /// @brief the priority of the edge (used during network creation)
     const int myPriority;
 
+    /// @brief the kilometrage/mileage at the start of the edge
+    const double myDistance;
+
     /// Edge width [m]
     double myWidth;
 
@@ -825,6 +861,9 @@ protected:
     /// @brief the traveltime on the empty edge (cached value for speedup)
     double myEmptyTraveltime;
 
+    /// @brief flat penalty when computing traveltime
+    double myTimePenalty;
+
     /// @brief whether this edge had a vehicle with less than max speed on it
     mutable bool myAmDelayed;
 
@@ -864,20 +903,31 @@ protected:
     /// @brief The bounding rectangle of end nodes incoming or outgoing edges for taz connectors or of my own start and end node for normal edges
     Boundary myBoundary;
 
+    /// @brief List of waiting vehicles
+    mutable std::vector<SUMOVehicle*> myWaiting;
+
+#ifdef HAVE_FOX
+    /// @brief Mutex for accessing waiting vehicles
+    mutable FXMutex myWaitingMutex;
+
+    /// @brief Mutex for accessing successor edges
+    mutable FXMutex mySuccessorMutex;
+#endif
+
 private:
 
-    /// @brief the oppositing superposble edge
+    /// @brief the oppositing superposable edge
     const MSEdge* myBidiEdge;
 
     /// @brief Invalidated copy constructor.
     MSEdge(const MSEdge&);
 
     /// @brief assignment operator.
-    MSEdge& operator=(const MSEdge&);
+    MSEdge& operator=(const MSEdge&) = delete;
 
     bool isSuperposable(const MSEdge* other);
 
-    void addToAllowed(const SVCPermissions permissions, const std::vector<MSLane*>* allowedLanes, AllowedLanesCont& laneCont) const;
+    void addToAllowed(const SVCPermissions permissions, std::shared_ptr<const std::vector<MSLane*> > allowedLanes, AllowedLanesCont& laneCont) const;
 };
 
 
