@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -15,7 +15,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 09 Apr 2001
-/// @version $Id$
 ///
 // Stores edges and lanes, performs moving of vehicle
 /****************************************************************************/
@@ -34,8 +33,16 @@
 #include <iostream>
 #include <list>
 #include <set>
+#include <queue>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/Named.h>
+#include <utils/router/SUMOAbstractRouter.h>
+#include <utils/vehicle/SUMOVehicle.h>
+
+#include <utils/foxtools/FXSynchQue.h>
+#ifdef HAVE_FOX
+#include <utils/foxtools/FXWorkerThread.h>
+#endif
 
 
 // ===========================================================================
@@ -121,9 +128,12 @@ public:
      *
      * @see MSLane::executeMovements
      * @see MSLane::integrateNewVehicle
-     * @todo When moving to parallel processing, the usage of myWithVehicles2Integrate would get insecure!!
      */
     void executeMovements(SUMOTime t);
+
+    void needsVehicleIntegration(MSLane* const l) {
+        myWithVehicles2Integrate.push_back(l);
+    }
     /// @}
 
 
@@ -135,7 +145,7 @@ public:
      *
      * @see MSEdge::changeLanes
      */
-    void changeLanes(SUMOTime t);
+    void changeLanes(const SUMOTime t);
 
 
     /** @brief Detect collisions
@@ -161,13 +171,6 @@ public:
     }
 
 
-    /** @brief Returns the list of names of all known edges
-     *
-     * @return a vector of names of all known edges
-     */
-    std::vector<std::string> getEdgeNames() const;
-
-
     /** @brief Informs the control that the given lane got active
      *
      * @param[in] l The activated lane
@@ -175,9 +178,17 @@ public:
      */
     void gotActive(MSLane* l);
 
+    /// @brief trigger collision checking for inactive lane
+    void checkCollisionForInactive(MSLane* l);
+
     /// @brief apply additional restrictions
     void setAdditionalRestrictions();
 
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool& getThreadPool() {
+        return myThreadPool;
+    }
+#endif
 
 public:
     /**
@@ -200,6 +211,34 @@ public:
         bool haveNeighbors;
     };
 
+#ifdef HAVE_FOX
+    /**
+     * @class WorkerThread
+     * @brief the thread which provides the router instance as context
+     */
+    class WorkerThread : public FXWorkerThread {
+    public:
+        WorkerThread(FXWorkerThread::Pool& pool)
+            : FXWorkerThread(pool), myRouter(nullptr) {}
+        bool setRouter(SUMOAbstractRouter<MSEdge, SUMOVehicle>* router) {
+            if (myRouter == nullptr) {
+                myRouter = router;
+                return true;
+            }
+            return false;
+        }
+        SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouter() const {
+            return *myRouter;
+        }
+        virtual ~WorkerThread() {
+            stop();
+            delete myRouter;
+        }
+    private:
+        SUMOAbstractRouter<MSEdge, SUMOVehicle>* myRouter;
+    };
+#endif
+
 private:
     /// @brief Loaded edges
     MSEdgeVector myEdges;
@@ -214,7 +253,7 @@ private:
     std::list<MSLane*> myActiveLanes;
 
     /// @brief A storage for lanes which shall be integrated because vehicles have moved onto them
-    std::vector<MSLane*> myWithVehicles2Integrate;
+    FXSynchQue<MSLane*, std::vector<MSLane*> > myWithVehicles2Integrate;
 
     /// @brief Lanes which changed the state without informing the control
     std::set<MSLane*, ComparatorNumericalIdLess> myChangedStateLanes;
@@ -222,6 +261,17 @@ private:
     /// @brief The list of active (not empty) lanes
     std::vector<SUMOTime> myLastLaneChange;
 
+    /// @brief Additional lanes for which collision checking must be performed
+    std::set<MSLane*, ComparatorNumericalIdLess> myInactiveCheckCollisions;
+
+    double myMinLengthGeometryFactor;
+
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool myThreadPool;
+#endif
+
+    std::priority_queue<std::pair<int, int> > myRNGLoad;
+
 private:
     /// @brief Copy constructor.
     MSEdgeControl(const MSEdgeControl&);
