@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -16,7 +16,6 @@
 /// @author  Mario Krumnow
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // Sets and checks options for microsim; inits global outputs and settings
 /****************************************************************************/
@@ -39,7 +38,7 @@
 #include <utils/common/StringUtils.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/iodevices/OutputDevice.h>
-#include <utils/xml/SUMOVehicleParserHelper.h>
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 #include <microsim/MSBaseVehicle.h>
 #include <microsim/MSJunction.h>
 #include <microsim/MSRoute.h>
@@ -100,7 +99,7 @@ MSFrame::fillOptions() {
     oc.addDescription("load-state", "Input", "Loads a network state from FILE");
     oc.doRegister("load-state.offset", new Option_String("0", "TIME"));//!!! check, describe
     oc.addDescription("load-state.offset", "Input", "Shifts all times loaded from a saved state by the given offset");
-    oc.doRegister("load-state.remove-vehicles", new Option_String(""));
+    oc.doRegister("load-state.remove-vehicles", new Option_StringVector(StringVector({""})));
     oc.addDescription("load-state.remove-vehicles", "Input", "Removes vehicles with the given IDs from the loaded state");
 
     //  register output options
@@ -140,6 +139,8 @@ MSFrame::fillOptions() {
     oc.addDescription("fcd-output.geo", "Output", "Save the Floating Car Data using geo-coordinates (lon/lat)");
     oc.doRegister("fcd-output.signals", new Option_Bool(false));
     oc.addDescription("fcd-output.signals", "Output", "Add the vehicle signal state to the FCD output (brake lights etc.)");
+    oc.doRegister("fcd-output.distance", new Option_Bool(false));
+    oc.addDescription("fcd-output.distance", "Output", "Add kilometrage to the FCD output (linear referencing)");
     oc.doRegister("fcd-output.filter-edges.input-file", new Option_FileName());
     oc.addDescription("fcd-output.filter-edges.input-file", "Output", "Restrict fcd output to the edge selection from the given input file");
 
@@ -202,9 +203,15 @@ MSFrame::fillOptions() {
     oc.doRegister("vehroute-output.skip-ptlines", new Option_Bool(false));
     oc.addDescription("vehroute-output.skip-ptlines", "Output", "Skip vehroute output for public transport vehicles");
 
+    oc.doRegister("vehroute-output.incomplete", new Option_Bool(false));
+    oc.addDescription("vehroute-output.incomplete", "Output", "Include invalid routes and route stubs in vehroute output");
+
     oc.doRegister("link-output", new Option_FileName());
     oc.addDescription("link-output", "Output", "Save links states into FILE");
 
+    oc.doRegister("railsignal-block-output", new Option_FileName());
+    oc.addDescription("railsignal-block-output", "Output", "Save railsignal-blocks into FILE");
+
     oc.doRegister("bt-output", new Option_FileName());
     oc.addDescription("bt-output", "Output", "Save bluetooth visibilities into FILE (in conjunction with device.btreceiver and device.btsender)");
 
@@ -223,7 +230,7 @@ MSFrame::fillOptions() {
 #ifdef _DEBUG
     oc.doRegister("movereminder-output", new Option_FileName());
     oc.addDescription("movereminder-output", "Output", "Save movereminder states of selected vehicles into FILE");
-    oc.doRegister("movereminder-output.vehicles", new Option_String());
+    oc.doRegister("movereminder-output.vehicles", new Option_StringVector());
     oc.addDescription("movereminder-output.vehicles", "Output", "List of vehicle ids which shall save their movereminder states");
 #endif
 
@@ -231,7 +238,7 @@ MSFrame::fillOptions() {
     oc.addDescription("save-state.times", "Output", "Use INT[] as times at which a network state written");
     oc.doRegister("save-state.period", new Option_String("-1", "TIME"));
     oc.addDescription("save-state.period", "Output", "save state repeatedly after TIME period");
-    oc.doRegister("save-state.prefix", new Option_FileName("state"));
+    oc.doRegister("save-state.prefix", new Option_FileName(StringVector({ "state" })));
     oc.addDescription("save-state.prefix", "Output", "Prefix for network states");
     oc.doRegister("save-state.suffix", new Option_String(".sbx"));
     oc.addDescription("save-state.suffix", "Output", "Suffix for network states (.sbx or .xml)");
@@ -251,6 +258,9 @@ MSFrame::fillOptions() {
     oc.doRegister("step-method.ballistic", new Option_Bool(false));
     oc.addDescription("step-method.ballistic", "Processing", "Whether to use ballistic method for the positional update of vehicles (default is a semi-implicit Euler method).");
 
+    oc.doRegister("threads", new Option_Integer(1));
+    oc.addDescription("threads", "Processing", "Defines the number of threads for parallel simulation");
+
     oc.doRegister("lateral-resolution", new Option_Float(-1));
     oc.addDescription("lateral-resolution", "Processing", "Defines the resolution in m when handling lateral positioning within a lane (with -1 all vehicles drive at the center of their lane");
 
@@ -321,6 +331,12 @@ MSFrame::fillOptions() {
     oc.doRegister("tls.all-off", new Option_Bool(false));
     oc.addDescription("tls.all-off", "Processing", "Switches off all traffic lights.");
 
+    oc.doRegister("tls.actuated.show-detectors", new Option_Bool(false));
+    oc.addDescription("tls.actuated.show-detectors", "Processing", "Sets default visibility for actuation detectors");
+
+    oc.doRegister("tls.delay_based.detector-range", new Option_Float(100));
+    oc.addDescription("tls.delay_based.detector-range", "Processing", "Sets default range for detecting delayed vehicles");
+
     oc.doRegister("time-to-impatience", new Option_String("300", "TIME"));
     oc.addDescription("time-to-impatience", "Processing", "Specify how long a vehicle may wait until impatience grows from 0 to 1, defaults to 300, non-positive values disable impatience growth");
 
@@ -340,6 +356,9 @@ MSFrame::fillOptions() {
     oc.doRegister("emergencydecel.warning-threshold", new Option_Float(1));
     oc.addDescription("emergencydecel.warning-threshold", "Processing", "Sets the fraction of emergency decel capability that must be used to trigger a warning.");
 
+    oc.doRegister("parking.maneuver", new Option_Bool(false));
+    oc.addDescription("parking.maneuver", "Processing", "Whether parking simulation includes manoeuvering time and associated lane blocking");
+
     // pedestrian model
     oc.doRegister("pedestrian.model", new Option_String("striping"));
     oc.addDescription("pedestrian.model", "Processing", "Select among pedestrian models ['nonInteracting', 'striping', 'remote']");
@@ -348,20 +367,27 @@ MSFrame::fillOptions() {
     oc.addDescription("pedestrian.striping.stripe-width", "Processing", "Width of parallel stripes for segmenting a sidewalk (meters) for use with model 'striping'");
 
     oc.doRegister("pedestrian.striping.dawdling", new Option_Float(0.2));
-    oc.addDescription("pedestrian.striping.dawdling", "Processing", "factor for random slow-downs [0,1] for use with model 'striping'");
+    oc.addDescription("pedestrian.striping.dawdling", "Processing", "Factor for random slow-downs [0,1] for use with model 'striping'");
 
     oc.doRegister("pedestrian.striping.jamtime", new Option_String("300", "TIME"));
     oc.addDescription("pedestrian.striping.jamtime", "Processing", "Time in seconds after which pedestrians start squeezing through a jam when using model 'striping' (non-positive values disable squeezing)");
+    oc.doRegister("pedestrian.striping.jamtime.crossing", new Option_String("10", "TIME"));
+    oc.addDescription("pedestrian.striping.jamtime.crossing", "Processing", "Time in seconds after which pedestrians start squeezing through a jam while on a pedestrian crossing when using model 'striping' (non-positive values disable squeezing)");
 
     oc.doRegister("pedestrian.remote.address", new Option_String("localhost:9000"));
     oc.addDescription("pedestrian.remote.address", "Processing", "The address (host:port) of the external simulation");
 
+    oc.doRegister("ride.stop-tolerance", new Option_Float(10.));
+    oc.addDescription("ride.stop-tolerance", "Processing", "Tolerance to apply when matching pedestrian and vehicle positions on boarding at individual stops");
+
     // generic routing options
     oc.doRegister("routing-algorithm", new Option_String("dijkstra"));
     oc.addDescription("routing-algorithm", "Routing",
                       "Select among routing algorithms ['dijkstra', 'astar', 'CH', 'CHWrapper']");
     oc.doRegister("weights.random-factor", new Option_Float(1.));
     oc.addDescription("weights.random-factor", "Routing", "Edge weights for routing are dynamically disturbed by a random factor drawn uniformly from [1,FLOAT)");
+    oc.doRegister("weights.minor-penalty", new Option_Float(1.5));
+    oc.addDescription("weights.minor-penalty", "Routing", "Apply the given time penalty when computing minimum routing costs for minor-link internal lanes");
 
     oc.doRegister("astar.all-distances", new Option_FileName());
     oc.addDescription("astar.all-distances", "Routing", "Initialize lookup table for astar from the given file (generated by marouter --all-pairs-output)");
@@ -372,12 +398,12 @@ MSFrame::fillOptions() {
     oc.doRegister("persontrip.walkfactor", new Option_Float(double(0.75)));
     oc.addDescription("persontrip.walkfactor", "Routing", "Use FLOAT as a factor on pedestrian maximum speed during intermodal routing");
 
-    oc.doRegister("persontrip.transfer.car-walk", new Option_String("parkingAreas"));
+    oc.doRegister("persontrip.transfer.car-walk", new Option_StringVector(StringVector({ "parkingAreas" })));
     oc.addDescription("persontrip.transfer.car-walk", "Routing", "Where are mode changes from car to walking allowed (possible values: 'parkingAreas', 'ptStops', 'allJunctions' and combinations)");
 
     // devices
     oc.addOptionSubTopic("Emissions");
-    oc.doRegister("phemlight-path", new Option_FileName("./PHEMlight/"));
+    oc.doRegister("phemlight-path", new Option_FileName(StringVector({ "./PHEMlight/" })));
     oc.addDescription("phemlight-path", "Emissions", "Determines where to load PHEMlight definitions from.");
 
     oc.addOptionSubTopic("Communication");
@@ -389,7 +415,7 @@ MSFrame::fillOptions() {
     oc.addSynonyme("duration-log.disable", "no-duration-log", false);
     oc.addDescription("duration-log.disable", "Report", "Disable performance reports for individual simulation steps");
 
-    oc.doRegister("duration-log.statistics", new Option_Bool(false));
+    oc.doRegister("duration-log.statistics", 't', new Option_Bool(false));
     oc.addDescription("duration-log.statistics", "Report", "Enable statistics on vehicle trips");
 
     oc.doRegister("no-step-log", new Option_Bool(false));
@@ -401,11 +427,7 @@ MSFrame::fillOptions() {
     oc.addDescription("remote-port", "TraCI Server", "Enables TraCI Server if set");
     oc.doRegister("num-clients", new Option_Integer(1));
     oc.addDescription("num-clients", "TraCI Server", "Expected number of connecting clients");
-#ifdef HAVE_PYTHON
-    oc.doRegister("python-script", new Option_String());
-    oc.addDescription("python-script", "TraCI Server", "Runs TraCI script with embedded python");
-#endif
-    //
+
     oc.addOptionSubTopic("Mesoscopic");
     oc.doRegister("mesosim", new Option_Bool(false));
     oc.addDescription("mesosim", "Mesoscopic", "Enables mesoscopic simulation");
@@ -442,6 +464,9 @@ MSFrame::fillOptions() {
 
     // add rand options
     RandHelper::insertRandOptions();
+    oc.doRegister("thread-rngs", new Option_Integer(64));
+    oc.addDescription("thread-rngs", "Random Number",
+                      "Number of pre-allocated random number generators to ensure repeatable multi-threaded simulations (should be at least the number of threads for repeatable simulations).");
 
     // add GUI options
     // the reason that we include them in vanilla sumo as well is to make reusing config files easy
@@ -455,12 +480,18 @@ MSFrame::fillOptions() {
     oc.doRegister("game", 'G', new Option_Bool(false));
     oc.addDescription("game", "GUI Only", "Start the GUI in gaming mode");
 
+    oc.doRegister("game.mode", new Option_String("tls"));
+    oc.addDescription("game.mode", "GUI Only", "Select the game type ('tls', 'drt')");
+
     oc.doRegister("start", 'S', new Option_Bool(false));
     oc.addDescription("start", "GUI Only", "Start the simulation after loading");
 
-    oc.doRegister("breakpoints", new Option_String());
+    oc.doRegister("breakpoints", 'B', new Option_StringVector());
     oc.addDescription("breakpoints", "GUI Only", "Use TIME[] as times when the simulation should halt");
 
+    oc.doRegister("edgedata-files", new Option_FileName());
+    oc.addDescription("edgedata-files", "GUI Only", "Load edge/lane weights for visualization from FILE");
+
     oc.doRegister("demo", 'D', new Option_Bool(false));
     oc.addDescription("demo", "GUI Only", "Restart the simulation after ending (demo mode)");
 
@@ -470,10 +501,10 @@ MSFrame::fillOptions() {
     oc.doRegister("registry-viewport", new Option_Bool(false));
     oc.addDescription("registry-viewport", "GUI Only", "Load current viewport from registry");
 
-    oc.doRegister("window-size", new Option_String());
+    oc.doRegister("window-size", new Option_StringVector());
     oc.addDescription("window-size", "GUI Only", "Create initial window with the given x,y size");
 
-    oc.doRegister("window-pos", new Option_String());
+    oc.doRegister("window-pos", new Option_StringVector());
     oc.addDescription("window-pos", "GUI Only", "Create initial window at the given x,y position");
 
     oc.doRegister("tracker-interval", new Option_Float(1.0));
@@ -491,6 +522,10 @@ MSFrame::fillOptions() {
     // gui testing - debug
     oc.doRegister("gui-testing-debug", new Option_Bool(false));
     oc.addDescription("gui-testing-debug", "GUI Only", "Enable output messages during GUI-Testing");
+
+    // gui testing - settings output
+    oc.doRegister("gui-testing.setting-output", new Option_FileName());
+    oc.addDescription("gui-testing.setting-output", "GUI Only", "Save gui settings in the given settingsoutput file");
 }
 
 
@@ -512,6 +547,7 @@ MSFrame::buildStreams() {
 
     //OutputDevice::createDeviceByOption("vtk-output", "vtk-export");
     OutputDevice::createDeviceByOption("link-output", "link-output");
+    OutputDevice::createDeviceByOption("railsignal-block-output", "railsignal-block-output");
     OutputDevice::createDeviceByOption("bt-output", "bt-output");
     OutputDevice::createDeviceByOption("lanechange-output", "lanechanges");
     OutputDevice::createDeviceByOption("stop-output", "stops", "stopinfo_file.xsd");
@@ -556,8 +592,10 @@ MSFrame::checkOptions() {
     if (oc.getBool("meso-junction-control.limited") && !oc.getBool("meso-junction-control")) {
         oc.set("meso-junction-control", "true");
     }
-    if (oc.getBool("mesosim") && oc.isDefault("pedestrian.model")) {
-        oc.set("pedestrian.model", "nonInteracting");
+    if (oc.getBool("mesosim")) {
+        if (oc.isDefault("pedestrian.model")) {
+            oc.set("pedestrian.model", "nonInteracting");
+        }
     }
     const SUMOTime begin = string2time(oc.getString("begin"));
     const SUMOTime end = string2time(oc.getString("end"));
@@ -565,6 +603,7 @@ MSFrame::checkOptions() {
         WRITE_ERROR("The begin time should not be negative.");
         ok = false;
     }
+    checkStepLengthMultiple(begin, " for begin");
     if (end != string2time("-1")) {
         if (end < begin) {
             WRITE_ERROR("The end time should be after the begin time.");
@@ -575,6 +614,14 @@ MSFrame::checkOptions() {
         WRITE_ERROR("the minimum step-length is 0.001");
         ok = false;
     }
+    const SUMOTime period = string2time(oc.getString("device.fcd.period"));
+    if (period > 0) {
+        checkStepLengthMultiple(period, " for device.fcd.period");
+    }
+    const SUMOTime statePeriod = string2time(oc.getString("save-state.period"));
+    if (statePeriod > 0) {
+        checkStepLengthMultiple(period, " for save-state.period");
+    }
 #ifdef _DEBUG
     if (oc.isSet("movereminder-output.vehicles") && !oc.isSet("movereminder-output")) {
         WRITE_ERROR("option movereminder-output.vehicles requires option movereminder-output to be set");
@@ -590,11 +637,6 @@ MSFrame::checkOptions() {
     if (oc.getBool("ignore-accidents")) {
         WRITE_WARNING("The option 'ignore-accidents' is deprecated. Use 'collision.action none' instead.");
     }
-#ifdef HAVE_PYTHON
-    if (oc.isSet("python-script")) {
-        WRITE_WARNING("The option 'python-script' is deprecated. Use libsumo or TraCI instead.");
-    }
-#endif
     if (oc.getBool("duration-log.statistics") && oc.isDefault("verbose")) {
         oc.set("verbose", "true");
     }
@@ -638,6 +680,27 @@ MSFrame::checkOptions() {
             ok = false;
         }
     };
+#ifndef HAVE_FOX
+    if (oc.getInt("threads") > 1) {
+        WRITE_ERROR("Parallel simulation is only possible when compiled with Fox.");
+        ok = false;
+    }
+#endif
+    if (oc.getInt("threads") > oc.getInt("thread-rngs")) {
+        WRITE_WARNING("Number of threads exceeds number of thread-rngs. Simulation runs with the same seed may produce different results");
+    }
+    if (oc.getString("game.mode") != "tls" && oc.getString("game.mode") != "drt") {
+        WRITE_ERROR("game.mode must be one of ['tls', 'drt']");
+    }
+
+    if (oc.isSet("persontrip.transfer.car-walk")) {
+        for (const std::string& opt : OptionsCont::getOptions().getStringVector("persontrip.transfer.car-walk")) {
+            if (opt != "parkingAreas" && opt != "ptStops" && opt != "allJunctions") {
+                WRITE_ERROR("Invalid transfer option '" + opt + "'. Must be one of 'parkingAreas', 'ptStops' and 'allJunctions'");
+                ok = false;
+            }
+        }
+    }
 
     ok &= MSDevice::checkOptions(oc);
     ok &= SystemFrame::checkOptions();
@@ -663,6 +726,7 @@ MSFrame::setMSGlobals(OptionsCont& oc) {
     MSGlobals::gCheckRoutes = !oc.getBool("ignore-route-errors");
     MSGlobals::gLaneChangeDuration = string2time(oc.getString("lanechange.duration"));
     MSGlobals::gLateralResolution = oc.getFloat("lateral-resolution");
+    MSGlobals::gSublane = (MSGlobals::gLaneChangeDuration > 0 || MSGlobals::gLateralResolution > 0);
     MSGlobals::gStateLoaded = oc.isSet("load-state");
     MSGlobals::gUseMesoSim = oc.getBool("mesosim");
     MSGlobals::gMesoLimitedJunctionControl = oc.getBool("meso-junction-control.limited");
@@ -699,9 +763,16 @@ MSFrame::setMSGlobals(OptionsCont& oc) {
         // value already checked in checkOptions()
         MSGlobals::gDefaultEmergencyDecel = StringUtils::toDouble(defaultEmergencyDecelOption);
     }
+    MSGlobals::gNumSimThreads = oc.getInt("threads");
+    MSGlobals::gNumThreads = MAX2(MSGlobals::gNumSimThreads, oc.getInt("device.rerouting.threads"));
 
     MSGlobals::gEmergencyDecelWarningThreshold = oc.getFloat("emergencydecel.warning-threshold");
-    
+    MSGlobals::gMinorPenalty = oc.getFloat("weights.minor-penalty");
+
+    MSGlobals::gModelParkingManoeuver = oc.getBool("parking.maneuver");
+
+    MSGlobals::gStopTolerance = oc.getFloat("ride.stop-tolerance");
+
 #ifdef _DEBUG
     if (oc.isSet("movereminder-output")) {
         MSBaseVehicle::initMoveReminderOutput(oc);
@@ -710,6 +781,4 @@ MSFrame::setMSGlobals(OptionsCont& oc) {
 }
 
 
-
 /****************************************************************************/
-
