@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2003-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2003-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    late summer 2003
-/// @version $Id$
 ///
 // Some static variables for faster access
 /****************************************************************************/
@@ -66,6 +65,18 @@ bool MSGlobals::gUnitTests(false);
 
 bool MSGlobals::gComputeLC;
 
+int MSGlobals::gNumSimThreads;
+int MSGlobals::gNumThreads;
+
 double MSGlobals::gEmergencyDecelWarningThreshold(1);
+
+double MSGlobals::gMinorPenalty(0);
+
+bool MSGlobals::gModelParkingManoeuver;
+
+bool MSGlobals::gSublane(false);
+
+double MSGlobals::gStopTolerance;
+
 /****************************************************************************/
 
