@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSLeaderInfo.h
 /// @author  Jakob Erdmann
 /// @date    Oct 2015
-/// @version $Id$
 ///
 // Information about vehicles ahead (may be multiple vehicles if
 // lateral-resolution is active)
@@ -97,6 +96,10 @@ public:
         return myHasVehicles;
     }
 
+    const std::vector<const MSVehicle*>& getVehicles() const {
+        return myVehicles;
+    }
+
     /// @brief whether a stopped vehicle is leader
     bool hasStoppedVehicle() const;
 
@@ -163,6 +166,10 @@ public:
     /// @brief print a debugging representation
     virtual std::string toString() const;
 
+    const std::vector<double>& getDistances() const {
+        return myDistances;
+    }
+
 protected:
 
     std::vector<double> myDistances;
