@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A connnection between lanes
 /****************************************************************************/
@@ -102,7 +101,8 @@ public:
                                       const SUMOTime _arrivalTimeBraking,
                                       const double _arrivalSpeedBraking,
                                       const SUMOTime _waitingTime,
-                                      const double _dist
+                                      const double _dist,
+                                      const double _speed
                                      ) :
             arrivalTime(_arrivalTime), leavingTime(_leavingTime),
             arrivalSpeed(_arrivalSpeed), leaveSpeed(_leaveSpeed),
@@ -110,7 +110,8 @@ public:
             arrivalTimeBraking(_arrivalTimeBraking),
             arrivalSpeedBraking(_arrivalSpeedBraking),
             waitingTime(_waitingTime),
-            dist(_dist) {
+            dist(_dist),
+            speed(_speed) {
         }
 
         /// @brief The time the vehicle's front arrives at the link
@@ -131,6 +132,8 @@ public:
         const SUMOTime waitingTime;
         /// @brief The distance up to the current link
         const double dist;
+        /// @brief The current speed
+        const double speed;
 
     private:
         /// invalidated assignment operator
@@ -138,6 +141,8 @@ public:
 
     };
 
+    typedef std::map<const SUMOVehicle*, const ApproachingVehicleInformation, ComparatorNumericalIdLess> ApproachInfos;
+    typedef std::vector<const SUMOVehicle*> BlockingFoes;
 
     /** @brief Constructor for simulation which uses internal lanes
      *
@@ -165,15 +170,23 @@ public:
                                const std::vector<MSLink*>& foeLinks, const std::vector<MSLane*>& foeLanes,
                                MSLane* internalLaneBefore = 0);
 
-    /// @brief add walkingarea as foe
+    /// @brief add walkingarea as foe (when entering the junction)
     void addWalkingAreaFoe(const MSLane* lane) {
         myWalkingAreaFoe = lane;
     }
 
-    /// @brief get walkingarea as foe
+    /// @brief add walkingarea as foe (when leaving the junction)
+    void addWalkingAreaFoeExit(const MSLane* lane) {
+        myWalkingAreaFoeExit = lane;
+    }
+
+    /// @brief get walkingarea as foes
     const MSLane* getWalkingAreaFoe() {
         return myWalkingAreaFoe;
     }
+    const MSLane* getWalkingAreaFoeExit() {
+        return myWalkingAreaFoeExit;
+    }
 
     /** @brief Sets the information about an approaching vehicle
      *
@@ -198,7 +211,7 @@ public:
     ApproachingVehicleInformation getApproaching(const SUMOVehicle* veh) const;
 
     /// @brief return all approaching vehicles
-    const std::map<const SUMOVehicle*, ApproachingVehicleInformation, ComparatorNumericalIdLess>& getApproaching() const {
+    const ApproachInfos& getApproaching() const {
         return myApproachingVehicles;
     }
 
@@ -212,9 +225,9 @@ public:
     bool opened(SUMOTime arrivalTime, double arrivalSpeed, double leaveSpeed, double vehicleLength,
                 double impatience, double decel, SUMOTime waitingTime,
                 double posLat = 0,
-                std::vector<const SUMOVehicle*>* collectFoes = 0,
+                BlockingFoes* collectFoes = nullptr,
                 bool ignoreRed = false,
-                const SUMOVehicle* ego = 0) const;
+                const SUMOVehicle* ego = nullptr) const;
 
     /** @brief Returns the information whether this link is blocked
      * Valid after the vehicles have set their requests
@@ -232,7 +245,7 @@ public:
      **/
     bool blockedAtTime(SUMOTime arrivalTime, SUMOTime leaveTime, double arrivalSpeed, double leaveSpeed,
                        bool sameTargetLane, double impatience, double decel, SUMOTime waitingTime,
-                       std::vector<const SUMOVehicle*>* collectFoes = 0, const SUMOVehicle* ego = 0) const;
+                       BlockingFoes* collectFoes = nullptr, const SUMOVehicle* ego = nullptr) const;
 
 
     bool isBlockingAnyone() const {
@@ -254,6 +267,9 @@ public:
      */
     bool hasApproachingFoe(SUMOTime arrivalTime, SUMOTime leaveTime, double speed, double decel) const;
 
+    /// @brief get the foe vehicle that is closest to the intersection or nullptr along with the foe link
+    std::pair<const SUMOVehicle*, const MSLink*>  getFirstApproachingFoe() const;
+
     MSJunction* getJunction() const {
         return myJunction;
     }
@@ -427,10 +443,13 @@ public:
      */
     LinkLeaders getLeaderInfo(const MSVehicle* ego, double dist, std::vector<const MSPerson*>* collectBlockers = 0, bool isShadowLink = false) const;
 
+    /// @brief check for persons on walkingarea in the path of ego vehicle
+    void checkWalkingAreaFoe(const MSVehicle* ego, const MSLane* foeLane, std::vector<const MSPerson*>* collectBlockers, LinkLeaders& result) const;
+
     /// @brief return the speed at which ego vehicle must approach the zipper link
     double getZipperSpeed(const MSVehicle* ego, const double dist, double vSafe,
                           SUMOTime arrivalTime,
-                          std::vector<const SUMOVehicle*>* collectFoes) const;
+                          BlockingFoes* collectFoes) const;
 
     /// @brief return the via lane if it exists and the lane otherwise
     MSLane* getViaLaneOrLane() const;
@@ -511,6 +530,9 @@ public:
         return myLateralShift;
     }
 
+    /// @brief get string description for this link
+    std::string  getDescription() const;
+
 private:
     /// @brief return whether the given vehicles may NOT merge safely
     static inline bool unsafeMergeSpeeds(double leaderSpeed, double followerSpeed, double leaderDecel, double followerDecel) {
@@ -518,10 +540,7 @@ private:
         return (leaderSpeed * leaderSpeed / leaderDecel) <= (followerSpeed * followerSpeed / followerDecel);
     }
 
-    /// @brief returns whether the given lane may still be occupied by a vehicle currently on it
-    static bool maybeOccupied(MSLane* lane);
-
-    /// @brief whether fllower could stay behind leader (possibly by braking)
+    /// @brief whether follower could stay behind leader (possibly by braking)
     static bool couldBrakeForLeader(double followDist, double leaderDist, const MSVehicle* follow, const MSVehicle* leader);
 
     MSLink* computeParallelLink(int direction);
@@ -544,7 +563,7 @@ private:
     /// @brief The lane approaching this link
     MSLane* myLaneBefore;
 
-    std::map<const SUMOVehicle*, ApproachingVehicleInformation, ComparatorNumericalIdLess> myApproachingVehicles;
+    ApproachInfos myApproachingVehicles;
     std::set<MSLink*> myBlockedFoeLinks;
 
     /// @brief The position within this respond
@@ -614,7 +633,11 @@ private:
     // TODO: documentation
     std::vector<MSLink*> myFoeLinks;
     std::vector<const MSLane*> myFoeLanes;
+
+    /// @brief walkingArea that must be checked when entering the intersection
     const MSLane* myWalkingAreaFoe;
+    /// @brief walkingArea that must be checked when leaving the intersection
+    const MSLane* myWalkingAreaFoeExit;
 
     /// @brief whether on of myFoeLanes is a crossing
     bool myHavePedestrianCrossingFoe;
