@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -18,7 +18,6 @@
 /// @author  Mario Krumnow
 /// @author  Christoph Sommer
 /// @date    Tue, 06 Mar 2001
-/// @version $Id$
 ///
 // The simulated network and simulation perfomer
 /****************************************************************************/
@@ -58,11 +57,11 @@
 #include <utils/options/OptionsCont.h>
 #include <utils/options/OptionsIO.h>
 #include <utils/shapes/ShapeContainer.h>
-#include <utils/vehicle/DijkstraRouter.h>
-#include <utils/vehicle/AStarRouter.h>
-#include <utils/vehicle/IntermodalRouter.h>
-#include <utils/vehicle/PedestrianRouter.h>
-#include <utils/xml/SUMORouteLoaderControl.h>
+#include <utils/router/DijkstraRouter.h>
+#include <utils/router/AStarRouter.h>
+#include <utils/router/IntermodalRouter.h>
+#include <utils/router/PedestrianRouter.h>
+#include <utils/vehicle/SUMORouteLoaderControl.h>
 #include <utils/xml/XMLSubSys.h>
 #include <traci-server/TraCIServer.h>
 #include <libsumo/Simulation.h>
@@ -88,12 +87,15 @@
 #include <microsim/pedestrians/MSPModel.h>
 #include <microsim/pedestrians/MSPerson.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSRailSignal.h>
 #include <microsim/trigger/MSChargingStation.h>
+#include <utils/router/FareModul.h>
 
 #include "MSTransportableControl.h"
 #include "MSEdgeControl.h"
 #include "MSJunctionControl.h"
 #include "MSInsertionControl.h"
+#include "MSDynamicShapeUpdater.h"
 #include "MSEventControl.h"
 #include "MSEdge.h"
 #include "MSJunction.h"
@@ -169,6 +171,20 @@ MSNet::getInstance(void) {
     throw ProcessError("A network was not yet constructed.");
 }
 
+void
+MSNet::initStatic() {
+    if (!MSGlobals::gUseMesoSim) {
+        MSVehicle::Influencer::init();
+    }
+}
+
+void
+MSNet::cleanupStatic() {
+    if (!MSGlobals::gUseMesoSim) {
+        MSVehicle::Influencer::cleanup();
+    }
+}
+
 
 MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
              MSEventControl* endOfTimestepEvents,
@@ -176,12 +192,14 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
              ShapeContainer* shapeCont):
     myAmInterrupted(false),
     myVehiclesMoved(0),
+    myPersonsMoved(0),
     myHavePermissions(false),
     myHasInternalLinks(false),
     myHasElevation(false),
-    myRouterTT(nullptr),
-    myRouterEffort(nullptr),
-    myPedestrianRouter(nullptr) {
+    myHasPedestrianNetwork(false),
+    myHasBidiEdges(false),
+    myEdgeDataEndTime(-1),
+    myDynamicShapeUpdater(nullptr) {
     if (myInstance != nullptr) {
         throw ProcessError("A network was already constructed.");
     }
@@ -212,6 +230,7 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
         MSGlobals::gMesoNet = new MELoop(string2time(oc.getString("meso-recheck")));
     }
     myInstance = this;
+    initStatic();
 }
 
 
@@ -236,9 +255,6 @@ MSNet::closeBuilding(const OptionsCont& oc, MSEdgeControl* edges, MSJunctionCont
     myStateDumpPrefix = oc.getString("save-state.prefix");
     myStateDumpSuffix = oc.getString("save-state.suffix");
 
-    // set requests/responses
-    myJunctions->postloadInitContainer();
-
     // initialise performance computation
     mySimBeginMillis = SysUtils::getCurrentMillis();
     myHasInternalLinks = hasInternalLinks;
@@ -246,12 +262,15 @@ MSNet::closeBuilding(const OptionsCont& oc, MSEdgeControl* edges, MSJunctionCont
         WRITE_WARNING("Opposite direction driving does not work together with the sublane model.");
     }
     myHasElevation = checkElevation();
+    myHasPedestrianNetwork = checkWalkingarea();
+    myHasBidiEdges = checkBidiEdges();
     myLefthand = lefthand;
     myVersion = version;
 }
 
 
 MSNet::~MSNet() {
+    cleanupStatic();
     // delete controls
     delete myJunctions;
     delete myDetectorControl;
@@ -276,9 +295,18 @@ MSNet::~MSNet() {
     myInsertionEvents = nullptr;
     delete myShapeContainer;
     delete myEdgeWeights;
-    delete myRouterTT;
-    delete myRouterEffort;
-    delete myPedestrianRouter;
+    for (auto& router : myRouterTT) {
+        delete router.second;
+    }
+    myRouterTT.clear();
+    for (auto& router : myRouterEffort) {
+        delete router.second;
+    }
+    myRouterEffort.clear();
+    for (auto& router : myPedestrianRouter) {
+        delete router.second;
+    }
+    myPedestrianRouter.clear();
     for (auto& router : myIntermodalRouter) {
         delete router.second;
     }
@@ -316,15 +344,6 @@ MSNet::simulate(SUMOTime start, SUMOTime stop) {
     SimulationState state = SIMSTATE_RUNNING;
     // state loading may have changed the start time so we need to reinit it
     myStep = start;
-#ifdef HAVE_PYTHON
-    if (OptionsCont::getOptions().isSet("python-script")) {
-        TraCIServer::runEmbedded(OptionsCont::getOptions().getString("python-script"));
-        closeSimulation(start);
-        WRITE_MESSAGE("Simulation ended at time: " + time2string(getCurrentTimeStep()));
-        WRITE_MESSAGE("Reason: Script ended");
-        return state;
-    }
-#endif
     while (state == SIMSTATE_RUNNING) {
         if (myLogStepNumber) {
             preSimStepOutput();
@@ -372,13 +391,16 @@ MSNet::generateStatistics(SUMOTime start) {
     msg << "Performance: " << "\n" << " Duration: " << duration << "ms" << "\n";
     if (duration != 0) {
         msg << " Real time factor: " << (STEPS2TIME(myStep - start) * 1000. / (double)duration) << "\n";
-        msg.setf(std::ios::fixed , std::ios::floatfield);    // use decimal format
+        msg.setf(std::ios::fixed, std::ios::floatfield);     // use decimal format
         msg.setf(std::ios::showpoint);    // print decimal point
         msg << " UPS: " << ((double)myVehiclesMoved / ((double)duration / 1000)) << "\n";
+        if (myPersonsMoved > 0) {
+            msg << " UPS-Persons: " << ((double)myPersonsMoved / ((double)duration / 1000)) << "\n";
+        }
     }
     // print vehicle statistics
     const std::string discardNotice = ((myVehicleControl->getLoadedVehicleNo() != myVehicleControl->getDepartedVehicleNo()) ?
-                                        " (Loaded: " + toString(myVehicleControl->getLoadedVehicleNo()) + ")" : "");
+                                       " (Loaded: " + toString(myVehicleControl->getLoadedVehicleNo()) + ")" : "");
     msg << "Vehicles: " << "\n"
         << " Inserted: " << myVehicleControl->getDepartedVehicleNo() << discardNotice << "\n"
         << " Running: " << myVehicleControl->getRunningVehicleNo() << "\n"
@@ -431,6 +453,9 @@ MSNet::closeSimulation(SUMOTime start) {
     if (OptionsCont::getOptions().isSet("chargingstations-output")) {
         writeChargingStationOutput();
     }
+    if (OptionsCont::getOptions().isSet("railsignal-block-output")) {
+        writeRailSignalBlocks();
+    }
     if (myLogExecutionTime) {
         WRITE_MESSAGE(generateStatistics(start));
     }
@@ -444,20 +469,22 @@ MSNet::simulationStep() {
               << ", myStep = " << myStep
               << std::endl;
 #endif
-    if (myLogExecutionTime) {
-        myTraCIStepDuration = SysUtils::getCurrentMillis();
-    }
     TraCIServer* t = TraCIServer::getInstance();
-    if (t != nullptr && !t->isEmbedded()) {
+    if (t != nullptr) {
+        if (myLogExecutionTime) {
+            myTraCIStepDuration = SysUtils::getCurrentMillis();
+        }
         t->processCommandsUntilSimStep(myStep);
 #ifdef DEBUG_SIMSTEP
         bool loadRequested = !TraCI::getLoadArgs().empty();
-        bool closed = TraCIServer::wasClosed();
-        assert(t->getTargetTime() >= myStep || loadRequested || closed);
+        assert(t->getTargetTime() >= myStep || loadRequested || TraCIServer::wasClosed());
 #endif
-    }
-    if (myLogExecutionTime) {
-        myTraCIStepDuration = SysUtils::getCurrentMillis() - myTraCIStepDuration;
+        if (myLogExecutionTime) {
+            myTraCIStepDuration = SysUtils::getCurrentMillis() - myTraCIStepDuration;
+        }
+        if (TraCIServer::wasClosed()) {
+            return;
+        }
     }
 #ifdef DEBUG_SIMSTEP
     std::cout << SIMTIME << ": TraCI target time: " << t->getTargetTime() << std::endl;
@@ -467,13 +494,15 @@ MSNet::simulationStep() {
         mySimStepDuration = SysUtils::getCurrentMillis();
     }
     // simulation state output
-    std::vector<SUMOTime>::iterator timeIt = find(myStateDumpTimes.begin(), myStateDumpTimes.end(), myStep);
+    std::vector<SUMOTime>::iterator timeIt = std::find(myStateDumpTimes.begin(), myStateDumpTimes.end(), myStep);
     if (timeIt != myStateDumpTimes.end()) {
         const int dist = (int)distance(myStateDumpTimes.begin(), timeIt);
         MSStateHandler::saveState(myStateDumpFiles[dist], myStep);
     }
     if (myStateDumpPeriod > 0 && myStep % myStateDumpPeriod == 0) {
-        MSStateHandler::saveState(myStateDumpPrefix + "_" + time2string(myStep) + myStateDumpSuffix, myStep);
+        std::string timeStamp = time2string(myStep);
+        std::replace(timeStamp.begin(), timeStamp.end(), ':', '-');
+        MSStateHandler::saveState(myStateDumpPrefix + "_" + timeStamp + myStateDumpSuffix, myStep);
     }
     myBeginOfTimestepEvents->execute(myStep);
 #ifdef HAVE_FOX
@@ -487,6 +516,7 @@ MSNet::simulationStep() {
 
     if (MSGlobals::gUseMesoSim) {
         MSGlobals::gMesoNet->simulate(myStep);
+        myVehicleControl->removePending();
     } else {
         // assure all lanes with vehicles are 'active'
         myEdges->patchActiveLanes();
@@ -550,6 +580,9 @@ MSNet::simulationStep() {
     if (myLogExecutionTime) {
         mySimStepDuration = SysUtils::getCurrentMillis() - mySimStepDuration;
         myVehiclesMoved += myVehicleControl->getRunningVehicleNo();
+        if (myPersonControl != nullptr) {
+            myPersonsMoved += myPersonControl->getRunningNumber();
+        }
     }
     myStep += DELTA_T;
 }
@@ -563,16 +596,16 @@ MSNet::simulationState(SUMOTime stopTime) const {
     if (TraCIServer::getInstance() != nullptr && !TraCIServer::getInstance()->getLoadArgs().empty()) {
         return SIMSTATE_LOADING;
     }
-    if ((stopTime < 0 || myStep > stopTime) && TraCIServer::getInstance() == nullptr) {
+    if ((stopTime < 0 || myStep > stopTime) && TraCIServer::getInstance() == nullptr && (stopTime > 0 || myStep > myEdgeDataEndTime)) {
         if ((myVehicleControl->getActiveVehicleCount() == 0)
                 && (myInserter->getPendingFlowCount() == 0)
                 && (myPersonControl == nullptr || !myPersonControl->hasNonWaiting())
                 && (myContainerControl == nullptr || !myContainerControl->hasNonWaiting())) {
             if (myPersonControl) {
-                myPersonControl->abortWaitingForVehicle();
+                myPersonControl->abortAnyWaitingForVehicle();
             }
             if (myContainerControl) {
-                myContainerControl->abortWaitingForVehicle();
+                myContainerControl->abortAnyWaitingForVehicle();
             }
             myVehicleControl->abortWaiting();
             return SIMSTATE_NO_FURTHER_VEHICLES;
@@ -717,9 +750,12 @@ MSNet::writeOutput() {
         od.writeAttr("running", myVehicleControl->getRunningVehicleNo());
         od.writeAttr("waiting", myInserter->getWaitingVehicleNo());
         od.writeAttr("ended", myVehicleControl->getEndedVehicleNo());
+        od.writeAttr("arrived", myVehicleControl->getArrivedVehicleNo());
+        od.writeAttr("collisions", myVehicleControl->getCollisionCount());
+        od.writeAttr("teleports", myVehicleControl->getTeleportCount());
+        od.writeAttr("halting", myVehicleControl->getHaltingVehicleNo());
         od.writeAttr("meanWaitingTime", meanWaitingTime);
         od.writeAttr("meanTravelTime", meanTravelTime);
-        od.writeAttr("halting", myVehicleControl->getHaltingVehicleNo());
         std::pair<double, double> meanSpeed = myVehicleControl->getVehicleMeanSpeeds();
         od.writeAttr("meanSpeed", meanSpeed.first);
         od.writeAttr("meanSpeedRelative", meanSpeed.second);
@@ -751,14 +787,12 @@ MSNet::writeOutput() {
     }
 
     // write SSM output
-    for (std::set<MSDevice_SSM*>::iterator di = MSDevice_SSM::getInstances().begin(); di != MSDevice_SSM::getInstances().end(); ++di) {
-        MSDevice_SSM* dev = (*di);
+    for (MSDevice_SSM* dev : MSDevice_SSM::getInstances()) {
         dev->updateAndWriteOutput();
     }
 
     // write ToC output
-    for (std::set<MSDevice_ToC*>::iterator di = MSDevice_ToC::getInstances().begin(); di != MSDevice_ToC::getInstances().end(); ++di) {
-        MSDevice_ToC* dev = (*di);
+    for (MSDevice_ToC* dev : MSDevice_ToC::getInstances()) {
         if (dev->generatesOutput()) {
             dev->writeOutput();
         }
@@ -788,6 +822,11 @@ MSNet::getContainerControl() {
     return *myContainerControl;
 }
 
+MSDynamicShapeUpdater*
+MSNet::makeDynamicShapeUpdater() {
+    myDynamicShapeUpdater = std::unique_ptr<MSDynamicShapeUpdater> (new MSDynamicShapeUpdater(MSNet::getInstance()->getShapeContainer()));
+    return myDynamicShapeUpdater.get();
+}
 
 MSEdgeWeightsStorage&
 MSNet::getWeightsStorage() {
@@ -808,7 +847,7 @@ void
 MSNet::postSimStepOutput() const {
     if (myLogExecutionTime) {
         std::ostringstream oss;
-        oss.setf(std::ios::fixed , std::ios::floatfield);    // use decimal format
+        oss.setf(std::ios::fixed, std::ios::floatfield);     // use decimal format
         oss.setf(std::ios::showpoint);    // print decimal point
         oss << std::setprecision(gPrecision);
         if (mySimStepDuration != 0) {
@@ -828,7 +867,7 @@ MSNet::postSimStepOutput() const {
             << " BUF " << myInserter->getWaitingVehicleNo()
             << ")                                              ";
         std::string prev = "Step #" + time2string(myStep - DELTA_T);
-        std::cout << oss.str().substr(0, 78 - prev.length());
+        std::cout << oss.str().substr(0, 90 - prev.length());
     }
     std::cout << '\r';
 }
@@ -844,7 +883,7 @@ MSNet::addVehicleStateListener(VehicleStateListener* listener) {
 
 void
 MSNet::removeVehicleStateListener(VehicleStateListener* listener) {
-    std::vector<VehicleStateListener*>::iterator i = find(myVehicleStateListeners.begin(), myVehicleStateListeners.end(), listener);
+    std::vector<VehicleStateListener*>::iterator i = std::find(myVehicleStateListeners.begin(), myVehicleStateListeners.end(), listener);
     if (i != myVehicleStateListeners.end()) {
         myVehicleStateListeners.erase(i);
     }
@@ -853,8 +892,11 @@ MSNet::removeVehicleStateListener(VehicleStateListener* listener) {
 
 void
 MSNet::informVehicleStateListener(const SUMOVehicle* const vehicle, VehicleState to, const std::string& info) {
-    for (std::vector<VehicleStateListener*>::iterator i = myVehicleStateListeners.begin(); i != myVehicleStateListeners.end(); ++i) {
-        (*i)->vehicleStateChanged(vehicle, to, info);
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myStateListenerMutex, MSGlobals::gNumThreads > 1);
+#endif
+    for (VehicleStateListener* const listener : myVehicleStateListeners) {
+        listener->vehicleStateChanged(vehicle, to, info);
     }
 }
 
@@ -888,6 +930,17 @@ MSNet::getStoppingPlaceID(const MSLane* lane, const double pos, const SumoXMLTag
 }
 
 
+const NamedObjectCont<MSStoppingPlace*>&
+MSNet::getStoppingPlaces(SumoXMLTag category) const {
+    auto it = myStoppingPlaces.find(category);
+    if (it != myStoppingPlaces.end()) {
+        return it->second;
+    } else {
+        throw ProcessError("No stoppingPlace of type '" + toString(category) + "' found");
+    }
+}
+
+
 void
 MSNet::writeChargingStationOutput() const {
     if (myStoppingPlaces.count(SUMO_TAG_CHARGING_STATION) > 0) {
@@ -899,52 +952,63 @@ MSNet::writeChargingStationOutput() const {
 }
 
 
+void
+MSNet::writeRailSignalBlocks() const {
+    OutputDevice& output = OutputDevice::getDeviceByOption("railsignal-block-output");
+    for (auto tls : myLogics->getAllLogics()) {
+        MSRailSignal* rs = dynamic_cast<MSRailSignal*>(tls);
+        if (rs != nullptr) {
+            rs->writeBlocks(output);
+        }
+    }
+}
+
+
 SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSNet::getRouterTT(const MSEdgeVector& prohibited) const {
-    if (myRouterTT == nullptr) {
+MSNet::getRouterTT(const int rngIndex, const MSEdgeVector& prohibited) const {
+    if (myRouterTT.count(rngIndex) == 0) {
         const std::string routingAlgorithm = OptionsCont::getOptions().getString("routing-algorithm");
         if (routingAlgorithm == "dijkstra") {
-            myRouterTT = new DijkstraRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-                MSEdge::getAllEdges(), true, &MSNet::getTravelTime);
+            myRouterTT[rngIndex] = new DijkstraRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, &MSNet::getTravelTime, nullptr, false, nullptr, true);
         } else {
             if (routingAlgorithm != "astar") {
                 WRITE_WARNING("TraCI and Triggers cannot use routing algorithm '" + routingAlgorithm + "'. using 'astar' instead.");
             }
-            myRouterTT = new AStarRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-                MSEdge::getAllEdges(), true, &MSNet::getTravelTime);
+            myRouterTT[rngIndex] = new AStarRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, &MSNet::getTravelTime, nullptr, true);
         }
     }
-    dynamic_cast<SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle>*>(myRouterTT)->prohibit(prohibited);
-    return *myRouterTT;
+    myRouterTT[rngIndex]->prohibit(prohibited);
+    return *myRouterTT[rngIndex];
 }
 
 
 SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSNet::getRouterEffort(const MSEdgeVector& prohibited) const {
-    if (myRouterEffort == nullptr) {
-        myRouterEffort = new DijkstraRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-            MSEdge::getAllEdges(), true, &MSNet::getEffort, &MSNet::getTravelTime);
+MSNet::getRouterEffort(const int rngIndex, const MSEdgeVector& prohibited) const {
+    if (myRouterEffort.count(rngIndex) == 0) {
+        myRouterEffort[rngIndex] = new DijkstraRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, &MSNet::getEffort, &MSNet::getTravelTime, false, nullptr, true);
     }
-    dynamic_cast<SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle>*>(myRouterEffort)->prohibit(prohibited);
-    return *myRouterEffort;
+    myRouterEffort[rngIndex]->prohibit(prohibited);
+    return *myRouterEffort[rngIndex];
 }
 
 
 MSNet::MSPedestrianRouter&
-MSNet::getPedestrianRouter(const MSEdgeVector& prohibited) const {
-    if (myPedestrianRouter == nullptr) {
-        myPedestrianRouter = new MSPedestrianRouter();
+MSNet::getPedestrianRouter(const int rngIndex, const MSEdgeVector& prohibited) const {
+    if (myPedestrianRouter.count(rngIndex) == 0) {
+        myPedestrianRouter[rngIndex] = new MSPedestrianRouter();
     }
-    myPedestrianRouter->prohibit(prohibited);
-    return *myPedestrianRouter;
+    myPedestrianRouter[rngIndex]->prohibit(prohibited);
+    return *myPedestrianRouter[rngIndex];
 }
 
 
 MSNet::MSIntermodalRouter&
-MSNet::getIntermodalRouter(const int routingMode, const MSEdgeVector& prohibited) const {
-    if (myIntermodalRouter.count(routingMode) == 0) {
+MSNet::getIntermodalRouter(const int rngIndex, const int routingMode, const MSEdgeVector& prohibited) const {
+    const OptionsCont& oc = OptionsCont::getOptions();
+    const int key = rngIndex * oc.getInt("thread-rngs") + routingMode;
+    if (myIntermodalRouter.count(key) == 0) {
         int carWalk = 0;
-        for (const std::string& opt : OptionsCont::getOptions().getStringVector("persontrip.transfer.car-walk")) {
+        for (const std::string& opt : oc.getStringVector("persontrip.transfer.car-walk")) {
             if (opt == "parkingAreas") {
                 carWalk |= MSIntermodalRouter::Network::PARKING_AREAS;
             } else if (opt == "ptStops") {
@@ -954,15 +1018,14 @@ MSNet::getIntermodalRouter(const int routingMode, const MSEdgeVector& prohibited
             }
         }
         const std::string routingAlgorithm = OptionsCont::getOptions().getString("routing-algorithm");
-        if (routingMode == ROUTING_MODE_COMBINED) {
-            // replace nullptr here by your EffortCalculator
-            myIntermodalRouter[routingMode] = new MSIntermodalRouter(MSNet::adaptIntermodalRouter, carWalk, routingAlgorithm, routingMode, nullptr);
+        if (routingMode == libsumo::ROUTING_MODE_COMBINED) {
+            myIntermodalRouter[key] = new MSIntermodalRouter(MSNet::adaptIntermodalRouter, carWalk, routingAlgorithm, routingMode, new FareModul());
         } else {
-            myIntermodalRouter[routingMode] = new MSIntermodalRouter(MSNet::adaptIntermodalRouter, carWalk, routingAlgorithm, routingMode);
+            myIntermodalRouter[key] = new MSIntermodalRouter(MSNet::adaptIntermodalRouter, carWalk, routingAlgorithm, routingMode);
         }
     }
-    myIntermodalRouter[routingMode]->prohibit(prohibited);
-    return *myIntermodalRouter[routingMode];
+    myIntermodalRouter[key]->prohibit(prohibited);
+    return *myIntermodalRouter[key];
 }
 
 
@@ -973,29 +1036,24 @@ MSNet::adaptIntermodalRouter(MSIntermodalRouter& router) {
         const MSEdge* const edge = &i.second->getLane().getEdge();
         router.getNetwork()->addAccess(i.first, edge, i.second->getAccessPos(edge), i.second->getAccessDistance(edge), SUMO_TAG_PARKING_AREA);
     }
+    EffortCalculator* const external = router.getExternalEffort();
     // add access to all public transport stops
     for (const auto& i : myInstance->myStoppingPlaces[SUMO_TAG_BUS_STOP]) {
         const MSEdge* const edge = &i.second->getLane().getEdge();
-        router.getNetwork()->addAccess(i.first, edge, i.second->getAccessPos(edge), i.second->getAccessDistance(edge), SUMO_TAG_BUS_STOP);
+        router.getNetwork()->addAccess(i.first, edge, i.second->getAccessPos(edge),
+                                       i.second->getAccessDistance(edge), SUMO_TAG_BUS_STOP);
         for (const auto& a : i.second->getAllAccessPos()) {
             router.getNetwork()->addAccess(i.first, &std::get<0>(a)->getEdge(), std::get<1>(a), std::get<2>(a), SUMO_TAG_BUS_STOP);
         }
+        if (external != nullptr) {
+            external->addStop(router.getNetwork()->getStopEdge(i.first)->getNumericalID(), *i.second);
+        }
     }
     myInstance->getInsertionControl().adaptIntermodalRouter(router);
     myInstance->getVehicleControl().adaptIntermodalRouter(router);
 }
 
 
-const NamedRTree&
-MSNet::getLanesRTree() const {
-    if (!myLanesRTree.first) {
-        MSLane::fill(myLanesRTree.second);
-        myLanesRTree.first = true;
-    }
-    return myLanesRTree.second;
-}
-
-
 bool
 MSNet::checkElevation() {
     const MSEdgeVector& edges = myEdges->getEdges();
@@ -1010,6 +1068,27 @@ MSNet::checkElevation() {
 }
 
 
+bool
+MSNet::checkWalkingarea() {
+    for (const MSEdge* e : myEdges->getEdges()) {
+        if (e->getFunction() == EDGEFUNC_WALKINGAREA) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+bool
+MSNet::checkBidiEdges() {
+    for (const MSEdge* e : myEdges->getEdges()) {
+        if (e->getBidiEdge() != nullptr) {
+            return true;
+        }
+    }
+    return false;
+}
+
 bool
 MSNet::warnOnce(const std::string& typeAndID) {
     if (myWarnedOnce.find(typeAndID) == myWarnedOnce.end()) {
@@ -1019,4 +1098,5 @@ MSNet::warnOnce(const std::string& typeAndID) {
     return false;
 }
 
+
 /****************************************************************************/
