@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -15,8 +15,8 @@
 /// @author  Eric Nicolay
 /// @author  Clemens Honomichl
 /// @author  Michael Behrisch
+/// @author  Leonhard Luecken
 /// @date    Mon, 12 Mar 2001
-/// @version $Id$
 ///
 // The simulated network and simulation performer
 /****************************************************************************/
@@ -37,14 +37,19 @@
 #include <iostream>
 #include <cmath>
 #include <iomanip>
+#include <memory>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/NamedObjectCont.h>
 #include <utils/common/NamedRTree.h>
-#include <utils/vehicle/SUMOAbstractRouter.h>
+#include <utils/router/SUMOAbstractRouter.h>
 #include <microsim/trigger/MSChargingStation.h>
 #include "MSJunction.h"
 
+#ifdef HAVE_FOX
+#include <utils/foxtools/FXConditionalLock.h>
+#endif
+
 
 // ===========================================================================
 // class declarations
@@ -64,6 +69,8 @@ class MSTLLogicControl;
 class MSTrafficLightLogic;
 class MSDetectorControl;
 class ShapeContainer;
+class MSDynamicShapeUpdater;
+class PolygonDynamics;
 class BinaryInputDevice;
 class MSEdgeWeightsStorage;
 class SUMOVehicle;
@@ -116,6 +123,19 @@ public:
      */
     static MSNet* getInstance();
 
+    /**
+     * @brief Returns whether this is a GUI Net
+     */
+    virtual bool isGUINet() const {
+        return false;
+    }
+
+    /// @brief Place for static initializations of simulation components (called after successful net build)
+    static void initStatic();
+
+    /// @brief Place for static initializations of simulation components (called after successful net build)
+    static void cleanupStatic();
+
 
     /** @brief Returns whether the network was already constructed
     * @return whether the network was already constructed
@@ -440,6 +460,18 @@ public:
         return *myShapeContainer;
     }
 
+    /** @brief Returns the dynamic shapes updater
+     * @see PolygonDynamics
+     */
+    MSDynamicShapeUpdater* getDynamicShapeUpdater() {
+        return myDynamicShapeUpdater.get();
+    }
+
+    /** @brief Creates and returns a dynamic shapes updater
+     * @see PolygonDynamics
+     */
+    MSDynamicShapeUpdater* makeDynamicShapeUpdater();
+
     /** @brief Returns the net's internal edge travel times/efforts container
      *
      * If the net does not have such a container, it is built.
@@ -480,13 +512,22 @@ public:
     std::string getStoppingPlaceID(const MSLane* lane, const double pos, const SumoXMLTag category) const;
     /// @}
 
+    const NamedObjectCont<MSStoppingPlace*>& getStoppingPlaces(SumoXMLTag category) const;
 
     /// @brief write charging station output
     void writeChargingStationOutput() const;
 
+    /// @brief write rail signal block output
+    void writeRailSignalBlocks() const;
+
     /// @brief creates a wrapper for the given logic (see GUINet)
     virtual void createTLWrapper(MSTrafficLightLogic*) {};
 
+    /// @brief return wheter the given logic (or rather it's wrapper) is selected in the GUI
+    virtual bool isSelected(const MSTrafficLightLogic*) const {
+        return false;
+    }
+
     /// @name Notification about vehicle state changes
     /// @{
 
@@ -515,7 +556,9 @@ public:
         /// @brief The vehicle is involved in a collision
         VEHICLE_STATE_COLLISION,
         /// @brief The vehicle had to brake harder than permitted
-        VEHICLE_STATE_EMERGENCYSTOP
+        VEHICLE_STATE_EMERGENCYSTOP,
+        /// @brief Vehicle maneuvering either entering or exiting a parking space
+        VEHICLE_STATE_MANEUVERING
     };
 
 
@@ -586,21 +629,16 @@ public:
     /* @brief get the router, initialize on first use
      * @param[in] prohibited The vector of forbidden edges (optional)
      */
-    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT(
-        const MSEdgeVector& prohibited = MSEdgeVector()) const;
-    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterEffort(
-        const MSEdgeVector& prohibited = MSEdgeVector()) const;
-    MSPedestrianRouter& getPedestrianRouter(const MSEdgeVector& prohibited = MSEdgeVector()) const;
-    MSIntermodalRouter& getIntermodalRouter(const int routingMode = 0, const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT(const int rngIndex,
+            const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterEffort(const int rngIndex,
+            const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    MSPedestrianRouter& getPedestrianRouter(const int rngIndex, const MSEdgeVector& prohibited = MSEdgeVector()) const;
+    MSIntermodalRouter& getIntermodalRouter(const int rngIndex, const int routingMode = 0, const MSEdgeVector& prohibited = MSEdgeVector()) const;
 
     static void adaptIntermodalRouter(MSIntermodalRouter& router);
 
 
-    /** @brief Returns an RTree that contains lane IDs
-     * @return An Rtree containing lane IDs
-     */
-    const NamedRTree& getLanesRTree() const;
-
     /// @brief return whether the network contains internal links
     bool hasInternalLinks() const {
         return myHasInternalLinks;
@@ -611,6 +649,16 @@ public:
         return myHasElevation;
     }
 
+    /// @brief return whether the network contains walkingareas and crossings
+    bool hasPedestrianNetwork() const {
+        return myHasPedestrianNetwork;
+
+    }
+    /// @brief return whether the network contains bidirectional rail edges
+    bool hasBidiEdges() const {
+        return myHasBidiEdges;
+    }
+
     /// @brief return whether the network was built for lefthand traffic
     bool lefthand() const {
         return myLefthand;
@@ -636,6 +684,11 @@ protected:
     /// @brief check all lanes for elevation data
     bool checkElevation();
 
+    /// @brief check all lanes for type walkingArea
+    bool checkWalkingarea();
+
+    /// @brief check wether bidirectional edges occur in the network
+    bool checkBidiEdges();
 
 protected:
     /// @brief Unique instance of MSNet
@@ -698,13 +751,14 @@ protected:
     bool myLogStepNumber;
 
     /// @brief The last simulation step duration
-    long myTraCIStepDuration, mySimStepDuration;
+    long myTraCIStepDuration = 0, mySimStepDuration = 0;
 
     /// @brief The overall simulation duration
     long mySimBeginMillis;
 
     /// @brief The overall number of vehicle movements
     long long int myVehiclesMoved;
+    long long int myPersonsMoved;
     //}
 
 
@@ -737,34 +791,53 @@ protected:
     /// @brief Whether the network contains elevation data
     bool myHasElevation;
 
+    /// @brief Whether the network contains pedestrian network elements
+    bool myHasPedestrianNetwork;
+
+    /// @brief Whether the network contains bidirectional rail edges
+    bool myHasBidiEdges;
+
     /// @brief Whether the network was built for left-hand traffic
     bool myLefthand;
 
     /// @brief the network version
     double myVersion;
 
+    /// @brief end of loaded edgeData
+    SUMOTime myEdgeDataEndTime;
+
     /// @brief Dictionary of bus / container stops
     std::map<SumoXMLTag, NamedObjectCont<MSStoppingPlace*> > myStoppingPlaces;
 
     /// @brief Container for vehicle state listener
     std::vector<VehicleStateListener*> myVehicleStateListeners;
 
+#ifdef HAVE_FOX
+    /// @brief to avoid concurrent access to the state update function
+    FXMutex myStateListenerMutex;
+#endif
+
     /// @brief container to record warnings that shall only be issued once
     std::map<std::string, bool> myWarnedOnce;
 
     /* @brief The router instance for routing by trigger and by traci
      * @note MSDevice_Routing has its own instance since it uses a different weight function
      * @note we provide one member for every switchable router type
-     * because the class structure makes it inconvenient to use a superclass*/
-    mutable SUMOAbstractRouter<MSEdge, SUMOVehicle>* myRouterTT;
-    mutable SUMOAbstractRouter<MSEdge, SUMOVehicle>* myRouterEffort;
-    mutable MSPedestrianRouter* myPedestrianRouter;
+     * because the class structure makes it inconvenient to use a superclass
+     */
+    mutable std::map<int, SUMOAbstractRouter<MSEdge, SUMOVehicle>*> myRouterTT;
+    mutable std::map<int, SUMOAbstractRouter<MSEdge, SUMOVehicle>*> myRouterEffort;
+    mutable std::map<int, MSPedestrianRouter*> myPedestrianRouter;
     mutable std::map<int, MSIntermodalRouter*> myIntermodalRouter;
 
-
     /// @brief An RTree structure holding lane IDs
     mutable std::pair<bool, NamedRTree> myLanesRTree;
 
+    /// @brief Updater for dynamic shapes that are tracking traffic objects
+    ///        (ensures removal of shape dynamics when the objects are removed)
+    /// @see utils/shapes/PolygonDynamics
+    std::unique_ptr<MSDynamicShapeUpdater> myDynamicShapeUpdater;
+
 
     /// @brief string constants for simstep stages
     static const std::string STAGE_EVENTS;
