@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2015-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2015-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Mirco Sturari
 /// @author  Jakob Erdmann
 /// @date    Tue, 19.01.2016
-/// @version $Id$
 ///
 // A area where vehicles can park next to the road
 /****************************************************************************/
@@ -23,14 +22,17 @@
 #include <config.h>
 
 #include <cassert>
+#include <utils/common/WrappingCommand.h>
 #include <utils/vehicle/SUMOVehicle.h>
 #include <utils/geom/Position.h>
 #include <utils/geom/GeomHelper.h>
+#include <microsim/MSEventControl.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicleType.h>
 #include "MSLane.h"
 #include "MSTransportable.h"
 #include "MSParkingArea.h"
+#include "MSGlobals.h"
 
 //#define DEBUG_RESERVATIONS
 //#define DEBUG_COND2(obj) (obj.getID() == "v.3")
@@ -53,10 +55,13 @@ MSParkingArea::MSParkingArea(const std::string& id,
     myWidth(width),
     myLength(length),
     myAngle(angle),
+    myEgressBlocked(false),
     myReservationTime(-1),
     myReservations(0),
-    myReservationMaxLength(0)
-{
+    myReservationMaxLength(0),
+    myNumAlternatives(0),
+    myLastStepOccupancy(0),
+    myUpdateEvent(nullptr) {
     // initialize unspecified defaults
     if (myWidth == 0) {
         myWidth = SUMO_const_laneWidth;
@@ -80,9 +85,9 @@ MSParkingArea::MSParkingArea(const std::string& id,
         const Position s = myShape.positionAtOffset(spaceDim * (i + 1));
         Position pos = myAngle == 0 ? s : (f + s) * 0.5;
         addLotEntry(pos.x(), pos.y(), pos.z(),
-            myWidth, myLength,
-            ((double) atan2((s.x() - f.x()), (f.y() - s.y())) * (double) 180.0 / (double) M_PI) + myAngle);
-        mySpaceOccupancies.back().myEndPos = myBegPos + spaceDim * (i + 1);
+                    myWidth, myLength,
+                    ((double) atan2((s.x() - f.x()), (f.y() - s.y())) * (double) 180.0 / (double) M_PI) + myAngle);
+        mySpaceOccupancies.back().myEndPos = myBegPos + MAX2(POSITION_EPS, spaceDim * (i + 1));
     }
     computeLastFreePos();
 }
@@ -99,18 +104,58 @@ MSParkingArea::addLotEntry(double x, double y, double z,
     lsd.myWidth = width;
     lsd.myLength = length;
     lsd.myRotation = angle;
-    lsd.myEndPos = myEndPos;
+    // If we are modelling parking set the end position to the lot position relative to the lane
+    //   rather than the end of the parking area - this results in vehicles stopping nearer the space
+    //   and re-entering the lane nearer the space. (If we are not modelling parking the vehicle will usually
+    //    enter the space and re-enter at the end of the parking area.)
+    if (MSGlobals::gModelParkingManoeuver) {
+        const double offset = this->getLane().getShape().nearest_offset_to_point2D(lsd.myPosition);
+        if (offset <  getBeginLanePosition()) {
+            lsd.myEndPos =  getBeginLanePosition() + POSITION_EPS;
+        } else {
+            if (this->getLane().getLength() > offset) {
+                lsd.myEndPos = offset;
+            } else {
+                lsd.myEndPos = this->getLane().getLength() - POSITION_EPS;
+            }
+        }
+        // Work out the angle of the lot relative to the lane  (+90 parallels the way the bay is drawn )
+        int relativeAngle = static_cast<int>(lsd.myRotation + 90. - RAD2DEG(this->getLane().getShape().rotationAtOffset(lsd.myEndPos)));
+        if (relativeAngle < 0) {
+            relativeAngle += 360;
+        }
+
+        // use this to set the manoeuver angle - real life manoeuver will always be < 180 degrees - hence the modulus
+        //   if p2.y is -ve the lot is on LHS of lane relative to lane direction
+        Position p2 = this->getLane().getShape().transformToVectorCoordinates(lsd.myPosition);
+        if (p2.y() < (0. + POSITION_EPS)) {
+            lsd.myManoeuverAngle = abs(relativeAngle) % 180;
+        } else { // lot is on RHS of lane
+            lsd.myManoeuverAngle = abs(abs(relativeAngle) % 180 - 180) % 180;
+        }
+    } else {
+        lsd.myEndPos = myEndPos;
+        lsd.myManoeuverAngle = int(angle); // unused unless gModelParkingManoeuver is true
+    }
+
+
     mySpaceOccupancies.push_back(lsd);
     myCapacity++;
     computeLastFreePos();
 }
 
+int
+MSParkingArea::getLastFreeLotAngle() const {
+    assert(myLastFreeLot >= 0);
+    assert(myLastFreeLot < (int)mySpaceOccupancies.size());
+    return (mySpaceOccupancies[myLastFreeLot].myManoeuverAngle);
+}
 
 
 
 double
 MSParkingArea::getLastFreePos(const SUMOVehicle& forVehicle) const {
-    if (myCapacity == getOccupancy()) {
+    if (myCapacity == (int)myEndPositions.size()) {
         // keep enough space so that  parking vehicles can leave
         return myLastFreePos - forVehicle.getVehicleType().getMinGap() - POSITION_EPS;
     } else {
@@ -130,6 +175,18 @@ MSParkingArea::getVehiclePosition(const SUMOVehicle& forVehicle) const {
     return Position::INVALID;
 }
 
+
+double
+MSParkingArea::getInsertionPosition(const SUMOVehicle& forVehicle) const {
+    for (const auto& lsd : mySpaceOccupancies) {
+        if (lsd.vehicle == &forVehicle) {
+            return lsd.myEndPos;
+        }
+    }
+    return -1;
+}
+
+
 double
 MSParkingArea::getVehicleAngle(const SUMOVehicle& forVehicle) const {
     for (const auto& lsd : mySpaceOccupancies) {
@@ -143,18 +200,25 @@ MSParkingArea::getVehicleAngle(const SUMOVehicle& forVehicle) const {
 
 void
 MSParkingArea::enter(SUMOVehicle* what, double beg, double end) {
-    if (myLastFreeLot >= 0) {
-        assert(myLastFreeLot < (int)mySpaceOccupancies.size());
-        mySpaceOccupancies[myLastFreeLot].vehicle = what;
-        myEndPositions[what] = std::pair<double, double>(beg, end);
-        computeLastFreePos();
+    assert(myLastFreePos >= 0);
+    assert(myLastFreeLot < (int)mySpaceOccupancies.size());
+    if (myUpdateEvent == nullptr) {
+        myUpdateEvent = new WrappingCommand<MSParkingArea>(this, &MSParkingArea::updateOccupancy);
+        MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(myUpdateEvent);
     }
+    mySpaceOccupancies[myLastFreeLot].vehicle = what;
+    myEndPositions[what] = std::pair<double, double>(beg, end);
+    computeLastFreePos();
 }
 
 
 void
 MSParkingArea::leaveFrom(SUMOVehicle* what) {
     assert(myEndPositions.find(what) != myEndPositions.end());
+    if (myUpdateEvent == nullptr) {
+        myUpdateEvent = new WrappingCommand<MSParkingArea>(this, &MSParkingArea::updateOccupancy);
+        MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(myUpdateEvent);
+    }
     for (auto& lsd : mySpaceOccupancies) {
         if (lsd.vehicle == what) {
             lsd.vehicle = nullptr;
@@ -166,36 +230,64 @@ MSParkingArea::leaveFrom(SUMOVehicle* what) {
 }
 
 
+SUMOTime
+MSParkingArea::updateOccupancy(SUMOTime /* currentTime */) {
+    myLastStepOccupancy = getOccupancy();
+    myUpdateEvent = nullptr;
+    return 0;
+}
+
+
 void
 MSParkingArea::computeLastFreePos() {
     myLastFreeLot = -1;
     myLastFreePos = myBegPos;
+    myEgressBlocked = false;
     for (auto& lsd : mySpaceOccupancies) {
-        if (lsd.vehicle == nullptr) {
-            myLastFreeLot = lsd.index;
-            myLastFreePos = lsd.myEndPos;
+        if (lsd.vehicle == nullptr
+                || (getOccupancy() == getCapacity()
+                    && lsd.vehicle->remainingStopDuration() <= 0
+                    && !lsd.vehicle->isStoppedTriggered())) {
+            if (lsd.vehicle == nullptr) {
+                myLastFreeLot = lsd.index;
+                myLastFreePos = lsd.myEndPos;
+            } else {
+                // vehicle wants to exit the parking area
+                myLastFreeLot = lsd.index;
+                myLastFreePos = lsd.myEndPos - lsd.vehicle->getVehicleType().getLength() - POSITION_EPS;
+                myEgressBlocked = true;
+            }
             break;
         } else {
-            myLastFreePos = MIN2(myLastFreePos, 
-                    lsd.myEndPos - lsd.vehicle->getVehicleType().getLength() - NUMERICAL_EPS);
+            myLastFreePos = MIN2(myLastFreePos,
+                                 lsd.myEndPos - lsd.vehicle->getVehicleType().getLength() - NUMERICAL_EPS);
         }
     }
 }
 
 
-double 
+double
 MSParkingArea::getLastFreePosWithReservation(SUMOTime t, const SUMOVehicle& forVehicle) {
     if (forVehicle.getLane() != &myLane) {
         // for different lanes, do not consider reservations to avoid lane-order
         // dependency in parallel simulation
 #ifdef DEBUG_RESERVATIONS
-        if (DEBUG_COND2(forVehicle)) std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " other lane\n";
+        if (DEBUG_COND2(forVehicle)) {
+            std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " other lane\n";
+        }
 #endif
-        return getLastFreePos(forVehicle);
+        if (myNumAlternatives > 0 && getOccupancy() == getCapacity()) {
+            // ensure that the vehicle reaches the rerouter lane
+            return MAX2(myBegPos, MIN2(POSITION_EPS, myEndPos));
+        } else {
+            return getLastFreePos(forVehicle);
+        }
     }
     if (t > myReservationTime) {
 #ifdef DEBUG_RESERVATIONS
-        if (DEBUG_COND2(forVehicle)) std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " first reservation\n";
+        if (DEBUG_COND2(forVehicle)) {
+            std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " first reservation\n";
+        }
 #endif
         myReservationTime = t;
         myReservations = 1;
@@ -209,22 +301,24 @@ MSParkingArea::getLastFreePosWithReservation(SUMOTime t, const SUMOVehicle& forV
     } else {
         if (myCapacity > getOccupancy() + myReservations) {
 #ifdef DEBUG_RESERVATIONS
-        if (DEBUG_COND2(forVehicle)) std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " res=" << myReservations << " enough space\n";
+            if (DEBUG_COND2(forVehicle)) {
+                std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() << " res=" << myReservations << " enough space\n";
+            }
 #endif
             myReservations++;
             myReservationMaxLength = MAX2(myReservationMaxLength, forVehicle.getVehicleType().getLength());
             return getLastFreePos(forVehicle);
-        } else{
+        } else {
             if (myCapacity == 0) {
                 return getLastFreePos(forVehicle);
             } else {
 #ifdef DEBUG_RESERVATIONS
-        if (DEBUG_COND2(forVehicle)) std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID() 
-            << " res=" << myReservations << " resTime=" << myReservationTime << " reserved full, maxLen=" << myReservationMaxLength << " endPos=" << mySpaceOccupancies[0].myEndPos << "\n";
+                if (DEBUG_COND2(forVehicle)) std::cout << SIMTIME << " pa=" << getID() << " freePosRes veh=" << forVehicle.getID()
+                                                           << " res=" << myReservations << " resTime=" << myReservationTime << " reserved full, maxLen=" << myReservationMaxLength << " endPos=" << mySpaceOccupancies[0].myEndPos << "\n";
 #endif
-                return (mySpaceOccupancies[0].myEndPos 
-                        - myReservationMaxLength 
-                        - forVehicle.getVehicleType().getMinGap() 
+                return (mySpaceOccupancies[0].myEndPos
+                        - myReservationMaxLength
+                        - forVehicle.getVehicleType().getMinGap()
                         - NUMERICAL_EPS);
             }
         }
@@ -258,8 +352,18 @@ MSParkingArea::getCapacity() const {
 
 int
 MSParkingArea::getOccupancy() const {
-    return (int)myEndPositions.size();
+    return (int)myEndPositions.size() - (myEgressBlocked ? 1 : 0);
 }
 
 
+int
+MSParkingArea::getOccupancyIncludingBlocked() const {
+    return (int)myEndPositions.size();
+}
+
+void
+MSParkingArea::notifyEgressBlocked() {
+    computeLastFreePos();
+}
+
 /****************************************************************************/
