@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2015-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2015-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSParkingArea.h
 /// @author  Mirco Sturari
 /// @date    Tue, 19.01.2016
-/// @version $Id$
 ///
 // A area where vehicles can park next to the road
 /****************************************************************************/
@@ -39,6 +38,7 @@ class MSLane;
 class SUMOVehicle;
 class MSTransportable;
 class Position;
+class Command;
 
 
 // ===========================================================================
@@ -97,9 +97,25 @@ public:
     /** @brief Returns the area occupancy
      *
      * @return The occupancy computed as number of vehicles in myEndPositions
+     * (reduced by 1 if at least one vehicle has finished parking but is blocked
+     * from entering the road)
      */
     int getOccupancy() const;
 
+    /** @brief Returns the area occupancy
+     *
+     * @return The occupancy computed as number of vehicles in myEndPositions
+     */
+    int getOccupancyIncludingBlocked() const;
+
+    /** @brief Returns the area occupancy at the end of the last simulation step
+     *
+     * @return The occupancy computed as number of vehicles in myEndPositions
+     */
+    int getLastStepOccupancy() const {
+        return myLastStepOccupancy;
+    }
+
 
     /** @brief Called if a vehicle enters this stop
      *
@@ -107,9 +123,9 @@ public:
      *
      * Recomputes the free space using "computeLastFreePos" then.
      *
-     * @param[in] what The vehicle that enters the bus stop
+     * @param[in] what The vehicle that enters the parking area
      * @param[in] beg The begin halting position of the vehicle
-     * @param[in] what The end halting position of the vehicle
+     * @param[in] end The end halting position of the vehicle
      * @see computeLastFreePos
      */
     void enter(SUMOVehicle* what, double beg, double end);
@@ -121,12 +137,22 @@ public:
      *
      * Recomputes the free space using "computeLastFreePos" then.
      *
-     * @param[in] what The vehicle that leaves the bus stop
+     * @param[in] what The vehicle that leaves the parking area
      * @see computeLastFreePos
      */
     void leaveFrom(SUMOVehicle* what);
 
 
+    /** @brief Called at the end of the time step
+     *
+     * Stores the current occupancy.
+     *
+     * @param[in] currentTime The current simulation time (unused)
+     * @return Always 0 (the event is not rescheduled)
+     */
+    SUMOTime updateOccupancy(SUMOTime currentTime);
+
+
     /** @brief Returns the last free position on this stop
      *
      * @return The last free position of this bus stop
@@ -148,6 +174,12 @@ public:
      */
     Position getVehiclePosition(const SUMOVehicle& forVehicle) const;
 
+    /** @brief Returns the insertion position of a parked vehicle
+     *
+     * @return The nsertion position of a parked vehicle along the lane
+     */
+    double getInsertionPosition(const SUMOVehicle& forVehicle) const;
+
 
     /** @brief Returns the angle of parked vehicle
      *
@@ -155,6 +187,12 @@ public:
      */
     double getVehicleAngle(const SUMOVehicle& forVehicle) const;
 
+    /** @brief Return the angle of myLastFreeLot - the next parking lot
+     *         only expected to be called after we have established there is space in the parking area
+     *
+     * @return The angle of the lot in degrees
+     */
+    int getLastFreeLotAngle() const;
 
     /** @brief Add a lot entry to parking area
      *
@@ -167,7 +205,7 @@ public:
      * @return Whether the lot entry could be added
      */
     virtual void addLotEntry(double x, double y, double z,
-            double width, double length, double angle);
+                             double width, double length, double angle);
 
 
     /** @brief Returns the lot rectangle width
@@ -190,6 +228,18 @@ public:
      */
     double getAngle() const;
 
+
+    /// @brief update state so that vehicles wishing to enter cooperate with exiting vehicles
+    void notifyEgressBlocked();
+
+    void setNumAlternatives(int alternatives) {
+        myNumAlternatives = MAX2(myNumAlternatives, alternatives);
+    }
+
+    int getNumAlternatives() const {
+        return myNumAlternatives;
+    }
+
 protected:
 
     /** @struct LotSpaceDefinition
@@ -210,6 +260,8 @@ protected:
         double myLength;
         /// @brief The position along the lane that the vehicle needs to reach for entering this lot
         double myEndPos;
+        ///@brief The angle between lane and lot through which a vehicle must manoeuver to enter the lot
+        int myManoeuverAngle;
     };
 
 
@@ -246,11 +298,23 @@ protected:
     /// @brief The roadside shape of this parkingArea
     PositionVector myShape;
 
+    /// @brief whether a vehicle wants to exit but is blocked
+    bool myEgressBlocked;
+
     /// @brief track parking reservations from the lane for the current time step
     SUMOTime myReservationTime;
     int myReservations;
     double myReservationMaxLength;
 
+    /// @brief the number of alternative parkingAreas that are assigned to parkingAreaRerouter
+    int myNumAlternatives;
+
+    /// @brief Changes to the occupancy in the current time step
+    int myLastStepOccupancy;
+
+    /// @brief Event for updating the occupancy
+    Command* myUpdateEvent;
+
 private:
 
     /// @brief Invalidated copy constructor.
