@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A vehicle route
 /****************************************************************************/
@@ -52,7 +51,7 @@ FXMutex MSRoute::myDictMutex(true);
 MSRoute::MSRoute(const std::string& id,
                  const ConstMSEdgeVector& edges,
                  const bool isPermanent, const RGBColor* const c,
-                 const std::vector<SUMOVehicleParameter::Stop>& stops) : 
+                 const std::vector<SUMOVehicleParameter::Stop>& stops) :
     Named(id), myEdges(edges), myAmPermanent(isPermanent),
     myReferenceCounter(isPermanent ? 1 : 0),
     myColor(c),
@@ -263,12 +262,14 @@ MSRoute::dict_saveState(OutputDevice& out) {
         out.writeAttr(SUMO_ATTR_STATE, (*it).second->myAmPermanent);
         out.writeAttr(SUMO_ATTR_EDGES, (*it).second->myEdges).closeTag();
     }
-    for (RouteDistDict::iterator it = myDistDict.begin(); it != myDistDict.end(); ++it) {
-        out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, (*it).first);
-        out.writeAttr(SUMO_ATTR_STATE, (*it).second.second);
-        out.writeAttr(SUMO_ATTR_ROUTES, (*it).second.first->getVals());
-        out.writeAttr(SUMO_ATTR_PROBS, (*it).second.first->getProbs());
-        out.closeTag();
+    for (const auto& item : myDistDict) {
+        if (item.second.first->getVals().size() > 0) {
+            out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, item.first);
+            out.writeAttr(SUMO_ATTR_STATE, item.second.second);
+            out.writeAttr(SUMO_ATTR_ROUTES, item.second.first->getVals());
+            out.writeAttr(SUMO_ATTR_PROBS, item.second.first->getProbs());
+            out.closeTag();
+        }
     }
 }
 
@@ -276,11 +277,20 @@ MSRoute::dict_saveState(OutputDevice& out) {
 double
 MSRoute::getDistanceBetween(double fromPos, double toPos,
                             const MSEdge* fromEdge, const MSEdge* toEdge, bool includeInternal, int routePosition) const {
+    //std::cout << SIMTIME << " getDistanceBetween from=" << fromEdge->getID() << " to=" << toEdge->getID() << " fromPos=" << fromPos << " toPos=" << toPos << " includeInternal=" << includeInternal << "\n";
     if (routePosition < 0 || routePosition >= (int)myEdges.size()) {
         throw ProcessError("Invalid routePosition " + toString(routePosition) + " for route with " + toString(myEdges.size()) + " edges");
     }
-    /// XXX routes that start and end within the same intersection are not supported
-    //std::cout << SIMTIME << " getDistanceBetween from=" << fromEdge->getID() << " to=" << toEdge->getID() << " fromPos=" << fromPos << " toPos=" << toPos << " includeInternal=" << includeInternal << "\n";
+    if (fromEdge->isInternal() && toEdge->isInternal() && fromEdge->getToJunction() == toEdge->getToJunction()) {
+        // internal edges within the same junction
+        if (fromEdge == toEdge) {
+            if (fromPos <= toPos) {
+                return toPos - fromPos;
+            }
+        } else if (fromEdge->getSuccessors().front() == toEdge) {
+            return fromEdge->getLength() - fromPos + toPos;
+        }
+    }
     if (fromEdge->isInternal()) {
         if (fromEdge == myEdges.front()) {
             const MSEdge* succ = fromEdge->getSuccessors().front();
