@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id$
 ///
 // Parser and container for routes during their loading
 /****************************************************************************/
@@ -24,35 +23,16 @@
 // ===========================================================================
 #include <config.h>
 
-#include <string>
-#include <map>
-#include <vector>
-#include <microsim/MSRoute.h>
-#include <microsim/MSEdge.h>
-#include <microsim/MSJunction.h>
-#include <microsim/MSVehicleType.h>
-#include <microsim/MSVehicle.h>
-#include <microsim/MSInsertionControl.h>
-#include <microsim/MSVehicleControl.h>
-#include <microsim/MSLane.h>
 #include "MSRouteHandler.h"
 #include "MSTransportableControl.h"
-#include <utils/xml/SUMOSAXHandler.h>
-#include <utils/xml/SUMOXMLDefinitions.h>
-#include <utils/common/MsgHandler.h>
-#include <utils/common/StringUtils.h>
+#include <microsim/MSEdge.h>
+#include <microsim/MSInsertionControl.h>
+#include <microsim/MSVehicleControl.h>
 #include <utils/common/StringTokenizer.h>
-#include <utils/common/UtilExceptions.h>
+#include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
-#include <utils/vehicle/IntermodalRouter.h>
-#include <utils/vehicle/PedestrianRouter.h>
-#include "MSNet.h"
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 
-#include "MSParkingArea.h"
-#include "MSStoppingPlace.h"
-#include <microsim/MSGlobals.h>
-#include <microsim/trigger/MSChargingStation.h>
-#include <utils/xml/SUMOVehicleParserHelper.h>
 
 
 // ===========================================================================
@@ -64,9 +44,8 @@ std::mt19937 MSRouteHandler::myParsingRNG;
 // ===========================================================================
 // method definitions
 // ===========================================================================
-MSRouteHandler::MSRouteHandler(const std::string& file,
-                               bool addVehiclesDirectly) :
-    SUMORouteHandler(file, addVehiclesDirectly ? "" : "routes"),
+MSRouteHandler::MSRouteHandler(const std::string& file, bool addVehiclesDirectly) :
+    SUMORouteHandler(file, addVehiclesDirectly ? "" : "routes", true),
     myActivePlan(nullptr),
     myActiveContainerPlan(nullptr),
     myAddVehiclesDirectly(addVehiclesDirectly),
@@ -77,8 +56,8 @@ MSRouteHandler::MSRouteHandler(const std::string& file,
 }
 
 
-MSRouteHandler::~MSRouteHandler() {
-}
+MSRouteHandler::~MSRouteHandler() {}
+
 
 void
 MSRouteHandler::deleteActivePlans() {
@@ -152,10 +131,15 @@ MSRouteHandler::parseFromViaTo(std::string element,
 void
 MSRouteHandler::myStartElement(int element,
                                const SUMOSAXAttributes& attrs) {
+    if (myActivePlan != nullptr && myActivePlan->empty() && myVehicleParameter->departProcedure == DEPART_TRIGGERED
+            && (element == SUMO_TAG_WALK || element == SUMO_TAG_PERSONTRIP || element == SUMO_TAG_STOP)) {
+        throw ProcessError("Triggered departure for person '" + myVehicleParameter->id + "' requires starting with a ride.");
+    }
     SUMORouteHandler::myStartElement(element, attrs);
     try {
         switch (element) {
             case SUMO_TAG_PERSON:
+            case SUMO_TAG_PERSONFLOW:
                 if (!MSNet::getInstance()->getVehicleControl().hasVType(myVehicleParameter->vtypeid)) {
                     const std::string error = "The type '" + myVehicleParameter->vtypeid + "' for person '" + myVehicleParameter->id + "' is not known.";
                     delete myVehicleParameter;
@@ -185,6 +169,25 @@ MSRouteHandler::myStartElement(int element,
                 }
                 double arrivalPos = attrs.getOpt<double>(SUMO_ATTR_ARRIVALPOS, myVehicleParameter->id.c_str(), ok,
                                     bs == nullptr ? -NUMERICAL_EPS : bs->getEndLanePosition());
+
+                SUMOVehicle* startVeh = nullptr;
+                if (myActivePlan->empty() && myVehicleParameter->departProcedure == DEPART_TRIGGERED) {
+                    if (st.size() != 1) {
+                        throw ProcessError("Triggered departure for person '" + pid + "' requires a unique lines value.");
+                    }
+                    // person starts
+                    MSVehicleControl& vehControl = MSNet::getInstance()->getVehicleControl();
+                    const std::string vehID = st.front();
+                    startVeh = vehControl.getVehicle(vehID);
+                    if (startVeh == nullptr) {
+                        throw ProcessError("Unknown vehicle '" + vehID + "' in triggered departure for person '" + pid + "'.");
+                    }
+                    if (startVeh->getParameter().departProcedure == DEPART_TRIGGERED) {
+                        throw ProcessError("Cannot use triggered vehicle '" + vehID + "' in triggered departure for person '" + pid + "'.");
+                    }
+                    myVehicleParameter->depart = startVeh->getParameter().depart;
+                }
+
                 if (attrs.hasAttribute(SUMO_ATTR_FROM)) {
                     const std::string fromID = attrs.get<std::string>(SUMO_ATTR_FROM, pid.c_str(), ok);
                     from = MSEdge::dictionary(fromID);
@@ -194,9 +197,12 @@ MSRouteHandler::myStartElement(int element,
                     if (!myActivePlan->empty() && myActivePlan->back()->getDestination() != from) {
                         throw ProcessError("Disconnected plan for person '" + myVehicleParameter->id + "' (" + fromID + "!=" + myActivePlan->back()->getDestination()->getID() + ").");
                     }
+                    if (startVeh != nullptr && startVeh->getRoute().getEdges().front() != from) {
+                        throw ProcessError("Disconnected plan for triggered person '" + pid + "' (" + fromID + "!=" + startVeh->getRoute().getEdges().front()->getID() + ").");
+                    }
                     if (myActivePlan->empty()) {
                         myActivePlan->push_back(new MSTransportable::Stage_Waiting(
-                                                    from, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
+                                                    from, nullptr, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
                     }
                 } else if (myActivePlan->empty()) {
                     throw ProcessError("The start edge for person '" + pid + "' is not known.");
@@ -241,7 +247,7 @@ MSRouteHandler::myStartElement(int element,
                         }
                         if (myActiveContainerPlan->empty()) {
                             myActiveContainerPlan->push_back(new MSTransportable::Stage_Waiting(
-                                                                 from, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
+                                                                 from, nullptr, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
                         }
                     } else if (myActiveContainerPlan->empty()) {
                         throw ProcessError("The start edge within a transport of container '" + containerId + "' is not known.");
@@ -317,7 +323,7 @@ MSRouteHandler::myStartElement(int element,
                 }
                 if (myActiveContainerPlan->empty()) {
                     myActiveContainerPlan->push_back(new MSTransportable::Stage_Waiting(
-                                                         myActiveRoute.front(), -1, myVehicleParameter->depart, departPos, "start", true));
+                                                         myActiveRoute.front(), nullptr, -1, myVehicleParameter->depart, departPos, "start", true));
                 }
                 myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Tranship(myActiveRoute, cs, speed, departPos, arrivalPos));
                 myActiveRoute.clear();
@@ -427,37 +433,14 @@ MSRouteHandler::openRoute(const SUMOSAXAttributes& attrs) {
 
 
 void
-MSRouteHandler::myEndElement(int element) {
-    SUMORouteHandler::myEndElement(element);
-    switch (element) {
-        case SUMO_TAG_VTYPE: {
-            MSVehicleType* vehType = MSVehicleType::build(*myCurrentVType);
-            delete myCurrentVType;
-            myCurrentVType = nullptr;
-            if (!MSNet::getInstance()->getVehicleControl().addVType(vehType)) {
-                const std::string id = vehType->getID();
-                delete vehType;
-                if (!MSGlobals::gStateLoaded) {
-                    throw ProcessError("Another vehicle type (or distribution) with the id '" + id + "' exists.");
-                }
-            } else {
-                if (myCurrentVTypeDistribution != nullptr) {
-                    myCurrentVTypeDistribution->add(vehType, vehType->getDefaultProbability());
-                }
-            }
-        }
-        break;
-        case SUMO_TAG_TRIP:
-            myVehicleParameter->parametersSet |= VEHPARS_FORCE_REROUTE;
-            closeRoute(true);
-            closeVehicle();
-            delete myVehicleParameter;
-            myVehicleParameter = nullptr;
-            myInsertStopEdgesAt = -1;
-            break;
-        default:
-            break;
-    }
+MSRouteHandler::openFlow(const SUMOSAXAttributes& /*attrs*/) {
+    // Currently unused
+}
+
+
+void
+MSRouteHandler::openTrip(const SUMOSAXAttributes& /*attrs*/) {
+    // Currently unsued
 }
 
 
@@ -747,6 +730,92 @@ MSRouteHandler::closePerson() {
     myActivePlan = nullptr;
 }
 
+
+void
+MSRouteHandler::closePersonFlow() {
+    if (myActivePlan->size() == 0) {
+        const std::string error = "personFlow '" + myVehicleParameter->id + "' has no plan.";
+        delete myVehicleParameter;
+        myVehicleParameter = nullptr;
+        deleteActivePlans();
+        throw ProcessError(error);
+    }
+    // let's check whether this person had to depart before the simulation starts
+    if (!(myAddVehiclesDirectly || checkLastDepart())
+            || (myVehicleParameter->depart < string2time(OptionsCont::getOptions().getString("begin")) && !myAmLoadingState)) {
+        delete myVehicleParameter;
+        myVehicleParameter = nullptr;
+        deleteActivePlans();
+        return;
+    }
+    // type existence has been checked on opening
+    MSVehicleType* type = MSNet::getInstance()->getVehicleControl().getVType(myVehicleParameter->vtypeid, &myParsingRNG);
+    // instantiate all persons of this flow
+    int i = 0;
+    registerLastDepart();
+    std::string baseID = myVehicleParameter->id;
+    if (myVehicleParameter->repetitionProbability > 0) {
+        if (myVehicleParameter->repetitionEnd == SUMOTime_MAX) {
+            throw ProcessError("probabilistic personFlow '" + myVehicleParameter->id + "' must specify end time");
+        } else {
+            for (SUMOTime t = myVehicleParameter->depart; t < myVehicleParameter->repetitionEnd; t += TIME2STEPS(1)) {
+                if (RandHelper::rand(&myParsingRNG) < myVehicleParameter->repetitionProbability) {
+                    addFlowPerson(t, type, baseID, i++);
+                }
+            }
+        }
+    } else {
+        SUMOTime depart = myVehicleParameter->depart;
+        for (; i < myVehicleParameter->repetitionNumber; i++) {
+            addFlowPerson(depart, type, baseID, i);
+            depart += myVehicleParameter->repetitionOffset;
+        }
+    }
+
+    myVehicleParameter = nullptr;
+    myActivePlan = nullptr;
+}
+
+void
+MSRouteHandler::addFlowPerson(SUMOTime depart, MSVehicleType* type, const std::string& baseID, int i) {
+    if (i > 0) {
+        // copy parameter and plan because the person takes over responsibility
+        SUMOVehicleParameter* copyParam = new SUMOVehicleParameter();
+        *copyParam = *myVehicleParameter;
+        myVehicleParameter = copyParam;
+        MSTransportable::MSTransportablePlan* copyPlan = new MSTransportable::MSTransportablePlan();
+        for (MSTransportable::Stage* s : *myActivePlan) {
+            copyPlan->push_back(s->clone());
+        }
+        myActivePlan = copyPlan;
+    }
+    myVehicleParameter->id = baseID + "." + toString(i);
+    myVehicleParameter->depart = depart;
+    MSTransportable* person = MSNet::getInstance()->getPersonControl().buildPerson(myVehicleParameter, type, myActivePlan, &myParsingRNG);
+    if (!MSNet::getInstance()->getPersonControl().add(person)) {
+        ProcessError error("Another person with the id '" + myVehicleParameter->id + "' exists.");
+        delete person;
+        throw error;
+    }
+}
+
+void
+MSRouteHandler::closeVType() {
+    MSVehicleType* vehType = MSVehicleType::build(*myCurrentVType);
+    if (!MSNet::getInstance()->getVehicleControl().addVType(vehType)) {
+        const std::string id = vehType->getID();
+        delete vehType;
+        if (!MSGlobals::gStateLoaded) {
+            throw ProcessError("Another vehicle type (or distribution) with the id '" + id + "' exists.");
+        }
+    } else {
+        if (myCurrentVTypeDistribution != nullptr) {
+            myCurrentVTypeDistribution->add(vehType, vehType->getDefaultProbability());
+        }
+    }
+}
+
+
 void
 MSRouteHandler::closeContainer() {
     if (myActiveContainerPlan->size() == 0) {
@@ -821,6 +890,14 @@ MSRouteHandler::closeFlow() {
 }
 
 
+void
+MSRouteHandler::closeTrip() {
+    myVehicleParameter->parametersSet |= VEHPARS_FORCE_REROUTE;
+    closeRoute(true);
+    closeVehicle();
+}
+
+
 void
 MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
     std::string errorSuffix;
@@ -839,6 +916,7 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
         return;
     }
     const MSEdge* edge = nullptr;
+    MSStoppingPlace* toStop = nullptr;
     // try to parse the assigned bus stop
     if (stop.busstop != "") {
         // ok, we have a bus stop
@@ -847,6 +925,7 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
             WRITE_ERROR("The busStop '" + stop.busstop + "' is not known" + errorSuffix);
             return;
         }
+        toStop = bs;
         const MSLane& l = bs->getLane();
         stop.lane = l.getID();
         stop.endPos = bs->getEndLanePosition();
@@ -860,6 +939,7 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
             WRITE_ERROR("The containerStop '" + stop.containerstop + "' is not known" + errorSuffix);
             return;
         }
+        toStop = cs;
         const MSLane& l = cs->getLane();
         stop.lane = l.getID();
         stop.endPos = cs->getEndLanePosition();
@@ -873,6 +953,7 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
             WRITE_ERROR("The parkingArea '" + stop.parkingarea + "' is not known" + errorSuffix);
             return;
         }
+        toStop = pa;
         const MSLane& l = pa->getLane();
         stop.lane = l.getID();
         stop.endPos = pa->getEndLanePosition();
@@ -881,15 +962,16 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
     } else if (stop.chargingStation != "") {
         // ok, we have a charging station
         MSStoppingPlace* cs = MSNet::getInstance()->getStoppingPlace(stop.chargingStation, SUMO_TAG_CHARGING_STATION);
-        if (cs != nullptr) {
-            const MSLane& l = cs->getLane();
-            stop.lane = l.getID();
-            stop.endPos = cs->getEndLanePosition();
-            stop.startPos = cs->getBeginLanePosition();
-        } else {
+        if (cs == nullptr) {
             WRITE_ERROR("The chargingStation '" + stop.chargingStation + "' is not known" + errorSuffix);
             return;
         }
+        toStop = cs;
+        const MSLane& l = cs->getLane();
+        stop.lane = l.getID();
+        stop.endPos = cs->getEndLanePosition();
+        stop.startPos = cs->getBeginLanePosition();
+        edge = &l.getEdge();
     } else {
         // no, the lane and the position should be given
         // get the lane
@@ -911,7 +993,7 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
                     edge = myActivePlan->back()->getDestination();
                     stop.lane = edge->getLanes()[0]->getID();
                     stop.endPos = myActivePlan->back()->getArrivalPos();
-                    stop.startPos = stop.endPos - POSITION_EPS;
+                    stop.startPos = MAX2(0., stop.endPos - MIN_STOP_LENGTH);
                 }
             } else {
                 WRITE_ERROR("A stop must be placed on a busStop, a chargingStation, a containerStop a parkingArea or a lane" + errorSuffix);
@@ -919,54 +1001,65 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
             }
         }
         edge = &MSLane::dictionary(stop.lane)->getEdge();
-        if (myActivePlan &&
-                !myActivePlan->empty() &&
-                myActivePlan->back()->getDestination() != edge) {
-            throw ProcessError("Disconnected plan for person '" + myVehicleParameter->id + "' (" + edge->getID() + "!=" + myActivePlan->back()->getDestination()->getID() + ").");
-        }
-        if (myActivePlan && myActivePlan->empty()) {
-            myActivePlan->push_back(new MSTransportable::Stage_Waiting(
-                                        edge, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
-        }
-        if (myActiveContainerPlan &&
-                !myActiveContainerPlan->empty() &&
-                myActiveContainerPlan->back()->getDestination() != &MSLane::dictionary(stop.lane)->getEdge()) {
-            throw ProcessError("Disconnected plan for container '" + myVehicleParameter->id + "' (" + MSLane::dictionary(stop.lane)->getEdge().getID() + "!=" + myActiveContainerPlan->back()->getDestination()->getID() + ").");
-        }
-        if (myActiveContainerPlan && myActiveContainerPlan->empty()) {
-            myActiveContainerPlan->push_back(new MSTransportable::Stage_Waiting(
-                                                 &MSLane::dictionary(stop.lane)->getEdge(), -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start", true));
-        }
         stop.endPos = attrs.getOpt<double>(SUMO_ATTR_ENDPOS, nullptr, ok, MSLane::dictionary(stop.lane)->getLength());
         if (attrs.hasAttribute(SUMO_ATTR_POSITION)) {
             WRITE_WARNING("Deprecated attribute 'pos' in description of stop" + errorSuffix);
             stop.endPos = attrs.getOpt<double>(SUMO_ATTR_POSITION, nullptr, ok, stop.endPos);
         }
-        stop.startPos = attrs.getOpt<double>(SUMO_ATTR_STARTPOS, nullptr, ok, MAX2(0., stop.endPos - 2 * POSITION_EPS));
+        stop.startPos = attrs.getOpt<double>(SUMO_ATTR_STARTPOS, nullptr, ok, MAX2(0., stop.endPos - MIN_STOP_LENGTH));
         const bool friendlyPos = attrs.getOpt<bool>(SUMO_ATTR_FRIENDLY_POS, nullptr, ok, false);
-        if (!ok || !checkStopPos(stop.startPos, stop.endPos, MSLane::dictionary(stop.lane)->getLength(), POSITION_EPS, friendlyPos)) {
+        if (!ok || (checkStopPos(stop.startPos, stop.endPos, MSLane::dictionary(stop.lane)->getLength(), POSITION_EPS, friendlyPos) != StopPos::STOPPOS_VALID)) {
             WRITE_ERROR("Invalid start or end position for stop on lane '" + stop.lane + "'" + errorSuffix);
             return;
         }
     }
-    if (myActivePlan != nullptr) {
+    if (myActivePlan) {
+        if (myActivePlan->empty()) {
+            double departPos = toStop == nullptr || myVehicleParameter->wasSet(VEHPARS_DEPARTPOS_SET)
+                               ? myVehicleParameter->departPos
+                               : (toStop->getBeginLanePosition() + toStop->getEndLanePosition()) / 2;
+            myActivePlan->push_back(new MSTransportable::Stage_Waiting(
+                                        edge, toStop, -1, myVehicleParameter->depart, departPos, "start", true));
+        } else if (myActivePlan->back()->getDestination() != edge) {
+            throw ProcessError("Disconnected plan for person '" + myVehicleParameter->id + "' (" + edge->getID() + "!=" + myActivePlan->back()->getDestination()->getID() + ").");
+        }  else if (myActivePlan->back()->getStageType() == MSTransportable::WAITING
+                    && (attrs.hasAttribute(SUMO_ATTR_ENDPOS) || attrs.hasAttribute(SUMO_ATTR_STARTPOS))) {
+            const double start = SUMOVehicleParameter::interpretEdgePos(stop.startPos, edge->getLength(), SUMO_ATTR_STARTPOS, "stopping at " + edge->getID());
+            const double end = SUMOVehicleParameter::interpretEdgePos(stop.endPos, edge->getLength(), SUMO_ATTR_ENDPOS, "stopping at " + edge->getID());
+            const double prevAr = myActivePlan->back()->getArrivalPos();
+            if (start > prevAr + NUMERICAL_EPS || end < prevAr - NUMERICAL_EPS) {
+                WRITE_WARNING("Disconnected plan for person '" + myVehicleParameter->id
+                              + "' (stop range " + toString(start) + "-" + toString(end) + " does not cover previous arrival position " + toString(prevAr) +  + ").");
+            }
+        }
         std::string actType = attrs.getOpt<std::string>(SUMO_ATTR_ACTTYPE, nullptr, ok, "waiting");
         double pos = (stop.startPos + stop.endPos) / 2.;
         if (!myActivePlan->empty()) {
             pos = myActivePlan->back()->getArrivalPos();
         }
         myActivePlan->push_back(new MSTransportable::Stage_Waiting(
-                                    &MSLane::dictionary(stop.lane)->getEdge(), stop.duration, stop.until, pos, actType, false));
-    } else if (myActiveContainerPlan != nullptr) {
+                                    &MSLane::dictionary(stop.lane)->getEdge(), toStop, stop.duration, stop.until, pos, actType, false));
+
+    } else if (myActiveContainerPlan) {
+        if (myActiveContainerPlan->empty()) {
+            double departPos = toStop == nullptr || myVehicleParameter->wasSet(VEHPARS_DEPARTPOS_SET)
+                               ? myVehicleParameter->departPos
+                               : (toStop->getBeginLanePosition() + toStop->getEndLanePosition()) / 2;
+            myActiveContainerPlan->push_back(new MSTransportable::Stage_Waiting(
+                                                 &MSLane::dictionary(stop.lane)->getEdge(), toStop, -1, myVehicleParameter->depart, departPos, "start", true));
+        } else if (myActiveContainerPlan->back()->getDestination() != &MSLane::dictionary(stop.lane)->getEdge()) {
+            throw ProcessError("Disconnected plan for container '" + myVehicleParameter->id + "' (" + MSLane::dictionary(stop.lane)->getEdge().getID() + "!=" + myActiveContainerPlan->back()->getDestination()->getID() + ").");
+        }
         std::string actType = attrs.getOpt<std::string>(SUMO_ATTR_ACTTYPE, nullptr, ok, "waiting");
         myActiveContainerPlan->push_back(new MSTransportable::Stage_Waiting(
-                                             &MSLane::dictionary(stop.lane)->getEdge(), stop.duration, stop.until, stop.startPos, actType, false));
+                                             &MSLane::dictionary(stop.lane)->getEdge(), toStop, stop.duration, stop.until, stop.startPos, actType, false));
     } else if (myVehicleParameter != nullptr) {
         myVehicleParameter->stops.push_back(stop);
     } else {
         myActiveRouteStops.push_back(stop);
     }
     if (myInsertStopEdgesAt >= 0) {
+        //std::cout << " myInsertStopEdgesAt=" << myInsertStopEdgesAt << " edge=" << edge->getID() << " myRoute=" << toString(myActiveRoute) << "\n";
         myActiveRoute.insert(myActiveRoute.begin() + myInsertStopEdgesAt, edge);
         myInsertStopEdgesAt++;
     }
@@ -1006,7 +1099,7 @@ MSRouteHandler::parseWalkPositions(const SUMOSAXAttributes& attrs, const std::st
         }
         if (attrs.hasAttribute(SUMO_ATTR_ARRIVALPOS)) {
             const double length = toEdge != nullptr ? toEdge->getLength() : bs->getLane().getLength();
-            const double arrPos = SUMOVehicleParserHelper::parseWalkPos(SUMO_ATTR_ARRIVALPOS, description, length,
+            const double arrPos = SUMOVehicleParserHelper::parseWalkPos(SUMO_ATTR_ARRIVALPOS, myHardFail, description, length,
                                   attrs.get<std::string>(SUMO_ATTR_ARRIVALPOS, description.c_str(), ok), &myParsingRNG);
             if (arrPos >= bs->getBeginLanePosition() && arrPos < bs->getEndLanePosition()) {
                 arrivalPos = arrPos;
@@ -1020,7 +1113,7 @@ MSRouteHandler::parseWalkPositions(const SUMOSAXAttributes& attrs, const std::st
             throw ProcessError("No destination edge for " + description + ".");
         }
         if (attrs.hasAttribute(SUMO_ATTR_ARRIVALPOS)) {
-            arrivalPos = SUMOVehicleParserHelper::parseWalkPos(SUMO_ATTR_ARRIVALPOS, description, toEdge->getLength(),
+            arrivalPos = SUMOVehicleParserHelper::parseWalkPos(SUMO_ATTR_ARRIVALPOS, myHardFail, description, toEdge->getLength(),
                          attrs.get<std::string>(SUMO_ATTR_ARRIVALPOS, description.c_str(), ok), &myParsingRNG);
         } else {
             arrivalPos = toEdge->getLength() / 2.;
@@ -1056,17 +1149,10 @@ MSRouteHandler::addPersonTrip(const SUMOSAXAttributes& attrs) {
 
     const std::string modes = attrs.getOpt<std::string>(SUMO_ATTR_MODES, id, ok, "");
     SVCPermissions modeSet = 0;
-    for (StringTokenizer st(modes); st.hasNext();) {
-        const std::string mode = st.next();
-        if (mode == "car") {
-            modeSet |= SVC_PASSENGER;
-        } else if (mode == "bicycle") {
-            modeSet |= SVC_BICYCLE;
-        } else if (mode == "public") {
-            modeSet |= SVC_BUS;
-        } else {
-            throw InvalidArgument("Unknown person mode '" + mode + "'.");
-        }
+    std::string errorMsg;
+    // try to parse person modes
+    if (!SUMOVehicleParameter::parsePersonModes(modes, "person", id, modeSet, errorMsg)) {
+        throw InvalidArgument(errorMsg);
     }
     const std::string types = attrs.getOpt<std::string>(SUMO_ATTR_VTYPES, id, ok, "");
     for (StringTokenizer st(types); st.hasNext();) {
@@ -1083,8 +1169,16 @@ MSRouteHandler::addPersonTrip(const SUMOSAXAttributes& attrs) {
     const double walkFactor = attrs.getOpt<double>(SUMO_ATTR_WALKFACTOR, id, ok, OptionsCont::getOptions().getFloat("persontrip.walkfactor"));
     const double departPosLat = attrs.getOpt<double>(SUMO_ATTR_DEPARTPOS_LAT, nullptr, ok, 0);
     if (ok) {
+        if (myActivePlan->empty()) {
+            double initialDepartPos = myVehicleParameter->departPos;
+            if (myVehicleParameter->departPosProcedure == DEPART_POS_RANDOM) {
+                initialDepartPos = RandHelper::rand(from->getLength(), &myParsingRNG);
+            }
+            myActivePlan->push_back(new MSTransportable::Stage_Waiting(from, nullptr, -1, myVehicleParameter->depart, initialDepartPos, "start", true));
+        }
         myVehicleParameter->parametersSet |= VEHPARS_FORCE_REROUTE;
-        myActivePlan->push_back(new MSTransportable::Stage_Trip(from, to == nullptr ? &stoppingPlace->getLane().getEdge() : to, stoppingPlace, duration, modeSet, types, speed, walkFactor, departPosLat, attrs.hasAttribute(SUMO_ATTR_ARRIVALPOS), arrivalPos));
+        MSStoppingPlace* fromStop = myActivePlan->empty() ? nullptr : myActivePlan->back()->getDestinationStop();
+        myActivePlan->push_back(new MSTransportable::Stage_Trip(from, fromStop, to == nullptr ? &stoppingPlace->getLane().getEdge() : to, stoppingPlace, duration, modeSet, types, speed, walkFactor, departPosLat, attrs.hasAttribute(SUMO_ATTR_ARRIVALPOS), arrivalPos));
     }
     myActiveRoute.clear();
 }
@@ -1124,7 +1218,7 @@ MSRouteHandler::addWalk(const SUMOSAXAttributes& attrs) {
             if (myVehicleParameter->departPosProcedure == DEPART_POS_RANDOM) {
                 initialDepartPos = RandHelper::rand(myActiveRoute.front()->getLength(), &myParsingRNG);
             }
-            myActivePlan->push_back(new MSTransportable::Stage_Waiting(myActiveRoute.front(), -1, myVehicleParameter->depart, initialDepartPos, "start", true));
+            myActivePlan->push_back(new MSTransportable::Stage_Waiting(myActiveRoute.front(), nullptr, -1, myVehicleParameter->depart, initialDepartPos, "start", true));
         }
         parseWalkPositions(attrs, myVehicleParameter->id, myActiveRoute.front(), myActiveRoute.back(), departPos, arrivalPos, bs, myActivePlan->back(), ok);
         if (myActiveRoute.empty()) {
@@ -1147,4 +1241,28 @@ MSRouteHandler::addWalk(const SUMOSAXAttributes& attrs) {
 }
 
 
+void
+MSRouteHandler::addPerson(const SUMOSAXAttributes& /*attrs*/) {
+}
+
+
+void
+MSRouteHandler::addContainer(const SUMOSAXAttributes& /*attrs*/) {
+}
+
+
+void
+MSRouteHandler::addRide(const SUMOSAXAttributes& /*attrs*/) {
+}
+
+
+void
+MSRouteHandler::addTransport(const SUMOSAXAttributes& /*attrs*/) {
+}
+
+
+void
+MSRouteHandler::addTranship(const SUMOSAXAttributes& /*attrs*/) {
+}
+
 /****************************************************************************/
