@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2012-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2012-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Thu, 13 Dec 2012
-/// @version $Id$
 ///
 // Parser and output filter for routes and vehicles state saving and loading
 /****************************************************************************/
@@ -44,10 +43,10 @@ class MESegment;
  */
 class MSStateHandler : public MSRouteHandler {
 public:
-    /// standard constructor
+    /// @brief standard constructor
     MSStateHandler(const std::string& file, const SUMOTime offset);
 
-    /// standard destructor
+    /// @brief standard destructor
     virtual ~MSStateHandler();
 
     /** @brief Saves the current state
@@ -56,6 +55,7 @@ public:
      */
     static void saveState(const std::string& file, SUMOTime step);
 
+    /// @brief get time
     SUMOTime getTime() const {
         return myTime;
     }
@@ -74,7 +74,6 @@ protected:
     void myStartElement(int element,
                         const SUMOSAXAttributes& attrs);
 
-
     /** @brief Called when a closing tag occurs
      *
      * @param[in] element ID of the currently opened element
@@ -88,10 +87,19 @@ protected:
     void closeVehicle();
 
 private:
+    /// @brief offset
     const SUMOTime myOffset;
+
+    /// @brief time
     SUMOTime myTime;
+
+    /// @brief segment
     MESegment* mySegment;
-    std::pair<int, int> myEdgeAndLane;
+
+    /// @brief current lane being loaded
+    MSLane* myCurrentLane;
+
+    /// @brief que index
     int myQueIndex;
 
     /// @brief cached attrs (used when loading vehicles)
@@ -112,11 +120,10 @@ private:
 
 private:
     /// @brief Invalidated copy constructor
-    MSStateHandler(const MSStateHandler& s);
+    MSStateHandler(const MSStateHandler& s) = delete;
 
     /// @brief Invalidated assignment operator
-    MSStateHandler& operator=(const MSStateHandler& s);
-
+    MSStateHandler& operator=(const MSStateHandler& s) = delete;
 };
 
 
