@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id$
 ///
 // Stores all persons or containers in the net and handles their waiting for cars.
 /****************************************************************************/
@@ -120,7 +119,10 @@ public:
     int getActiveCount();
 
     /// aborts the plan for any transportable that is still waiting for a ride
-    void abortWaitingForVehicle();
+    void abortAnyWaitingForVehicle();
+
+    /// let the given transportable abort waiting for a vehicle (when removing stage via TraCI)
+    void abortWaitingForVehicle(MSTransportable* t);
 
     /// aborts waiting stage of transportable
     void abortWaiting(MSTransportable* t);
@@ -194,6 +196,12 @@ public:
         return myJammedNumber;
     }
 
+    /** @brief Returns the number of vehicles waiting for a ride
+     */
+    int getWaitingForVehicleNumber() const {
+        return myWaitingForVehicleNumber;
+    }
+
     /// @}
 
 protected:
