@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -19,7 +19,6 @@
 /// @author  Axel Wegener
 /// @author  Leonhard Luecken
 /// @date    Mon, 12 Mar 2001
-/// @version $Id$
 ///
 // Representation of a vehicle in the micro simulation
 /****************************************************************************/
@@ -44,6 +43,7 @@
 #include "MSBaseVehicle.h"
 #include "MSLink.h"
 #include "MSLane.h"
+#include "MSNet.h"
 
 #define INVALID_SPEED 299792458 + 1 // nothing can go faster than the speed of light! Refs. #2577
 
@@ -66,7 +66,7 @@ class Position;
 class MSContainer;
 class MSJunction;
 class MSLeaderInfo;
-//class MSDriverState;
+class MSDevice_DriverState;
 class MSSimpleDriverState;
 
 // ===========================================================================
@@ -275,17 +275,6 @@ public:
      */
     bool replaceRoute(const MSRoute* route, const std::string& info, bool onInit = false, int offset = 0, bool addStops = true, bool removeStops = true);
 
-
-    /** @brief Returns whether the vehicle wil pass the given edge
-     * @param[in] The edge to find in the vehicle's current route
-     * @return Whether the given edge will be passed by the vehicle
-     * @todo Move to MSRoute?
-     */
-    bool willPass(const MSEdge* const edge) const;
-
-    int getRoutePosition() const;
-    void resetRoutePosition(int index);
-
     /** @brief Returns the vehicle's internal edge travel times/efforts container
      *
      * If the vehicle does not have such a container, it is built.
@@ -391,7 +380,7 @@ public:
      *  @param[in] accel the assumed acceleration
      *  @return distance covered in next integration step
      */
-    double getDeltaPos(double accel);
+    double getDeltaPos(const double accel) const;
 
 
     /// @name state setter/getter
@@ -441,6 +430,10 @@ public:
         return myState.myPosLat;
     }
 
+    void setLateralPositionOnLane(double posLat) {
+        myState.myPosLat = posLat;
+    }
+
     /** @brief Get the vehicle's lateral position on the lane:
      * @return The lateral position of the vehicle (in m distance between right
      * side of vehicle and ride side of the lane it is on
@@ -501,6 +494,8 @@ public:
         return myAcceleration;
     }
 
+    /// @brief get apparent deceleration based on vType parameters and current acceleration
+    double getCurrentApparentDecel() const;
 
     /** @brief Returns the vehicle's action step length in millisecs,
      *         i.e. the interval between two action points.
@@ -701,6 +696,14 @@ public:
     /// @brief Set a custom vehicle angle in rad, optionally updates furtherLanePosLat
     void setAngle(double angle, bool straightenFurther = false);
 
+    /** @brief Sets the action steplength of the vehicle
+     *
+     * @param actionStepLength New value
+     * @param resetActionOffset whether the action offset should be reset to zero,
+     *        i.e., the next action step should follow immediately.
+     */
+    void setActionStepLength(double actionStepLength, bool resetActionOffset = true);
+
     /** Returns true if the two vehicles overlap. */
     static bool overlap(const MSVehicle* veh1, const MSVehicle* veh2) {
         if (veh1->myState.myPos < veh2->myState.myPos) {
@@ -709,7 +712,6 @@ public:
         return veh1->myState.myPos - veh1->getVehicleType().getLengthWithGap() < veh2->myState.myPos;
     }
 
-
     /** Returns true if vehicle's speed is below 60km/h. This is only relevant
         on highways. Overtaking on the right is allowed then. */
     bool congested() const {
@@ -897,9 +899,8 @@ public:
      * @return The vehicle driver's state
      *
      */
-    inline std::shared_ptr<MSSimpleDriverState> getDriverState() const {
-        return myDriverState;
-    }
+
+    std::shared_ptr<MSSimpleDriverState> getDriverState() const;
 
 
     /** @brief Returns the vehicle's car following model variables
@@ -922,35 +923,37 @@ public:
         /// @brief The edge in the route to stop at
         MSRouteIterator edge;
         /// @brief The lane to stop at
-        const MSLane* lane;
+        const MSLane* lane = nullptr;
         /// @brief (Optional) bus stop if one is assigned to the stop
-        MSStoppingPlace* busstop;
+        MSStoppingPlace* busstop = nullptr;
         /// @brief (Optional) container stop if one is assigned to the stop
-        MSStoppingPlace* containerstop;
+        MSStoppingPlace* containerstop = nullptr;
         /// @brief (Optional) parkingArea if one is assigned to the stop
-        MSParkingArea* parkingarea;
+        MSParkingArea* parkingarea = nullptr;
         /// @brief (Optional) charging station if one is assigned to the stop
-        MSStoppingPlace* chargingStation;
+        MSStoppingPlace* chargingStation = nullptr;
         /// @brief The stop parameter
         const SUMOVehicleParameter::Stop pars;
         /// @brief The stopping duration
-        SUMOTime duration;
+        SUMOTime duration = -1;
         /// @brief whether an arriving person lets the vehicle continue
-        bool triggered;
+        bool triggered = false;
         /// @brief whether an arriving container lets the vehicle continue
-        bool containerTriggered;
+        bool containerTriggered = false;
         /// @brief Information whether the stop has been reached
-        bool reached;
+        bool reached = false;
         /// @brief The number of still expected persons
-        int numExpectedPerson;
+        int numExpectedPerson = 0;
         /// @brief The number of still expected containers
-        int numExpectedContainer;
+        int numExpectedContainer = 0;
         /// @brief The time at which the vehicle is able to board another person
-        SUMOTime timeToBoardNextPerson;
+        SUMOTime timeToBoardNextPerson = 0;
         /// @brief The time at which the vehicle is able to load another container
-        SUMOTime timeToLoadNextContainer;
+        SUMOTime timeToLoadNextContainer = 0;
         /// @brief Whether this stop was triggered by a collision
-        bool collision;
+        bool collision = false;
+        /// @brief the maximum time at which persons may board this vehicle
+        SUMOTime endBoarding = SUMOTime_MAX;
 
         /// @brief Write the current stop configuration (used for state saving)
         void write(OutputDevice& dev) const;
@@ -960,6 +963,7 @@ public:
 
         /// @brief get a short description for showing in the gui
         std::string getDescription() const;
+
     private:
         /// @brief Invalidated assignment operator
         Stop& operator=(const Stop& src);
@@ -980,14 +984,13 @@ public:
      */
     bool replaceParkingArea(MSParkingArea* parkingArea, std::string& errorMsg);
 
-    /** @brief Create a DriverState for the vehicle
-     */
-    void createDriverState();
-
-    /** @brief get the current parking area stop
+    /** @brief get the upcoming parking area stop or nullptr
      */
     MSParkingArea* getNextParkingArea();
 
+    /** @brief get the current  parking area stop or nullptr */
+    MSParkingArea* getCurrentParkingArea();
+
     /** @brief Returns whether the vehicle has to stop somewhere
      * @return Whether the vehicle has to stop somewhere
      */
@@ -998,7 +1001,7 @@ public:
     /** @brief Whether this vehicle is equipped with a MSDriverState
      */
     inline bool hasDriverState() const {
-        return (myDriverState != nullptr);
+        return myDriverState != nullptr;
     }
 
     /** @brief Returns whether the vehicle is at a stop
@@ -1009,6 +1012,12 @@ public:
     /// @brief Returns the remaining stop duration for a stopped vehicle or 0
     SUMOTime remainingStopDuration() const;
 
+    /** @brief Returns whether the vehicle stops at the given stopping place */
+    bool stopsAt(MSStoppingPlace* stop) const;
+
+    /** @brief Returns whether the vehicle stops at the given edge */
+    bool stopsAtEdge(const MSEdge* edge) const;
+
     /** @brief Returns whether the vehicle will stop on the current edge
      */
     bool willStop() const;
@@ -1051,7 +1060,7 @@ public:
 
     /** @brief return whether the given position is within range of the current stop
      */
-    bool isStoppedInRange(double pos) const;
+    bool isStoppedInRange(const double pos, const double tolerance) const;
     /// @}
 
     int getLaneIndex() const;
@@ -1094,7 +1103,6 @@ public:
      */
     double getTimeGapOnLane() const;
 
-
     /// @name Emission retrieval
     //@{
 
@@ -1146,8 +1154,6 @@ public:
     double getHarmonoise_NoiseEmissions() const;
     //@}
 
-
-
     /// @name Interaction with persons
     //@{
 
@@ -1340,24 +1346,139 @@ public:
     /// @brief get bounding polygon
     PositionVector getBoundingPoly() const;
 
-    /** @class Influencer
-     * @brief Changes the wished vehicle speed / lanes
-     *
-     * The class is used for passing velocities or velocity profiles obtained via TraCI to the vehicle.
-     * The speed adaptation is controlled by the stored speedTimeLine
-     * Additionally, the variables myConsiderSafeVelocity, myConsiderMaxAcceleration, and myConsiderMaxDeceleration
-     * control whether the safe velocity, the maximum acceleration, and the maximum deceleration
-     * have to be regarded.
-     *
-     * Furthermore this class is used to affect lane changing decisions according to
-     * LaneChangeMode and any given laneTimeLine
+    /** @enum ManoeuvreType
+     *  @brief  flag identifying which, if any, manoeuvre is in progress
      */
+    enum ManoeuvreType {
+        /// @brief Manoeuvre into stopping place
+        MANOEUVRE_ENTRY,
+        /// @brief Manoeuvre out of stopping place
+        MANOEUVRE_EXIT,
+        /// @brief not manouevring
+        MANOEUVRE_NONE
+    };
+
+    /// @brief accessor function to myManoeuvre equivalent
+    /// @note Setup of exit manoeuvre is invoked from MSVehicleTransfer
+    bool setExitManoeuvre();
+    /// @brief accessor function to myManoeuvre equivalent
+    void setManoeuvreType(const MSVehicle::ManoeuvreType mType);
+
+    /// @brief accessor function to myManoeuvre equivalent
+    bool manoeuvreIsComplete() const;
+    /// @brief accessor function to myManoeuvre equivalent
+    MSVehicle::ManoeuvreType getManoeuvreType() const;
+
+
+    /** @class Manoeuvre
+      * @brief  Container for manouevering time associated with stopping.
+      *
+      *  Introduced to cater for lane blocking whilst entering stop/leaving stop
+      *   and assure that emissions during manoeuvre are included in model
+      */
+    class Manoeuvre {
+
+    public:
+        /// Constructor.
+        Manoeuvre();
+
+        /// Copy constructor.
+        Manoeuvre(const Manoeuvre& manoeuvre);
+
+        /// Assignment operator.
+        Manoeuvre& operator=(const Manoeuvre& manoeuvre);
+
+        /// Operator !=
+        bool operator!=(const Manoeuvre& manoeuvre);
+
+        /// @brief Setup the entry manoeuvre for this vehicle (Sets completion time and manoeuvre type)
+        bool configureEntryManoeuvre(MSVehicle* veh);
+
+        /// @brief Setup the myManoeuvre for exiting (Sets completion time and manoeuvre type)
+        bool configureExitManoeuvre(MSVehicle* veh);
+
+        /// @brief Configure an entry manoeuvre if nothing is configured - otherwise check if complete
+        bool entryManoeuvreIsComplete(MSVehicle* veh);
+
+        /// @brief Check if specific manoeuver is ongoing and whether the completion time is beyond currentTime
+        bool
+        manoeuvreIsComplete(const ManoeuvreType checkType) const;
+
+        /// @brief Check if any manoeuver is ongoing and whether the completion time is beyond currentTime
+        bool
+        manoeuvreIsComplete() const;
+
+        /// @brief Accessor for manoeuvre angle
+        int getManoeuvreAngle() const;
+
+        /// @brief Accessor (get) for manoeuvre type
+        MSVehicle::ManoeuvreType getManoeuvreType() const;
+
+        /// @brief Accessor (set) for manoeuvre type
+        void setManoeuvreType(const MSVehicle::ManoeuvreType mType);
+
+    private:
+        /// @brief  The name of the vehicle associated with the Manoeuvre  - for debug output
+        std::string myManoeuvreVehicleID;
+
+        /// @brief  The name of the stop associated with the Manoeuvre  - for debug output
+        std::string myManoeuvreStop;
+
+        /// @brief Time at which the Manoeuvre for this stop started
+        SUMOTime myManoeuvreStartTime;
+
+        /// @brief Time at which this manoeuvre should complete
+        SUMOTime myManoeuvreCompleteTime;
+
+        /// @brief Manoeuvre type - currently entry, exit or none
+        ManoeuvreType myManoeuvreType;
+
+        // @brief Angle (degrees) through which manoeuver will turn vehicle - used to determine manouevre timing
+        int myManoeuvreAngle;
+    };
+
+    // Current or previous (completed) manoeuvre
+    Manoeuvre myManoeuvre;
+
+    /** @class Influencer
+      * @brief Changes the wished vehicle speed / lanes
+      *
+      * The class is used for passing velocities or velocity profiles obtained via TraCI to the vehicle.
+      * The speed adaptation is controlled by the stored speedTimeLine
+      * Additionally, the variables myConsiderSafeVelocity, myConsiderMaxAcceleration, and myConsiderMaxDeceleration
+      * control whether the safe velocity, the maximum acceleration, and the maximum deceleration
+      * have to be regarded.
+      *
+      * Furthermore this class is used to affect lane changing decisions according to
+      * LaneChangeMode and any given laneTimeLine
+      */
     class Influencer {
     private:
+
+        /// @brief A static instance of this class in GapControlState deactivates gap control
+        ///        for vehicles whose reference vehicle is removed from the road network
+        class GapControlVehStateListener : public MSNet::VehicleStateListener {
+            /** @brief Called if a vehicle changes its state
+             * @param[in] vehicle The vehicle which changed its state
+             * @param[in] to The state the vehicle has changed to
+             * @param[in] info Additional information on the state change
+             * @note This deactivates the corresponding gap control when a reference vehicle is removed.
+             */
+            void vehicleStateChanged(const SUMOVehicle* const vehicle, MSNet::VehicleState to, const std::string& info = "");
+        };
+
+
         /// @brief Container for state and parameters of the gap control
         struct GapControlState {
             GapControlState();
-            void activate(double tauOriginal, double tauTarget, double additionalGap, double duration, double changeRate, double maxDecel);
+            virtual ~GapControlState();
+            /// @brief Static initalization (adds vehicle state listener)
+            static void init();
+            /// @brief Static cleanup (removes vehicle state listener)
+            static void cleanup();
+            /// @brief Start gap control with given params
+            void activate(double tauOriginal, double tauTarget, double additionalGap, double duration, double changeRate, double maxDecel, const MSVehicle* refVeh);
+            /// @brief Stop gap control
             void deactivate();
             /// @brief Original value for the desired headway (will be reset after duration has expired)
             double tauOriginal;
@@ -1376,6 +1497,8 @@ public:
             double changeRate;
             /// @brief Maximal deceleration to be applied due to the adapted headway
             double maxDecel;
+            /// @brief reference vehicle for the gap - if it is null, the current leader on the ego's lane is used as a reference
+            const MSVehicle* referenceVeh;
             /// @brief Whether the gap control is active
             bool active;
             /// @brief Whether the desired gap was attained during the current activity phase (induces the remaining duration to decrease)
@@ -1386,15 +1509,24 @@ public:
             SUMOTime lastUpdate;
             /// @brief cache storage for the headway increments of the current operation
             double timeHeadwayIncrement, spaceHeadwayIncrement;
+
+            /// @brief stores reference vehicles currently in use by a gapController
+            static std::map<const MSVehicle*, GapControlState*> refVehMap;
+
+        private:
+            static GapControlVehStateListener vehStateListener;
         };
     public:
         /// @brief Constructor
         Influencer();
 
-
         /// @brief Destructor
         ~Influencer();
 
+        /// @brief Static initalization
+        static void init();
+        /// @brief Static cleanup
+        static void cleanup();
 
         /** @brief Sets a new velocity timeline
          * @param[in] speedTimeLine The time line of speeds to use
@@ -1403,7 +1535,7 @@ public:
 
         /** @brief Activates the gap control with the given parameters, @see GapControlState
          */
-        void activateGapController(double originalTau, double newTimeHeadway, double newSpaceHeadway, double duration, double changeRate, double maxDecel);
+        void activateGapController(double originalTau, double newTimeHeadway, double newSpaceHeadway, double duration, double changeRate, double maxDecel, MSVehicle* refVeh = nullptr);
 
         /** @brief Deactivates the gap control
          */
@@ -1414,6 +1546,11 @@ public:
          */
         void setLaneTimeLine(const std::vector<std::pair<SUMOTime, int> >& laneTimeLine);
 
+        /** @brief Adapts lane timeline when moving to a new lane and the lane index changes
+         * @param[in] indexShift The change in laneIndex
+         */
+        void adaptLaneTimeLine(int indexShift);
+
         /** @brief Sets a new sublane-change request
          * @param[in] latDist The lateral distance for changing
          */
@@ -1558,7 +1695,7 @@ public:
             return myTraciLaneChangePriority == LCP_ALWAYS;
         }
 
-        SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT() const;
+        SUMOAbstractRouter<MSEdge, SUMOVehicle>& getRouterTT(const int rngIndex) const;
 
     private:
         /// @brief The velocity time line to apply
@@ -1638,7 +1775,7 @@ public:
     const Influencer* getInfluencer() const;
 
     bool hasInfluencer() const {
-        return myInfluencer != 0;
+        return myInfluencer != nullptr;
     }
 
     /// @brief allow TraCI to influence a lane change decision
@@ -1658,7 +1795,10 @@ public:
     static int nextLinkPriority(const std::vector<MSLane*>& conts);
 
     /// @brief whether the given vehicle must be followed at the given junction
-    bool isLeader(const MSLink* link, const MSVehicle* veh) const; 
+    bool isLeader(const MSLink* link, const MSVehicle* veh) const;
+
+    // @brief get the position of the back bumper;
+    const Position getBackPosition() const;
 
     /// @name state io
     //@{
@@ -1704,7 +1844,7 @@ protected:
      *  @param[in/out] vSafeMin The minimal safe (or admissible) velocity (used her for ensuring the clearing of junctions in time).
      *  @param[in/out] vSafeMinDist The distance to the next link, which should either be crossed this step, or in front of which the vehicle need to stop.
      */
-    void processLinkAproaches(double& vSafe, double& vSafeMin, double& vSafeMinDist);
+    void processLinkApproaches(double& vSafe, double& vSafeMin, double& vSafeMinDist);
 
 
     /** @brief This method checks if the vehicle has advanced over one or several lanes
@@ -1745,7 +1885,7 @@ protected:
     void updateTimeLoss(double vNext);
 
     /// @brief whether the vehicle is a train that can reverse its direction at the current point in its route
-    bool canReverse() const;
+    bool canReverse(double speedThreshold = SUMO_const_haltingSpeed) const;
 
     /** @brief sets the braking lights on/off
      */
@@ -1763,8 +1903,12 @@ protected:
     void updateOccupancyAndCurrentBestLane(const MSLane* startLane);
 
     /** @brief Returns the list of still pending stop edges
+     * also returns the first and last stop position
      */
-    const ConstMSEdgeVector getStopEdges() const;
+    const ConstMSEdgeVector getStopEdges(double& firstPos, double& lastPos) const;
+
+    /// @brief return list of route indices for the remaining stops
+    std::vector<std::pair<int, double> > getStopIndices() const;
 
     /// @brief get distance for coming to a stop (used for rerouting checks)
     double getBrakeGap() const;
@@ -1786,8 +1930,7 @@ protected:
     State myState;
 
     /// @brief This vehicle's driver state @see MSDriverState
-//    std::shared_ptr<MSDriverState> myDriverState;
-    std::shared_ptr<MSSimpleDriverState> myDriverState;
+    MSDevice_DriverState* myDriverState;
 
     /// @brief The flag myActionStep indicates whether the current time step is an action point for the vehicle.
     bool myActionStep;
@@ -1851,7 +1994,7 @@ protected:
     /// @brief the angle in radians (@todo consider moving this into myState)
     double myAngle;
 
-    /// @brief distance to the next stop or -1 if there is none
+    /// @brief distance to the next stop or doubleMax if there is none
     double myStopDist;
 
     /// @brief amount of time for which the vehicle is immune from collisions
@@ -1945,7 +2088,7 @@ protected:
     void checkRewindLinkLanes(const double lengthsInFront, DriveItemVector& lfLinks) const;
 
     /// @brief unregister approach from all upcoming links
-    void removeApproachingInformation(DriveItemVector& lfLinks) const;
+    void removeApproachingInformation(const DriveItemVector& lfLinks) const;
 
 
     /// @brief estimate leaving speed when accelerating across a link
@@ -2001,9 +2144,6 @@ protected:
     // @brief return the lane on which the back of this vehicle resides
     const MSLane* getBackLane() const;
 
-    // @brief get the position of the back bumper;
-    const Position getBackPosition() const;
-
     /** @brief updates the vehicles state, given a next value for its speed.
      *         This value can be negative in case of the ballistic update to indicate
      *         a stop within the next timestep. (You can call this a 'hack' to
@@ -2020,6 +2160,10 @@ protected:
     /// @brief decide whether a red (or yellow light) may be ignore
     bool ignoreRed(const MSLink* link, bool canBrake) const;
 
+
+    /// @brief check whether all stop.edge MSRouteIterators are valid and in order
+    bool haveValidStopEdges() const;
+
 private:
     /* @brief The vehicle's knowledge about edge efforts/travel times; @see MSEdgeWeightsStorage
      * @note member is initialized on first access */
@@ -2031,6 +2175,8 @@ private:
     /// @brief An instance of a velocity/lane influencing instance; built in "getInfluencer"
     Influencer* myInfluencer;
 
+
+
 private:
     /// @brief invalidated default constructor
     MSVehicle();
