@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 10. Dec 2003
-/// @version $Id$
 ///
 // The class responsible for building and deletion of vehicles
 /****************************************************************************/
@@ -31,12 +30,16 @@
 #include "MSRouteHandler.h"
 #include <microsim/devices/MSVehicleDevice.h>
 #include <utils/common/FileHelpers.h>
+#include <utils/common/Named.h>
 #include <utils/common/RGBColor.h>
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/iodevices/BinaryInputDevice.h>
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/options/OptionsCont.h>
-#include <utils/vehicle/IntermodalRouter.h>
+#include <utils/router/IntermodalRouter.h>
+#ifdef HAVE_FOX
+#include <utils/foxtools/FXConditionalLock.h>
+#endif
 
 
 // ===========================================================================
@@ -60,7 +63,8 @@ MSVehicleControl::MSVehicleControl() :
     myWaitingForPerson(0),
     myWaitingForContainer(0),
     myMaxSpeedFactor(1),
-    myMinDeceleration(SUMOVTypeParameter::getDefaultDecel(SVC_IGNORING)) {
+    myMinDeceleration(SUMOVTypeParameter::getDefaultDecel(SVC_IGNORING)),
+    myPendingRemovals(MSGlobals::gNumSimThreads > 1) {
     SUMOVTypeParameter defType(DEFAULT_VTYPE_ID, SVC_PASSENGER);
     myVTypeDict[DEFAULT_VTYPE_ID] = MSVehicleType::build(defType);
     SUMOVTypeParameter defPedType(DEFAULT_PEDTYPE_ID, SVC_PEDESTRIAN);
@@ -69,55 +73,94 @@ MSVehicleControl::MSVehicleControl() :
     SUMOVTypeParameter defBikeType(DEFAULT_BIKETYPE_ID, SVC_BICYCLE);
     defBikeType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
     myVTypeDict[DEFAULT_BIKETYPE_ID] = MSVehicleType::build(defBikeType);
-    OptionsCont& oc = OptionsCont::getOptions();
-    myScale = oc.getFloat("scale");
+    myScale = OptionsCont::getOptions().getFloat("scale");
 }
 
 
 MSVehicleControl::~MSVehicleControl() {
     // delete vehicles
     for (VehicleDictType::iterator i = myVehicleDict.begin(); i != myVehicleDict.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
     myVehicleDict.clear();
     // delete vehicle type distributions
     for (VTypeDistDictType::iterator i = myVTypeDistDict.begin(); i != myVTypeDistDict.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
     myVTypeDistDict.clear();
     // delete vehicle types
     for (VTypeDictType::iterator i = myVTypeDict.begin(); i != myVTypeDict.end(); ++i) {
-        //delete(*i).second;
+        delete (*i).second;
     }
     myVTypeDict.clear();
 }
 
+
 SUMOVehicle*
 MSVehicleControl::buildVehicle(SUMOVehicleParameter* defs,
                                const MSRoute* route, MSVehicleType* type,
                                const bool ignoreStopErrors, const bool fromRouteFile) {
     myLoadedVehNo++;
     MSVehicle* built = new MSVehicle(defs, route, type, type->computeChosenSpeedDeviation(fromRouteFile ? MSRouteHandler::getParsingRNG() : nullptr));
-    built->addStops(ignoreStopErrors);
+    try {
+        built->addStops(ignoreStopErrors);
+    } catch (ProcessError&) {
+        delete built;
+        throw;
+    }
     MSNet::getInstance()->informVehicleStateListener(built, MSNet::VEHICLE_STATE_BUILT);
     return built;
 }
 
 
 void
-MSVehicleControl::scheduleVehicleRemoval(SUMOVehicle* veh) {
+MSVehicleControl::scheduleVehicleRemoval(SUMOVehicle* veh, bool checkDuplicate) {
     assert(myRunningVehNo > 0);
-    myTotalTravelTime += STEPS2TIME(MSNet::getInstance()->getCurrentTimeStep() - veh->getDeparture());
-    myRunningVehNo--;
-    MSNet::getInstance()->informVehicleStateListener(veh, MSNet::VEHICLE_STATE_ARRIVED);
-    for (MSVehicleDevice* const dev : veh->getDevices()) {
-        dev->generateOutput();
+    if (!checkDuplicate || !isPendingRemoval(veh)) {
+        myPendingRemovals.push_back(veh);
     }
-    if (OptionsCont::getOptions().isSet("tripinfo-output")) {
-        // close tag after tripinfo (possibly including emissions from another device) have been written
-        OutputDevice::getDeviceByOption("tripinfo-output").closeTag();
+}
+
+
+bool
+MSVehicleControl::isPendingRemoval(SUMOVehicle* veh) {
+#ifdef HAVE_FOX
+    return myPendingRemovals.contains(veh);
+#else
+    return std::find(myPendingRemovals.begin(), myPendingRemovals.end(), veh) == myPendingRemovals.end();
+#endif
+}
+
+void
+MSVehicleControl::removePending() {
+    OutputDevice* tripinfoOut = OptionsCont::getOptions().isSet("tripinfo-output") ? &OutputDevice::getDeviceByOption("tripinfo-output") : nullptr;
+#ifdef HAVE_FOX
+    std::vector<SUMOVehicle*>& vehs = myPendingRemovals.getContainer();
+#else
+    std::vector<SUMOVehicle*>& vehs = myPendingRemovals;
+#endif
+    std::sort(vehs.begin(), vehs.end(), ComparatorNumericalIdLess());
+    for (SUMOVehicle* const veh : vehs) {
+        myTotalTravelTime += STEPS2TIME(MSNet::getInstance()->getCurrentTimeStep() - veh->getDeparture());
+        myRunningVehNo--;
+        MSNet::getInstance()->informVehicleStateListener(veh, MSNet::VEHICLE_STATE_ARRIVED);
+        for (MSVehicleDevice* const dev : veh->getDevices()) {
+            dev->generateOutput();
+        }
+        if (tripinfoOut != nullptr) {
+            // close tag after tripinfo (possibly including emissions from another device) have been written
+            tripinfoOut->closeTag();
+        }
+        deleteVehicle(veh);
     }
-    deleteVehicle(veh);
+    vehs.clear();
+    if (tripinfoOut != nullptr) {
+        // there seem to be people who think reading an unfinished xml is a good idea ;-)
+        tripinfoOut->flush();
+    }
+#ifdef HAVE_FOX
+    myPendingRemovals.unlock();
+#endif
 }
 
 
@@ -127,7 +170,7 @@ MSVehicleControl::vehicleDeparted(const SUMOVehicle& v) {
     myTotalDepartureDelay += STEPS2TIME(v.getDeparture() - STEPFLOOR(v.getParameter().depart));
     MSNet::getInstance()->informVehicleStateListener(&v, MSNet::VEHICLE_STATE_DEPARTED);
     myMaxSpeedFactor = MAX2(myMaxSpeedFactor, v.getChosenSpeedFactor());
-    if ((v.getVClass() & (SVC_SHIP | SVC_PEDESTRIAN | SVC_RAIL | SVC_RAIL_ELECTRIC | SVC_RAIL_URBAN)) == 0) {
+    if ((v.getVClass() & (SVC_PEDESTRIAN | SVC_NON_ROAD)) == 0) {
         // only  worry about deceleration of road users
         myMinDeceleration = MIN2(myMinDeceleration, v.getVehicleType().getCarFollowModel().getMaxDecel());
     }
@@ -181,7 +224,7 @@ MSVehicleControl::addVehicle(const std::string& id, SUMOVehicle* v) {
                 // position will be checked against person position later
                 static_cast<MSVehicle*>(v)->setTentativeLaneAndPosition(firstEdge->getLanes()[0], v->getParameter().departPos);
             }
-            addWaiting(v->getRoute().getEdges().front(), v);
+            firstEdge->addWaiting(v);
             registerOneWaiting(pars.departProcedure == DEPART_TRIGGERED);
         }
         if (pars.line != "" && pars.repetitionNumber < 0) {
@@ -212,6 +255,10 @@ MSVehicleControl::deleteVehicle(SUMOVehicle* veh, bool discard) {
     if (veh != nullptr) {
         myVehicleDict.erase(veh->getID());
     }
+    auto ptVehIt = std::find(myPTVehicles.begin(), myPTVehicles.end(), veh);
+    if (ptVehIt != myPTVehicles.end()) {
+        myPTVehicles.erase(ptVehIt);
+    }
     delete veh;
 }
 
@@ -333,7 +380,7 @@ MSVehicleControl::insertVTypeIDs(std::vector<std::string>& into) const {
 }
 
 
-std::set<std::string>
+const std::set<std::string>
 MSVehicleControl::getVTypeDistributionMembership(const std::string& id) const {
     std::map<std::string, std::set<std::string>>::const_iterator it = myVTypeToDist.find(id);
     if (it == myVTypeToDist.end()) {
@@ -343,58 +390,6 @@ MSVehicleControl::getVTypeDistributionMembership(const std::string& id) const {
 }
 
 
-void
-MSVehicleControl::addWaiting(const MSEdge* const edge, SUMOVehicle* vehicle) {
-    if (myWaiting.find(edge) == myWaiting.end()) {
-        myWaiting[edge] = std::vector<SUMOVehicle*>();
-    }
-    myWaiting[edge].push_back(vehicle);
-}
-
-
-void
-MSVehicleControl::removeWaiting(const MSEdge* const edge, const SUMOVehicle* vehicle) {
-    if (myWaiting.find(edge) != myWaiting.end()) {
-        std::vector<SUMOVehicle*>::iterator it = std::find(myWaiting[edge].begin(), myWaiting[edge].end(), vehicle);
-        if (it != myWaiting[edge].end()) {
-            myWaiting[edge].erase(it);
-        }
-    }
-}
-
-
-SUMOVehicle*
-MSVehicleControl::getWaitingVehicle(const MSEdge* const edge, const std::set<std::string>& lines, const double position, const std::string ridingID) {
-    if (myWaiting.find(edge) != myWaiting.end()) {
-        // for every vehicle waiting vehicle at this edge
-        std::vector<SUMOVehicle*> waitingTooFarAway;
-        for (std::vector<SUMOVehicle*>::const_iterator it = myWaiting[edge].begin(); it != myWaiting[edge].end(); ++it) {
-            const std::string& line = (*it)->getParameter().line == "" ? (*it)->getParameter().id : (*it)->getParameter().line;
-            double vehiclePosition = (*it)->getPositionOnLane();
-            // if the line of the vehicle is contained in the set of given lines and the vehicle is stopped and is positioned
-            // in the interval [position - t, position + t] for a tolerance t=10
-            if (lines.count(line)) {
-                if ((position - 10 <= vehiclePosition) && (vehiclePosition <= position + 10)) {
-                    return (*it);
-                } else if ((*it)->isStoppedTriggered() ||
-                           (*it)->getParameter().departProcedure == DEPART_TRIGGERED) {
-                    // maybe we are within the range of the stop
-                    if ((*it)->isStoppedInRange(position)) {
-                        return (*it);
-                    } else {
-                        waitingTooFarAway.push_back(*it);
-                    }
-                }
-            }
-        }
-        for (std::vector<SUMOVehicle*>::iterator it = waitingTooFarAway.begin(); it != waitingTooFarAway.end(); ++it) {
-            WRITE_WARNING(ridingID + " at edge '" + edge->getID() + "' position " + toString(position) + " cannot use waiting vehicle '" + (*it)->getID() + "' at position " + toString((*it)->getPositionOnLane()) + " because it is too far away.");
-        }
-    }
-    return nullptr;
-}
-
-
 void
 MSVehicleControl::abortWaiting() {
     for (VehicleDictType::iterator i = myVehicleDict.begin(); i != myVehicleDict.end(); ++i) {
@@ -473,4 +468,3 @@ MSVehicleControl::adaptIntermodalRouter(MSNet::MSIntermodalRouter& router) const
 
 
 /****************************************************************************/
-
