@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 10. Dec 2003
-/// @version $Id$
 ///
 // The class responsible for building and deletion of vehicles
 /****************************************************************************/
@@ -29,6 +28,10 @@
 #include <string>
 #include <map>
 #include <set>
+#ifdef HAVE_FOX
+#include <fx.h>
+#include <utils/foxtools/FXSynchQue.h>
+#endif
 #include <utils/distribution/RandomDistributor.h>
 #include <utils/common/SUMOTime.h>
 #include <utils/common/SUMOVehicleClass.h>
@@ -151,7 +154,21 @@ public:
      *
      * @param[in] veh The vehicle to remove
      */
-    void scheduleVehicleRemoval(SUMOVehicle* veh);
+    void scheduleVehicleRemoval(SUMOVehicle* veh, bool checkDuplicate = false);
+
+
+    /** @brief Removes a vehicle after it has ended
+     *
+     * Writes output to tripinfos and vehroutes if wished; decrements
+     *  the number of running vehicles and increments the number of ended
+     *  vehicles. Then deletes the vehicle using "deleteVehicle".
+     *
+     * This method should be called for each vehicle that was inserted
+     *  into the network and quits its ride.
+     *
+     * @param[in] veh The vehicle to remove
+     */
+    void removePending();
 
 
     /** @brief Returns the begin of the internal vehicle map
@@ -376,9 +393,9 @@ public:
 
     /** @brief Returns the named vehicle type or a sample from the named distribution
      * @param[in] id The id of the vehicle type to return. If left out, the default type is returned.
-     * @return The named vehicle type, or 0 if no such type exists
+     * @return The named vehicle type, or nullptr if no such type exists
      */
-    MSVehicleType* getVType(const std::string& id = DEFAULT_VTYPE_ID, std::mt19937* rng = 0);
+    MSVehicleType* getVType(const std::string& id = DEFAULT_VTYPE_ID, std::mt19937* rng = nullptr);
 
 
     /** @brief Inserts ids of all known vehicle types and vehicle type distributions to the given vector
@@ -390,24 +407,10 @@ public:
     /** @brief Return the distribution IDs the vehicle type is a member of
     * @param[in] vehType The vehicle type to look for membership in distributions
     */
-    std::set<std::string> getVTypeDistributionMembership(const std::string& id) const;
+    const std::set<std::string> getVTypeDistributionMembership(const std::string& id) const;
 
     /// @}
 
-    /// @brief Adds a vehicle to the list of waiting vehiclse to a given edge
-    void addWaiting(const MSEdge* const edge, SUMOVehicle* vehicle);
-
-    /// @brief Removes a vehicle from the list of waiting vehicles to a given edge
-    void removeWaiting(const MSEdge* const edge, const SUMOVehicle* vehicle);
-
-    /* @brief returns a vehicle of the given lines that is waiting for a for a person or a container at this edge at the given positions
-     * @param[in] edge The edge at which the vehicle is positioned.
-     * @param[in] lines The set of lines from which at least one must correspond to the line of the vehicle
-     * @param[in] position The vehicle shall be positioned in the interval [position - t, position + t], where t is some tolerance
-     * @param[in] ridingID The id of the person or container that wants to ride
-     */
-    SUMOVehicle* getWaitingVehicle(const MSEdge* const edge, const std::set<std::string>& lines, const double position, const std::string ridingID);
-
     /** @brief increases the count of vehicles waiting for a transport to allow recognition of person / container related deadlocks
      */
     void registerOneWaiting(const bool isPerson) {
@@ -507,6 +510,9 @@ private:
      */
     bool checkVType(const std::string& id);
 
+    /// @brief whether the given vehicle is scheduled for removal
+    bool isPendingRemoval(SUMOVehicle* veh);
+
 protected:
     /// @name Vehicle statistics (always accessible)
     /// @{
@@ -590,9 +596,6 @@ private:
     /// @brief Whether the default bicycle type was already used or can still be replaced
     bool myDefaultBikeTypeMayBeDeleted;
 
-    /// the lists of waiting vehicles to a given edge
-    std::map<const MSEdge* const, std::vector<SUMOVehicle*> > myWaiting;
-
     /// the number of vehicles wainting for persons contained in myWaiting which can only continue by being triggered
     int myWaitingForPerson;
 
@@ -611,6 +614,13 @@ private:
     /// @brief List of vehicles which belong to public transport
     std::vector<SUMOVehicle*> myPTVehicles;
 
+    /// @brief List of vehicles which are going to be removed
+#ifdef HAVE_FOX
+    FXSynchQue<SUMOVehicle*, std::vector<SUMOVehicle*> > myPendingRemovals;
+#else
+    std::vector<SUMOVehicle*> myPendingRemovals;
+#endif
+
 private:
     /// @brief invalidated copy constructor
     MSVehicleControl(const MSVehicleControl& s);
