@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id$
 ///
 // The car-following model and parameter
 /****************************************************************************/
@@ -515,6 +514,11 @@ public:
      */
     static MSVehicleType* build(SUMOVTypeParameter& from);
 
+    /// @brief   Accessor function for parameter equivalent returning entry time for a specific manoeuver angle
+    SUMOTime getEntryManoeuvreTime(const int angle) const;
+    /// @brief   Accessor function for parameter equivalent returning exit time for a specific manoeuver angle
+    SUMOTime getExitManoeuvreTime(const int angle) const;
+
 
     /** @brief Duplicates the microsim vehicle type giving the newly created type the given id,
      *         marking it as vehicle specific
@@ -541,7 +545,15 @@ public:
      * @return Whether this vehicle type is based on a different one, and belongs to one vehicle only
      */
     bool isVehicleSpecific() const {
-        return myOriginalType != 0;
+        return myOriginalType != nullptr;
+    }
+
+
+    /** @brief Returns the id of the original vehicle type if this is a vehicle specific type, the id otherwise
+     * @return the original vehicle type id
+     */
+    const std::string& getOriginalID() const {
+        return myOriginalType != nullptr ? myOriginalType->getID() : getID();
     }
 
 
@@ -555,6 +567,10 @@ public:
      */
     void check();
 
+protected:
+    /// @brief init Rail Visualization Parameters
+    void initRailVisualizationParameters();
+
 private:
     /// @brief the parameter container
     SUMOVTypeParameter myParameter;
@@ -581,11 +597,10 @@ private:
 
 private:
     /// @brief Invalidated copy constructor
-    MSVehicleType(const MSVehicleType&);
+    MSVehicleType(const MSVehicleType&) = delete;
 
     /// @brief Invalidated assignment operator
-    MSVehicleType& operator=(const MSVehicleType&);
-
+    MSVehicleType& operator=(const MSVehicleType&) = delete;
 };
 
 
