@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    15 Feb 2004
-/// @version $Id$
 ///
 // Writes e2 state of a link for the time the link has yellow/red
 /****************************************************************************/
@@ -50,7 +49,7 @@ public:
      */
     Command_SaveTLCoupledLaneDet(MSTLLogicControl::TLSLogicVariants& tlls,
                                  MSDetectorFileOutput* dtf, SUMOTime begin, OutputDevice& device,
-                                 MSLink* link);
+                                 const MSLink* link);
 
 
     /// @brief Destructor
@@ -72,7 +71,7 @@ public:
 
 private:
     /// @brief The link to observe
-    MSLink* myLink;
+    const MSLink* myLink;
 
     /// @brief The state the link had the last time
     LinkState myLastState;
