@@ -1,13 +1,11 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSCFModel_Krauss.h
 /// @author  Tobias Mayer
@@ -15,7 +13,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 28 Jul 2009
-/// @version $Id$
 ///
 // Krauss car-following model, with acceleration decrease and faster start
 /****************************************************************************/
@@ -25,11 +22,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "MSCFModel_KraussOrig1.h"
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -45,30 +38,18 @@
 class MSCFModel_Krauss : public MSCFModel_KraussOrig1 {
 public:
     /** @brief Constructor
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] emergencyDecel The maximum emergency deceleration
-     * @param[in] apparentDecel The deceleration as expected by others
-     * @param[in] dawdle The driver imperfection
-     * @param[in] headwayTime The driver's desired headway
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_Krauss(const MSVehicleType* vtype, double accel, double decel,
-                     double emergencyDecel, double apparentDecel, double dawdle, double headwayTime);
+    MSCFModel_Krauss(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
     ~MSCFModel_Krauss();
 
-
     /// @name Implementations of the MSCFModel interface
     /// @{
-
-    /** @brief Applies interaction with stops and lane changing model influences
-     * @param[in] veh The ego vehicle
-     * @param[in] vPos The possible velocity
-     * @return The velocity after applying interactions with stops and lane change model influences
-     */
-    virtual double moveHelper(MSVehicle* const veh, double vPos) const;
+    /// @brief apply dawdling
+    double patchSpeedBeforeLC(const MSVehicle* veh, double vMin, double vMax) const;
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
      * this uses the maximumSafeStopSpeed
@@ -86,9 +67,10 @@ public:
      * @param[in] speed The vehicle's speed
      * @param[in] gap2pred The (netto) distance to the LEADER
      * @param[in] predSpeed The speed of LEADER
+     * @param[in] pred The leading vehicle (LEADER)
      * @return EGO's safe speed
      */
-    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel) const;
+    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Returns the model's name
@@ -115,7 +97,7 @@ protected:
      * @param[in] sigma The sigma value to use
      * @return The speed after dawdling
      */
-    double dawdle2(double speed, double sigma) const;
+    double dawdle2(double speed, double sigma, std::mt19937* rng) const;
 
 };
 
