@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSCFModel_KraussX.h
 /// @author  Jakob Erdmann
 /// @date    27 Feb 2017
-/// @version $Id$
 ///
 // Experimental extensions to the Krauss car-following model
 /****************************************************************************/
@@ -78,7 +77,7 @@ private:
      * @return The speed after dawdling
      *
      */
-    double dawdleX(double vOld, double vMin, double vMax) const;
+    double dawdleX(double vOld, double vMin, double vMax, std::mt19937* rng) const;
 
     /// @brief extension parameter nr1
     double myTmp1;
