@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2012-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2012-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Gregor L\"ammel
 /// @author  Leander Flamm
 /// @date    Tue, 08 Feb 2017
-/// @version $Id$
 ///
 // <description missing>
 /****************************************************************************/
@@ -141,15 +140,18 @@ double MSCFModel_Rail::maxNextSpeed(double speed, const MSVehicle* const veh) co
 
 double MSCFModel_Rail::minNextSpeed(double speed, const MSVehicle* const veh) const {
 
-    double slope = veh->getSlope();
-    double gr = myTrainParams.weight * G * sin(DEG2RAD(slope)); //kN
-    double res = getInterpolatedValueFromLookUpMap(speed, &(myTrainParams.resistance)); // kN
-    double totalRes = res + gr; //kN
-
-
-    double a = myTrainParams.decl + totalRes / myTrainParams.rotWeight;
-
-    return speed - a * DELTA_T / 1000.;
+    const double slope = veh->getSlope();
+    const double gr = myTrainParams.weight * G * sin(DEG2RAD(slope)); //kN
+    const double res = getInterpolatedValueFromLookUpMap(speed, &(myTrainParams.resistance)); // kN
+    const double totalRes = res + gr; //kN
+    const double a = myTrainParams.decl + totalRes / myTrainParams.rotWeight;
+    const double vMin = speed - a * DELTA_T / 1000.;
+    if (MSGlobals::gSemiImplicitEulerUpdate) {
+        return MAX2(vMin, 0.);
+    } else {
+        // NOTE: ballistic update allows for negative speeds to indicate a stop within the next timestep
+        return vMin;
+    }
 
 }
 
