@@ -1,19 +1,16 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSCFModel_Wiedemann.h
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2011
-/// @version $Id$
 ///
 // The psycho-physical model of Wiedemann
 /****************************************************************************/
@@ -23,11 +20,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include "MSCFModel.h"
 #include <microsim/MSLane.h>
@@ -48,20 +41,9 @@ class MSCFModel_Wiedemann : public MSCFModel {
 public:
 
     /** @brief Constructor
-     *
-     * @param[in] security The security parameter in [0,1] (dimensionless)
-     * @param[in] estimation The estimation capability parameter in [0,1] (dimensionless)
-     * @param[in] accel The maximum acceleration
-     * @param[in] decel The maximum deceleration
-     * @param[in] emergencyDecel The maximum emergency deceleration
-     * @param[in] apparentDecel The deceleration as expected by others
-     *
-     * @note other parameters of the wiedemann model:
-     * - speed: included in MSVehicleType
+     *  @param[in] vtype the type for which this model is built and also the parameter object to configure this model
      */
-    MSCFModel_Wiedemann(const MSVehicleType* vtype,
-                        double accel, double decel, double emergencyDecel, double apparentDecel,
-                        double security, double estimation);
+    MSCFModel_Wiedemann(const MSVehicleType* vtype);
 
 
     /// @brief Destructor
@@ -76,7 +58,7 @@ public:
      * @param[in] vPos The possible velocity
      * @return The velocity after applying interactions with stops and lane change model influences
      */
-    double moveHelper(MSVehicle* const veh, double vPos) const;
+    double finalizeSpeed(MSVehicle* const veh, double vPos) const;
 
 
     /** @brief Computes the vehicle's safe speed (no dawdling)
@@ -87,7 +69,7 @@ public:
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel) const;
+    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
@@ -109,8 +91,16 @@ public:
      * @todo evaluate signature
      * @see MSCFModel::interactionGap
      */
-    double interactionGap(const MSVehicle* const , double vL) const;
+    double interactionGap(const MSVehicle* const, double vL) const;
 
+    /** @brief Returns the minimum gap to reserve if the leader is braking at maximum (>=0)
+     * @param[in] veh The vehicle itself, for obtaining other values
+     * @param[in] pred The leader vehicle, for obtaining other values
+     * @param[in] speed EGO's speed
+     * @param[in] leaderSpeed LEADER's speed
+     * @param[in] leaderMaxDecel LEADER's max. deceleration rate
+     */
+    double getSecureGap(const MSVehicle* const veh, const MSVehicle* const pred, const double speed, const double leaderSpeed, const double leaderMaxDecel) const;
 
     /** @brief Returns the model's name
      * @return The model's name
@@ -153,7 +143,7 @@ private:
     /// @{
     double fullspeed(double v, double vpref, double dx, double bx) const; // also 'WUNSCH'
     double following(double sign) const; // also 'FOLGEN'
-    double approaching(double dv, double dx, double bx) const;  // also 'BREMSBX'
+    double approaching(double dv, double dx, double abx) const;  // also 'BREMSBX'
     double emergency(double dv, double dx) const; // also 'BREMSAX'
     /// @}
 
@@ -167,23 +157,19 @@ private:
     /// @brief The driver's estimation parameter // also 'ZF2'
     const double myEstimation;
 
-    /// @brief front-bumper to front-bumper distance
+    /// @brief the minimum front-bumper to front-bumper distance when standing
     const double myAX;
 
     /// @brief perception threshold modifier
     const double myCX;
 
-    /// @brief The vehicle's minimum acceleration [m/s^2]
+    /// @brief The vehicle's minimum acceleration [m/s^2] // also b_null
     const double myMinAccel;
 
     /// @brief free-flow distance in m
     static const double D_MAX;
     /// @}
 
-    /// @brief vsafe from krauss since Wiedemann is deficient at approaching
-    // standing obstacles (see MSCFModel_Krauss::_vsafe)
-    double krauss_vsafe(double gap, double predSpeed) const;
-
 private:
     /// @brief Invalidated assignment operator
     MSCFModel_Wiedemann& operator=(const MSCFModel_Wiedemann& s);
