@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2007-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2007-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 04 Dec 2007
-/// @version $Id$
 ///
 // Abstract in-vehicle device
 /****************************************************************************/
@@ -31,6 +30,7 @@
 #include <set>
 #include <random>
 #include <microsim/MSMoveReminder.h>
+#include <microsim/MSVehicleType.h>
 #include <microsim/MSVehicleControl.h>
 #include <utils/common/Named.h>
 #include <utils/common/StringUtils.h>
@@ -46,7 +46,7 @@ class SUMOVehicle;
 class MSTransportable;
 class SUMOSAXAttributes;
 class MSVehicleDevice;
-class MSPersonDevice;
+class MSTransportableDevice;
 
 
 // ===========================================================================
@@ -85,7 +85,7 @@ public:
     * @param[in] p The person for which a device may be built
     * @param[filled] into The vector to store the built device in
     */
-    static void buildPersonDevices(MSTransportable& p, std::vector<MSPersonDevice*>& into);
+    static void buildTransportableDevices(MSTransportable& p, std::vector<MSTransportableDevice*>& into);
 
     static std::mt19937* getEquipmentRNG() {
         return &myEquipmentRNG;
@@ -94,17 +94,12 @@ public:
     /// @brief return the name for this type of device
     virtual const std::string deviceName() const = 0;
 
-    /** @brief Determines whether a transportable should get a certain device
-     **/
-    static bool equippedByParameter(const MSTransportable* t, const std::string& deviceName, bool outputOptionSet);
-
     /// @brief perform cleanup for all devices
     static void cleanupAll();
 
 public:
     /** @brief Constructor
      *
-     * @param[in] holder The vehicle that holds this device
      * @param[in] id The ID of the device
      */
     MSDevice(const std::string& id) : Named(id) {
@@ -165,7 +160,7 @@ protected:
      * @param[in] optionsTopic The options topic into which the options shall be added
      * @param[filled] oc The options container to add the options to
      */
-    static void insertDefaultAssignmentOptions(const std::string& deviceName, const std::string& optionsTopic, OptionsCont& oc, const bool isPerson=false);
+    static void insertDefaultAssignmentOptions(const std::string& deviceName, const std::string& optionsTopic, OptionsCont& oc, const bool isPerson = false);
 
 
     /** @brief Determines whether a vehicle should get a certain device
@@ -174,7 +169,7 @@ protected:
      * @param[in] deviceName The name of the device type
      * @param[in] v The vehicle to determine whether it shall be equipped or not
      */
-    template<class DEVICEHOLDER> 
+    template<class DEVICEHOLDER>
     static bool equippedByDefaultAssignmentOptions(const OptionsCont& oc, const std::string& deviceName, DEVICEHOLDER& v, bool outputOptionSet, const bool isPerson = false);
     /// @}
 
@@ -242,6 +237,11 @@ MSDevice::equippedByDefaultAssignmentOptions(const OptionsCont& oc, const std::s
         parameterGiven = true;
         haveByParameter = StringUtils::toBool(v.getVehicleType().getParameter().getParameter(key, "false"));
     }
+    //std::cout << " deviceName=" << deviceName << " holder=" << v.getID()
+    //    << " nameGiven=" << nameGiven << " haveByName=" << haveByName
+    //    << " parameterGiven=" << parameterGiven << " haveByParameter=" << haveByParameter
+    //    << " numberGiven=" << numberGiven << " haveByNumber=" << haveByNumber
+    //    << " outputOptionSet=" << outputOptionSet << "\n";
     if (haveByName) {
         return true;
     } else if (parameterGiven) {
