@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2013-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2013-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Tamas Kurczveil
 /// @author  Pablo Alvarez Lopez
 /// @date    20-12-13
-/// @version $Id$
 ///
 // The Battery parameters for the vehicle
 /****************************************************************************/
@@ -77,7 +76,7 @@ public:
     *
     * @return True (always).
     */
-    bool notifyMove(SUMOVehicle& veh, double oldPos,  double newPos, double newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos,  double newPos, double newSpeed);
     /// @}
 
     /// @brief return the name for this type of device
@@ -162,6 +161,11 @@ public:
     /// @brief Increase myVehicleStopped
     void increaseVehicleStoppedTimer();
 
+    /// @brief retrieve parameters for the energy consumption model
+    const std::map<int, double>& getEnergyParams() const {
+        return myParam;
+    }
+
 protected:
     /// @brief Parameter, The actual vehicles's Battery Capacity in kWh, [myActualBatteryCapacity <= myMaximumBatteryCapacity]
     double myActualBatteryCapacity;
