@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Fri, 30.01.2009
-/// @version $Id$
 ///
 // A device which collects vehicular emissions
 /****************************************************************************/
@@ -22,13 +21,13 @@
 // ===========================================================================
 #include <config.h>
 
-#include "MSDevice_Emissions.h"
 #include <microsim/MSNet.h>
 #include <microsim/MSLane.h>
 #include <microsim/MSVehicleControl.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/emissions/PollutantsInterface.h>
 #include <utils/iodevices/OutputDevice.h>
+#include "MSDevice_Emissions.h"
 
 
 // ===========================================================================
@@ -66,15 +65,35 @@ MSDevice_Emissions::~MSDevice_Emissions() {
 
 
 bool
-MSDevice_Emissions::notifyMove(SUMOVehicle& veh, double /*oldPos*/, double /*newPos*/, double newSpeed) {
+MSDevice_Emissions::notifyMove(SUMOTrafficObject& veh, double /*oldPos*/, double /*newPos*/, double newSpeed) {
     const SUMOEmissionClass c = veh.getVehicleType().getEmissionClass();
     const double a = veh.getAcceleration();
     const double slope = veh.getSlope();
-    myEmissions.addScaled(PollutantsInterface::computeAll(c, newSpeed, a, slope), TS);
+    myEmissions.addScaled(PollutantsInterface::computeAll(c, newSpeed, a, slope,
+                          static_cast<const SUMOVehicle&>(veh).getEmissionParameters()), TS);
     return true;
 }
 
 
+void
+MSDevice_Emissions::notifyMoveInternal(const SUMOTrafficObject& veh,
+                                       const double /* frontOnLane */,
+                                       const double timeOnLane,
+                                       const double /* meanSpeedFrontOnLane */,
+                                       const double meanSpeedVehicleOnLane,
+                                       const double /* travelledDistanceFrontOnLane */,
+                                       const double /* travelledDistanceVehicleOnLane */,
+                                       const double /* meanLengthOnLane */) {
+
+    // called by meso (see MSMeanData_Emissions::MSLaneMeanDataValues::notifyMoveInternal)
+    const double a = veh.getAcceleration();
+    myEmissions.addScaled(PollutantsInterface::computeAll(veh.getVehicleType().getEmissionClass(),
+                          meanSpeedVehicleOnLane, a, veh.getSlope(),
+                          static_cast<const SUMOVehicle&>(veh).getEmissionParameters()), timeOnLane);
+}
+
+
+
 void
 MSDevice_Emissions::generateOutput() const {
     if (OptionsCont::getOptions().isSet("tripinfo-output")) {
@@ -92,6 +111,5 @@ MSDevice_Emissions::generateOutput() const {
 }
 
 
-
 /****************************************************************************/
 
