@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2007-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2007-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 04 Dec 2007
-/// @version $Id$
 ///
 // A device that performs vehicle rerouting based on current edge speeds
 /****************************************************************************/
@@ -48,7 +47,7 @@ class MSLane;
  * @brief A device that performs vehicle rerouting based on current edge speeds
  *
  * The routing-device system consists of in-vehicle devices that perform the routing
- *  and simulation-wide static methods for collecting edge weights and 
+ *  and simulation-wide static methods for collecting edge weights and
  *  parallelizing in MSRoutingEngine.
  *
  * A device is assigned to a vehicle using the common explicit/probability - procedure.
@@ -115,7 +114,7 @@ public:
      * @see MSEventHandler
      * @see WrappingCommand
      */
-    bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
     /// @}
 
     /// @brief return the name for this type of device
@@ -151,6 +150,10 @@ public:
         mySkipRouting = currentTime;
     }
 
+    SUMOTime getPeriod() const {
+        return myPeriod;
+    }
+
     /// @brief try to retrieve the given parameter from this device. Throw exception for unsupported key
     std::string getParameter(const std::string& key) const;
 
