@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Melanie Weber
 /// @author  Andreas Kendziorra
 /// @date    Fri, 30.01.2009
-/// @version $Id$
 ///
 // A device which is used to keep track of persons and containers riding with a vehicle
 /****************************************************************************/
@@ -71,7 +70,7 @@ public:
      * @see MSMoveReminder
      * @see MSMoveReminder::notifyMove
      */
-    bool notifyMove(SUMOVehicle& veh, double oldPos, double newPos, double newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double newSpeed);
 
 
     /** @brief Adds passengers on vehicle insertion
@@ -82,7 +81,7 @@ public:
      * @see MSMoveReminder::notifyEnter
      * @see MSMoveReminder::Notification
      */
-    bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
     /** @brief Passengers leaving on arrival
@@ -94,7 +93,7 @@ public:
      *
      * @return True if it did not leave the net.
      */
-    bool notifyLeave(SUMOVehicle& veh, double lastPos,
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos,
                      MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
     /// @}
 
@@ -140,7 +139,7 @@ protected:
     /** @brief Internal notification about the vehicle moves, see MSMoveReminder::notifyMoveInternal()
      *
      */
-    void notifyMoveInternal(const SUMOVehicle& veh,
+    void notifyMoveInternal(const SUMOTrafficObject& veh,
                             const double frontOnLane,
                             const double timeOnLane,
                             const double meanSpeedFrontOnLane,
