@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2013-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2013-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Jakob Erdmann
 /// @author  Leonhard Luecken
 /// @date    Tue, 06.10.2015
-/// @version $Id$
 ///
 // A lane change model for heterogeneous traffic (based on sub-lanes)
 /****************************************************************************/
@@ -229,6 +228,8 @@ protected:
     /// @brief decide in which direction to move in case both directions are desirable
     StateAndDist decideDirection(StateAndDist sd1, StateAndDist sd2) const;
 
+    /// @brief return the most important change reason
+    static int lowest_bit(int changeReason);
 
 protected:
 
@@ -337,6 +338,12 @@ protected:
     /// @brief compute speedGain when moving by the given amount
     double computeSpeedGain(double latDistSublane, double defaultNextSpeed) const;
 
+    /// @brief get lateral position of this vehicle
+    double getPosLat();
+
+    /// @brief get lateral drift for the current step
+    double getLateralDrift();
+
 protected:
     /// @brief a value for tracking the probability that a change to the right is beneficial
     double mySpeedGainProbabilityRight;
@@ -403,6 +410,8 @@ protected:
     double myLookaheadLeft;
     // @brief the factor by which the speedGain-threshold for the leftdiffers from the threshold for the right
     double mySpeedGainRight;
+    // @brief lane discipline factor
+    double myLaneDiscipline;
     //@}
 
     /// @name derived parameters
@@ -413,6 +422,9 @@ protected:
     double myChangeProbThresholdLeft;
     // @brief threshold value for accepting speed loss to achieve desired sublane alignment
     double mySpeedLossProbThreshold;
+
+    // @brief state of lane keeping imperfection
+    double mySigmaState;
     //@}
 
 };
