@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -16,7 +16,6 @@
 /// @author  Jakob Erdmann
 /// @author  Leonhard Luecken
 /// @date    Mon Feb 03 2014 14:13 CET
-/// @version $Id$
 ///
 // An areal detector covering to a sequence of consecutive lanes
 /****************************************************************************/
@@ -36,13 +35,15 @@
 #include <microsim/MSMoveReminder.h>
 #include <microsim/output/MSDetectorFileOutput.h>
 #include <utils/common/UtilExceptions.h>
-#include <utils/vehicle/SUMOVehicle.h>
 #include <cassert>
 
+
 // ===========================================================================
 // class declarations
 // ===========================================================================
 class OutputDevice;
+class SUMOVehicle;
+class SUMOTrafficObject;
 
 
 // ===========================================================================
@@ -288,7 +289,7 @@ public:
      * @see MSMoveReminder
      * @see MSMoveReminder::notifyMove
      */
-    virtual bool notifyMove(SUMOVehicle& veh, double oldPos, double newPos,
+    virtual bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos,
                             double newSpeed);
 
 
@@ -303,7 +304,7 @@ public:
      * @param[in] isLaneChange whether the vehicle changed from the lane
      * @see MSMoveReminder::notifyLeave
      */
-    virtual bool notifyLeave(SUMOVehicle& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    virtual bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
     /** @brief Adds the vehicle to known vehicles if not beyond the dector
@@ -318,7 +319,7 @@ public:
      * @see MSMoveReminder::notifyEnter
      * @see MSMoveReminder::Notification
      */
-    virtual bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane);
+    virtual bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane);
     /// @}
 
 
@@ -508,7 +509,7 @@ public:
     /** @brief Returns an estimate of the number of vehicles currently on the detector */
     int getEstimatedCurrentVehicleNumber(double speedThreshold) const;
 
-    /** @brief Returns an estimate of the lenght of the queue of vehicles currently stopped on the detector */
+    /** @brief Returns an estimate of the length of the queue of vehicles currently stopped on the detector */
     double getEstimateQueueLength() const;
     /// @}
 
