@@ -1,13 +1,11 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSE3Collector.cpp
 /// @author  Christian Roessel
@@ -16,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Tue Dec 02 2003 22:17 CET
-/// @version $Id$
 ///
 // A detector of vehicles passing an area between entry/exit points
 /****************************************************************************/
@@ -24,11 +21,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <algorithm>
 
@@ -36,6 +29,17 @@
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicle.h>
 
+//#define DEBUG_E3_NOTIFY_MOVE
+//#define DEBUG_E3_NOTIFY_ENTER
+//#define DEBUG_E3_NOTIFY_LEAVE
+//#define DEBUG_E3_DETECTORUPDATE
+
+//#define DEBUG_COND(obj) ((obj.getID() == ""))
+//#define DEBUG_COND_VEH(veh) ((veh).getID() == "")
+//#define DEBUG_COND_VEH(veh) ((veh).isSelected())
+//#define DEBUG_COND(collector) (true)
+//#define DEBUG_COND_VEH(veh) (true)
+
 
 // ===========================================================================
 // method definitions
@@ -46,15 +50,69 @@
 MSE3Collector::MSE3EntryReminder::MSE3EntryReminder(
     const MSCrossSection& crossSection, MSE3Collector& collector) :
     MSMoveReminder(collector.getID() + "_entry", crossSection.myLane),
-    myCollector(collector), myPosition(crossSection.myPosition) {}
+    myCollector(collector), myPosition(crossSection.myPosition) {
+}
 
 
 bool
-MSE3Collector::MSE3EntryReminder::notifyMove(SUMOVehicle& veh, double oldPos,
+MSE3Collector::MSE3EntryReminder::notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane) {
+#ifdef DEBUG_E3_NOTIFY_ENTER
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3EntryReminder::notifyEnter() (" << getDescription() << "on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " enteredLane=" << enteredLane->getID()
+                  << " reason=" << reason
+                  << "\n";
+    }
+#endif
+    if (reason != NOTIFICATION_JUNCTION) {
+        const double posOnLane = veh.getBackPositionOnLane(enteredLane) + veh.getVehicleType().getLength();
+        if (myLane == enteredLane && posOnLane > myPosition) {
+#ifdef HAVE_FOX
+            FXConditionalLock lock(myCollector.myContainerMutex, MSGlobals::gNumSimThreads > 1);
+#endif
+            const auto& itVeh = myCollector.myEnteredContainer.find(&veh);
+            if (itVeh == myCollector.myEnteredContainer.end() ||
+                    itVeh->second.entryReminder != this) {
+#ifdef DEBUG_E3_NOTIFY_ENTER
+                if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+                    std::cout << "  assume already known\n";
+                }
+#endif
+                // if the vehicle changes into a covered section we assume it was already registered on another lane
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
+
+bool
+MSE3Collector::MSE3EntryReminder::notifyMove(SUMOTrafficObject& veh, double oldPos,
         double newPos, double newSpeed) {
+#ifdef DEBUG_E3_NOTIFY_MOVE
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3EntryReminder::notifyMove() (" << getDescription() << "on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " entered. oldPos=" << oldPos << " newPos=" << newPos << " newSpeed=" << newSpeed
+                  << " myPosition=" << myPosition
+                  << "\n";
+    }
+#endif
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myCollector.myContainerMutex, MSGlobals::gNumSimThreads > 1);
+#endif
     if (myCollector.myEnteredContainer.find(&veh) == myCollector.myEnteredContainer.end() && newPos > myPosition) {
         if (oldPos > myPosition) {
             // was behind the detector already in the last step
+#ifdef DEBUG_E3_NOTIFY_MOVE
+            if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+                std::cout << "    already behind\n";
+            }
+#endif
             return false;
         } else {
             // entered in this step
@@ -63,7 +121,12 @@ MSE3Collector::MSE3EntryReminder::notifyMove(SUMOVehicle& veh, double oldPos,
             assert(!MSGlobals::gSemiImplicitEulerUpdate || newSpeed != 0); // how could it move across the detector otherwise
             const double timeBeforeEnter = MSCFModel::passingTime(oldPos, myPosition, newPos, oldSpeed, newSpeed);
             const double fractionTimeOnDet = TS - timeBeforeEnter;
-            myCollector.enter(veh, entryTime - fractionTimeOnDet, fractionTimeOnDet);
+            myCollector.enter(veh, entryTime - fractionTimeOnDet, fractionTimeOnDet, this);
+#ifdef DEBUG_E3_NOTIFY_MOVE
+            if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+                std::cout << "    enter\n";
+            }
+#endif
         }
     }
     return true;
@@ -71,10 +134,23 @@ MSE3Collector::MSE3EntryReminder::notifyMove(SUMOVehicle& veh, double oldPos,
 
 
 bool
-MSE3Collector::MSE3EntryReminder::notifyLeave(SUMOVehicle& veh, double, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
+MSE3Collector::MSE3EntryReminder::notifyLeave(SUMOTrafficObject& veh, double, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
+#ifdef DEBUG_E3_NOTIFY_LEAVE
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3EntryReminder::notifyLeave() (" << getDescription() << "on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " reason=" << reason
+                  << "\n";
+    }
+#endif
     if (reason >= MSMoveReminder::NOTIFICATION_ARRIVED) {
-        WRITE_WARNING("Vehicle '" + veh.getID() + "' arrived inside " + toString(SUMO_TAG_E3DETECTOR) + " '" + myCollector.getID() + "'.");
-        myCollector.myEnteredContainer.erase(&veh);
+#ifdef HAVE_FOX
+        FXConditionalLock lock(myCollector.myContainerMutex, MSGlobals::gNumSimThreads > 1);
+#endif
+        if (myCollector.myEnteredContainer.erase(&veh) > 0) {
+            WRITE_WARNING("Vehicle '" + veh.getID() + "' arrived inside " + toString(SUMO_TAG_E3DETECTOR) + " '" + myCollector.getID() + "'.");
+        }
         return false;
     }
     return true;
@@ -91,12 +167,55 @@ MSE3Collector::MSE3LeaveReminder::MSE3LeaveReminder(
 
 
 bool
-MSE3Collector::MSE3LeaveReminder::notifyMove(SUMOVehicle& veh, double oldPos,
+MSE3Collector::MSE3LeaveReminder::notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane) {
+#ifdef DEBUG_E3_NOTIFY_ENTER
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3LeaveReminder::notifyEnter() (" << getDescription() << "on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " enteredLane=" << enteredLane->getID()
+                  << " reason=" << reason
+                  << "\n";
+    }
+#endif
+    // this method does not access containers, so no locking here
+    if (reason != NOTIFICATION_JUNCTION) {
+        const double backPosOnLane = veh.getBackPositionOnLane(enteredLane);
+        if (backPosOnLane > myPosition) {
+            // if the vehicle changes into a covered section we assume it was already registered on another lane
+            // however, if it is not fully past the detector we still need to track it
+#ifdef DEBUG_E3_NOTIFY_ENTER
+            if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+                std::cout << "  assume already known\n";
+            }
+#endif
+            return false;
+        }
+    }
+    return true;
+}
+
+
+bool
+MSE3Collector::MSE3LeaveReminder::notifyMove(SUMOTrafficObject& veh, double oldPos,
         double newPos, double newSpeed) {
+#ifdef DEBUG_E3_NOTIFY_MOVE
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3LeaveReminder::notifyMove() (" << getDescription() << " on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " entered. oldPos=" << oldPos << " newPos=" << newPos << " newSpeed=" << newSpeed
+                  << " myPosition=" << myPosition
+                  << "\n";
+    }
+#endif
     if (newPos < myPosition) {
         // crossSection not yet reached
         return true;
     }
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myCollector.myContainerMutex, MSGlobals::gNumSimThreads > 1);
+#endif
     const double oldSpeed = veh.getPreviousSpeed();
     if (oldPos < myPosition) {
         assert(!MSGlobals::gSemiImplicitEulerUpdate || newSpeed != 0); // how could it move across the detector otherwise
@@ -104,6 +223,11 @@ MSE3Collector::MSE3LeaveReminder::notifyMove(SUMOVehicle& veh, double oldPos,
 //        const double leaveTimeFront = SIMTIME - TS + (myPosition - oldPos) / newSpeed;
         const double leaveTimeFront = SIMTIME - TS + timeBeforeLeave;
         myCollector.leaveFront(veh, leaveTimeFront);
+#ifdef DEBUG_E3_NOTIFY_MOVE
+        if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+            std::cout << "    leaveFront\n";
+        }
+#endif
     }
     const double backPos = newPos - veh.getVehicleType().getLength();
     if (backPos < myPosition) {
@@ -116,20 +240,49 @@ MSE3Collector::MSE3LeaveReminder::notifyMove(SUMOVehicle& veh, double oldPos,
     assert(!MSGlobals::gSemiImplicitEulerUpdate || newSpeed != 0); // how could it move across the detector otherwise
     const double timeBeforeLeave = MSCFModel::passingTime(oldBackPos, myPosition, backPos, oldSpeed, newSpeed);
     myCollector.leave(veh, leaveStep - TS + timeBeforeLeave, timeBeforeLeave);
+#ifdef DEBUG_E3_NOTIFY_MOVE
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << "    leave\n";
+    }
+#endif
     return false;
 }
 
 
 bool
-MSE3Collector::MSE3LeaveReminder::notifyLeave(SUMOVehicle&  veh , double /* lastPos */, MSMoveReminder::Notification reason, const MSLane* /* enteredLane */) {
-    if (reason == MSMoveReminder::NOTIFICATION_LANE_CHANGE) {
+MSE3Collector::MSE3LeaveReminder::notifyLeave(SUMOTrafficObject&  veh, double /* lastPos */, MSMoveReminder::Notification reason, const MSLane* enteredLane) {
+#ifdef DEBUG_E3_NOTIFY_LEAVE
+    if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+        std::cout << SIMTIME
+                  << " MSE3LeaveReminder::notifyLeave() (" << getDescription() << "on lane '" << myLane->getID() << "')"
+                  << " vehicle '" << veh.getID() << "'"
+                  << " reason=" << reason
+                  << "\n";
+    }
+#endif
+    if (reason == MSMoveReminder::NOTIFICATION_LANE_CHANGE && &enteredLane->getEdge() == &myLane->getEdge()) {
+        // keep the detector when changing while still on the exit detector but already on a new lane (#4803)
+#ifdef DEBUG_E3_NOTIFY_LEAVE
+        if (DEBUG_COND(myCollector) && DEBUG_COND_VEH(veh)) {
+            std::cout << "  remove reminder, keep in container\n";
+        }
+#endif
         return false;
     }
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myCollector.myContainerMutex, MSGlobals::gNumSimThreads > 1);
+#endif
     if (reason == MSMoveReminder::NOTIFICATION_TELEPORT) {
         WRITE_WARNING("Vehicle '" + veh.getID() + "' teleported from " + toString(SUMO_TAG_E3DETECTOR) + " '" + myCollector.getID() + "'.");
         myCollector.myEnteredContainer.erase(&veh);
         return false;
     }
+    if (reason >= MSMoveReminder::NOTIFICATION_ARRIVED) {
+        if (myCollector.myEnteredContainer.erase(&veh) > 0) {
+            WRITE_WARNING("Vehicle '" + veh.getID() + "' arrived inside " + toString(SUMO_TAG_E3DETECTOR) + " '" + myCollector.getID() + "'.");
+        }
+        return false;
+    }
     return true;
 }
 
@@ -141,10 +294,12 @@ MSE3Collector::MSE3Collector(const std::string& id,
                              const CrossSectionVector& exits,
                              double haltingSpeedThreshold,
                              SUMOTime haltingTimeThreshold,
-                             const std::string& vTypes)
-    : MSDetectorFileOutput(id, vTypes), myEntries(entries), myExits(exits),
-      myHaltingTimeThreshold(haltingTimeThreshold), myHaltingSpeedThreshold(haltingSpeedThreshold),
-      myCurrentMeanSpeed(0), myCurrentHaltingsNumber(0), myLastResetTime(-1) {
+                             const std::string& vTypes,
+                             bool openEntry) :
+    MSDetectorFileOutput(id, vTypes), myEntries(entries), myExits(exits),
+    myHaltingTimeThreshold(haltingTimeThreshold), myHaltingSpeedThreshold(haltingSpeedThreshold),
+    myCurrentMeanSpeed(0), myCurrentHaltingsNumber(0), myLastResetTime(-1),
+    myOpenEntry(openEntry) {
     // Set MoveReminders to entries and exits
     for (CrossSectionVectorConstIt crossSec1 = entries.begin(); crossSec1 != entries.end(); ++crossSec1) {
         myEntryReminders.push_back(new MSE3EntryReminder(*crossSec1, *this));
@@ -174,7 +329,10 @@ MSE3Collector::reset() {
 
 
 void
-MSE3Collector::enter(const SUMOVehicle& veh, const double entryTimestep, const double fractionTimeOnDet) {
+MSE3Collector::enter(const SUMOTrafficObject& veh, const double entryTimestep, const double fractionTimeOnDet, MSE3EntryReminder* entryReminder) {
+    if (!vehicleApplies(veh)) {
+        return;
+    }
     if (myEnteredContainer.find(&veh) != myEnteredContainer.end()) {
         WRITE_WARNING("Vehicle '" + veh.getID() + "' reentered " + toString(SUMO_TAG_E3DETECTOR) + " '" + getID() + "'.");
         return;
@@ -185,7 +343,7 @@ MSE3Collector::enter(const SUMOVehicle& veh, const double entryTimestep, const d
     v.frontLeaveTime = 0;
     v.backLeaveTime = 0;
     v.speedSum = speedFraction;
-    v.haltingBegin = veh.getSpeed() < myHaltingSpeedThreshold ? entryTimestep : -1;
+    v.haltingBegin = veh.getSpeed() < myHaltingSpeedThreshold ? TIME2STEPS(entryTimestep) : -1;
     v.intervalSpeedSum = entryTimestep >= STEPS2TIME(myLastResetTime) ? speedFraction : 0;
     v.haltings = 0;
     v.intervalHaltings = 0;
@@ -200,15 +358,20 @@ MSE3Collector::enter(const SUMOVehicle& veh, const double entryTimestep, const d
         v.timeLoss = static_cast<const MSVehicle&>(veh).getTimeLoss();
         v.intervalTimeLoss = v.timeLoss;
     }
-
+    v.entryReminder = entryReminder;
     myEnteredContainer[&veh] = v;
 }
 
 
 void
-MSE3Collector::leaveFront(const SUMOVehicle& veh, const double leaveTimestep) {
+MSE3Collector::leaveFront(const SUMOTrafficObject& veh, const double leaveTimestep) {
+    if (!vehicleApplies(veh)) {
+        return;
+    }
     if (myEnteredContainer.find(&veh) == myEnteredContainer.end()) {
-        WRITE_WARNING("Vehicle '" + veh.getID() + "' left " + toString(SUMO_TAG_E3DETECTOR) + " '" + getID() + "' without entering it.");
+        if (!myOpenEntry) {
+            WRITE_WARNING("Vehicle '" + veh.getID() + "' left " + toString(SUMO_TAG_E3DETECTOR) + " '" + getID() + "' without entering it.");
+        }
     } else {
         myEnteredContainer[&veh].frontLeaveTime = leaveTimestep;
     }
@@ -216,9 +379,14 @@ MSE3Collector::leaveFront(const SUMOVehicle& veh, const double leaveTimestep) {
 
 
 void
-MSE3Collector::leave(const SUMOVehicle& veh, const double leaveTimestep, const double fractionTimeOnDet) {
+MSE3Collector::leave(const SUMOTrafficObject& veh, const double leaveTimestep, const double fractionTimeOnDet) {
+    if (!vehicleApplies(veh)) {
+        return;
+    }
     if (myEnteredContainer.find(&veh) == myEnteredContainer.end()) {
-        WRITE_WARNING("Vehicle '" + veh.getID() + "' left " + toString(SUMO_TAG_E3DETECTOR) + " '" + getID() + "' without entering it.");
+        if (!myOpenEntry) {
+            WRITE_WARNING("Vehicle '" + veh.getID() + "' left " + toString(SUMO_TAG_E3DETECTOR) + " '" + getID() + "' without entering it.");
+        }
     } else {
         E3Values values = myEnteredContainer[&veh];
         values.backLeaveTime = leaveTimestep;
@@ -233,7 +401,7 @@ MSE3Collector::leave(const SUMOVehicle& veh, const double leaveTimestep, const d
             values.timeLoss = static_cast<const MSVehicle&>(veh).getTimeLoss() - values.timeLoss;
         }
         myEnteredContainer.erase(&veh);
-        myLeftContainer[&veh] = values;
+        myLeftContainer.push_back(values);
     }
 }
 
@@ -249,13 +417,13 @@ MSE3Collector::writeXMLOutput(OutputDevice& dev,
     double meanSpeed = 0.;
     double meanHaltsPerVehicle = 0.;
     double meanTimeLoss = 0.;
-    for (std::map<const SUMOVehicle*, E3Values>::iterator i = myLeftContainer.begin(); i != myLeftContainer.end(); ++i) {
-        meanHaltsPerVehicle += (double)(*i).second.haltings;
-        meanTravelTime += (*i).second.frontLeaveTime - (*i).second.entryTime;
-        const double steps = (*i).second.backLeaveTime - (*i).second.entryTime;
+    for (const E3Values& values : myLeftContainer) {
+        meanHaltsPerVehicle += (double)values.haltings;
+        meanTravelTime += values.frontLeaveTime - values.entryTime;
+        const double steps = values.backLeaveTime - values.entryTime;
         meanOverlapTravelTime += steps;
-        meanSpeed += ((*i).second.speedSum / steps);
-        meanTimeLoss += STEPS2TIME((*i).second.timeLoss);
+        meanSpeed += (values.speedSum / steps);
+        meanTimeLoss += STEPS2TIME(values.timeLoss);
     }
     meanTravelTime = vehicleSum != 0 ? meanTravelTime / (double)vehicleSum : -1;
     meanOverlapTravelTime = vehicleSum != 0 ? meanOverlapTravelTime / (double)vehicleSum : -1;
@@ -274,7 +442,7 @@ MSE3Collector::writeXMLOutput(OutputDevice& dev,
     double meanIntervalHaltsPerVehicleWithin = 0.;
     double meanIntervalDurationWithin = 0.;
     double meanTimeLossWithin = 0.;
-    for (std::map<const SUMOVehicle*, E3Values>::iterator i = myEnteredContainer.begin(); i != myEnteredContainer.end(); ++i) {
+    for (std::map<const SUMOTrafficObject*, E3Values>::iterator i = myEnteredContainer.begin(); i != myEnteredContainer.end(); ++i) {
         meanHaltsPerVehicleWithin += (double)(*i).second.haltings;
         meanIntervalHaltsPerVehicleWithin += (double)(*i).second.intervalHaltings;
         const double end = (*i).second.backLeaveTime == 0 ? STEPS2TIME(stopTime) : (*i).second.backLeaveTime;
@@ -336,8 +504,15 @@ void
 MSE3Collector::detectorUpdate(const SUMOTime step) {
     myCurrentMeanSpeed = 0;
     myCurrentHaltingsNumber = 0;
-    for (std::map<const SUMOVehicle*, E3Values>::iterator pair = myEnteredContainer.begin(); pair != myEnteredContainer.end(); ++pair) {
-        const SUMOVehicle* veh = pair->first;
+    for (std::map<const SUMOTrafficObject*, E3Values>::iterator pair = myEnteredContainer.begin(); pair != myEnteredContainer.end(); ++pair) {
+        const SUMOTrafficObject* veh = pair->first;
+#ifdef DEBUG_E3_DETECTORUPDATE
+        //if (DEBUG_COND(*this) && DEBUG_COND_VEH(*veh)) {
+        if (DEBUG_COND(*this)) {
+            std::cout << SIMTIME << " vehPtr=" << veh << "\n";
+            std::cout << "       veh=" << veh->getID() << "\n";
+        }
+#endif
         E3Values& values = pair->second;
         myCurrentMeanSpeed += veh->getSpeed();
         values.hadUpdate = true;
@@ -345,9 +520,11 @@ MSE3Collector::detectorUpdate(const SUMOTime step) {
         values.intervalSpeedSum += veh->getSpeed() * TS;
         if (veh->getSpeed() < myHaltingSpeedThreshold) {
             if (values.haltingBegin == -1) {
-                values.haltingBegin = STEPS2TIME(step);
+                values.haltingBegin = step;
             }
-            if (step - values.haltingBegin > myHaltingTimeThreshold) {
+            SUMOTime haltingDuration = step - values.haltingBegin;
+            if (haltingDuration >= myHaltingTimeThreshold
+                    && haltingDuration < (myHaltingTimeThreshold + DELTA_T)) {
                 values.haltings++;
                 values.intervalHaltings++;
                 myCurrentHaltingsNumber++;
@@ -385,7 +562,7 @@ MSE3Collector::getVehiclesWithin() const {
 std::vector<std::string>
 MSE3Collector::getCurrentVehicleIDs() const {
     std::vector<std::string> ret;
-    for (std::map<const SUMOVehicle*, E3Values>::const_iterator pair = myEnteredContainer.begin(); pair != myEnteredContainer.end(); ++pair) {
+    for (std::map<const SUMOTrafficObject*, E3Values>::const_iterator pair = myEnteredContainer.begin(); pair != myEnteredContainer.end(); ++pair) {
         ret.push_back((*pair).first->getID());
     }
     std::sort(ret.begin(), ret.end());
