@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2004-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2004-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2004-11-23
-/// @version $Id$
 ///
 // An unextended detector measuring at a fixed position on a fixed lane.
 /****************************************************************************/
@@ -108,7 +107,7 @@ public:
      * @return True if vehicle enters the induction loop
      * @see Notification
      */
-    bool notifyEnter(SUMOVehicle& veh, Notification reason, const MSLane* enteredLane = 0);
+    bool notifyEnter(SUMOTrafficObject& veh, Notification reason, const MSLane* enteredLane = 0);
 
     /** @brief Checks whether the vehicle shall be counted and/or shall still touch this MSMoveReminder
      *
@@ -126,7 +125,7 @@ public:
      * @see enterDetectorByMove
      * @see leaveDetectorByMove
      */
-    bool notifyMove(SUMOVehicle& veh, double oldPos, double newPos, double newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double newSpeed);
 
 
     /** @brief Dismisses the vehicle if it is on the detector due to a lane change
@@ -143,7 +142,7 @@ public:
      * @see MSMoveReminder
      * @see MSMoveReminder::notifyLeave
      */
-    bool notifyLeave(SUMOVehicle& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
     //@}
@@ -160,7 +159,7 @@ public:
      *
      * @return The speed [m/s] of the vehicle if one is on the detector, -1 otherwise
      */
-    double getCurrentSpeed() const;
+    double getSpeed(const int offset) const;
 
 
     /** @brief Returns the length of the vehicle on the detector
@@ -170,7 +169,7 @@ public:
      *
      * @return The length [m] of the vehicle if one is on the detector, -1 otherwise
      */
-    double getCurrentLength() const;
+    double getVehicleLength(const int offset) const;
 
 
     /** @brief Returns the current occupancy
@@ -182,7 +181,7 @@ public:
      * @return This detector's current occupancy
      * @todo recheck (especially if more than one vehicle has passed)
      */
-    double getCurrentOccupancy() const;
+    double getOccupancy() const;
 
 
     /** @brief Returns the number of vehicles that have passed the detector
@@ -194,7 +193,7 @@ public:
      * @return The number of vehicles that have passed the detector
      * @todo recheck (especially if more than one vehicle has passed)
      */
-    int getCurrentPassedNumber() const;
+    double getPassedNumber(const int offset) const;
 
 
     /** @brief Returns the ids of vehicles that have passed the detector
@@ -202,14 +201,17 @@ public:
      * @return The ids of vehicles that have passed the detector
      * @todo recheck (especially if more than one vehicle has passed)
      */
-    std::vector<std::string> getCurrentVehicleIDs() const;
+    std::vector<std::string> getVehicleIDs(const int offset) const;
 
 
     /** @brief Returns the time since the last vehicle left the detector
      *
-     * @return Timesteps from last leaving (detection) of the detector
+     * @return seconds from last leaving (detection) of the detector
      */
     double getTimeSinceLastDetection() const;
+
+    ///@brief return last time a vehicle was on the detector
+    SUMOTime getLastDetectionTime() const;
     //@}
 
 
@@ -283,6 +285,10 @@ public:
      */
     virtual std::vector<VehicleData> collectVehiclesOnDet(SUMOTime t, bool leaveTime = false) const;
 
+    /// @brief allows for special color in the gui version
+    virtual void setSpecialColor(const RGBColor* /*color*/) {};
+
+    virtual void setVisible(bool /*show*/) {};
 
 protected:
     /// @name Methods that add and remove vehicles from internal container
@@ -292,7 +298,7 @@ protected:
      * @param veh The entering vehicle.
      * @param entryTimestep Timestep (not necessary integer) of entrance.
      */
-    virtual void enterDetectorByMove(SUMOVehicle& veh, double entryTimestep);
+    virtual void enterDetectorByMove(SUMOTrafficObject& veh, double entryTimestep);
 
 
     /** @brief Processes a vehicle that leaves the detector
@@ -303,14 +309,14 @@ protected:
      * @param veh The leaving vehicle.
      * @param leaveTimestep Timestep (not necessary integer) of leaving.
      */
-    virtual void leaveDetectorByMove(SUMOVehicle& veh, double leaveTimestep);
+    virtual void leaveDetectorByMove(SUMOTrafficObject& veh, double leaveTimestep);
 
 
     /** @brief Removes a vehicle from the detector's map myVehiclesOnDet.
      * @param veh The leaving vehicle.
      * @param lastPos The last position of the leaving vehicle.
      */
-    virtual void leaveDetectorByLaneChange(SUMOVehicle& veh, double lastPos);
+    virtual void leaveDetectorByLaneChange(SUMOTrafficObject& veh, double lastPos);
     /// @}
 
 
@@ -337,13 +343,9 @@ protected:
     /// @brief Leave-time of the last vehicle detected [s]
     double myLastLeaveTime;
 
-    /// @brief Occupancy by the last vehicle detected.
-    double myLastOccupancy;
-
     /// @brief The number of entered vehicles
     int myEnteredVehicleNumber;
 
-
     /// @brief Type of myVehicleDataCont.
     typedef std::deque< VehicleData > VehicleDataCont;
 
@@ -353,13 +355,8 @@ protected:
     /// @brief Data of vehicles that have completely passed the detector in the last time interval
     VehicleDataCont myLastVehicleDataCont;
 
-
-    /// @brief Type of myVehiclesOnDet
-
-    typedef std::map< SUMOVehicle*, double > VehicleMap;
-
     /// @brief Data for vehicles that have entered the detector (vehicle -> enter time)
-    VehicleMap myVehiclesOnDet;
+    std::map<SUMOTrafficObject*, double> myVehiclesOnDet;
 
 private:
     /// @brief Invalidated copy constructor.
