@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2011-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2011-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    2011-09.08
-/// @version $Id$
 ///
 // An instantaneous induction loop
 /****************************************************************************/
@@ -38,6 +37,7 @@
 class MSLane;
 class MSVehicle;
 class OutputDevice;
+class SUMOTrafficObject;
 
 
 // ===========================================================================
@@ -90,7 +90,7 @@ public:
      * @see enterDetectorByMove
      * @see leaveDetectorByMove
      */
-    bool notifyMove(SUMOVehicle& veh, double oldPos, double newPos, double newSpeed);
+    bool notifyMove(SUMOTrafficObject& veh, double oldPos, double newPos, double newSpeed);
 
 
     /** @brief Dismisses the vehicle if it is on the detector due to a lane change
@@ -107,7 +107,7 @@ public:
      * @see MSMoveReminder
      * @see MSMoveReminder::notifyLeave
      */
-    bool notifyLeave(SUMOVehicle& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+    bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
     //@}
 
 
@@ -149,7 +149,7 @@ protected:
      * @param[in] add An optional attribute to report
      * @param[in] addValue The value of the optional attribute
      */
-    void write(const char* state, double t, SUMOVehicle& veh, double speed, const char* add = 0, double addValue = -1);
+    void write(const char* state, double t, SUMOTrafficObject& veh, double speed, const char* add = 0, double addValue = -1);
 
 
 protected:
@@ -163,7 +163,7 @@ protected:
     double myLastExitTime;
 
     /// @brief The last exit time
-    std::map<SUMOVehicle*, double> myEntryTimes;
+    std::map<SUMOTrafficObject*, double> myEntryTimes;
 
 private:
     /// @brief Invalidated copy constructor.
