@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 17.11.2009
-/// @version $Id$
 ///
 // Data collector for edges/lanes
 /****************************************************************************/
@@ -39,7 +38,7 @@
 class OutputDevice;
 class MSEdge;
 class MSLane;
-class SUMOVehicle;
+class SUMOTrafficObject;
 
 typedef std::vector<MSEdge*> MSEdgeVector;
 
@@ -94,7 +93,7 @@ public:
          * @see MSMoveReminder::notifyEnter
          * @see MSMoveReminder::Notification
          */
-        virtual bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        virtual bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
         /** @brief Checks whether the reminder still has to be notified about the vehicle moves
@@ -110,7 +109,7 @@ public:
          *
          * @return True if vehicle hasn't passed the reminder completely.
          */
-        bool notifyMove(SUMOVehicle& veh, double oldPos,
+        bool notifyMove(SUMOTrafficObject& veh, double oldPos,
                         double newPos, double newSpeed);
 
 
@@ -122,7 +121,7 @@ public:
          * @see MSMoveReminder
          * @see MSMoveReminder::notifyLeave
          */
-        virtual bool notifyLeave(SUMOVehicle& veh, double lastPos,
+        virtual bool notifyLeave(SUMOTrafficObject& veh, double lastPos,
                                  MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
@@ -208,7 +207,7 @@ public:
         /** @brief Internal notification about the vehicle moves
          *  @see MSMoveReminder::notifyMoveInternal().
          */
-        void notifyMoveInternal(const SUMOVehicle& veh, const double frontOnLane, const double timeOnLane, const double meanSpeedFrontOnLane, const double meanSpeedVehicleOnLane, const double travelledDistanceFrontOnLane, const double travelledDistanceVehicleOnLane, const double meanLengthOnLane);
+        void notifyMoveInternal(const SUMOTrafficObject& veh, const double frontOnLane, const double timeOnLane, const double meanSpeedFrontOnLane, const double meanSpeedVehicleOnLane, const double travelledDistanceFrontOnLane, const double travelledDistanceVehicleOnLane, const double meanLengthOnLane);
 
 
         /** @brief Called if the vehicle leaves the reminder's lane
@@ -220,7 +219,7 @@ public:
          * @see MSMoveReminder
          * @see MSMoveReminder::notifyLeave
          */
-        bool notifyLeave(SUMOVehicle& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        bool notifyLeave(SUMOTrafficObject& veh, double lastPos, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
 
 
         /** @brief Computes current values and adds them to their sums
@@ -234,7 +233,7 @@ public:
          * @see MSMoveReminder::notifyEnter
          * @return Always true
          */
-        bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
+        bool notifyEnter(SUMOTrafficObject& veh, MSMoveReminder::Notification reason, const MSLane* enteredLane = 0);
         //@}
 
         bool isEmpty() const;
@@ -279,7 +278,7 @@ public:
         };
 
         /// @brief The map of vehicles to data entries
-        std::map<const SUMOVehicle*, TrackerEntry*> myTrackedData;
+        std::map<const SUMOTrafficObject*, TrackerEntry*> myTrackedData;
 
         /// @brief The currently active meandata "intervals"
         std::list<TrackerEntry*> myCurrentData;
@@ -297,6 +296,7 @@ public:
      * @param[in] withEmpty Information whether empty lanes/edges shall be written
      * @param[in] withInternal Information whether internal lanes/edges shall be written
      * @param[in] trackVehicles Information whether vehicles shall be tracked
+     * @param[in] detectPersons Whether pedestrians shall be detected instead of vehicles
      * @param[in] maxTravelTime the maximum travel time to use when calculating per vehicle output
      * @param[in] defaultEffort the value to use when calculating defaults
      * @param[in] minSamples the minimum number of sample seconds before the values are valid
@@ -306,7 +306,8 @@ public:
                const SUMOTime dumpBegin, const SUMOTime dumpEnd,
                const bool useLanes, const bool withEmpty,
                const bool printDefaults, const bool withInternal,
-               const bool trackVehicles, const double minSamples,
+               const bool trackVehicles, const int detectPersons,
+               const double minSamples,
                const double maxTravelTime,
                const std::string& vTypes);
 
