@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Bjoern Hendriks
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id$
 ///
 // Realises dumping the complete network state
 /****************************************************************************/
@@ -117,17 +116,11 @@ MSXMLRawOut::writeEdge(OutputDevice& of, const MSEdge& edge, SUMOTime timestep)
 
 void
 MSXMLRawOut::writeLane(OutputDevice& of, const MSLane& lane) {
-    of.openTag("lane") << " id=\"" << lane.myID << "\"";
-    if (lane.getVehicleNumber() != 0) {
-        for (std::vector<MSVehicle*>::const_iterator veh = lane.myVehBuffer.begin();
-                veh != lane.myVehBuffer.end(); ++veh) {
-            writeVehicle(of, **veh);
-        }
-        for (MSLane::VehCont::const_iterator veh = lane.myVehicles.begin();
-                veh != lane.myVehicles.end(); ++veh) {
-            writeVehicle(of, **veh);
-        }
+    of.openTag("lane").writeAttr(SUMO_ATTR_ID, lane.getID());
+    for (const MSBaseVehicle* const veh : lane.getVehiclesSecure()) {
+        writeVehicle(of, *veh);
     }
+    lane.releaseVehicles();
     of.closeTag();
 }
 
@@ -147,6 +140,7 @@ MSXMLRawOut::writeVehicle(OutputDevice& of, const MSBaseVehicle& veh) {
             if (MSAbstractLaneChangeModel::haveLateralDynamics()) {
                 const double posLat = microVeh.getLateralPositionOnLane();
                 of.writeAttr(SUMO_ATTR_POSITION_LAT, posLat);
+                of.writeAttr("speedLat", microVeh.getLaneChangeModel().getSpeedLat());
             }
             const int personNumber = microVeh.getPersonNumber();
             if (personNumber > 0) {
