@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2014-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2014-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSPModel.cpp
 /// @author  Jakob Erdmann
 /// @date    Mon, 13 Jan 2014
-/// @version $Id$
 ///
 // The pedestrian following model (prototype)
 /****************************************************************************/
@@ -27,6 +26,7 @@
 #include <microsim/MSEdge.h>
 #include <microsim/MSJunction.h>
 #include <microsim/MSLane.h>
+#include <microsim/MSGlobals.h>
 #include "MSPModel_Striping.h"
 #include "MSPModel_NonInteracting.h"
 #include "MSPModel.h"
@@ -50,6 +50,10 @@ const double MSPModel::SAFETY_GAP(1.0);
 
 const double MSPModel::SIDEWALK_OFFSET(3);
 
+#ifdef HAVE_FOX
+FXMutex MSPModel::myInitializationMutex(true);
+#endif
+
 // ===========================================================================
 // MSPModel method definitions
 // ===========================================================================
@@ -57,6 +61,9 @@ const double MSPModel::SIDEWALK_OFFSET(3);
 
 MSPModel*
 MSPModel::getModel() {
+#ifdef HAVE_FOX
+    FXConditionalLock lock(myInitializationMutex, MSGlobals::gNumSimThreads > 1);
+#endif
     if (myModel == nullptr) {
         const OptionsCont& oc = OptionsCont::getOptions();
         MSNet* net = MSNet::getInstance();
