@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2014-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2014-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSPModel.h
 /// @author  Jakob Erdmann
 /// @date    Mon, 13 Jan 2014
-/// @version $Id$
 ///
 // The pedestrian following model (prototype)
 /****************************************************************************/
@@ -29,6 +28,9 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/geom/GeomHelper.h>
 #include <microsim/pedestrians/MSPerson.h>
+#ifdef HAVE_FOX
+#include <fx.h>
+#endif
 
 // ===========================================================================
 // class declarations
@@ -118,6 +120,13 @@ public:
     /// @brief whether movements on intersections are modelled
     virtual bool usingInternalLanes() = 0;
 
+protected:
+#ifdef HAVE_FOX
+    /// @brief the mutex for model initialization
+    static FXMutex myInitializationMutex;
+#endif
+
+
 private:
     static MSPModel* myModel;
 
