@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Jan 2004
-/// @version $Id$
 ///
 // The definition of a single phase of a tls logic
 /****************************************************************************/
@@ -83,7 +82,10 @@ public:
     SUMOTime myLastSwitch;
 
     /// @brief The index of the phase that suceeds this one (or -1)
-    int nextPhase;
+    std::vector<int> nextPhases;
+
+    /// @brief Optional name or description for the current phase
+    std::string name;
 
 private:
     /// @brief The phase definition
@@ -104,7 +106,8 @@ private:
      */
     LaneIdVector targetLaneSet;
 
-    void init(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg, SUMOTime maxDurationArg, int nextPhase) {
+    void init(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg, SUMOTime maxDurationArg,
+              const std::vector<int> nextPhasesArg, const std::string& nameArg) {
         this->duration = durationArg;
         this->state = stateArg;
         this->minDuration = minDurationArg < 0 ? durationArg : minDurationArg;
@@ -113,13 +116,17 @@ private:
         this->myLastSwitch = string2time(OptionsCont::getOptions().getString("begin")); // SUMOTime-option
         //For SOTL phases
         //this->phaseType = phaseTypeArg;
-        this->nextPhase = nextPhase;
+        this->nextPhases = nextPhasesArg;
+        this->name = nameArg;
     }
 
-    void init(SUMOTime durationArg, SUMOTime minDurationArg, SUMOTime maxDurationArg, const std::string& stateArg, int nextPhase, LaneIdVector& targetLaneSetArg) {
-        init(durationArg, stateArg, minDurationArg, maxDurationArg, nextPhase);
+    void init(SUMOTime durationArg, SUMOTime minDurationArg, SUMOTime maxDurationArg, const std::string& stateArg,
+              const std::vector<int>& nextPhasesArg, const std::string& nameArg, LaneIdVector* targetLaneSetArg) {
+        init(durationArg, stateArg, minDurationArg, maxDurationArg, nextPhasesArg, nameArg);
         //For SOTL target phases
-        this->targetLaneSet = targetLaneSetArg;
+        if (targetLaneSetArg != nullptr) {
+            this->targetLaneSet = *targetLaneSetArg;
+        }
     }
 
 
@@ -131,14 +138,14 @@ public:
      * @param[in] durationArg The duration of the phase
      * @param[in] stateArg The state in the phase
      */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, int nextPhase) {
+    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, const std::vector<int>& nextPhases, const std::string& name = "") {
         //PhaseType phaseType;
         phaseType = PhaseType();
         phaseType[UNDEFINED_BIT] = 1;
         phaseType[TRANSIENT_NOTDECISIONAL_BIT] = 0;
         phaseType[TARGET_BIT] = 0;
         phaseType[COMMIT_BIT] = 0;
-        init(durationArg, stateArg, durationArg, durationArg, nextPhase);
+        init(durationArg, stateArg, durationArg, durationArg, nextPhases, name);
     }
 
 
@@ -149,53 +156,17 @@ public:
      * @param[in] minDurationArg The minimum duration of the phase
      * @param[in] maxDurationArg The maximum duration of the phase
      */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg=-1, SUMOTime maxDurationArg=-1, int nextPhase=-1) {
+    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg = -1, SUMOTime maxDurationArg = -1,
+                      const std::vector<int>& nextPhases = std::vector<int>(), const std::string& name = "") {
         //PhaseType phaseType;
         phaseType = PhaseType();
         phaseType[UNDEFINED_BIT] = 1;
         phaseType[TRANSIENT_NOTDECISIONAL_BIT] = 0;
         phaseType[TARGET_BIT] = 0;
         phaseType[COMMIT_BIT] = 0;
-        init(durationArg, stateArg, minDurationArg, maxDurationArg, nextPhase);
+        init(durationArg, stateArg, minDurationArg, maxDurationArg, nextPhases, name);
     }
 
-    /*
-     * @brief Constructor for definitions for SOTL target step
-     * In this phase the duration is fixed, because min and max duration are unspecified
-     * @param[in] transient_notdecisional true means this is a transient phase, false is a decisional one
-     * @param[in] commit true means this is a commit phase, i.e. the traffic light logic can jump to a target phase form this phase
-     * @param[in] targetLaneSet identifies the lane-set to be considered when deciding the target phase to jump to
-     * @see MSPhaseDefinition::PhaseType
-     */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, int nextPhase,  bool transient_notdecisional, bool commit, LaneIdVector& targetLaneSetArg) {
-        if (targetLaneSetArg.size() == 0) {
-            MsgHandler::getErrorInstance()->inform("MSPhaseDefinition::MSPhaseDefinition -> targetLaneSetArg cannot be empty for a target phase");
-        }
-        //PhaseType phaseType;
-        phaseType = PhaseType();
-        phaseType[UNDEFINED_BIT] = 0;
-        phaseType[TRANSIENT_NOTDECISIONAL_BIT] = transient_notdecisional;
-        phaseType[TARGET_BIT] = 1;
-        phaseType[COMMIT_BIT] = commit;
-        init(durationArg, durationArg, durationArg, stateArg, nextPhase, targetLaneSetArg);
-    }
-
-    /*
-     * @brief Constructor for definitions for SOTL non-target step
-     * In this phase the duration is fixed, because min and max duration are unspecified
-     * @param[in] phaseType Indicates the type of the step
-     */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, int nextPhase, bool transient_notdecisional, bool commit) {
-        //PhaseType phaseType;
-        phaseType = PhaseType();
-        phaseType[UNDEFINED_BIT] = 0;
-        phaseType[TRANSIENT_NOTDECISIONAL_BIT] = transient_notdecisional;
-        phaseType[TARGET_BIT] = 0;
-        phaseType[COMMIT_BIT] = commit;
-        init(durationArg, stateArg, durationArg, durationArg, nextPhase);
-    }
-
-
     /*
      * @brief Constructor for definitions for SOTL target step
      * In this phase the duration is constrained between min and max duration
@@ -203,8 +174,9 @@ public:
      * @param[in] targetLaneSet If not null, specifies this MSPhaseDefinition is a target step
      * @see MSPhaseDefinition::PhaseType
      */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg, SUMOTime maxDurationArg, int nextPhase, bool transient_notdecisional, bool commit, LaneIdVector& targetLaneSetArg) {
-        if (targetLaneSetArg.size() == 0) {
+    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg, SUMOTime maxDurationArg,
+                      const std::vector<int>& nextPhases, const std::string& name, bool transient_notdecisional, bool commit, LaneIdVector* targetLaneSetArg = nullptr) {
+        if (targetLaneSetArg != nullptr && targetLaneSetArg->size() == 0) {
             MsgHandler::getErrorInstance()->inform("MSPhaseDefinition::MSPhaseDefinition -> targetLaneSetArg cannot be empty for a target phase");
         }
         //PhaseType phaseType;
@@ -212,29 +184,11 @@ public:
         phaseType = PhaseType();
         phaseType[UNDEFINED_BIT] = 0;
         phaseType[TRANSIENT_NOTDECISIONAL_BIT] = transient_notdecisional;
-        phaseType[TARGET_BIT] = 1;
+        phaseType[TARGET_BIT] = targetLaneSetArg == nullptr ? 0 : 1;
         phaseType[COMMIT_BIT] = commit;
-        init(durationArg, minDurationArg, maxDurationArg, stateArg, nextPhase, targetLaneSetArg);
+        init(durationArg, minDurationArg, maxDurationArg, stateArg, nextPhases, name, targetLaneSetArg);
     }
 
-    /*
-     * @brief Constructor for definitions for SOTL target step
-     * In this phase the duration is constrained between min and max duration
-     * @param[in] phaseType Indicates the type of the step
-     * @see MSPhaseDefinition::PhaseType
-     */
-    MSPhaseDefinition(SUMOTime durationArg, const std::string& stateArg, SUMOTime minDurationArg, SUMOTime maxDurationArg, int nextPhase, bool transient_notdecisional, bool commit) {
-        //PhaseType phaseType;
-        phaseType = PhaseType();
-        phaseType[UNDEFINED_BIT] = 0;
-        phaseType[TRANSIENT_NOTDECISIONAL_BIT] = transient_notdecisional;
-        phaseType[TARGET_BIT] = 0;
-        phaseType[COMMIT_BIT] = commit;
-        init(durationArg, stateArg, minDurationArg, maxDurationArg, nextPhase);
-
-    }
-
-
     /// @brief Destructor
     virtual ~MSPhaseDefinition() { }
 
@@ -254,8 +208,16 @@ public:
         return targetLaneSet;
     }
 
-    int getNextPhase() const {
-        return nextPhase;
+    const std::vector<int>& getNextPhases() const {
+        return nextPhases;
+    }
+
+    const std::string& getName() const {
+        return name;
+    }
+
+    void setName(const std::string& _name) {
+        name = _name;
     }
 
     /** @brief Returns whether this phase is a pure "green" phase
