@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSRailCrossing.cpp
 /// @author  Jakob Erdmann
 /// @date    Dez 2015
-/// @version $Id$
 ///
 // A rail signal logic
 /****************************************************************************/
@@ -39,9 +38,9 @@
 // method definitions
 // ===========================================================================
 MSRailCrossing::MSRailCrossing(MSTLLogicControl& tlcontrol,
-                               const std::string& id, const std::string& programID,
+                               const std::string& id, const std::string& programID, SUMOTime delay,
                                const std::map<std::string, std::string>& parameters) :
-    MSSimpleTrafficLightLogic(tlcontrol, id, programID, TLTYPE_RAIL_CROSSING, Phases(), 0, DELTA_T, parameters),
+    MSSimpleTrafficLightLogic(tlcontrol, id, programID, TLTYPE_RAIL_CROSSING, Phases(), 0, delay, parameters),
     // XXX make this configurable
     mySecurityGap(TIME2STEPS(15)),
     myMinGreenTime(TIME2STEPS(5)),
@@ -79,7 +78,6 @@ MSRailCrossing::adaptLinkInformationFrom(const MSTrafficLightLogic& logic) {
 SUMOTime
 MSRailCrossing::trySwitch() {
     SUMOTime nextTry = updateCurrentPhase();
-    setTrafficLightSignals(MSNet::getInstance()->getCurrentTimeStep());
     //if (getID() == "cluster_1088529493_1260626727") std::cout << " myStep=" << myStep << " nextTry=" << nextTry << "\n";
     return nextTry;
 }
