@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    MSRailCrossing.h
 /// @author  Jakob Erdmann
 /// @date    Dez 2015
-/// @version $Id$
 ///
 // A rail signal logic
 /****************************************************************************/
@@ -45,7 +44,7 @@ public:
      * @param[in] parameters This tls' parameters
      */
     MSRailCrossing(MSTLLogicControl& tlcontrol,
-                   const std::string& id, const std::string& programID,
+                   const std::string& id, const std::string& programID, SUMOTime delay,
                    const std::map<std::string, std::string>& parameters);
 
 
