@@ -1,25 +1,22 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2010-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2010-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSSOTLE2Sensors.cpp
 /// @author  Gianfilippo Slager
 /// @author  Alessio Bonfietti
 /// @author  Federico Caselli
 /// @date    2010-02-25
-/// @version $Id$
 ///
 // The class for SOTL sensors of "E2" type
 /****************************************************************************/
 
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <microsim/output/MSDetectorControl.h>
 #include <microsim/MSEdge.h>
 #include <microsim/MSVehicleType.h>
@@ -46,7 +43,7 @@ void MSSOTLE2Sensors::buildSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb, double sensorLength) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -67,7 +64,7 @@ void MSSOTLE2Sensors::buildCountSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
                 controlledLanes.begin(); laneVector != controlledLanes.end();
@@ -84,7 +81,7 @@ void MSSOTLE2Sensors::buildCountOutSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -107,7 +104,7 @@ void MSSOTLE2Sensors::buildOutSensors(
     MSTrafficLightLogic::LaneVectorVector controlledLanes,
     NLDetectorBuilder& nb, double sensorLength) {
     //for each lane build an appropriate sensor on it
-    MSLane* currentLane = NULL;
+    MSLane* currentLane = nullptr;
 
     //input and ouput lanes
     for (MSTrafficLightLogic::LaneVectorVector::const_iterator laneVector =
@@ -124,7 +121,7 @@ void MSSOTLE2Sensors::buildOutSensors(
 void MSSOTLE2Sensors::buildCountSensorForLane(MSLane* lane, NLDetectorBuilder& nb) {
     double sensorPos;
     double lensorLength;
-    MSE2Collector* newSensor = NULL;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
@@ -166,7 +163,7 @@ void MSSOTLE2Sensors::buildCountSensorForLane(MSLane* lane, NLDetectorBuilder& n
 void MSSOTLE2Sensors::buildCountSensorForOutLane(MSLane* lane, NLDetectorBuilder& nb) {
     double sensorPos;
     double lensorLength;
-    MSE2Collector* newSensor = NULL;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
@@ -176,7 +173,7 @@ void MSSOTLE2Sensors::buildCountSensorForOutLane(MSLane* lane, NLDetectorBuilder
         sensorPos = (lane->getLength() - sensorLength)
                     - (SENSOR_START <= lane->getLength() ? SENSOR_START : 0);
 
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength =
             sensorLength <= (lane->getLength() - sensorPos) ?
             sensorLength : (lane->getLength() - sensorPos);
@@ -252,13 +249,13 @@ void MSSOTLE2Sensors::buildSensorForLane(MSLane* lane, NLDetectorBuilder& nb, do
     }
     double sensorPos;
     double lensorLength;
-    MSE2Collector* newSensor = NULL;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
         //Check and set zero if the lane is not long enough for the specified sensor start
         sensorPos = SENSOR_START <= lane->getLength() ? SENSOR_START : 0;
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength = sensorLength <= (lane->getLength() - sensorPos) ? sensorLength : (lane->getLength() - sensorPos);
         //TODO check this lengths
 //        DBG(
@@ -333,7 +330,7 @@ void MSSOTLE2Sensors::buildSensorForOutLane(MSLane* lane,
         NLDetectorBuilder& nb, double sensorLength) {
     double sensorPos;
     double lensorLength;
-    MSE2Collector* newSensor = NULL;
+    MSE2Collector* newSensor = nullptr;
     //Check not to have more than a sensor for lane
     if (m_sensorMap.find(lane->getID()) == m_sensorMap.end()) {
 
@@ -343,7 +340,7 @@ void MSSOTLE2Sensors::buildSensorForOutLane(MSLane* lane,
         sensorPos = (lane->getLength() - sensorLength)
                     - (SENSOR_START <= lane->getLength() ? SENSOR_START : 0);
 
-        //Check and trim if the lane is not long enough for the specified sensor lenght
+        //Check and trim if the lane is not long enough for the specified sensor length
         lensorLength =
             sensorLength <= (lane->getLength() - sensorPos) ?
             sensorLength : (lane->getLength() - sensorPos);
@@ -379,7 +376,7 @@ int MSSOTLE2Sensors::countVehicles(MSLane* lane) {
 }
 
 /*
- * Estimate queue lenght according to the distance of the last vehicles
+ * Estimate queue length according to the distance of the last vehicles
  */
 
 double MSSOTLE2Sensors::getEstimateQueueLength(std::string laneId) {
@@ -405,7 +402,7 @@ double MSSOTLE2Sensors::getEstimateQueueLength(std::string laneId) {
 }
 
 /*
- * Estimate queue lenght according to the distance of the last vehicles that exceed a threshold
+ * Estimate queue length according to the distance of the last vehicles that exceed a threshold
  */
 
 int MSSOTLE2Sensors::estimateVehicles(std::string laneId) {
@@ -509,7 +506,7 @@ void MSSOTLE2Sensors::setVehicleWeigths(const std::string& weightString) {
         split(*typesIt, '=', typeWeight);
         if (typeWeight.size() == 2) {
             std::string type = trim(typeWeight[0]);
-            int value = TplConvert::_2int(typeWeight[1].c_str());
+            int value = StringUtils::toInt(typeWeight[1]);
             logstr << type << "=" << value << " ";
             m_typeWeightMap[type] = value;
         }
