@@ -1,19 +1,16 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2013-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2013-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    MSSOTLPolicy5DStimulus.h
 /// @author  Riccardo Belletti
 /// @author  Simone Bacchilega
 /// @date    2014-09-30
-/// @version $Id$
 ///
 // The class the low-level policy stimulus
 /****************************************************************************/
@@ -24,11 +21,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 //#define SWARM_DEBUG
 #include <utils/common/SwarmDebug.h>
@@ -59,21 +52,21 @@ public:
 
     double getStimCox() {
         std::string key = getKeyPrefix() + "_STIM_COX";
-        return readParameter(key, stimCoxDVal);
+        return getDouble(key, stimCoxDVal);
     }
     void setStimCoxDefVal(double defVal) {
         stimCoxDVal = defVal;
     }
     double getStimOffsetIn() {
         std::string key = getKeyPrefix() + "_STIM_OFFSET_IN";
-        return readParameter(key, stimOffsetInDVal);
+        return getDouble(key, stimOffsetInDVal);
     }
     void setStimOffsetInDefVal(double defVal) {
         stimOffsetInDVal = defVal;
     }
     double getStimOffsetOut() {
         std::string key = getKeyPrefix() + "_STIM_OFFSET_OUT";
-        return readParameter(key, stimOffsetOutDVal);
+        return getDouble(key, stimOffsetOutDVal);
     }
 
     void setStimOffsetOutDefVal(double defVal) {
@@ -82,21 +75,21 @@ public:
 
     double getStimOffsetDispersionIn() {
         std::string key = getKeyPrefix() + "_STIM_OFFSET_DISPERSION_IN";
-        return readParameter(key, stimOffsetDispersionInDVal);
+        return getDouble(key, stimOffsetDispersionInDVal);
     }
     void setStimOffsetDispersionInDefVal(double defVal) {
         stimOffsetDispersionInDVal = defVal;
     }
     double getStimOffsetDispersionOut() {
         std::string key = getKeyPrefix() + "_STIM_OFFSET_DISPERSION_OUT";
-        return readParameter(key, stimOffsetDispersionOutDVal);
+        return getDouble(key, stimOffsetDispersionOutDVal);
     }
     void setStimOffsetDispersionOutDefVal(double defVal) {
         stimOffsetDispersionOutDVal = defVal;
     }
     double getStimDivisorIn() {
         std::string key = getKeyPrefix() + "_STIM_DIVISOR_IN";
-        return readParameter(key, stimDivInDVal);
+        return getDouble(key, stimDivInDVal);
     }
 
     void setStimDivisorInDefVal(double defVal) {
@@ -105,7 +98,7 @@ public:
 
     double getStimDivisorOut() {
         std::string key = getKeyPrefix() + "_STIM_DIVISOR_OUT";
-        return readParameter(key, stimDivOutDVal);
+        return getDouble(key, stimDivOutDVal);
     }
 
     void setStimDivisorOutDefVal(double defVal) {
@@ -114,7 +107,7 @@ public:
 
     double getStimDivisorDispersionIn() {
         std::string key = getKeyPrefix() + "_STIM_DIVISOR_DISPERSION_IN";
-        return readParameter(key, stimDivDispersionInDVal);
+        return getDouble(key, stimDivDispersionInDVal);
     }
 
     void setStimDivisorDispersionInDefVal(double defVal) {
@@ -122,7 +115,7 @@ public:
     }
     double getStimDivisorDispersionOut() {
         std::string key = getKeyPrefix() + "_STIM_DIVISOR_DISPERSION_OUT";
-        return readParameter(key, stimDivDispersionOutDVal);
+        return getDouble(key, stimDivDispersionOutDVal);
     }
 
     void setStimDivisorDispersionOutDefVal(double defVal) {
@@ -130,14 +123,14 @@ public:
     }
     double getStimCoxExpIn() {
         std::string key = getKeyPrefix() + "_STIM_COX_EXP_IN";
-        return readParameter(key, stimCoxExpInDVal);
+        return getDouble(key, stimCoxExpInDVal);
     }
     void setStimCoxExpInDefVal(double defVal) {
         stimCoxExpInDVal = defVal;
     }
     double getStimCoxExpOut() {
         std::string key = getKeyPrefix() + "_STIM_COX_EXP_OUT";
-        return readParameter(key, stimCoxExpOutDVal);
+        return getDouble(key, stimCoxExpOutDVal);
     }
     void setStimCoxExpOutDefVal(double defVal) {
         stimCoxExpOutDVal = defVal;
@@ -145,14 +138,14 @@ public:
 
     double getStimCoxExpDispersionIn() {
         std::string key = getKeyPrefix() + "_STIM_COX_EXP_DISPERSION_IN";
-        return readParameter(key, stimCoxExpDispersionInDVal);
+        return getDouble(key, stimCoxExpDispersionInDVal);
     }
     void setStimCoxExpDispersionInDefVal(double defVal) {
         stimCoxExpDispersionInDVal = defVal;
     }
     double getStimCoxExpDispersionOut() {
         std::string key = getKeyPrefix() + "_STIM_COX_EXP_DISPERSION_OUT";
-        return readParameter(key, stimCoxExpDispersionOutDVal);
+        return getDouble(key, stimCoxExpDispersionOutDVal);
     }
     void setStimCoxExpDispersionOutDefVal(double defVal) {
         stimCoxExpDispersionOutDVal = defVal;
