@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // The parent class for traffic light logics
 /****************************************************************************/
@@ -151,6 +150,9 @@ public:
      */
     virtual SUMOTime trySwitch() = 0;
 
+    /// @brief called when switching programs
+    virtual void activateProgram();
+    virtual void deactivateProgram();
 
     /** @brief Applies the current signal states to controlled links
      * @param[in] t The current time
@@ -338,6 +340,8 @@ public:
 
     /// @}
 
+    /// @brief whether this logic is selected in the GUI
+    bool isSelected() const;
 
 protected:
     /**
@@ -433,6 +437,9 @@ protected:
     /// @brief list of indices that are ignored in mesoscopic simulatino
     std::set<int> myIgnoredIndices;
 
+    /// @brief whether the current program is active
+    bool myAmActive;
+
 private:
     /// @brief initialize optional meso penalties
     void initMesoTLSPenalties();
