@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2005-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2005-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, May 2005
-/// @version $Id$
 ///
 // Calibrates the flow on an edge by removing an inserting vehicles
 /****************************************************************************/
@@ -39,7 +38,7 @@
 #include <utils/xml/XMLSubSys.h>
 #include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
-#include <utils/xml/SUMOVehicleParserHelper.h>
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 #include <utils/distribution/RandomDistributor.h>
 #include <utils/vehicle/SUMOVehicleParameter.h>
 #include "MSCalibrator.h"
@@ -63,23 +62,27 @@ MSCalibrator::MSCalibrator(const std::string& id,
                            const std::string& outputFilename,
                            const SUMOTime freq, const double length,
                            const MSRouteProbe* probe,
+                           const std::string& vTypes,
                            bool addLaneMeanData) :
     MSTrigger(id),
     MSRouteHandler(aXMLFilename, true),
+    MSDetectorFileOutput(id, vTypes, false), // detecting persons not yet supported
     myEdge(edge),
     myLane(lane),
     myPos(pos), myProbe(probe),
     myEdgeMeanData(nullptr, length, false, nullptr),
+    myMeanDataParent(id + "_dummyMeanData", 0, 0, false, false, false, false, false, false, 1, 0, 0, vTypes),
     myCurrentStateInterval(myIntervals.begin()),
     myOutput(nullptr), myFrequency(freq), myRemoved(0),
     myInserted(0), myClearedInJam(0),
     mySpeedIsDefault(true), myDidSpeedAdaption(false), myDidInit(false),
     myDefaultSpeed(myLane == nullptr ? myEdge->getSpeedLimit() : myLane->getSpeedLimit()),
     myHaveWarnedAboutClearingJam(false),
-    myAmActive(false) {
+    myAmActive(false),
+    myHaveInvalidJam(false) {
     if (outputFilename != "") {
         myOutput = &OutputDevice::getDevice(outputFilename);
-        myOutput->writeXMLHeader("calibratorstats", "calibratorstats_file.xsd");
+        writeXMLDetectorProlog(*myOutput);
     }
     if (aXMLFilename != "") {
         XMLSubSys::runParser(*this, aXMLFilename);
@@ -93,7 +96,7 @@ MSCalibrator::MSCalibrator(const std::string& id,
             MSLane* lane = myEdge->getLanes()[i];
             if (myLane == nullptr || myLane == lane) {
                 //std::cout << " cali=" << getID() << " myLane=" << Named::getIDSecure(myLane) << " checkLane=" << i << "\n";
-                MSMeanData_Net::MSLaneMeanDataValues* laneData = new MSMeanData_Net::MSLaneMeanDataValues(lane, lane->getLength(), true, nullptr);
+                MSMeanData_Net::MSLaneMeanDataValues* laneData = new MSMeanData_Net::MSLaneMeanDataValues(lane, lane->getLength(), true, &myMeanDataParent);
                 laneData->setDescription("meandata_calibrator_" + lane->getID());
                 LeftoverReminders.push_back(laneData);
                 myLaneMeanData.push_back(laneData);
@@ -115,7 +118,7 @@ MSCalibrator::init() {
         // calibration should happen after regular insertions have taken place
         MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(new CalibratorCommand(this));
     } else {
-        WRITE_WARNING("No flow intervals in calibrator '" + myID + "'.");
+        WRITE_WARNING("No flow intervals in calibrator '" + getID() + "'.");
     }
     myDidInit = true;
 }
@@ -123,7 +126,7 @@ MSCalibrator::init() {
 
 MSCalibrator::~MSCalibrator() {
     if (myCurrentStateInterval != myIntervals.end()) {
-        writeXMLOutput();
+        intervalEnd();
     }
     for (std::vector<VehicleRemover*>::iterator it = myVehicleRemovers.begin(); it != myVehicleRemovers.end(); ++it) {
         (*it)->disable();
@@ -147,12 +150,12 @@ MSCalibrator::myStartElement(int element,
             bool ok = true;
             state.q = attrs.getOpt<double>(SUMO_ATTR_VEHSPERHOUR, nullptr, ok, -1.);
             state.v = attrs.getOpt<double>(SUMO_ATTR_SPEED, nullptr, ok, -1.);
-            state.begin = attrs.getSUMOTimeReporting(SUMO_ATTR_BEGIN, myID.c_str(), ok);
+            state.begin = attrs.getSUMOTimeReporting(SUMO_ATTR_BEGIN, getID().c_str(), ok);
             if (state.begin < lastEnd) {
-                WRITE_ERROR("Overlapping or unsorted intervals in calibrator '" + myID + "'.");
+                WRITE_ERROR("Overlapping or unsorted intervals in calibrator '" + getID() + "'.");
             }
-            state.end = attrs.getOptSUMOTimeReporting(SUMO_ATTR_END, myID.c_str(), ok, -1);
-            state.vehicleParameter = SUMOVehicleParserHelper::parseVehicleAttributes(attrs, true, true);
+            state.end = attrs.getOptSUMOTimeReporting(SUMO_ATTR_END, getID().c_str(), ok, -1);
+            state.vehicleParameter = SUMOVehicleParserHelper::parseVehicleAttributes(attrs, true, true, true);
             LeftoverVehicleParameters.push_back(state.vehicleParameter);
             // vehicles should be inserted with max speed unless stated otherwise
             if (state.vehicleParameter->departSpeedProcedure == DEPART_SPEED_DEFAULT) {
@@ -173,15 +176,15 @@ MSCalibrator::myStartElement(int element,
             }
             if (state.vehicleParameter->vtypeid != DEFAULT_VTYPE_ID &&
                     MSNet::getInstance()->getVehicleControl().getVType(state.vehicleParameter->vtypeid) == nullptr) {
-                WRITE_ERROR("Unknown vehicle type '" + state.vehicleParameter->vtypeid + "' in calibrator '" + myID + "'.");
+                WRITE_ERROR("Unknown vehicle type '" + state.vehicleParameter->vtypeid + "' in calibrator '" + getID() + "'.");
             }
         } catch (EmptyData&) {
-            WRITE_ERROR("Mandatory attribute missing in definition of calibrator '" + myID + "'.");
+            WRITE_ERROR("Mandatory attribute missing in definition of calibrator '" + getID() + "'.");
         } catch (NumberFormatException&) {
-            WRITE_ERROR("Non-numeric value for numeric attribute in definition of calibrator '" + myID + "'.");
+            WRITE_ERROR("Non-numeric value for numeric attribute in definition of calibrator '" + getID() + "'.");
         }
         if (state.q < 0 && state.v < 0) {
-            WRITE_ERROR("Either 'vehsPerHour' or 'speed' has to be given in flow definition of calibrator '" + myID + "'.");
+            WRITE_ERROR("Either 'vehsPerHour' or 'speed' has to be given in flow definition of calibrator '" + getID() + "'.");
         }
         if (myIntervals.size() > 0 && myIntervals.back().end == -1) {
             myIntervals.back().end = state.begin;
@@ -207,28 +210,9 @@ MSCalibrator::myEndElement(int element) {
 
 
 void
-MSCalibrator::writeXMLOutput() {
+MSCalibrator::intervalEnd() {
     if (myOutput != nullptr) {
-        updateMeanData();
-        const int p = passed();
-        // meandata will be off if vehicles are removed on the next edge instead of this one
-        const int discrepancy = myEdgeMeanData.nVehEntered + myEdgeMeanData.nVehDeparted - myEdgeMeanData.nVehVaporized - passed();
-        assert(discrepancy >= 0);
-        const std::string ds = (discrepancy > 0 ? "\" vaporizedOnNextEdge=\"" + toString(discrepancy) : "");
-        const double durationSeconds = STEPS2TIME(myCurrentStateInterval->end - myCurrentStateInterval->begin);
-        (*myOutput) << "    <interval begin=\"" << time2string(myCurrentStateInterval->begin) <<
-                    "\" end=\"" << time2string(myCurrentStateInterval->end) <<
-                    "\" id=\"" << myID <<
-                    "\" nVehContrib=\"" << p <<
-                    "\" removed=\"" << myRemoved <<
-                    "\" inserted=\"" << myInserted <<
-                    "\" cleared=\"" << myClearedInJam <<
-                    "\" flow=\"" << p * 3600.0 / durationSeconds <<
-                    "\" aspiredFlow=\"" << myCurrentStateInterval->q <<
-                    "\" speed=\"" << myEdgeMeanData.getTravelledDistance() / myEdgeMeanData.getSamples() <<
-                    "\" aspiredSpeed=\"" << myCurrentStateInterval->v <<
-                    ds << //optional
-                    "\"/>\n";
+        writeXMLOutput(*myOutput, myCurrentStateInterval->begin, myCurrentStateInterval->end);
     }
     myDidSpeedAdaption = false;
     myInserted = 0;
@@ -287,7 +271,7 @@ MSCalibrator::removePending() {
             if (vehicle != nullptr) {
                 vehicle->onRemovalFromNet(MSMoveReminder::NOTIFICATION_VAPORIZED);
                 vehicle->getLane()->removeVehicle(vehicle, MSMoveReminder::NOTIFICATION_VAPORIZED);
-                vc.scheduleVehicleRemoval(vehicle);
+                vc.scheduleVehicleRemoval(vehicle, true);
             } else {
                 WRITE_WARNING("Calibrator '" + getID() + "' could not remove vehicle '" + *it + "'.");
             }
@@ -302,6 +286,7 @@ MSCalibrator::removePending() {
 SUMOTime
 MSCalibrator::execute(SUMOTime currentTime) {
     // get current simulation values (valid for the last simulation second)
+    myHaveInvalidJam = invalidJam(myLane == 0 ? -1 : myLane->getIndex());
     // XXX could we miss vehicle movements if this is called less often than every DELTA_T (default) ?
     updateMeanData();
     const bool hadRemovals = removePending();
@@ -342,11 +327,11 @@ MSCalibrator::execute(SUMOTime currentTime) {
     const int totalWishedNum = totalWished();
     int adaptedNum = passed() + myClearedInJam;
 #ifdef MSCalibrator_DEBUG
-    std::cout << time2string(currentTime) << " " << myID
+    std::cout << time2string(currentTime) << " " << getID()
               << " q=" << myCurrentStateInterval->q
               << " totalWished=" << totalWishedNum
               << " adapted=" << adaptedNum
-              << " jam=" << invalidJam(myLane == 0 ? -1 : myLane->getIndex())
+              << " jam=" << myHaveInvalidJam
               << " entered=" << myEdgeMeanData.nVehEntered
               << " departed=" << myEdgeMeanData.nVehDeparted
               << " arrived=" << myEdgeMeanData.nVehArrived
@@ -379,11 +364,11 @@ MSCalibrator::execute(SUMOTime currentTime) {
                 route = MSRoute::dictionary(pars->routeid);
             }
             if (route == nullptr) {
-                WRITE_WARNING("No valid routes in calibrator '" + myID + "'.");
+                WRITE_WARNING("No valid routes in calibrator '" + getID() + "'.");
                 break;
             }
             if (!route->contains(myEdge)) {
-                WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + myID + "' does not contain edge '" + myEdge->getID() + "'.");
+                WRITE_WARNING("Route '" + route->getID() + "' in calibrator '" + getID() + "' does not contain edge '" + myEdge->getID() + "'.");
                 break;
             }
             const int routeIndex = (int)std::distance(route->begin(),
@@ -392,9 +377,10 @@ MSCalibrator::execute(SUMOTime currentTime) {
             assert(route != 0 && vtype != 0);
             // build the vehicle
             SUMOVehicleParameter* newPars = new SUMOVehicleParameter(*pars);
-            newPars->id = myID + "." + toString((int)STEPS2TIME(myCurrentStateInterval->begin)) + "." + toString(myInserted);
+            newPars->id = getNewVehicleID();
             newPars->depart = currentTime;
             newPars->routeid = route->getID();
+            newPars->departLaneProcedure = DEPART_LANE_FIRST_ALLOWED; // ensure successful vehicle creation
             MSVehicle* vehicle;
             try {
                 vehicle = dynamic_cast<MSVehicle*>(MSNet::getInstance()->getVehicleControl().buildVehicle(
@@ -411,7 +397,7 @@ MSCalibrator::execute(SUMOTime currentTime) {
 #ifdef MSCalibrator_DEBUG
             std::cout << " resetting route pos: " << routeIndex << "\n";
 #endif
-            vehicle->resetRoutePosition(routeIndex);
+            vehicle->resetRoutePosition(routeIndex, pars->departLaneProcedure);
             if (myEdge->insertVehicle(*vehicle, currentTime)) {
                 if (!MSNet::getInstance()->getVehicleControl().addVehicle(vehicle->getID(), vehicle)) {
                     throw ProcessError("Emission of vehicle '" + vehicle->getID() + "' in calibrator '" + getID() + "'failed!");
@@ -432,7 +418,7 @@ MSCalibrator::execute(SUMOTime currentTime) {
         }
     }
     if (myCurrentStateInterval->end <= currentTime + myFrequency) {
-        writeXMLOutput();
+        intervalEnd();
     }
     return myFrequency;
 }
@@ -485,11 +471,12 @@ MSCalibrator::remainingVehicleCapacity(int laneIndex) const {
     const SUMOVehicleParameter* pars = myCurrentStateInterval->vehicleParameter;
     const MSVehicleType* vtype = MSNet::getInstance()->getVehicleControl().getVType(pars->vtypeid);
     const double spacePerVehicle = vtype->getLengthWithGap() + myEdge->getSpeedLimit() * vtype->getCarFollowModel().getHeadwayTime();
-    if (last == nullptr) {
-        // ensure vehicles can be inserted on short edges
-        return MAX2(1, (int)(myEdge->getLength() / spacePerVehicle));
+    int overallSpaceLeft = (int)ceil(lane->getLength() / spacePerVehicle) - lane->getVehicleNumber();
+    if (last != nullptr) {
+        int entrySpaceLeft = (int)(last->getPositionOnLane() / spacePerVehicle);
+        return MAX2(overallSpaceLeft, entrySpaceLeft);
     } else {
-        return (int)(last->getPositionOnLane() / spacePerVehicle);
+        return overallSpaceLeft;
     }
 }
 
@@ -517,10 +504,15 @@ MSCalibrator::updateMeanData() {
     }
 }
 
-bool MSCalibrator::VehicleRemover::notifyEnter(SUMOVehicle& veh, Notification /* reason */, const MSLane* /* enteredLane */) {
+
+bool
+MSCalibrator::VehicleRemover::notifyEnter(SUMOTrafficObject& veh, Notification /* reason */, const MSLane* /* enteredLane */) {
     if (myParent == nullptr) {
         return false;
     }
+    if (!myParent->vehicleApplies(veh)) {
+        return false;
+    }
     if (myParent->isActive()) {
         myParent->updateMeanData();
         const bool calibrateFlow = myParent->myCurrentStateInterval->q >= 0;
@@ -535,13 +527,13 @@ bool MSCalibrator::VehicleRemover::notifyEnter(SUMOVehicle& veh, Notification /*
             if (myParent->scheduleRemoval(vehicle)) {
                 myParent->myRemoved++;
             }
-        } else if (myParent->invalidJam(myLaneIndex)) {
+        } else if (myParent->myHaveInvalidJam) {
 #ifdef MSCalibrator_DEBUG
             std::cout << time2string(MSNet::getInstance()->getCurrentTimeStep()) << " " << myParent->getID()
                       << " vaporizing " << vehicle->getID() << " to clear jam\n";
 #endif
             if (!myParent->myHaveWarnedAboutClearingJam) {
-                WRITE_WARNING("Clearing jam at calibrator '" + myParent->myID + "' at time "
+                WRITE_WARNING("Clearing jam at calibrator '" + myParent->getID() + "' at time "
                               + time2string(MSNet::getInstance()->getCurrentTimeStep()));
                 myParent->myHaveWarnedAboutClearingJam = true;
             }
@@ -554,6 +546,42 @@ bool MSCalibrator::VehicleRemover::notifyEnter(SUMOVehicle& veh, Notification /*
 }
 
 
+void
+MSCalibrator::writeXMLOutput(OutputDevice& dev, SUMOTime startTime, SUMOTime stopTime) {
+    updateMeanData();
+    const int p = passed();
+    // meandata will be off if vehicles are removed on the next edge instead of this one
+    const int discrepancy = myEdgeMeanData.nVehEntered + myEdgeMeanData.nVehDeparted - myEdgeMeanData.nVehVaporized - passed();
+    assert(discrepancy >= 0);
+    const std::string ds = (discrepancy > 0 ? "\" vaporizedOnNextEdge=\"" + toString(discrepancy) : "");
+    const double durationSeconds = STEPS2TIME(stopTime - startTime);
+    dev << "    <interval begin=\"" << time2string(startTime) <<
+        "\" end=\"" << time2string(stopTime) <<
+        "\" id=\"" << getID() <<
+        "\" nVehContrib=\"" << p <<
+        "\" removed=\"" << myRemoved <<
+        "\" inserted=\"" << myInserted <<
+        "\" cleared=\"" << myClearedInJam <<
+        "\" flow=\"" << p * 3600.0 / durationSeconds <<
+        "\" aspiredFlow=\"" << myCurrentStateInterval->q <<
+        "\" speed=\"" << myEdgeMeanData.getTravelledDistance() / myEdgeMeanData.getSamples() <<
+        "\" aspiredSpeed=\"" << myCurrentStateInterval->v <<
+        ds << //optional
+        "\"/>\n";
+}
+
+void
+MSCalibrator::writeXMLDetectorProlog(OutputDevice& dev) const {
+    dev.writeXMLHeader("calibratorstats", "calibratorstats_file.xsd");
+}
+
+std::string
+MSCalibrator::getNewVehicleID() {
+    // avoid name clash for subsecond interval spacing
+    const double beginS = STEPS2TIME(myCurrentStateInterval->begin);
+    const int precision = beginS == int(beginS) ? 0 : 2;
+    return getID() + "." + toString(beginS, precision) + "." + toString(myInserted);
+}
 
 /****************************************************************************/
 
