@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2012-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2012-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    02. March 2012
-/// @version $Id$
 ///
 // Algorithms for network computation
 /****************************************************************************/
@@ -60,6 +59,7 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node, bool war
         (*k)->setTurningDestination(nullptr);
     }
     std::vector<Combination> combinations;
+    const bool geometryLike = node->geometryLike();
     for (std::vector<NBEdge*>::const_iterator j = outgoing.begin(); j != outgoing.end(); ++j) {
         NBEdge* outedge = *j;
         for (std::vector<NBEdge*>::const_iterator k = incoming.begin(); k != incoming.end(); ++k) {
@@ -73,7 +73,10 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node, bool war
             }
             double angle = fabs(signedAngle);
             // std::cout << "incoming=" << e->getID() << " outgoing=" << outedge->getID() << " relAngle=" << NBHelpers::relAngle(e->getAngleAtNode(node), outedge->getAngleAtNode(node)) << "\n";
-            if (e->getFromNode() == outedge->getToNode() && angle > 120) {
+            const bool badPermissions = ((outedge->getPermissions() & e->getPermissions() & ~SVC_PEDESTRIAN) == 0
+                                         && !geometryLike
+                                         && outedge->getPermissions() != e->getPermissions());
+            if (e->getFromNode() == outedge->getToNode() && angle > 120 && !badPermissions) {
                 // they connect the same nodes; should be the turnaround direction
                 // we'll assign a maximum number
                 //
@@ -89,6 +92,10 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node, bool war
             if (angle < 160) {
                 continue;
             }
+            if (badPermissions) {
+                // penalty
+                angle -= 90;
+            }
             Combination c;
             c.from = e;
             c.to = outedge;
@@ -99,11 +106,14 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node, bool war
     // sort combinations so that the ones with the highest angle are at the begin
     std::sort(combinations.begin(), combinations.end(), combination_by_angle_sorter());
     std::set<NBEdge*> seen;
+    //std::cout << "check combinations at " << node->getID() << "\n";
     for (std::vector<Combination>::const_iterator j = combinations.begin(); j != combinations.end(); ++j) {
+        //std::cout << " from=" << (*j).from->getID() << " to=" << (*j).to->getID() << " a=" << (*j).angle << "\n";
         if (seen.find((*j).from) != seen.end() || seen.find((*j).to) != seen.end()) {
             // do not regard already set edges
             if ((*j).angle > 360 && warn) {
-                WRITE_WARNING("Ambiguity in turnarounds computation at junction '" + node->getID() + "'.");
+                WRITE_WARNINGF("Ambiguity in turnarounds computation at junction '%'.", node->getID());
+                //std::cout << "  already seen: " << toString(seen) << "\n";
                 warn = false;
             }
             continue;
@@ -152,16 +162,25 @@ NBNodesEdgesSorter::swapWhenReversed(const NBNode* const n,
 void
 NBNodeTypeComputer::computeNodeTypes(NBNodeCont& nc, NBTrafficLightLogicCont& tlc) {
     validateRailCrossings(nc, tlc);
+    const OptionsCont& oc = OptionsCont::getOptions();
+    const double rightBeforeLeftSpeed = oc.getFloat("junctions.right-before-left.speed-threshold");
     for (std::map<std::string, NBNode*>::const_iterator i = nc.begin(); i != nc.end(); ++i) {
-        NBNode* n = (*i).second;
+        NBNode* const n = (*i).second;
         // the type may already be set from the data
         if (n->myType != NODETYPE_UNKNOWN && n->myType != NODETYPE_DEAD_END) {
+            n->myTypeWasGuessed = false;
+            continue;
+        }
+        // check whether the node was set to be unregulated by the user
+        if (oc.getBool("keep-nodes-unregulated") || oc.isInStringVector("keep-nodes-unregulated.explicit", n->getID())
+                || (oc.getBool("keep-nodes-unregulated.district-nodes") && (n->isNearDistrict() || n->isDistrict()))) {
+            n->myType = NODETYPE_NOJUNCTION;
             continue;
         }
         // check whether the node is a waterway node. Set to unregulated by default
         bool waterway = true;
-        for (EdgeVector::const_iterator i = n->getEdges().begin(); i != n->getEdges().end(); ++i) {
-            if (!isWaterway((*i)->getPermissions())) {
+        for (NBEdge* e : n->getEdges()) {
+            if (!isWaterway(e->getPermissions())) {
                 waterway = false;
                 break;
             }
@@ -181,6 +200,11 @@ NBNodeTypeComputer::computeNodeTypes(NBNodeCont& nc, NBTrafficLightLogicCont& tl
             n->myType = NODETYPE_PRIORITY;
             continue;
         }
+        if (isRailwayNode(n)) {
+            // priority instead of unregulated to ensure that collisions can be detected
+            n->myType = NODETYPE_PRIORITY;
+            continue;
+        }
         // determine the type
         SumoXMLNodeType type = NODETYPE_RIGHT_BEFORE_LEFT;
         for (EdgeVector::const_iterator i = n->myIncomingEdges.begin(); i != n->myIncomingEdges.end(); i++) {
@@ -191,11 +215,11 @@ NBNodeTypeComputer::computeNodeTypes(NBNodeCont& nc, NBTrafficLightLogicCont& tl
                 }
                 // @todo check against a legal document
                 // @todo figure out when NODETYPE_PRIORITY_STOP is appropriate
-                const double s1 = (*i)->getSpeed() * (double) 3.6;
-                const double s2 = (*j)->getSpeed() * (double) 3.6;
+                const double s1 = (*i)->getSpeed();
+                const double s2 = (*j)->getSpeed();
                 const int p1 = (*i)->getPriority();
                 const int p2 = (*j)->getPriority();
-                if (fabs(s1 - s2) > (double) 9.5 || MAX2(s1, s2) >= (double) 49. || p1 != p2) {
+                if (fabs(s1 - s2) > (9.5 / 3.6) || MAX2(s1, s2) >= rightBeforeLeftSpeed || p1 != p2) {
                     type = NODETYPE_PRIORITY;
                     break;
                 }
@@ -203,6 +227,7 @@ NBNodeTypeComputer::computeNodeTypes(NBNodeCont& nc, NBTrafficLightLogicCont& tl
         }
         // save type
         n->myType = type;
+        n->myTypeWasGuessed = true;
     }
 }
 
@@ -236,7 +261,7 @@ NBNodeTypeComputer::validateRailCrossings(NBNodeCont& nc, NBTrafficLightLogicCon
                 n->myType = NODETYPE_PRIORITY;
             } else if (numNonRailwayNonPed > 2) {
                 // does not look like a rail crossing (roads in conflict). maybe a traffic light?
-                WRITE_WARNING("Converting invalid rail_crossing to traffic_light at junction '" + n->getID() + "'");
+                WRITE_WARNINGF("Converting invalid rail_crossing to traffic_light at junction '%'.", n->getID());
                 TrafficLightType type = SUMOXMLDefinitions::TrafficLightTypes.get(OptionsCont::getOptions().getString("tls.default-type"));
                 NBTrafficLightDefinition* tlDef = new NBOwnTLDef(n->getID(), n, 0, type);
                 n->myType = NODETYPE_TRAFFIC_LIGHT;
@@ -245,7 +270,7 @@ NBNodeTypeComputer::validateRailCrossings(NBNodeCont& nc, NBTrafficLightLogicCon
                     n->removeTrafficLight(tlDef);
                     n->myType = NODETYPE_PRIORITY;
                     delete tlDef;
-                    WRITE_WARNING("Could not allocate tls '" + n->getID() + "'.");
+                    WRITE_WARNINGF("Could not allocate tls '%'.", n->getID());
                 }
             }
         }
@@ -254,7 +279,7 @@ NBNodeTypeComputer::validateRailCrossings(NBNodeCont& nc, NBTrafficLightLogicCon
 
 
 bool
-NBNodeTypeComputer::isRailwayNode(NBNode* n) {
+NBNodeTypeComputer::isRailwayNode(const NBNode* n) {
     int numRailway = 0;
     int numNonRailway = 0;
     for (NBEdge* e : n->getIncomingEdges()) {
@@ -272,31 +297,25 @@ NBNodeTypeComputer::isRailwayNode(NBNode* n) {
 // ---------------------------------------------------------------------------
 void
 NBEdgePriorityComputer::computeEdgePriorities(NBNodeCont& nc) {
-    for (std::map<std::string, NBNode*>::const_iterator i = nc.begin(); i != nc.end(); ++i) {
-        computeEdgePrioritiesSingleNode((*i).second);
-    }
-}
-
-
-void
-NBEdgePriorityComputer::computeEdgePrioritiesSingleNode(NBNode* node) {
-    // preset all junction's edge priorities to zero
-    for (EdgeVector::iterator j = node->myAllEdges.begin(); j != node->myAllEdges.end(); ++j) {
-        (*j)->setJunctionPriority(node, NBEdge::MINOR_ROAD);
-    }
-    node->markBentPriority(false);
-    // check if the junction is not a real junction
-    if (node->myIncomingEdges.size() == 1 && node->myOutgoingEdges.size() == 1) {
-        return;
-    }
-    // compute the priorities on junction when needed
-    if (node->getType() != NODETYPE_RIGHT_BEFORE_LEFT) {
-        if (node->getRightOfWay() == RIGHT_OF_WAY_EDGEPRIORITY) {
-            for (NBEdge* e : node->getIncomingEdges()) {
-                e->setJunctionPriority(node, e->getPriority());
+    for (const auto& node : nc) {
+        // preset all junction's edge priorities to zero
+        for (NBEdge* const edge : node.second->myAllEdges) {
+            edge->setJunctionPriority(node.second, NBEdge::MINOR_ROAD);
+        }
+        node.second->markBentPriority(false);
+        // check if the junction is not a real junction
+        if (node.second->myIncomingEdges.size() == 1 && node.second->myOutgoingEdges.size() == 1) {
+            continue;
+        }
+        // compute the priorities on junction when needed
+        if (node.second->getType() != NODETYPE_RIGHT_BEFORE_LEFT && node.second->getType() != NODETYPE_ALLWAY_STOP && node.second->getType() != NODETYPE_NOJUNCTION) {
+            if (node.second->getRightOfWay() == RIGHT_OF_WAY_EDGEPRIORITY) {
+                for (NBEdge* e : node.second->getIncomingEdges()) {
+                    e->setJunctionPriority(node.second, e->getPriority());
+                }
+            } else {
+                setPriorityJunctionPriorities(*node.second);
             }
-        } else {
-            setPriorityJunctionPriorities(*node);
         }
     }
 }
@@ -426,6 +445,7 @@ NBEdgePriorityComputer::setPriorityJunctionPriorities(NBNode& n) {
     markBestParallel(n, bestFirst, bestSecond);
 }
 
+
 void
 NBEdgePriorityComputer::markBestParallel(const NBNode& n, NBEdge* bestFirst, NBEdge* bestSecond) {
     // edges running parallel to the main direction should also be prioritised
