@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2012-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2012-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    29. March 2012
-/// @version $Id$
 ///
 // Algorithms for highway on-/off-ramps computation
 /****************************************************************************/
@@ -177,12 +176,12 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
     NBEdge* last = cont;
     NBEdge* curr = cont;
     std::set<NBEdge*> incremented;
-    if (addLanes && toAdd > 0 && find(incremented.begin(), incremented.end(), cont) == incremented.end()) {
+    if (addLanes && toAdd > 0 && std::find(incremented.begin(), incremented.end(), cont) == incremented.end()) {
         double currLength = 0;
         while (curr != nullptr && currLength + curr->getGeometry().length() - POSITION_EPS < rampLength) {
             if (find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
                 curr->incLaneNo(toAdd);
-                if (curr->getStep() < NBEdge::LANES2LANES_USER) {
+                if (curr->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER) {
                     curr->invalidateConnections(true);
                 }
                 incremented.insert(curr);
@@ -195,7 +194,7 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
                 }
             }
             NBNode* nextN = curr->getToNode();
-            if (nextN->getOutgoingEdges().size() == 1) {
+            if (nextN->getOutgoingEdges().size() == 1 && nextN->getIncomingEdges().size() == 1) {
                 curr = nextN->getOutgoingEdges()[0];
                 if (curr->getNumLanes() != firstLaneNumber) {
                     // the number of lanes changes along the computation; we'll stop...
@@ -213,7 +212,7 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
             }
         }
         // check whether a further split is necessary
-        if (curr != nullptr && !dontSplit && currLength - POSITION_EPS < rampLength && curr->getNumLanes() == firstLaneNumber && find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
+        if (curr != nullptr && !dontSplit && currLength - POSITION_EPS < rampLength && curr->getNumLanes() == firstLaneNumber && std::find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
             // there is enough place to build a ramp; do it
             bool wasFirst = first == curr;
             NBNode* rn = new NBNode(curr->getID() + "-AddedOnRampNode", curr->getGeometry().positionAtOffset(rampLength - currLength));
@@ -251,12 +250,12 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
     }
     // set connections from ramp/highway to added ramp
     if (addLanes) {
-        if (potHighway->getStep() < NBEdge::LANES2LANES_USER) {
+        if (potHighway->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER) {
             if (!potHighway->addLane2LaneConnections(0, first, potRamp->getNumLanes(), MIN2(first->getNumLanes() - potRamp->getNumLanes(), potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true)) {
                 throw ProcessError("Could not set connection!");
             }
         }
-        if (potRamp->getStep() < NBEdge::LANES2LANES_USER) {
+        if (potRamp->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER) {
             if (!potRamp->addLane2LaneConnections(0, first, 0, potRamp->getNumLanes(), NBEdge::L2L_VALIDATED, true)) {
                 throw ProcessError("Could not set connection!");
             }
@@ -282,12 +281,12 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
     NBEdge* last = prev;
     NBEdge* curr = prev;
     std::set<NBEdge*> incremented;
-    if (toAdd > 0 && find(incremented.begin(), incremented.end(), prev) == incremented.end()) {
+    if (toAdd > 0 && std::find(incremented.begin(), incremented.end(), prev) == incremented.end()) {
         double currLength = 0;
         while (curr != nullptr && currLength + curr->getGeometry().length() - POSITION_EPS < rampLength) {
             if (find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
                 curr->incLaneNo(toAdd);
-                if (curr->getStep() < NBEdge::LANES2LANES_USER) {
+                if (curr->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER) {
                     curr->invalidateConnections(true);
                 }
                 incremented.insert(curr);
@@ -296,9 +295,9 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
                 last = curr;
             }
             NBNode* prevN = curr->getFromNode();
-            if (prevN->getIncomingEdges().size() == 1) {
+            if (prevN->getIncomingEdges().size() == 1 && prevN->getOutgoingEdges().size() == 1) {
                 curr = prevN->getIncomingEdges()[0];
-                if (curr->getStep() < NBEdge::LANES2LANES_USER && toAdd != 0) {
+                if (curr->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER && toAdd != 0) {
                     // curr might be an onRamp. In this case connections need to be rebuilt
                     curr->invalidateConnections();
                 }
@@ -318,7 +317,7 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
             }
         }
         // check whether a further split is necessary
-        if (curr != nullptr && !dontSplit && currLength - POSITION_EPS < rampLength && curr->getNumLanes() == firstLaneNumber && find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
+        if (curr != nullptr && !dontSplit && currLength - POSITION_EPS < rampLength && curr->getNumLanes() == firstLaneNumber && std::find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
             // there is enough place to build a ramp; do it
             bool wasFirst = first == curr;
             Position pos = curr->getGeometry().positionAtOffset(curr->getGeometry().length() - (rampLength  - currLength));
@@ -346,7 +345,7 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
         }
     }
     // set connections from added ramp to ramp/highway
-    if (first->getStep() < NBEdge::LANES2LANES_USER) {
+    if (first->getStep() < NBEdge::EdgeBuildingStep::LANES2LANES_USER) {
         if (!first->addLane2LaneConnections(potRamp->getNumLanes(), potHighway, 0, MIN2(first->getNumLanes() - 1, potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true)) {
             throw ProcessError("Could not set connection!");
         }
@@ -501,7 +500,8 @@ NBRampsComputer::fulfillsRampConstraints(
         }
     }
     // are the angles between highway and other / ramp and other more or less straight?
-    const NBNode* node = potHighway->getToNode() == potRamp->getToNode() ? potHighway->getToNode() : potHighway->getFromNode();
+    const NBNode* node = ((potHighway->getToNode() == potRamp->getToNode() && potHighway->getToNode() == other->getFromNode())
+                          ? potHighway->getToNode() : potHighway->getFromNode());
     double angle = fabs(NBHelpers::relAngle(potHighway->getAngleAtNode(node), other->getAngleAtNode(node)));
     if (angle >= 60) {
         return false;
