@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // The class holds a description of a connection between two edges
 /****************************************************************************/
@@ -44,7 +43,8 @@ NBConnection::NBConnection(NBEdge* from, NBEdge* to) :
     myFrom(from), myTo(to),
     myFromID(from->getID()), myToID(to->getID()),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex) {
+    myTlIndex(InvalidTlIndex),
+    myTlIndex2(InvalidTlIndex) {
 }
 
 
@@ -53,15 +53,17 @@ NBConnection::NBConnection(const std::string& fromID, NBEdge* from,
     myFrom(from), myTo(to),
     myFromID(fromID), myToID(toID),
     myFromLane(-1), myToLane(-1),
-    myTlIndex(InvalidTlIndex) {
+    myTlIndex(InvalidTlIndex),
+    myTlIndex2(InvalidTlIndex) {
 }
 
 
 NBConnection::NBConnection(NBEdge* from, int fromLane,
-                           NBEdge* to, int toLane, int tlIndex) :
+                           NBEdge* to, int toLane, int tlIndex, int tlIndex2) :
     myFrom(from), myTo(to),
     myFromLane(fromLane), myToLane(toLane),
-    myTlIndex(tlIndex) {
+    myTlIndex(tlIndex),
+    myTlIndex2(tlIndex2) {
     /* @todo what should we assert here?
     assert(myFromLane<0||from->getNumLanes()>(int) myFromLane);
     assert(myToLane<0||to->getNumLanes()>(int) myToLane);
@@ -78,7 +80,8 @@ NBConnection::NBConnection(const NBConnection& c) :
     myFrom(c.myFrom), myTo(c.myTo),
     myFromID(c.myFromID), myToID(c.myToID),
     myFromLane(c.myFromLane), myToLane(c.myToLane),
-    myTlIndex(c.myTlIndex) {
+    myTlIndex(c.myTlIndex),
+    myTlIndex2(c.myTlIndex2) {
 }
 
 
@@ -182,7 +185,8 @@ NBConnection::operator==(const NBConnection& c) const {
     return (myFrom    == c.myFrom     && myTo    == c.myTo &&
             myFromID  == c.myFromID   && myToID  == c.myToID &&
             myFromLane == c.myFromLane && myToLane == c.myToLane &&
-            myTlIndex == c.myTlIndex);
+            myTlIndex == c.myTlIndex &&
+            myTlIndex2 == c.myTlIndex2);
 }
 
 
