@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // A class representing a single district
 /****************************************************************************/
@@ -68,7 +67,7 @@ NBDistrict::mirrorX() {
 
 bool
 NBDistrict::addSource(NBEdge* const source, double weight) {
-    EdgeVector::iterator i = find(mySources.begin(), mySources.end(), source);
+    EdgeVector::iterator i = std::find(mySources.begin(), mySources.end(), source);
     if (i != mySources.end()) {
         return false;
     }
@@ -81,7 +80,7 @@ NBDistrict::addSource(NBEdge* const source, double weight) {
 
 bool
 NBDistrict::addSink(NBEdge* const sink, double weight) {
-    EdgeVector::iterator i = find(mySinks.begin(), mySinks.end(), sink);
+    EdgeVector::iterator i = std::find(mySinks.begin(), mySinks.end(), sink);
     if (i != mySinks.end()) {
         return false;
     }
