@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // The representation of a single edge during network building
 /****************************************************************************/
@@ -58,11 +57,38 @@ class NBVehicle;
 // ===========================================================================
 // class definitions
 // ===========================================================================
+/**
+ * @class NBRouterEdge
+ * @brief Superclass for NBEdge and NBEdge::Connection to initialize Router
+ */
+class NBRouterEdge {
+public:
+    virtual const std::string& getID() const = 0;
+    virtual double getSpeed() const = 0;
+    virtual double getLength() const = 0;
+    virtual int getNumericalID() const = 0;
+    virtual const ConstRouterEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const = 0;
+    virtual bool isInternal() const {
+        return false;
+    }
+    inline bool prohibits(const NBVehicle* const /*veh*/) const {
+        return false;
+    }
+    inline bool restricts(const NBVehicle* const /*veh*/) const {
+        return false;
+    }
+
+    static inline double getTravelTimeStatic(const NBRouterEdge* const edge, const NBVehicle* const /*veh*/, double /*time*/) {
+        return edge->getLength() / edge->getSpeed();
+    }
+};
+
+
 /**
  * @class NBEdge
  * @brief The representation of a single edge during network building
  */
-class NBEdge : public Named, public Parameterised {
+class NBEdge : public Named, public Parameterised, public NBRouterEdge {
     friend class NBEdgeCont;
 
     /** used for visualization (NETEDIT) */
@@ -79,7 +105,7 @@ public:
      *  information, a counter holding the current step is needed. This is done
      *  by using this enumeration.
      */
-    enum EdgeBuildingStep {
+    enum class EdgeBuildingStep {
         /// @brief The edge has been loaded and connections shall not be added
         INIT_REJECT_CONNECTIONS,
         /// @brief The edge has been loaded, nothing is computed yet
@@ -113,7 +139,7 @@ public:
     /** @struct Lane
      * @brief An (internal) definition of a single lane of an edge
      */
-    struct Lane : public Parameterised {
+    struct Lane final : public Parameterised {
         /// @brief constructor
         Lane(NBEdge* e, const std::string& _origID);
 
@@ -151,13 +177,16 @@ public:
 
         /// @brief A custom shape for this lane set by the user
         PositionVector customShape;
+
+        /// @brief the type of this lane
+        std::string type;
     };
 
 
     /** @struct Connection
      * @brief A structure which describes a connection between edges or lanes
      */
-    struct Connection : public Parameterised {
+    struct Connection final : public Parameterised, public NBRouterEdge {
         /** @brief Constructor
          * @param[in] fromLane_ The lane the connections starts at
          * @param[in] toEdge_ The edge the connections yields in
@@ -173,10 +202,8 @@ public:
                    double speed_ = UNSPECIFIED_SPEED,
                    bool haveVia_ = false,
                    bool uncontrolled_ = false,
-                   const PositionVector& customShape_ = PositionVector::EMPTY);
-
-        /// @brief destructor
-        ~Connection() { }
+                   const PositionVector& customShape_ = PositionVector::EMPTY,
+                   SVCPermissions permissions = SVC_UNSPECIFIED);
 
         /// @brief The lane the connections starts at
         int fromLane;
@@ -193,6 +220,9 @@ public:
         /// @brief The index of this connection within the controlling traffic light
         int tlLinkIndex;
 
+        /// @brief The index of the internal junction within the controlling traffic light (optional)
+        int tlLinkIndex2;
+
         /// @brief Information about being definitely free to drive (on-ramps)
         bool mayDefinitelyPass;
 
@@ -211,6 +241,9 @@ public:
         /// @brief custom shape for connection
         PositionVector customShape;
 
+        /// @brief List of vehicle types that are allowed on this connection
+        SVCPermissions permissions;
+
         /// @brief id of Connection
         std::string id;
 
@@ -249,6 +282,30 @@ public:
 
         /// @brief computed length (average of all internal lane shape lengths that share an internal edge)
         double length;
+
+        /// @name NBRouterEdge interface
+        /// @{
+        static ConstRouterEdgePairVector myViaSuccessors; // always empty
+        const std::string& getID() const {
+            return id;
+        }
+        double getSpeed() const {
+            return vmax;
+        }
+        double getLength() const {
+            return shape.length() + viaShape.length();
+        }
+        int getNumericalID() const {
+            throw ProcessError("NBEdge::Connection does not implement getNumericalID()");
+        }
+        bool isInternal() const {
+            return true;
+        }
+        const ConstRouterEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const {
+            UNUSED_PARAMETER(vClass);
+            return myViaSuccessors;
+        }
+        /// }@
     };
 
     /// @brief Dummy edge to use when a reference must be supplied in the no-arguments constructor (FOX technicality)
@@ -556,6 +613,10 @@ public:
         return myEndOffset;
     }
 
+    double getDistance() const {
+        return myDistance;
+    }
+
     /** @brief Returns the stopOffset to the end of the edge
      * @return The offset to the end of the edge
      */
@@ -578,9 +639,15 @@ public:
         return mySignalOffset;
     }
 
+    /// @brief Returns the node that (possibly) represents a traffic signal controlling at the end of this edge
+    NBNode* getSignalNode() const {
+        return mySignalNode;
+    }
+
     /// @brief sets the offset of a traffic signal from the end of this edge
-    void setSignalOffset(double offset) {
+    void setSignalOffset(double offset, NBNode* signalNode) {
         mySignalOffset = offset;
+        mySignalNode = signalNode;
     }
 
     /** @brief Returns the lane definitions
@@ -597,6 +664,9 @@ public:
      */
     int getFirstNonPedestrianLaneIndex(int direction, bool exclusive = false) const;
 
+    /// @brief return index of the first lane that allows the given permissions
+    int getSpecialLane(SVCPermissions permissions) const;
+
     /** @brief return the first lane that permits at least 1 vClass or the last lane if search direction of there is no such lane
      * @param[in] direction The direction in which the lanes shall be checked
      */
@@ -675,16 +745,19 @@ public:
     /// @brief shift geometry at the given node to avoid overlap
     void shiftPositionAtNode(NBNode* node, NBEdge* opposite);
 
+    /// @brief return position taking into account loaded length
+    Position geometryPositionAtOffset(double offset) const;
+
     /** @brief Recomputeds the lane shapes to terminate at the node shape
      * For every lane the intersection with the fromNode and toNode is
      * calculated and the lane shorted accordingly. The edge length is then set
-     * to the average of all lane lenghts (which may differ). This average length is used as the lane
+     * to the average of all lane lengths (which may differ). This average length is used as the lane
      * length when writing the network.
      * @note All lanes of an edge in a sumo net must have the same nominal length
      *  but may differ in actual geomtric length.
      * @note Depends on previous call to NBNodeCont::computeNodeShapes
      */
-    void computeEdgeShape(double smoothElevationThreshold=-1);
+    void computeEdgeShape(double smoothElevationThreshold = -1);
 
     /** @brief Returns the shape of the nth lane
      * @return The shape of the lane given by its index (counter from right)
@@ -705,13 +778,6 @@ public:
         return myLaneSpreadFunction;
     }
 
-    /** @brief Splits this edge at geometry points
-     * @param[in] ec The edge cont to add new edges to
-     * @param[in] nc The node cont to add new nodes to
-     * @return Whether the geometry was changed
-     */
-    bool splitGeometry(NBEdgeCont& ec, NBNodeCont& nc);
-
     /** @brief Removes points with a distance lesser than the given
      * @param[in] minDist The minimum distance between two position to keep the second
      */
@@ -722,7 +788,7 @@ public:
      * @param[in] minRadius The minimum turning radius allowed at the start and end
      * @param[in] fix Whether to prune geometry points to avoid sharp turns at start and end
      */
-    void checkGeometry(const double maxAngle, const double minRadius, bool fix);
+    void checkGeometry(const double maxAngle, const double minRadius, bool fix, bool silent);
     //@}
 
     /// @name Setting and getting connections
@@ -772,7 +838,8 @@ public:
                                 double visibility = UNSPECIFIED_VISIBILITY_DISTANCE,
                                 double speed = UNSPECIFIED_SPEED,
                                 const PositionVector& customShape = PositionVector::EMPTY,
-                                const bool uncontrolled = UNSPECIFIED_CONNECTION_UNCONTROLLED);
+                                const bool uncontrolled = UNSPECIFIED_CONNECTION_UNCONTROLLED,
+                                SVCPermissions = SVC_UNSPECIFIED);
 
     /** @brief Builds no connections starting at the given lanes
      *
@@ -816,20 +883,20 @@ public:
                        double visibility = UNSPECIFIED_VISIBILITY_DISTANCE,
                        double speed = UNSPECIFIED_SPEED,
                        const PositionVector& customShape = PositionVector::EMPTY,
-                       const bool uncontrolled = UNSPECIFIED_CONNECTION_UNCONTROLLED);
-
-    /// @brief insert a previously created NBEdge::connection
-    void insertConnection(NBEdge::Connection connection);
+                       const bool uncontrolled = UNSPECIFIED_CONNECTION_UNCONTROLLED,
+                       SVCPermissions permissions = SVC_UNSPECIFIED);
 
     /** @brief Returns connections from a given lane
      *
      * This method goes through "myConnections" and copies those which are
      *  starting at the given lane.
      * @param[in] lane The lane which connections shall be returned
+     * @param[in] to The target Edge (ignore nullptr)
+     * @param[in] toLane The target lane (ignore if > 0)
      * @return The connections from the given lane
      * @see NBEdge::Connection
      */
-    std::vector<Connection> getConnectionsFromLane(int lane) const;
+    std::vector<Connection> getConnectionsFromLane(int lane, NBEdge* to = nullptr, int toLane = -1) const;
 
     /** @brief Returns the specified connection
      * This method goes through "myConnections" and returns the specified one
@@ -893,7 +960,7 @@ public:
     /** @brief Returns the list of lanes that may be used to reach the given edge
      * @return Lanes approaching the given edge
      */
-    std::vector<int> getConnectionLanes(NBEdge* currentOutgoing) const;
+    std::vector<int> getConnectionLanes(NBEdge* currentOutgoing, bool withBikes = true) const;
 
     /// @brief sorts the outgoing connections by their angle relative to their junction
     void sortOutgoingConnectionsByAngle();
@@ -915,10 +982,10 @@ public:
      * @param[in] tryLater If the connection does not exist, try again during recheckLanes()
      * @param[in] adaptToLaneRemoval we are in the process of removing a complete lane, adapt all connections accordingly
      */
-    void removeFromConnections(NBEdge* toEdge, int fromLane = -1, int toLane = -1, bool tryLater = false, const bool adaptToLaneRemoval = false);
+    void removeFromConnections(NBEdge* toEdge, int fromLane = -1, int toLane = -1, bool tryLater = false, const bool adaptToLaneRemoval = false, const bool keepPossibleTurns = false);
 
     /// @brief remove an existent connection of edge
-    bool removeFromConnections(NBEdge::Connection connectionToRemove);
+    bool removeFromConnections(const NBEdge::Connection& connectionToRemove);
 
     /// @brief invalidate current connections of edge
     void invalidateConnections(bool reallowSetting = false);
@@ -964,15 +1031,15 @@ public:
     }
 
     /// @brief Marks this edge being within an intersection
-    void setInternal() {
-        myAmInnerEdge = true;
+    void setInsideTLS() {
+        myAmInTLS = true;
     }
 
     /** @brief Returns whether this edge was marked as being within an intersection
      * @return Whether this edge was marked as being within an intersection
      */
-    bool isInternal() const {
-        return myAmInnerEdge;
+    bool isInsideTLS() const {
+        return myAmInTLS;
     }
     /// @}
 
@@ -994,9 +1061,12 @@ public:
      */
     int getJunctionPriority(const NBNode* const node) const;
 
-    /// @brief set loaded lenght
+    /// @brief set loaded length
     void setLoadedLength(double val);
 
+    /// @brief patch average lane length in regard to the opposite edge
+    void setAverageLengthWithOpposite(double val);
+
     /// @brief dimiss vehicle class information
     void dismissVehicleClassInformation();
 
@@ -1020,6 +1090,9 @@ public:
     /// @brief whether lanes differ in width
     bool hasLaneSpecificWidth() const;
 
+    /// @brief whether lanes differ in type
+    bool hasLaneSpecificType() const;
+
     /// @brief whether lanes differ in offset
     bool hasLaneSpecificEndOffset() const;
 
@@ -1052,7 +1125,7 @@ public:
      *  of this edge to the leftmost lane of myTurnDestination).
      * @param[in] noTLSControlled Whether the turnaround shall not be connected if this edge is controlled by a tls
      */
-    void appendTurnaround(bool noTLSControlled, bool onlyDeadends, bool checkPermissions);
+    void appendTurnaround(bool noTLSControlled, bool onlyDeadends, bool noGeometryLike, bool checkPermissions);
 
     /** @brief Returns the node at the given edges length (using an epsilon)
         @note When no node is existing at the given position, 0 is returned
@@ -1101,12 +1174,9 @@ public:
      */
     NBEdge* getTurnDestination(bool possibleDestination = false) const;
 
-    /// @brief get Lane ID (Secure)
+    /// @brief get lane ID
     std::string getLaneID(int lane) const;
 
-    /// @brief get Lane ID (Insecure)
-    std::string getLaneIDInsecure(int lane) const;
-
     /// @brief get lane speed
     double getLaneSpeed(int lane) const;
 
@@ -1159,6 +1229,9 @@ public:
     /// @brief restore an previously added BikeLane
     void restoreBikelane(std::vector<NBEdge::Lane> oldLanes, PositionVector oldGeometry, std::vector<NBEdge::Connection> oldConnections);
 
+    /// @brief add a lane of the given width, restricted to the given class and shift existing connections
+    void addRestrictedLane(double width, SUMOVehicleClass vclass);
+
     /// @brief set allowed/disallowed classes for the given lane or for all lanes if -1 is given
     void setPermissions(SVCPermissions permissions, int lane = -1);
 
@@ -1177,6 +1250,9 @@ public:
     /// @brief set lane specific width (negative lane implies set for all lanes)
     void setLaneWidth(int lane, double width);
 
+    /// @brief set lane specific type (negative lane implies set for all lanes)
+    void setLaneType(int lane, const std::string& type);
+
     /// @brief set lane specific end-offset (negative lane implies set for all lanes)
     void setEndOffset(int lane, double offset);
 
@@ -1199,6 +1275,11 @@ public:
     /// @brief set origID for all lanes
     void setOrigID(const std::string origID);
 
+    /// @brief set lane specific speed (negative lane implies set for all lanes)
+    void setDistance(double distance) {
+        myDistance = distance;
+    }
+
     /// @brief disable connections for TLS
     void disableConnection4TLS(int fromLane, NBEdge* toEdge, int toLane);
 
@@ -1217,7 +1298,7 @@ public:
     }
 
     /// @brief declares connections as fully loaded. This is needed to avoid recomputing connections if an edge has no connections intentionally.
-    void declareConnectionsAsLoaded(EdgeBuildingStep step = LANES2LANES_USER) {
+    void declareConnectionsAsLoaded(EdgeBuildingStep step = EdgeBuildingStep::LANES2LANES_USER) {
         myStep = step;
     }
 
@@ -1257,7 +1338,7 @@ public:
     void debugPrintConnections(bool outgoing = true, bool incoming = false) const;
 
     /// @brief compute the first intersection point between the given lane geometries considering their rspective widths
-    static double firstIntersection(const PositionVector& v1, const PositionVector& v2, double width2);
+    static double firstIntersection(const PositionVector& v1, const PositionVector& v2, double width2, const std::string& error = "");
 
     /** returns a modified version of laneShape which starts at the outside of startNode. laneShape may be shorted or extended
      * @note see [wiki:Developer/Network_Building_Process]
@@ -1293,9 +1374,15 @@ public:
 
     /** @brief Returns the following edges for the given vClass
      */
-    const NBConstEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
+    const ConstRouterEdgePairVector& getViaSuccessors(SUMOVehicleClass vClass = SVC_IGNORING) const;
 
     //@}
+    const std::string& getID() const {
+        return Named::getID();
+    }
+
+    /// @brief join adjacent lanes with the given permissions
+    bool joinLanes(SVCPermissions perms);
 
 private:
     /** @class ToEdgeConnectionsAdder
@@ -1346,16 +1433,18 @@ private:
         /// @brief enum of possible directions
         enum Direction { DIR_RIGHTMOST, DIR_LEFTMOST, DIR_FORWARD };
 
-        /// @brief list of the main direction within the following junction relative to the edge
-        std::vector<Direction> myDirs;
-
     public:
         /// @brief constructor
-        MainDirections(const EdgeVector& outgoing, NBEdge* parent, NBNode* to, int indexOfStraightest);
+        MainDirections(const EdgeVector& outgoing, NBEdge* parent, NBNode* to, const std::vector<int>& availableLanes);
 
         /// @brief destructor
         ~MainDirections();
 
+        /// @brief returns the index of the straightmost among the given outgoing edges
+        int getStraightest() const {
+            return myStraightest;
+        }
+
         /// @brief returns the information whether no following street has a higher priority
         bool empty() const;
 
@@ -1363,6 +1452,12 @@ private:
         bool includes(Direction d) const;
 
     private:
+        /// @brief the index of the straightmost among the given outgoing edges
+        int myStraightest;
+
+        /// @brief list of the main direction within the following junction relative to the edge
+        std::vector<Direction> myDirs;
+
         /// @brief Invalidated copy constructor.
         MainDirections(const MainDirections&);
 
@@ -1399,16 +1494,13 @@ private:
     void divideOnEdges(const EdgeVector* outgoing);
 
     /// @brief divide selected lanes on edges
-    void divideSelectedLanesOnEdges(const EdgeVector* outgoing, const std::vector<int>& availableLanes, const std::vector<int>* priorities);
+    void divideSelectedLanesOnEdges(const EdgeVector* outgoing, const std::vector<int>& availableLanes);
 
     /// @brief add some straight connections
-    void addStraightConnections(const EdgeVector* outgoing, const std::vector<int>& availableLanes, const std::vector<int>* priorities);
+    void addStraightConnections(const EdgeVector* outgoing, const std::vector<int>& availableLanes, const std::vector<int>& priorities);
 
     /// @brief recomputes the edge priorities and manipulates them for a distribution of lanes on edges which is more like in real-life
-    std::vector<int>* prepareEdgePriorities(const EdgeVector* outgoing);
-
-    /// @brief computes the sum of the given list's entries (sic!)
-    static int computePrioritySum(const std::vector<int>& priorities);
+    const std::vector<int> prepareEdgePriorities(const EdgeVector* outgoing, const std::vector<int>& availableLanes);
 
     /// @name Setting and getting connections
     /// @{
@@ -1430,10 +1522,10 @@ private:
     void computeAngle();
 
     /// @brief determine conflict between opposite left turns
-    bool bothLeftIntersect(const NBNode& n, const PositionVector& shape, LinkDirection dir, NBEdge* otherFrom, const NBEdge::Connection& otherCon, int numPoints, double width2) const;
+    bool bothLeftIntersect(const NBNode& n, const PositionVector& shape, LinkDirection dir, NBEdge* otherFrom, const NBEdge::Connection& otherCon, int numPoints, double width2, int shapeFlag = 0) const;
 
-    /// @brief add a lane of the given width, restricted to the given class and shift existing connections
-    void addRestrictedLane(double width, SUMOVehicleClass vclass);
+    /// @brief returns whether any lane already allows the given vclass exclusively
+    bool hasRestrictedLane(SUMOVehicleClass vclass) const;
 
     /// @brief restore a restricted lane
     void restoreRestrictedLane(SUMOVehicleClass vclass, std::vector<NBEdge::Lane> oldLanes, PositionVector oldGeometry, std::vector<NBEdge::Connection> oldConnections);
@@ -1469,6 +1561,9 @@ private:
     /// @brief The maximal speed
     double mySpeed;
 
+    /// @brief The mileage/kilometrage at the start of this edge in a linear coordination system
+    double myDistance;
+
     /** @brief List of connections to following edges
      * @see Connection
      */
@@ -1515,22 +1610,12 @@ private:
     /// @brief An optional length to use (-1 if not valid)
     double myLoadedLength;
 
-    /// @brief Information whether this is a junction-inner edge
-    bool myAmInnerEdge;
+    /// @brief Information whether this is lies within a joined tls
+    bool myAmInTLS;
 
     /// @brief Information whether this edge is a (macroscopic) connector
     bool myAmMacroscopicConnector;
 
-    /// @brief TLS Disabled Connections
-    struct TLSDisabledConnection {
-        int fromLane;
-        NBEdge* to;
-        int toLane;
-    };
-
-    /// @brief vector with the disabled connections
-    std::vector<TLSDisabledConnection> myTLSDisabledConnections;
-
     /// @brief The street name (or whatever arbitrary string you wish to attach)
     std::string myStreetName;
 
@@ -1539,6 +1624,7 @@ private:
 
     /// @brief the offset of a traffic light signal from the end of this edge (-1 for None)
     double mySignalOffset;
+    NBNode* mySignalNode;
 
     /// @brief intersection borders (because the node shape might be invalid)
     /// @{
@@ -1554,38 +1640,9 @@ private:
     mutable EdgeVector mySuccessors;
 
     // @brief a static list of successor edges. Set by NBEdgeCont and requires reset when the network changes
-    mutable NBConstEdgePairVector myViaSuccessors;
+    mutable ConstRouterEdgePairVector myViaSuccessors;
 
 public:
-    /// @class tls_disable_finder
-    class tls_disable_finder {
-    public:
-        /// @brief constructor
-        tls_disable_finder(const TLSDisabledConnection& tpl) : myDefinition(tpl) { }
-
-        /// @brief operator ()
-        bool operator()(const TLSDisabledConnection& e) const {
-            if (e.to != myDefinition.to) {
-                return false;
-            }
-            if (e.fromLane != myDefinition.fromLane) {
-                return false;
-            }
-            if (e.toLane != myDefinition.toLane) {
-                return false;
-            }
-            return true;
-        }
-
-    private:
-        /// @brief definition of disable connection
-        TLSDisabledConnection myDefinition;
-
-    private:
-        /// @brief invalidated assignment operator
-        tls_disable_finder& operator=(const tls_disable_finder& s);
-    };
-
 
     /// @class connections_toedge_finder
     class connections_toedge_finder {
