@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
@@ -204,15 +203,14 @@ public:
      * @brief A structure which describes changes of lane number or speed along the road
      */
     struct Split {
-        Split() : offset(0), offsetFactor(1) {}
         /// @brief The lanes after this change
         std::vector<int> lanes;
         /// @brief The position of this change
-        double pos;
+        double pos = INVALID_DOUBLE;
         /// @brief The speed after this change
-        double speed;
+        double speed = INVALID_DOUBLE;
         /// @brief The new node that is created for this split
-        NBNode* node;
+        NBNode* node = nullptr;
         /// @brief The id for the edge before the split
         std::string idBefore;
         /// @brief The id for the edge after the split
@@ -220,9 +218,9 @@ public:
         /// @brief the default node id
         std::string nameID;
         /// @brief lateral offset to edge geometry
-        double offset;
+        double offset = 0.;
         /// @brief direction in which to apply the offset (used by netgenerate for lefthand networks)
-        int offsetFactor;
+        int offsetFactor = 1;
     };
 
     void processSplits(NBEdge* e, std::vector<Split> splits,
@@ -343,7 +341,7 @@ public:
      * @param[in] nc The node container needed to build (geometry) nodes
      * @see NBEdge::splitGeometry
      */
-    void splitGeometry(NBNodeCont& nc);
+    void splitGeometry(NBDistrictCont& dc, NBNodeCont& nc);
 
 
     /** @brief
@@ -359,7 +357,7 @@ public:
      * @param[in] fix Whether to prune geometry points to avoid sharp turns at start and end
      * @see NBEdge::checkGeometry
      */
-    void checkGeometries(const double maxAngle, const double minRadius, bool fix);
+    void checkGeometries(const double maxAngle, const double minRadius, bool fix, bool fixRailways, bool silent = false);
     /// @}
 
 
@@ -417,7 +415,7 @@ public:
      * @todo Recheck whether a visitor-pattern should be used herefor
      * @see NBEdge::appendTurnaround
      */
-    void appendTurnarounds(bool noTLSControlled, bool onlyDeadends);
+    void appendTurnarounds(bool noTLSControlled, bool onlyDeadends, bool noGeometryLike);
 
 
     /** @brief Appends turnarounds to all edges stored in the container
@@ -441,7 +439,7 @@ public:
      * @todo Recheck usage
      * @see NBEdge::computeEdgeShape
      */
-    void computeEdgeShapes(double smoothElevationThreshold=-1);
+    void computeEdgeShapes(double smoothElevationThreshold = -1);
 
 
     /** @brief Computes the shapes of all lanes of all edges stored in the container
@@ -540,7 +538,11 @@ public:
     void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass,
                                   bool keepClear, double contPos, double visibility, double speed,
                                   const PositionVector& customShape,
-                                  bool warnOnly = false);
+                                  bool uncontrolled,
+                                  bool warnOnly,
+                                  SVCPermissions permissions = SVC_UNSPECIFIED);
+
+    bool hasPostProcessConnection(const std::string& from, const std::string& to = "");
 
 
     /** @brief Try to set any stored connections
@@ -552,7 +554,7 @@ public:
     void generateStreetSigns();
 
     /// @brief add sidwalks to edges within the given limits or permissions and return the number of edges affected
-    int guessSidewalks(double width, double minSpeed, double maxSpeed, bool fromPermissions);
+    int guessSpecialLanes(SUMOVehicleClass svc, double width, double minSpeed, double maxSpeed, bool fromPermissions, const std::string& excludeOpt);
 
 
     /** @brief Returns the determined roundabouts
@@ -563,6 +565,9 @@ public:
     /// @brief add user specified roundabout
     void addRoundabout(const EdgeSet& roundabout);
 
+    /// @brief remove roundabout that contains the given node
+    void removeRoundabout(const NBNode* node);
+
     /// @brief mark edge priorities and prohibit turn-arounds for all roundabout edges
     void markRoundabouts();
 
@@ -590,8 +595,12 @@ public:
      */
     EdgeVector getGeneratedFrom(const std::string& id) const;
 
+    /// @brief join adjacent lanes with the given permissions
+    int joinLanes(SVCPermissions perms);
+
     /// @brief return all edges
     EdgeVector getAllEdges() const;
+    RouterEdgeVector getAllRouterEdges() const;
 
 private:
     /// @brief compute the form factor for a loop of edges
@@ -616,13 +625,16 @@ private:
         PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_,
                               bool mayDefinitelyPass_, bool keepClear_, double contPos_, double visibility_, double speed_,
                               const PositionVector& customShape_,
-                              bool warnOnly_ = false) :
+                              bool uncontrolled_,
+                              bool warnOnly_, SVCPermissions permissions_) :
             from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_), contPos(contPos_),
             visibility(visibility_),
             speed(speed_),
             customShape(customShape_),
-            warnOnly(warnOnly_) {
-        }
+            uncontrolled(uncontrolled_),
+            permissions(permissions_),
+            warnOnly(warnOnly_)
+        {}
         /// @brief The id of the edge the connection starts at
         std::string from;
         /// @brief The number of the lane the connection starts at
@@ -643,12 +655,16 @@ private:
         double speed;
         /// @brief custom shape for connection
         PositionVector customShape;
+        /// @brief whether this connection shall not be controlled by a traffic light
+        bool uncontrolled;
+        /// @brief custom permissions for connection
+        SVCPermissions permissions;
         /// @brief whether a failure to set this connection is a warning or an error
         bool warnOnly;
     };
 
     /// @brief The list of connections to recheck
-    std::vector<PostProcessConnection> myConnections;
+    std::map<std::string, std::vector<PostProcessConnection> > myConnections;
 
 
     /// @brief The type of the dictionary where an edge may be found by its id
