@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // Some mathematical helper methods
 /****************************************************************************/
@@ -46,11 +45,11 @@
 double
 NBHelpers::relAngle(double angle1, double angle2) {
     angle2 -= angle1;
-    if (angle2 > 180) {
-        angle2 = (360 - angle2) * -1;
+    while (angle2 > 180.) {
+        angle2 -= 360.;
     }
-    while (angle2 < -180) {
-        angle2 = 360 + angle2;
+    while (angle2 < -180.) {
+        angle2 += 360.;
     }
     return angle2;
 }
