@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2011-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2011-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mar 2011
-/// @version $Id$
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
@@ -56,19 +55,18 @@ NBLoadedSUMOTLDef::NBLoadedSUMOTLDef(const std::string& id, const std::string& p
 
 
 NBLoadedSUMOTLDef::NBLoadedSUMOTLDef(NBTrafficLightDefinition* def, NBTrafficLightLogic* logic) :
-    // allow for adding a new program for the same def: take the programID from the new logic
-    NBTrafficLightDefinition(def->getID(), logic->getProgramID(), def->getOffset(), def->getType()),
+    // allow for adding a new program for the same def: take the offset and programID from the new logic
+    NBTrafficLightDefinition(def->getID(), logic->getProgramID(), logic->getOffset(), def->getType()),
     myTLLogic(new NBTrafficLightLogic(logic)),
     myOriginalNodes(def->getNodes().begin(), def->getNodes().end()),
     myReconstructAddedConnections(false),
     myReconstructRemovedConnections(false),
     myPhasesLoaded(false) {
-    assert(def->getOffset() == logic->getOffset());
     assert(def->getType() == logic->getType());
     myControlledLinks = def->getControlledLinks();
     myControlledNodes = def->getNodes();
     NBLoadedSUMOTLDef* sumoDef = dynamic_cast<NBLoadedSUMOTLDef*>(def);
-    updateParameter(def->getParametersMap());
+    updateParameters(def->getParametersMap());
     if (sumoDef != nullptr) {
         myReconstructAddedConnections = sumoDef->myReconstructAddedConnections;
         myReconstructRemovedConnections = sumoDef->myReconstructRemovedConnections;
@@ -94,13 +92,17 @@ NBLoadedSUMOTLDef::myCompute(int brakingTimeSeconds) {
 
 
 void
-NBLoadedSUMOTLDef::addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex, bool reconstruct) {
+NBLoadedSUMOTLDef::addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex, int linkIndex2, bool reconstruct) {
     assert(myTLLogic->getNumLinks() > 0); // logic should be loaded by now
     if (linkIndex >= (int)myTLLogic->getNumLinks()) {
         throw ProcessError("Invalid linkIndex " + toString(linkIndex) + " for traffic light '" + getID() +
                            "' with " + toString(myTLLogic->getNumLinks()) + " links.");
     }
-    NBConnection conn(from, fromLane, to, toLane, linkIndex);
+    if (linkIndex2 >= (int)myTLLogic->getNumLinks()) {
+        throw ProcessError("Invalid linkIndex2 " + toString(linkIndex2) + " for traffic light '" + getID() +
+                           "' with " + toString(myTLLogic->getNumLinks()) + " links.");
+    }
+    NBConnection conn(from, fromLane, to, toLane, linkIndex, linkIndex2);
     // avoid duplicates
     auto newEnd = remove_if(myControlledLinks.begin(), myControlledLinks.end(), connection_equal(conn));
     // remove_if does not remove, only re-order
@@ -121,7 +123,7 @@ NBLoadedSUMOTLDef::addConnection(NBEdge* from, NBEdge* to, int fromLane, int toL
 void
 NBLoadedSUMOTLDef::setTLControllingInformation() const {
     if (myReconstructAddedConnections) {
-        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+        NBOwnTLDef dummy(DummyID, myControlledNodes, 0, getType());
         dummy.setParticipantsInformation();
         dummy.setTLControllingInformation();
         for (std::vector<NBNode*>::const_iterator i = myControlledNodes.begin(); i != myControlledNodes.end(); i++) {
@@ -167,8 +169,8 @@ NBLoadedSUMOTLDef::replaceRemoved(NBEdge* removed, int removedLane, NBEdge* by,
 
 
 void
-NBLoadedSUMOTLDef::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur) {
-    myTLLogic->addStep(duration, state, minDur, maxDur);
+NBLoadedSUMOTLDef::addPhase(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur, const std::vector<int>& next, const std::string& name) {
+    myTLLogic->addStep(duration, state, minDur, maxDur, next, name);
 }
 
 
@@ -250,7 +252,7 @@ NBLoadedSUMOTLDef::collectEdges() {
     for (EdgeVector::iterator j = myIncomingEdges.begin(); j != myIncomingEdges.end();) {
         NBEdge* edge = *j;
         // an edge lies within the logic if it is outgoing as well as incoming
-        EdgeVector::iterator k = find(myOutgoing.begin(), myOutgoing.end(), edge);
+        EdgeVector::iterator k = std::find(myOutgoing.begin(), myOutgoing.end(), edge);
         if (k != myOutgoing.end()) {
             if (myControlledInnerEdges.count(edge->getID()) == 0) {
                 bool controlled = false;
@@ -264,7 +266,7 @@ NBLoadedSUMOTLDef::collectEdges() {
                     myControlledInnerEdges.insert(edge->getID());
                 } else {
                     myEdgesWithin.push_back(edge);
-                    (*j)->setInternal();
+                    (*j)->setInsideTLS();
                     ++j; //j = myIncomingEdges.erase(j);
                     continue;
                 }
@@ -453,7 +455,7 @@ NBLoadedSUMOTLDef::reconstructLogic() {
         if (!myPhasesLoaded && !(netedit && hasValidIndices())) {
             // rebuild the logic from scratch
             // XXX if a connection with the same from- and to-edge already exisits, its states could be copied instead
-            NBOwnTLDef dummy(DummyID, myControlledNodes, 0, TLTYPE_STATIC);
+            NBOwnTLDef dummy(DummyID, myControlledNodes, 0, getType());
             dummy.setParticipantsInformation();
             dummy.setProgramID(getProgramID());
             dummy.setTLControllingInformation();
@@ -488,7 +490,7 @@ NBLoadedSUMOTLDef::reconstructLogic() {
         for (NBConnectionVector::iterator it = myControlledLinks.begin(); it != myControlledLinks.end();) {
             const NBConnection con = (*it);
             if (// edge still exists
-                find(myIncomingEdges.begin(), myIncomingEdges.end(), con.getFrom()) != myIncomingEdges.end()
+                std::find(myIncomingEdges.begin(), myIncomingEdges.end(), con.getFrom()) != myIncomingEdges.end()
                 // connection still exists
                 && con.getFrom()->hasConnectionTo(con.getTo(), con.getToLane(), con.getFromLane())
                 // connection is still set to be controlled
@@ -632,5 +634,54 @@ NBLoadedSUMOTLDef::usingSignalGroups() const {
     return false;
 }
 
+void
+NBLoadedSUMOTLDef::guessMinMaxDuration() {
+    bool hasMinMaxDur = false;
+    for (auto phase : myTLLogic->getPhases()) {
+        if (phase.maxDur != UNSPECIFIED_DURATION) {
+            std::cout << " phase=" << phase.state << " maxDur=" << phase.maxDur << "\n";
+            hasMinMaxDur = true;
+        }
+    }
+    if (!hasMinMaxDur) {
+        const SUMOTime minMinDur = TIME2STEPS(OptionsCont::getOptions().getInt("tls.min-dur"));
+        const SUMOTime maxDur = TIME2STEPS(OptionsCont::getOptions().getInt("tls.max-dur"));
+        std::set<int> yellowIndices;
+        for (auto phase : myTLLogic->getPhases()) {
+            for (int i = 0; i < (int)phase.state.size(); i++) {
+                if (phase.state[i] == 'y' || phase.state[i] == 'Y') {
+                    yellowIndices.insert(i);
+                }
+            }
+        }
+        for (int ip = 0; ip < (int)myTLLogic->getPhases().size(); ip++) {
+            bool needMinMaxDur = false;
+            auto phase = myTLLogic->getPhases()[ip];
+            std::set<int> greenIndices;
+            if (phase.state.find_first_of("yY") != std::string::npos) {
+                continue;
+            }
+            for (int i = 0; i < (int)phase.state.size(); i++) {
+                if (yellowIndices.count(i) != 0 && phase.state[i] == 'G') {
+                    needMinMaxDur = true;
+                    greenIndices.insert(i);
+                }
+            }
+            if (needMinMaxDur) {
+                double maxSpeed = 0;
+                for (NBConnection& c : myControlledLinks) {
+                    if (greenIndices.count(c.getTLIndex()) != 0) {
+                        maxSpeed = MAX2(maxSpeed, c.getFrom()->getLaneSpeed(c.getFromLane()));
+                    }
+                }
+                // 5s at 50km/h, 10s at 80km/h, rounded to full seconds
+                const double minDurBySpeed = maxSpeed * 3.6 / 6 - 3.3;
+                SUMOTime minDur = MAX2(minMinDur, TIME2STEPS(floor(minDurBySpeed + 0.5)));
+                myTLLogic->setPhaseMinDuration(ip, minDur);
+                myTLLogic->setPhaseMaxDuration(ip, maxDur);
+            }
+        }
+    }
+}
 /****************************************************************************/
 
