@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2011-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2011-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -11,7 +11,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar 2011
-/// @version $Id$
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
@@ -97,7 +96,7 @@ public:
      * @param[in] minDur The minimum duration of the phase to add
      * @param[in] maxDur The maximum duration of the phase to add
      */
-    void addPhase(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur);
+    void addPhase(SUMOTime duration, const std::string& state, SUMOTime minDur, SUMOTime maxDur, const std::vector<int>& next, const std::string& name);
 
     /// @brief mark phases as load
     void phasesLoaded() {
@@ -106,7 +105,7 @@ public:
 
     /** @brief Adds a connection and immediately informs the edges
      */
-    void addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex, bool reconstruct = true);
+    void addConnection(NBEdge* from, NBEdge* to, int fromLane, int toLane, int linkIndex, int linkIndex2, bool reconstruct = true);
 
 
     /** @brief removes the given connection from the traffic light
@@ -148,6 +147,9 @@ public:
     /// @brief join nodes and states from the given logic (append red state)
     void joinLogic(NBTrafficLightDefinition* def);
 
+    /// @brief heuristically add minDur and maxDur when switching from tlType fixed to actuated
+    void guessMinMaxDuration();
+
 protected:
     /** @brief Collects the links participating in this traffic light
      *    (only if not previously loaded)
