@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -15,7 +15,6 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    20 Nov 2001
-/// @version $Id$
 ///
 // Instance responsible for building networks
 /****************************************************************************/
@@ -84,38 +83,30 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
 
     // MODIFYING THE SETS OF NODES AND EDGES
-
-    long before = SysUtils::getCurrentMillis();
-
-
     // Removes edges that are connecting the same node
-    PROGRESS_BEGIN_MESSAGE("Removing self-loops");
+    long before = PROGRESS_BEGIN_TIME_MESSAGE("Removing self-loops");
     myNodeCont.removeSelfLoops(myDistrictCont, myEdgeCont, myTLLCont);
     PROGRESS_TIME_MESSAGE(before);
     if (mayAddOrRemove && oc.exists("remove-edges.isolated") && oc.getBool("remove-edges.isolated")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Finding isolated roads");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Finding isolated roads");
         myNodeCont.removeIsolatedRoads(myDistrictCont, myEdgeCont);
         PROGRESS_TIME_MESSAGE(before);
     }
     if (mayAddOrRemove && oc.exists("keep-edges.components") && oc.getInt("keep-edges.components") > 0) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Finding largest components");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Finding largest components");
         myNodeCont.removeComponents(myDistrictCont, myEdgeCont, oc.getInt("keep-edges.components"));
         PROGRESS_TIME_MESSAGE(before);
     }
     if (mayAddOrRemove && oc.exists("keep-edges.postload") && oc.getBool("keep-edges.postload")) {
         if (oc.isSet("keep-edges.explicit") || oc.isSet("keep-edges.input-file")) {
-            before = SysUtils::getCurrentMillis();
-            PROGRESS_BEGIN_MESSAGE("Removing unwished edges");
+            before = PROGRESS_BEGIN_TIME_MESSAGE("Removing unwished edges");
             myEdgeCont.removeUnwishedEdges(myDistrictCont);
             PROGRESS_TIME_MESSAGE(before);
         }
     }
     // Processing pt stops and lines
     if (oc.exists("ptstop-output") && oc.isSet("ptstop-output")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Processing public transport stops");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Processing public transport stops");
         if (!(oc.exists("ptline-output") && oc.isSet("ptline-output"))) {
             myPTStopCont.localizePTStops(myEdgeCont);
         }
@@ -124,22 +115,26 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
 
     if (oc.exists("ptline-output") && oc.isSet("ptline-output")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Revising public transport stops based on pt lines");
-        myPTLineCont.process(myEdgeCont);
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Revising public transport stops based on pt lines");
+        myPTLineCont.process(myEdgeCont, myPTStopCont);
         PROGRESS_TIME_MESSAGE(before);
     }
 
-    if (oc.exists("ptline-output") && oc.isSet("ptline-output") && oc.exists("ptline-clean-up") && oc.getBool("ptline-clean-up")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Cleaning up public transport stops that are not served by any line");
-        myPTStopCont.postprocess(myPTLineCont.getServedPTStops());
-        PROGRESS_TIME_MESSAGE(before);
+    if (oc.exists("ptline-output") && oc.isSet("ptline-output")) {
+        if (oc.exists("ptline-clean-up") && oc.getBool("ptline-clean-up")) {
+            before = PROGRESS_BEGIN_TIME_MESSAGE("Cleaning up public transport stops that are not served by any line");
+            myPTStopCont.postprocess(myPTLineCont.getServedPTStops());
+            PROGRESS_TIME_MESSAGE(before);
+        } else {
+            int numDeletedStops = myPTStopCont.cleanupDeleted(myEdgeCont);
+            if (numDeletedStops > 0) {
+                WRITE_WARNING("Removed " + toString(numDeletedStops) + " pt stops because they could not be assigned to the network");
+            }
+        }
     }
 
     if (oc.exists("ptstop-output") && oc.isSet("ptstop-output")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Align pt stop id signs with corresponding edge id signs");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Align pt stop id signs with corresponding edge id signs");
         myPTStopCont.alignIdSigns();
         PROGRESS_TIME_MESSAGE(before);
     }
@@ -149,6 +144,10 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         NBTurningDirectionsComputer::computeTurnDirections(myNodeCont, false);
         NBRailwayTopologyAnalyzer::makeAllBidi(*this);
     } else if (oc.exists("railway.topology.repair") && oc.getBool("railway.topology.repair")) {
+        // correct railway angles for angle-based connectivity heuristic
+        myEdgeCont.checkGeometries(0,
+                                   oc.getFloat("geometry.min-radius"), false,
+                                   oc.getBool("geometry.min-radius.fix.railways"), true);
         NBTurningDirectionsComputer::computeTurnDirections(myNodeCont, false);
         NBRailwayTopologyAnalyzer::repairTopology(*this);
     }
@@ -157,7 +156,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         NBRailwayTopologyAnalyzer::analyzeTopology(*this);
     }
 
-    if (oc.getBool("junctions.join") || (oc.exists("ramps.guess") && oc.getBool("ramps.guess"))) {
+    if (oc.getBool("junctions.join") || (oc.exists("ramps.guess") && oc.getBool("ramps.guess")) || oc.getBool("tls.guess.joining")) {
         // preliminary geometry computations to determine the length of edges
         // This depends on turning directions and sorting of edge list
         // in case junctions are joined geometry computations have to be repeated
@@ -187,8 +186,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     int numJoined = myNodeCont.joinLoadedClusters(myDistrictCont, myEdgeCont, myTLLCont);
     if (mayAddOrRemove && oc.getBool("junctions.join")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Joining junction clusters");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Joining junction clusters");
         numJoined += myNodeCont.joinJunctions(oc.getFloat("junctions.join-dist"), myDistrictCont, myEdgeCont, myTLLCont, myPTStopCont);
         PROGRESS_TIME_MESSAGE(before);
     }
@@ -201,11 +199,17 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         WRITE_MESSAGE(" Joined " + toString(numJoined) + " junction cluster(s).");
     }
     //
+    if (mayAddOrRemove && oc.exists("join-lanes") && oc.getBool("join-lanes")) {
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Joining lanes");
+        const int num = myEdgeCont.joinLanes(SVC_IGNORING) + myEdgeCont.joinLanes(SVC_PEDESTRIAN);
+        PROGRESS_TIME_MESSAGE(before);
+        WRITE_MESSAGE("   Joined lanes on " + toString(num) + " edges.");
+    }
+    //
     if (mayAddOrRemove) {
         int no = 0;
         const bool removeGeometryNodes = oc.exists("geometry.remove") && oc.getBool("geometry.remove");
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Removing empty nodes" + std::string(removeGeometryNodes ? " and geometry nodes" : ""));
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Removing empty nodes" + std::string(removeGeometryNodes ? " and geometry nodes" : ""));
         // removeUnwishedNodes needs turnDirections. @todo: try to call this less often
         NBTurningDirectionsComputer::computeTurnDirections(myNodeCont, false);
         no = myNodeCont.removeUnwishedNodes(myDistrictCont, myEdgeCont, myTLLCont, myPTStopCont, myPTLineCont, myParkingCont, removeGeometryNodes);
@@ -230,16 +234,14 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     geoConvHelper.computeFinal(lefthand); // information needed for location element fixed at this point
 
     if (oc.exists("geometry.min-dist") && !oc.isDefault("geometry.min-dist")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Reducing geometries");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Reducing geometries");
         myEdgeCont.reduceGeometries(oc.getFloat("geometry.min-dist"));
         PROGRESS_TIME_MESSAGE(before);
     }
     // @note: removing geometry can create similar edges so joinSimilarEdges  must come afterwards
     // @note: likewise splitting can destroy similarities so joinSimilarEdges must come before
     if (mayAddOrRemove &&  oc.getBool("edges.join")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Joining similar edges");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Joining similar edges");
         myNodeCont.joinSimilarEdges(myDistrictCont, myEdgeCont, myTLLCont);
         PROGRESS_TIME_MESSAGE(before);
     }
@@ -249,15 +251,13 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         PROGRESS_DONE_MESSAGE();
     }
     //
-    if (mayAddOrRemove &&  oc.exists("geometry.split") && oc.getBool("geometry.split")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Splitting geometry edges");
-        myEdgeCont.splitGeometry(myNodeCont);
+    if (mayAddOrRemove && oc.exists("geometry.split") && oc.getBool("geometry.split")) {
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Splitting geometry edges");
+        myEdgeCont.splitGeometry(myDistrictCont, myNodeCont);
         PROGRESS_TIME_MESSAGE(before);
     }
     // turning direction
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing turning directions");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing turning directions");
     NBTurningDirectionsComputer::computeTurnDirections(myNodeCont);
     PROGRESS_TIME_MESSAGE(before);
     // correct edge geometries to avoid overlap
@@ -266,8 +266,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
 
     // GUESS TLS POSITIONS
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Assigning nodes to traffic lights");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Assigning nodes to traffic lights");
     if (oc.isSet("tls.set")) {
         std::vector<std::string> tlControlledNodes = oc.getStringVector("tls.set");
         TrafficLightType type = SUMOXMLDefinitions::TrafficLightTypes.get(oc.getString("tls.default-type"));
@@ -300,12 +299,23 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
             }
         }
     }
+    // guess bike lanes
+    if (mayAddOrRemove && ((oc.getBool("bikelanes.guess") || oc.getBool("bikelanes.guess.from-permissions")))) {
+        const int bikelanes = myEdgeCont.guessSpecialLanes(SVC_BICYCLE, oc.getFloat("default.bikelane-width"),
+                              oc.getFloat("bikelanes.guess.min-speed"),
+                              oc.getFloat("bikelanes.guess.max-speed"),
+                              oc.getBool("bikelanes.guess.from-permissions"),
+                              "bikelanes.guess.exclude");
+        WRITE_MESSAGE("Guessed " + toString(bikelanes) + " bike lanes.");
+    }
+
     // guess sidewalks
     if (mayAddOrRemove && ((oc.getBool("sidewalks.guess") || oc.getBool("sidewalks.guess.from-permissions")))) {
-        const int sidewalks = myEdgeCont.guessSidewalks(oc.getFloat("default.sidewalk-width"),
+        const int sidewalks = myEdgeCont.guessSpecialLanes(SVC_PEDESTRIAN, oc.getFloat("default.sidewalk-width"),
                               oc.getFloat("sidewalks.guess.min-speed"),
                               oc.getFloat("sidewalks.guess.max-speed"),
-                              oc.getBool("sidewalks.guess.from-permissions"));
+                              oc.getBool("sidewalks.guess.from-permissions"),
+                              "sidewalks.guess.exclude");
         WRITE_MESSAGE("Guessed " + toString(sidewalks) + " sidewalks.");
     }
 
@@ -324,19 +334,18 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         myEdgeCont.checkGeometries(
             DEG2RAD(oc.getFloat("geometry.max-angle")),
             oc.getFloat("geometry.min-radius"),
-            oc.getBool("geometry.min-radius.fix"));
+            oc.getBool("geometry.min-radius.fix"),
+            oc.getBool("geometry.min-radius.fix.railways"));
     }
 
     // GEOMETRY COMPUTATION
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Sorting nodes' edges");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Sorting nodes' edges");
     NBNodesEdgesSorter::sortNodesEdges(myNodeCont);
     PROGRESS_TIME_MESSAGE(before);
     myEdgeCont.computeLaneShapes();
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing node shapes");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing node shapes");
     if (oc.exists("geometry.junction-mismatch-threshold")) {
         myNodeCont.computeNodeShapes(oc.getFloat("geometry.junction-mismatch-threshold"));
     } else {
@@ -344,8 +353,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing edge shapes");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing edge shapes");
     myEdgeCont.computeEdgeShapes(oc.getBool("geometry.max-grade.fix") ? oc.getFloat("geometry.max-grade") / 100 : -1);
     PROGRESS_TIME_MESSAGE(before);
     // resort edges based on the node and edge shapes
@@ -358,8 +366,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
         const double speedFactor = oc.getFloat("speed.factor");
         const double speedMin = oc.getFloat("speed.minimum");
         if (speedOffset != 0 || speedFactor != 1 || speedMin > 0) {
-            before = SysUtils::getCurrentMillis();
-            PROGRESS_BEGIN_MESSAGE("Applying speed modifications");
+            before = PROGRESS_BEGIN_TIME_MESSAGE("Applying speed modifications");
             for (std::map<std::string, NBEdge*>::const_iterator i = myEdgeCont.begin(); i != myEdgeCont.end(); ++i) {
                 (*i).second->setSpeed(-1, MAX2((*i).second->getSpeed() * speedFactor + speedOffset, speedMin));
             }
@@ -369,8 +376,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
 
     // CONNECTIONS COMPUTATION
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing node types");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing node types");
     NBNodeTypeComputer::computeNodeTypes(myNodeCont, myTLLCont);
     PROGRESS_TIME_MESSAGE(before);
     //
@@ -400,19 +406,16 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
 
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing priorities");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing priorities");
     NBEdgePriorityComputer::computeEdgePriorities(myNodeCont);
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing approached edges");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing approached edges");
     myEdgeCont.computeEdge2Edges(oc.getBool("no-left-connections"));
     PROGRESS_TIME_MESSAGE(before);
     //
     if (oc.getBool("roundabouts.guess")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Guessing and setting roundabouts");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Guessing and setting roundabouts");
         const int numGuessed = myEdgeCont.guessRoundabouts();
         if (numGuessed > 0) {
             WRITE_MESSAGE(" Guessed " + toString(numGuessed) + " roundabout(s).");
@@ -421,21 +424,18 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     myEdgeCont.markRoundabouts();
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing approaching lanes");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing approaching lanes");
     myEdgeCont.computeLanes2Edges();
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Dividing of lanes on approached lanes");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Dividing of lanes on approached lanes");
     myNodeCont.computeLanes2Lanes();
     myEdgeCont.sortOutgoingLanesConnections();
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Processing turnarounds");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Processing turnarounds");
     if (!oc.getBool("no-turnarounds")) {
-        myEdgeCont.appendTurnarounds(oc.getBool("no-turnarounds.tls"), oc.getBool("no-turnarounds.except-deadend"));
+        myEdgeCont.appendTurnarounds(oc.getBool("no-turnarounds.tls"), oc.getBool("no-turnarounds.except-deadend"), oc.getBool("no-turnarounds.geometry"));
     } else {
         myEdgeCont.appendTurnarounds(explicitTurnarounds, oc.getBool("no-turnarounds.tls"));
     }
@@ -446,8 +446,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Rechecking of lane endings");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Rechecking of lane endings");
     myEdgeCont.recheckLanes();
     PROGRESS_TIME_MESSAGE(before);
 
@@ -469,26 +468,22 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     // join traffic lights (after building connections)
     if (oc.getBool("tls.join")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Joining traffic light nodes");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Joining traffic light nodes");
         myNodeCont.joinTLS(myTLLCont, oc.getFloat("tls.join-dist"));
         PROGRESS_TIME_MESSAGE(before);
     }
 
     // COMPUTING RIGHT-OF-WAY AND TRAFFIC LIGHT PROGRAMS
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing traffic light control information");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing traffic light control information");
     myTLLCont.setTLControllingInformation(myEdgeCont, myNodeCont);
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing node logics");
-    myNodeCont.computeLogics(myEdgeCont, oc);
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing node logics");
+    myNodeCont.computeLogics(myEdgeCont);
     PROGRESS_TIME_MESSAGE(before);
     //
-    before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Computing traffic light logics");
+    before = PROGRESS_BEGIN_TIME_MESSAGE("Computing traffic light logics");
     std::pair<int, int> numbers = myTLLCont.computeLogics(oc);
     PROGRESS_TIME_MESSAGE(before);
     std::string progCount = "";
@@ -498,8 +493,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     WRITE_MESSAGE(" " + toString(numbers.first) + " traffic light(s) " + progCount + "computed.");
     //
     if (oc.isSet("street-sign-output")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Generating street signs");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Generating street signs");
         myEdgeCont.generateStreetSigns();
         PROGRESS_TIME_MESSAGE(before);
     }
@@ -509,8 +503,7 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     }
     // FINISHING INNER EDGES
     if (!oc.getBool("no-internal-links")) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Building inner edges");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Building inner edges");
         // walking areas shall only be built if crossings are wished as well
         for (std::map<std::string, NBNode*>::const_iterator i = myNodeCont.begin(); i != myNodeCont.end(); ++i) {
             (*i).second->buildInnerEdges();
@@ -520,11 +513,10 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     // PATCH NODE SHAPES
     if (oc.getFloat("junctions.scurve-stretch") > 0) {
         // @note: nodes have collected correction hints in buildInnerEdges()
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("stretching junctions to smooth geometries");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("stretching junctions to smooth geometries");
         myEdgeCont.computeLaneShapes();
         myNodeCont.computeNodeShapes();
-        myEdgeCont.computeEdgeShapes();
+        myEdgeCont.computeEdgeShapes(oc.getBool("geometry.max-grade.fix") ? oc.getFloat("geometry.max-grade") / 100 : -1);
         for (std::map<std::string, NBNode*>::const_iterator i = myNodeCont.begin(); i != myNodeCont.end(); ++i) {
             (*i).second->buildInnerEdges();
         }
@@ -538,14 +530,12 @@ NBNetBuilder::compute(OptionsCont& oc, const std::set<std::string>& explicitTurn
     };
 
     if (oc.exists("geometry.check-overlap")  && oc.getFloat("geometry.check-overlap") > 0) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Checking overlapping edges");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Checking overlapping edges");
         myEdgeCont.checkOverlap(oc.getFloat("geometry.check-overlap"), oc.getFloat("geometry.check-overlap.vertical-threshold"));
         PROGRESS_TIME_MESSAGE(before);
     }
     if (geoConvHelper.getConvBoundary().getZRange() > 0 && oc.getFloat("geometry.max-grade") > 0) {
-        before = SysUtils::getCurrentMillis();
-        PROGRESS_BEGIN_MESSAGE("Checking edge grade");
+        before = PROGRESS_BEGIN_TIME_MESSAGE("Checking edge grade");
         // user input is in %
         myEdgeCont.checkGrade(oc.getFloat("geometry.max-grade") / 100);
         PROGRESS_TIME_MESSAGE(before);
@@ -600,8 +590,7 @@ NBNetBuilder::computeSingleNode(NBNode* node, OptionsCont& oc, const std::set<st
 
 void
 NBNetBuilder::moveToOrigin(GeoConvHelper& geoConvHelper, bool lefthand) {
-    long before = SysUtils::getCurrentMillis();
-    PROGRESS_BEGIN_MESSAGE("Moving network to origin");
+    long before = PROGRESS_BEGIN_TIME_MESSAGE("Moving network to origin");
     Boundary boundary = geoConvHelper.getConvBoundary();
     const double x = -boundary.xmin();
     const double y = -(lefthand ? boundary.ymax() : boundary.ymin());
@@ -676,24 +665,7 @@ NBNetBuilder::transformCoordinates(PositionVector& from, bool includeInBoundary,
         for (int i = 0; i < (int) from.size(); i++) {
             transformCoordinate(copy[i], false);
         }
-        // check lengths and insert new points where needed (in the original
-        // coordinate system)
-        int inserted = 0;
-        for (int i = 0; i < (int)copy.size() - 1; i++) {
-            Position start = from[i + inserted];
-            Position end = from[i + inserted + 1];
-            double length = copy[i].distanceTo(copy[i + 1]);
-            const Position step = (end - start) * (maxLength / length);
-            int steps = 0;
-            while (length > maxLength) {
-                length -= maxLength;
-                steps++;
-                from.insert(from.begin() + i + inserted + 1, start + (step * steps));
-                inserted++;
-            }
-        }
-        // now perform the transformation again so that height mapping can be
-        // performed for the new points
+        addGeometrySegments(from, copy, maxLength);
     }
     bool ok = true;
     for (int i = 0; i < (int) from.size(); i++) {
@@ -702,6 +674,27 @@ NBNetBuilder::transformCoordinates(PositionVector& from, bool includeInBoundary,
     return ok;
 }
 
+int
+NBNetBuilder::addGeometrySegments(PositionVector& from, const PositionVector& cartesian, const double maxLength) {
+    // check lengths and insert new points where needed (in the original
+    // coordinate system)
+    int inserted = 0;
+    for (int i = 0; i < (int)cartesian.size() - 1; i++) {
+        Position start = from[i + inserted];
+        Position end = from[i + inserted + 1];
+        double length = cartesian[i].distanceTo(cartesian[i + 1]);
+        const Position step = (end - start) * (maxLength / length);
+        int steps = 0;
+        while (length > maxLength) {
+            length -= maxLength;
+            steps++;
+            from.insert(from.begin() + i + inserted + 1, start + (step * steps));
+            inserted++;
+        }
+    }
+    return inserted;
+}
+
 
 bool
 NBNetBuilder::runningNetedit() {
