@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Gregor Laemmel
 /// @date    Fri, 29.04.2005
-/// @version $Id$
 ///
 // Instance responsible for building networks
 /****************************************************************************/
@@ -31,6 +30,7 @@
 #include <iostream>
 #include <vector>
 #include <set>
+#include <utils/shapes/ShapeContainer.h>
 #include "NBEdgeCont.h"
 #include "NBTypeCont.h"
 #include "NBNodeCont.h"
@@ -187,6 +187,10 @@ public:
         return myParkingCont;
     }
 
+    ShapeContainer& getShapeCont() {
+        return myShapeCont;
+    }
+
     /// @brief notify about style of loaded network (Without Crossings)
     bool haveNetworkCrossings() {
         return myNetworkHaveCrossings;
@@ -208,6 +212,9 @@ public:
     static bool transformCoordinate(Position& from, bool includeInBoundary = true, GeoConvHelper* from_srs = 0);
     static bool transformCoordinates(PositionVector& from, bool includeInBoundary = true, GeoConvHelper* from_srs = 0);
 
+    /// @brief insertion geometry points to ensure maximum segment length between points
+    static int addGeometrySegments(PositionVector& from, const PositionVector& cartesian, const double maxLength);
+
     /// @brief whether netbuilding takes place in the context of NETEDIT
     static bool runningNetedit();
 
@@ -252,6 +259,9 @@ protected:
 
     NBParkingCont myParkingCont;
 
+    /// @brief container for loaded polygon data
+    ShapeContainer myShapeCont;
+
     /// @brief flag to indicate that network has crossings
     bool myNetworkHaveCrossings;
 
