@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Yun-Pang Floetteroed
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // The representation of a single node
 /****************************************************************************/
@@ -86,22 +85,12 @@ public:
      *  but may also be transposed in full when there is not enough space.
      */
     class ApproachingDivider : public Bresenham::BresenhamCallBack {
-    private:
-        /// @brief The list of edges that approach the current edge
-        EdgeVector* myApproaching;
-
-        /// @brief The approached current edge
-        NBEdge* myCurrentOutgoing;
-
-        /// @brief The available lanes to which connections shall be built
-        std::vector<int> myAvailableLanes;
-
     public:
         /**@brief Constructor
          * @param[in] approaching The list of the edges that approach the outgoing edge
          * @param[in] currentOutgoing The outgoing edge
          */
-        ApproachingDivider(EdgeVector* approaching, NBEdge* currentOutgoing);
+        ApproachingDivider(const EdgeVector& approaching, NBEdge* currentOutgoing);
 
         /// @brief Destructor
         ~ApproachingDivider();
@@ -117,6 +106,23 @@ public:
         /// @brief the method that spreads the wished number of lanes from the the lane given by the bresenham-call to both left and right
         std::deque<int>* spread(const std::vector<int>& approachingLanes, int dest) const;
 
+    private:
+        /// @brief The list of edges that approach the current edge
+        const EdgeVector& myApproaching;
+
+        /// @brief The approached current edge
+        NBEdge* myCurrentOutgoing;
+
+        /// @brief The available lanes to which connections shall be built
+        std::vector<int> myAvailableLanes;
+
+        /// @brief whether the outgoing edge is exclusively used by bikes
+        bool myIsBikeEdge;
+
+    private:
+        /// @brief Invalidated assignment operator.
+        ApproachingDivider& operator=(const ApproachingDivider&) = delete;
+
     };
 
     /** @struct Crossing
@@ -165,17 +171,14 @@ public:
         /// @brief constructor
         WalkingArea(const std::string& _id, double _width) :
             id(_id),
-            width(_width),
-            hasCustomShape(false),
-            minNextCrossingEdges(std::numeric_limits<int>::max()),
-            minPrevCrossingEdges(std::numeric_limits<int>::max()) {
+            width(_width) {
         }
         /// @brief the (edge)-id of this walkingArea
         std::string id;
         /// @brief This lane's width
         double width;
         /// @brief This lane's width
-        double length;
+        double length = INVALID_DOUBLE;
         /// @brief The polygonal shape
         PositionVector shape;
         /// @brief the lane-id of the next crossing(s)
@@ -185,11 +188,11 @@ public:
         /// @brief the lane-id of the previous sidewalk lane or ""
         std::vector<std::string> prevSidewalks;
         /// @brief whether this walkingArea has a custom shape
-        bool hasCustomShape;
+        bool hasCustomShape = false;
         /// @brief minimum number of edges crossed by nextCrossings
-        int minNextCrossingEdges;
+        int minNextCrossingEdges = std::numeric_limits<int>::max();
         /// @brief minimum number of edges crossed by incoming crossings
-        int minPrevCrossingEdges;
+        int minPrevCrossingEdges = std::numeric_limits<int>::max();
     };
 
     struct WalkingAreaCustomShape {
@@ -208,6 +211,8 @@ public:
     static const int AVOID_WIDE_RIGHT_TURN;
     static const int AVOID_WIDE_LEFT_TURN;
     static const int FOUR_CONTROL_POINTS;
+    static const int AVOID_INTERSECTING_LEFT_TURNS;
+    static const int SCURVE_IGNORE;
 
 public:
     /**@brief Constructor
@@ -282,6 +287,11 @@ public:
     RightOfWay getRightOfWay() const {
         return myRightOfWay;
     }
+
+    /// @brief Returns fringe type
+    FringeType getFringeType() const {
+        return myFringeType;
+    }
     /// @}
 
     /// @name Methods for dealing with assigned traffic lights
@@ -355,7 +365,7 @@ public:
     void computeLanes2Lanes();
 
     /// @brief computes the node's type, logic and traffic light
-    void computeLogic(const NBEdgeCont& ec, OptionsCont& oc);
+    void computeLogic(const NBEdgeCont& ec);
 
     /// @brief compute right-of-way logic for all lane-to-lane connections
     void computeLogic2(bool checkLaneFoes);
@@ -507,6 +517,11 @@ public:
         myRightOfWay = rightOfWay;
     }
 
+    /// @brief set method for computing right-of-way
+    void setFringeType(FringeType fringeType) {
+        myFringeType = fringeType;
+    }
+
     /// @brief return whether the shape was set by the user
     bool hasCustomShape() const {
         return myHaveCustomPoly;
@@ -543,7 +558,7 @@ public:
      * @param[in] recordError The node itself if the displacement error during shape computation shall be recorded
      * @return The shape of the internal lane
      */
-    PositionVector computeInternalLaneShape(NBEdge* fromE, const NBEdge::Connection& con, int numPoints, NBNode* recordError = 0) const;
+    PositionVector computeInternalLaneShape(NBEdge* fromE, const NBEdge::Connection& con, int numPoints, NBNode* recordError = 0, int shapeFlag = 0) const;
 
     /**@brief Compute a smooth curve between the given geometries
      * @param[in] begShape The geometry at the start
@@ -623,6 +638,7 @@ public:
 
     /// @brief whether this is structurally similar to a geometry node
     bool geometryLike() const;
+    bool geometryLike(const EdgeVector& incoming, const EdgeVector& outgoing) const;
 
     /// @brief update the type of this node as a roundabout
     void setRoundabout();
@@ -732,7 +748,7 @@ public:
     static bool isTrafficLight(SumoXMLNodeType type);
 
     /// @brief check if node is a simple continuation
-    bool isSimpleContinuation(bool checkLaneNumbers = true) const;
+    bool isSimpleContinuation(bool checkLaneNumbers = true, bool checkWidth = false) const;
 
     /// @brief mark whether a priority road turns at this node
     void markBentPriority(bool isBent) {
@@ -740,22 +756,27 @@ public:
     }
 
     /// @brief return whether a priority road turns at this node
-    bool isBentPriority() {
+    bool isBentPriority() const {
         return myIsBentPriority;
     }
 
+    /// @brief return whether a priority road turns at this node
+    bool typeWasGuessed() const {
+        return myTypeWasGuessed;
+    }
+
     /// @brief detects whether a given junction splits or merges lanes while keeping constant road width
     bool isConstantWidthTransition() const;
 
     /// @brief return list of unique endpoint coordinates of all edges at this node
-    std::vector<Position> getEndPoints() const;
+    std::vector<std::pair<Position, std::string> > getEndPoints() const;
 
 private:
     /// @brief sets the priorites in case of a priority junction
     void setPriorityJunctionPriorities();
 
     /// @brief returns a list of edges which are connected to the given outgoing edge
-    EdgeVector* getEdgesThatApproach(NBEdge* currentOutgoing);
+    void getEdgesThatApproach(NBEdge* currentOutgoing, EdgeVector& approaching);
 
     /// @brief replace incoming connections prohibitions
     void replaceInConnectionProhibitions(NBEdge* which, NBEdge* by, int whichLaneOff, int byLaneOff);
@@ -780,7 +801,9 @@ private:
 
     /// @brief returns whether sub is a subset of super
     static bool includes(const std::set<NBEdge*, ComparatorIdLess>& super,
-                  const std::set<const NBEdge*, ComparatorIdLess>& sub);
+                         const std::set<const NBEdge*, ComparatorIdLess>& sub);
+
+    NBEdge* getNextCompatibleOutgoing(const NBEdge* incoming, SVCPermissions vehPerm, EdgeVector::const_iterator start, bool clockwise) const;
 
 private:
     /// @brief The position the node lies at
@@ -834,6 +857,9 @@ private:
     /// @brief how to compute right of way for this node
     RightOfWay myRightOfWay;
 
+    /// @brief fringe type of this node
+    FringeType myFringeType;
+
     /// @brief whether to discard all pedestrian crossings
     bool myDiscardAllCrossings;
 
@@ -848,6 +874,9 @@ private:
      */
     bool myIsBentPriority;
 
+    /// @brief whether the node type was guessed rather than loaded
+    bool myTypeWasGuessed;
+
 
 private:
     /// @brief invalidated copy constructor
