@@ -1,13 +1,11 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
 /****************************************************************************/
 /// @file    NBNodeCont.h
 /// @author  Daniel Krajzewicz
@@ -16,7 +14,6 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // Container for nodes during the netbuilding process
 /****************************************************************************/
@@ -27,11 +24,7 @@
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -65,6 +58,11 @@ class NBPTStopCont;
  */
 class NBNodeCont {
 public:
+    /// @brief Definition of a node cluster container
+    typedef std::set<NBNode*, ComparatorIdLess> NodeSet;
+    typedef std::vector<NodeSet> NodeClusters;
+    typedef std::pair<NBNode*, double> NodeAndDist;
+
     /// @brief Constructor
     NBNodeCont();
 
@@ -136,13 +134,25 @@ public:
     /** @brief add ids of nodes which shall be joined into a single node
      * @param[in] cluster The cluster to add
      */
-    void addCluster2Join(std::set<std::string> cluster);
+    void addCluster2Join(std::set<std::string> cluster, NBNode* node);
 
     /// @brief Joins loaded junction clusters (see NIXMLNodesHandler)
     int joinLoadedClusters(NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc);
 
     /// @brief Joins junctions that are very close together
     int joinJunctions(double maxDist, NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc, NBPTStopCont& sc);
+
+    /// @brief remove geometry-like fringe nodes from cluster
+    void pruneClusterFringe(NodeSet& cluster) const;
+
+    /// @brief determine wether the cluster is not too complex for joining
+    bool feasibleCluster(const NodeSet& cluster, const NBEdgeCont& ec, const NBPTStopCont& sc, std::string& reason) const;
+
+    /// @brief try to find a joinable subset (recursively)
+    bool reduceToCircle(NodeSet& cluster, int circleSize, NodeSet startNodes, std::vector<NBNode*> cands = std::vector<NBNode*>()) const;
+
+    /// @brief find closest neighbor for building circle
+    NBEdge* shortestEdge(const NodeSet& cluster, const NodeSet& startNodes, const std::vector<NBNode*>& exclude) const;
     /// @}
 
     /// @name Adapting the input
@@ -230,14 +240,24 @@ public:
     void setAsTLControlled(NBNode* node, NBTrafficLightLogicCont& tlc, TrafficLightType type, std::string id = "");
     /// @}
 
+    /** @brief Returns whether the node with the id was deleted explicitly
+     */
+    bool wasRemoved(std::string id) const {
+        return myExtractedNodes.count(id) != 0;
+    }
+
+
     /// @brief Renames the node. Throws exception if newID already exists
     void rename(NBNode* node, const std::string& newID);
 
     /// divides the incoming lanes on outgoing lanes
     void computeLanes2Lanes();
 
-    /// build the list of outgoing edges and lanes
-    void computeLogics(const NBEdgeCont& ec, OptionsCont& oc);
+    /// @brief build the list of outgoing edges and lanes
+    void computeLogics(const NBEdgeCont& ec);
+
+    /// @brief compute right-of-way logic for all lane-to-lane connections
+    void computeLogics2(const NBEdgeCont& ec, OptionsCont& oc);
 
     /// @brief Returns the number of nodes stored in this container
     int size() const {
@@ -272,10 +292,11 @@ public:
      * @param[out] hasTLS Whether the new node has a traffic light
      * @param[out] tlType The type of traffic light (if any)
      */
-    void analyzeCluster(std::set<NBNode*> cluster, std::string& id, Position& pos, bool& hasTLS, TrafficLightType& type);
+    void analyzeCluster(NodeSet cluster, std::string& id, Position& pos,
+                        bool& hasTLS, TrafficLightType& type, SumoXMLNodeType& nodeType);
 
     /// @brief gets all joined clusters (see doc for myClusters2Join)
-    void registerJoinedCluster(const std::set<NBNode*>& cluster);
+    void registerJoinedCluster(const NodeSet& cluster);
 
     /// @brief gets all joined clusters (see doc for myClusters2Join)
     const std::vector<std::set<std::string> >& getJoinedClusters() const {
@@ -287,18 +308,19 @@ public:
      */
     void discardTrafficLights(NBTrafficLightLogicCont& tlc, bool geometryLike, bool guessSignals);
 
+    /* @brief discards rail signals
+     */
+    void discardRailSignals();
+
     /// @brief mark a node as being created form a split
     void markAsSplit(const NBNode* node) {
         mySplit.insert(node);
     }
 
     /// @brief remap node IDs accoring to options --numerical-ids and --reserved-ids
-    int remapIDs(bool numericaIDs, bool reservedIDs);
+    int remapIDs(bool numericaIDs, bool reservedIDs, const std::string& prefix);
 
 private:
-    /// @brief Definition of a node cluster container
-    typedef std::vector<std::set<NBNode*> > NodeClusters;
-    typedef std::pair<NBNode*, double> NodeAndDist;
 
     /// @name Helper methods for for joining nodes
     /// @{
@@ -313,6 +335,7 @@ private:
 
     /// @brief joins the given node clusters
     void joinNodeClusters(NodeClusters clusters, NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc);
+    void joinNodeCluster(NodeSet clusters, NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc, NBNode* predefined = nullptr);
 
     /// @}
 
@@ -320,9 +343,16 @@ private:
     /// @{
     /** @brief Returns whethe the given node cluster should be controlled by a tls
      * @param[in] c The node cluster
+     * @param[in] laneSpeedThreshold threshold for determining whether a node or cluster should be tls controlled
      * @return Whether this node cluster shall be controlled by a tls
      */
-    bool shouldBeTLSControlled(const std::set<NBNode*>& c) const;
+    bool shouldBeTLSControlled(const NodeSet& c, double laneSpeedThreshold) const;
+
+    /// @brief check wheter the set of nodes only contains pedestrian crossings
+    bool onlyCrossings(const NodeSet& c) const;
+
+    /// @brief check wheter the set of nodes contains traffic lights with custom id
+    bool customTLID(const NodeSet& c) const;
     /// @}
 
 
@@ -337,13 +367,13 @@ private:
     NodeCont myNodes;
 
     /// @brief The extracted nodes which are kept for reference
-    std::set<NBNode*> myExtractedNodes;
+    NodeCont myExtractedNodes;
 
     /// @brief set of node ids which should not be joined
     std::set<std::string> myJoinExclusions;
 
     /// @brief loaded sets of node ids to join (cleared after use)
-    std::vector<std::set<std::string> > myClusters2Join;
+    std::vector<std::pair<std::set<std::string>, NBNode*> > myClusters2Join;
 
     /// @brief sets of node ids which were joined
     std::vector<std::set<std::string> > myJoinedClusters;
