@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Sascha Krieg
 /// @date    Tue, 29.05.2005
-/// @version $Id$
 ///
 // A traffic light logics which must be computed (only nodes/edges are given)
 /****************************************************************************/
@@ -168,6 +167,10 @@ protected:
 
 
 protected:
+
+    /// @brief test whether a joined tls with layout 'opposites' would be built without dedicated left-turn phase
+    bool corridorLike() const;
+
     /** @brief Returns the weight of a stream given its direction
      * @param[in] dir The direction of the stream
      * @return This stream's weight
@@ -215,8 +218,25 @@ protected:
     static EdgeVector getConnectedOuterEdges(const EdgeVector& incoming);
 
 
-    /// @brief allow connections that follow on of the chosen edges
-    std::string allowFollowersOfChosen(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+    /// @brief allow connections that are compatible with the chosen edges
+    std::string allowCompatible(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                                const std::vector<int>& fromLanes, const std::vector<int>& toLanes);
+
+    std::string allowSingleEdge(std::string state, const EdgeVector& fromEdges);
+
+    std::string allowFollowers(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges);
+
+    std::string allowPredecessors(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                                  const std::vector<int>& fromLanes, const std::vector<int>& toLanes);
+
+    std::string allowUnrelated(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges,
+                               const std::vector<bool>& isTurnaround,
+                               const std::vector<NBNode::Crossing*>& crossings);
+
+    std::string allowByVClass(std::string state, const EdgeVector& fromEdges, const EdgeVector& toEdges, SVCPermissions perm);
+
+    /// @brief whether the given index is forbidden by a green link in the current state
+    bool forbidden(const std::string& state, int index, const EdgeVector& fromEdges, const EdgeVector& toEdges);
 
     /** @brief change 'G' to 'g' for conflicting connections
      * @param[in] state
