@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    NBPTStop.cpp
 /// @author  Gregor Laemmel
 /// @date    Tue, 20 Mar 2017
-/// @version $Id$
 ///
 // The representation of a single pt stop
 /****************************************************************************/
@@ -41,6 +40,7 @@ NBPTStop::NBPTStop(std::string ptStopId, Position position, std::string edgeId,
     myName(name),
     myPermissions(svcPermissions),
     myBidiStop(nullptr),
+    myIsLoose(origEdgeId == ""),
     myIsMultipleStopPositions(false) {
 }
 
@@ -74,7 +74,7 @@ NBPTStop::getPosition() const {
 
 
 void
-NBPTStop::computExtent(double center, double edgeLength) {
+NBPTStop::computeExtent(double center, double edgeLength) {
     myStartPos = MAX2(0.0, center - myPTStopLength / 2.);
     myEndPos = MIN2(center + myPTStopLength / 2., edgeLength);
 }
@@ -164,9 +164,9 @@ NBPTStop::getLength() const {
 
 
 bool
-NBPTStop::setEdgeId(std::string edgeId, NBEdgeCont& ec) {
+NBPTStop::setEdgeId(std::string edgeId, const NBEdgeCont& ec) {
     myEdgeId = edgeId;
-    return findLaneAndComputeBusStopExtend(ec);
+    return findLaneAndComputeBusStopExtent(ec);
 }
 
 
@@ -195,12 +195,12 @@ NBPTStop::setMyPTStopLength(double myPTStopLength) {
 
 
 bool
-NBPTStop::findLaneAndComputeBusStopExtend(NBEdgeCont& ec) {
+NBPTStop::findLaneAndComputeBusStopExtent(const NBEdgeCont& ec) {
     NBEdge* edge = ec.getByID(myEdgeId);
     if (edge != nullptr) {
         int laneNr = -1;
         for (const auto& it : edge->getLanes()) {
-            if ((it.permissions & getPermissions()) > 0) {
+            if ((it.permissions & getPermissions()) == getPermissions()) {
                 ++laneNr;
                 break;
             }
@@ -209,12 +209,13 @@ NBPTStop::findLaneAndComputeBusStopExtend(NBEdgeCont& ec) {
         if (laneNr != -1) {
             myLaneId = edge->getLaneID(laneNr);
             const PositionVector& shape = edge->getLaneShape(laneNr);
-            double offset = shape.nearest_offset_to_point2D(getPosition(), true);
-            computExtent(offset, shape.length());
+            double offset = shape.nearest_offset_to_point2D(getPosition(), false);
+            offset = offset * edge->getLoadedLength() / edge->getLength();
+            computeExtent(offset, edge->getLoadedLength());
             return true;
         }
     }
-    return false;
+    return myEdgeId == ""; // loose stop. Try later when processing lines
 }
 
 
