@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    NBPTStop.h
 /// @author  Gregor Laemmel
 /// @date    Tue, 20 Mar 2017
-/// @version $Id$
 ///
 // The representation of a single pt stop
 /****************************************************************************/
@@ -59,7 +58,6 @@ public:
     const std::string getName() const;
     const Position& getPosition() const;
     SVCPermissions getPermissions() const;
-    void computExtent(double center, double d);
     void write(OutputDevice& device);
     void reshiftPosition(const double offsetX, const double offsetY);
 
@@ -67,10 +65,10 @@ public:
     bool getIsMultipleStopPositions() const;
     void setIsMultipleStopPositions(bool multipleStopPositions);
     double getLength() const;
-    bool setEdgeId(std::string edgeId, NBEdgeCont& ec);
+    bool setEdgeId(std::string edgeId, const NBEdgeCont& ec);
     void registerAdditionalEdge(std::string wayId, std::string edgeId);
     void addPlatformCand(NBPTPlatform platform);
-    bool findLaneAndComputeBusStopExtend(NBEdgeCont& ec);
+    bool findLaneAndComputeBusStopExtent(const NBEdgeCont& ec);
 
     void setMyPTStopId(std::string id);
     void addAccess(std::string laneID, double offset, double length);
@@ -89,6 +87,13 @@ public:
         return myBidiStop;
     }
 
+    bool isLoose() const {
+        return myIsLoose;
+    }
+
+private:
+    void computeExtent(double center, double d);
+
 private:
     std::string myPTStopId;
     Position myPosition;
@@ -120,6 +125,8 @@ private:
 
     NBPTStop* myBidiStop;
 
+    /// @brief whether the stop was not part of the road network and must be mapped
+    bool myIsLoose;
 
 private:
     /// @brief Invalidated assignment operator.
