@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    NBPTStopCont.h
 /// @author  Gregor Laemmel
 /// @date    Tue, 20 Mar 2017
-/// @version $Id$
 ///
 // Container for pt stops during the netbuilding process
 /****************************************************************************/
@@ -68,7 +67,7 @@ public:
      *
      * @param cont
      */
-    void cleanupDeleted(NBEdgeCont& cont);
+    int cleanupDeleted(NBEdgeCont& cont);
 
     void assignLanes(NBEdgeCont& cont);
 
@@ -84,6 +83,8 @@ public:
     /// @brief add edges that must be kept
     void addEdges2Keep(const OptionsCont& oc, std::set<std::string>& into);
 
+    NBPTStop* findStop(const std::string& origEdgeID, Position pos, double threshold = 1) const;
+
 private:
     /// @brief Definition of the map of names to pt stops
     typedef std::map<std::string, NBPTStop*> PTStopsCont;
