@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -13,7 +13,6 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Tue, 20 Nov 2001
-/// @version $Id$
 ///
 // This class computes the logic of a junction
 /****************************************************************************/
@@ -161,11 +160,11 @@ NBRequest::buildBitfieldLogic() {
 
 void
 NBRequest::computeRightOutgoingLinkCrossings(NBEdge* from, NBEdge* to) {
-    EdgeVector::const_iterator pfrom = find(myAll.begin(), myAll.end(), from);
+    EdgeVector::const_iterator pfrom = std::find(myAll.begin(), myAll.end(), from);
     while (*pfrom != to) {
         NBContHelper::nextCCW(myAll, pfrom);
         if ((*pfrom)->getToNode() == myJunction) {
-            EdgeVector::const_iterator pto = find(myAll.begin(), myAll.end(), to);
+            EdgeVector::const_iterator pto = std::find(myAll.begin(), myAll.end(), to);
             while (*pto != from) {
                 if (!((*pto)->getToNode() == myJunction)) {
                     setBlocking(from, to, *pfrom, *pto);
@@ -179,11 +178,11 @@ NBRequest::computeRightOutgoingLinkCrossings(NBEdge* from, NBEdge* to) {
 
 void
 NBRequest::computeLeftOutgoingLinkCrossings(NBEdge* from, NBEdge* to) {
-    EdgeVector::const_iterator pfrom = find(myAll.begin(), myAll.end(), from);
+    EdgeVector::const_iterator pfrom = std::find(myAll.begin(), myAll.end(), from);
     while (*pfrom != to) {
         NBContHelper::nextCW(myAll, pfrom);
         if ((*pfrom)->getToNode() == myJunction) {
-            EdgeVector::const_iterator pto = find(myAll.begin(), myAll.end(), to);
+            EdgeVector::const_iterator pto = std::find(myAll.begin(), myAll.end(), to);
             while (*pto != from) {
                 if (!((*pto)->getToNode() == myJunction)) {
                     setBlocking(from, to, *pfrom, *pto);
@@ -253,8 +252,8 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
     }
 #ifdef DEBUG_SETBLOCKING
     if (DEBUGCOND) std::cout << "setBlocking"
-            << " 1:" << from1->getID() << "->" << to1->getID() 
-            << " 2:" << from2->getID() << "->" << to2->getID() << "\n";
+                                 << " 1:" << from1->getID() << "->" << to1->getID()
+                                 << " 2:" << from2->getID() << "->" << to2->getID() << "\n";
 #endif
     // check the priorities if required by node type
     if (myJunction->getType() != NODETYPE_RIGHT_BEFORE_LEFT) {
@@ -262,9 +261,9 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
         int from2p = from2->getJunctionPriority(myJunction);
 #ifdef DEBUG_SETBLOCKING
         if (DEBUGCOND) std::cout << "setBlocking"
-            << " 1:" << from1->getID() << "->" << to1->getID() 
-            << " 2:" << from2->getID() << "->" << to2->getID()
-            << " p1=" << from1p << " p2=" << from2p << "\n";
+                                     << " 1:" << from1->getID() << "->" << to1->getID()
+                                     << " 2:" << from2->getID() << "->" << to2->getID()
+                                     << " p1=" << from1p << " p2=" << from2p << "\n";
 #endif
         // check if one of the connections is higher priorised when incoming into
         //  the junction, the connection road will yield
@@ -284,9 +283,9 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
         LinkDirection ld2 = myJunction->getDirection(from2, to2);
 #ifdef DEBUG_SETBLOCKING
         if (DEBUGCOND) std::cout << "setBlocking"
-            << " 1:" << from1->getID() << "->" << to1->getID() 
-            << " 2:" << from2->getID() << "->" << to2->getID()
-            << " dir1=" << toString(ld1) << " dir2=" << toString(ld2) << "\n";
+                                     << " 1:" << from1->getID() << "->" << to1->getID()
+                                     << " 2:" << from2->getID() << "->" << to2->getID()
+                                     << " dir1=" << toString(ld1) << " dir2=" << toString(ld2) << "\n";
 #endif
         if (ld1 == LINKDIR_STRAIGHT) {
             if (ld2 != LINKDIR_STRAIGHT) {
@@ -325,7 +324,7 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
 
     // compute the yielding due to the right-before-left rule
     // get the position of the incoming lanes in the junction-wheel
-    EdgeVector::const_iterator c1 = find(myAll.begin(), myAll.end(), from1);
+    EdgeVector::const_iterator c1 = std::find(myAll.begin(), myAll.end(), from1);
     NBContHelper::nextCW(myAll, c1);
     // go through next edges clockwise...
     while (*c1 != from1 && *c1 != from2) {
@@ -337,7 +336,7 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
         NBContHelper::nextCW(myAll, c1);
     }
     // get the position of the incoming lanes in the junction-wheel
-    EdgeVector::const_iterator c2 = find(myAll.begin(), myAll.end(), from2);
+    EdgeVector::const_iterator c2 = std::find(myAll.begin(), myAll.end(), from2);
     NBContHelper::nextCW(myAll, c2);
     // go through next edges clockwise...
     while (*c2 != from2 && *c2 != from1) {
@@ -349,17 +348,17 @@ NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
         NBContHelper::nextCW(myAll, c2);
     }
 #ifdef DEBUG_SETBLOCKING
-        if (DEBUGCOND) std::cout << "setBlocking"
-            << " 1:" << from1->getID() << "->" << to1->getID() 
-            << " 2:" << from2->getID() << "->" << to2->getID()
-            << " noDecision\n";
+    if (DEBUGCOND) std::cout << "setBlocking"
+                                 << " 1:" << from1->getID() << "->" << to1->getID()
+                                 << " 2:" << from2->getID() << "->" << to2->getID()
+                                 << " noDecision\n";
 #endif
 }
 
 
 int
 NBRequest::distanceCounterClockwise(NBEdge* from, NBEdge* to) {
-    EdgeVector::const_iterator p = find(myAll.begin(), myAll.end(), from);
+    EdgeVector::const_iterator p = std::find(myAll.begin(), myAll.end(), from);
     int ret = 0;
     do {
         ret++;
@@ -371,7 +370,7 @@ NBRequest::distanceCounterClockwise(NBEdge* from, NBEdge* to) {
     return ret;
 }
 
-const std::string& 
+const std::string&
 NBRequest::getFoes(int linkIndex) const {
     assert(linkIndex >= 0);
     assert(linkIndex < (int)myFoes.size());
@@ -420,7 +419,7 @@ NBRequest::computeLogic(const bool checkLaneFoes) {
     for (i = myIncoming.begin(); i != myIncoming.end(); i++) {
         int noLanes = (*i)->getNumLanes();
         for (int k = 0; k < noLanes; k++) {
-            pos = computeLaneResponse(*i, k, pos, checkLaneFoes);
+            pos = computeLaneResponse(*i, k, pos, checkLaneFoes || myJunction->getType() == NODETYPE_ZIPPER);
         }
     }
     // crossings
@@ -661,15 +660,16 @@ NBRequest::getResponseString(const NBEdge* const from, const NBEdge::Connection&
                                   << "\n";
                     }
 #endif
-                    const bool hasLaneConflict = (!(checkLaneFoes || checkLaneFoesByClass(queryCon, *i, connected[k]) 
-                                || checkLaneFoesByCooperation(from, queryCon, *i, connected[k]))
-                            || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane));
+                    const bool hasLaneConflict = (!(checkLaneFoes || checkLaneFoesByClass(queryCon, *i, connected[k])
+                                                    || checkLaneFoesByCooperation(from, queryCon, *i, connected[k]))
+                                                  || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane));
                     if ((myForbids[idx2][idx] && hasLaneConflict)
                             || NBNode::rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane, lefthand)
                             || mergeConflict(from, queryCon, *i, connected[k], false)
                             || oppositeLeftTurnConflict(from, queryCon, *i, connected[k], false)
                             || myJunction->rightOnRedConflict(c.tlLinkIndex, connected[k].tlLinkIndex)
-                            || (myJunction->tlsContConflict(from, c, *i, connected[k]) && hasLaneConflict)
+                            || (myJunction->tlsContConflict(from, c, *i, connected[k]) && hasLaneConflict
+                                && !OptionsCont::getOptions().getBool("tls.ignore-internal-junction-jam"))
                        ) {
                         result += '1';
                     } else {
@@ -713,8 +713,8 @@ NBRequest::getFoesString(NBEdge* from, NBEdge* to, int fromLane, int toLane, con
             int size = (int) connected.size();
             for (int k = size; k-- > 0;) {
                 const bool hasLaneConflict = (!(checkLaneFoes || checkLaneFoesByClass(queryCon, *i, connected[k])
-                            || checkLaneFoesByCooperation(from, queryCon, *i, connected[k]))
-                        || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane));
+                                                || checkLaneFoesByCooperation(from, queryCon, *i, connected[k]))
+                                              || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane));
                 if ((foes(from, to, (*i), connected[k].toEdge) && hasLaneConflict)
                         || NBNode::rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane, lefthand)
                         || myJunction->turnFoes(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane, lefthand)
@@ -825,7 +825,7 @@ NBRequest::checkLaneFoesByClass(const NBEdge::Connection& con,
 
 bool
 NBRequest::checkLaneFoesByCooperation(const NBEdge* from, const NBEdge::Connection& con,
-        const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon) const {
+                                      const NBEdge* prohibitorFrom,  const NBEdge::Connection& prohibitorCon) const {
     if (con.toEdge != prohibitorCon.toEdge) {
         return false;
     }
@@ -841,7 +841,7 @@ NBRequest::checkLaneFoesByCooperation(const NBEdge* from, const NBEdge::Connecti
     }
     for (const auto& c : prohibitorFrom->getConnections()) {
         if (c.toEdge == con.toEdge && fromTargetLanes.count(c.toLane) != 0) {
-            //std::cout << " con=" << con->getDescription(from) << " foe=" << prohibitorCon.getDescription(prohibitorFrom) 
+            //std::cout << " con=" << con->getDescription(from) << " foe=" << prohibitorCon.getDescription(prohibitorFrom)
             //    << " no cooperation (targets=" << joinToString(fromTargetLanes, ' ') << " index=" << c.toLane << "\n";
             return false;
         }
@@ -873,8 +873,8 @@ NBRequest::laneConflict(const NBEdge* from, const NBEdge* to, int toLane,
 
 int
 NBRequest::getIndex(const NBEdge* const from, const NBEdge* const to) const {
-    EdgeVector::const_iterator fp = find(myIncoming.begin(), myIncoming.end(), from);
-    EdgeVector::const_iterator tp = find(myOutgoing.begin(), myOutgoing.end(), to);
+    EdgeVector::const_iterator fp = std::find(myIncoming.begin(), myIncoming.end(), from);
+    EdgeVector::const_iterator tp = std::find(myOutgoing.begin(), myOutgoing.end(), to);
     if (fp == myIncoming.end() || tp == myOutgoing.end()) {
         return -1;
     }
@@ -968,6 +968,15 @@ NBRequest::mustBrake(const NBEdge* const from, const NBEdge* const to, int fromL
             }
         }
     }
+    // maybe we need to brake due to a zipper conflict
+    if (myJunction->getType() == NODETYPE_ZIPPER) {
+        for (int idx1 = 0; idx1 < numLinks(); idx1++) {
+            //assert(myDone[idx1][idx2]);
+            if (myDone[idx1][idx2] && myForbids[idx2][idx1]) {
+                return true;
+            }
+        }
+    }
     return false;
 }
 
