@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id$
 ///
 // The base class for traffic light logic definitions
 /****************************************************************************/
@@ -109,8 +108,8 @@ NBTrafficLightDefinition::compute(OptionsCont& oc) {
     if (amInvalid()) {
         // make a copy of myControlledNodes because it will be modified;
         std::vector<NBNode*> nodes = myControlledNodes;
-        for (std::vector<NBNode*>::iterator it = nodes.begin(); it != nodes.end(); it++) {
-            (*it)->removeTrafficLight(this);
+        for (auto it : nodes) {
+            it->removeTrafficLight(this);
         }
         WRITE_WARNING("The traffic light '" + getID() + "' does not control any links; it will not be build.");
         return nullptr;
@@ -123,7 +122,7 @@ NBTrafficLightDefinition::compute(OptionsCont& oc) {
         brakingTime = oc.getInt("tls.yellow.time");
     }
     NBTrafficLightLogic* ret = myCompute(brakingTime);
-    ret->updateParameter(getParametersMap());
+    ret->updateParameters(getParametersMap());
     return ret;
 }
 
@@ -200,7 +199,7 @@ NBTrafficLightDefinition::collectEdges() {
     for (EdgeVector::iterator j = myIncomingEdges.begin(); j != myIncomingEdges.end(); ++j) {
         NBEdge* edge = *j;
         // an edge lies within the logic if it is outgoing as well as incoming
-        EdgeVector::iterator k = find(myOutgoing.begin(), myOutgoing.end(), edge);
+        EdgeVector::iterator k = std::find(myOutgoing.begin(), myOutgoing.end(), edge);
         if (k != myOutgoing.end()) {
             myEdgesWithin.push_back(edge);
         } else  {
@@ -218,7 +217,7 @@ NBTrafficLightDefinition::collectEdges() {
         // edges that are marked as 'inner' will not get their own phase when
         // computing traffic light logics (unless they cannot be reached from the outside at all)
         if (reachable.count(edge) == 1) {
-            edge->setInternal();
+            edge->setInsideTLS();
             // legacy behavior
             if (uncontrolledWithin && myControlledInnerEdges.count(edge->getID()) == 0) {
                 myIncomingEdges.erase(find(myIncomingEdges.begin(), myIncomingEdges.end(), edge));
@@ -465,17 +464,23 @@ NBTrafficLightDefinition::collectAllLinks() {
                     if (el.toEdge != nullptr && el.toLane >= (int) el.toEdge->getNumLanes()) {
                         throw ProcessError("Connection '" + incoming->getID() + "_" + toString(j) + "->" + el.toEdge->getID() + "_" + toString(el.toLane) + "' yields in a not existing lane.");
                     }
-                    if (incoming->getToNode()->getType() != NODETYPE_RAIL_CROSSING || !isRailway(incoming->getPermissions())) {
-                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex++));
-                    } else {
+                    if (incoming->getToNode()->getType() == NODETYPE_RAIL_CROSSING
+                            && isRailway(incoming->getPermissions())) {
+                        // railways stay uncontrolled at rail crossing but they
+                        // must be registered in MSRailCrossing
                         myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, -1));
+                    } else if (incoming->getToNode()->getType() == NODETYPE_RAIL_SIGNAL
+                               && incoming->getToNode()->getDirection(incoming, el.toEdge) == LINKDIR_TURN) {
+                        // turnarounds stay uncontrolled at rail signal
+                    } else {
+                        myControlledLinks.push_back(NBConnection(incoming, el.fromLane, el.toEdge, el.toLane, tlIndex++));
                     }
                 }
             }
         }
     }
     if (myControlledLinks.size() > 0 && tlIndex == 0) {
-        WRITE_WARNING("The rail crossing '" + getID() + "' does not have any roads.");
+        WRITE_WARNINGF("The rail crossing '%' does not have any roads.", getID());
     }
 }
 
@@ -525,7 +530,7 @@ NBTrafficLightDefinition::rightOnRedConflict(int index, int foeIndex) const {
     return std::find(myRightOnRedConflicts.begin(), myRightOnRedConflicts.end(), std::make_pair(index, foeIndex)) != myRightOnRedConflicts.end();
 }
 
-std::string 
+std::string
 NBTrafficLightDefinition::getDescription() const {
     return getID() + ':' + getProgramID() + '@' + toString(this);
 }
