@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,14 +10,13 @@
 /// @file    GNENet.h
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id$
 ///
 // The lop level container for GNE-network-components such as GNEEdge and
 // GNEJunction.  Contains an internal instances of NBNetBuilder GNE components
 // wrap netbuild-components of this underlying NBNetBuilder and supply
 // visualisation and editing capabilities (adapted from GUINet)
 //
-// Workflow (rough draft)
+// WorkrouteFlow (rough draft)
 //   wrap NB-components
 //   do netedit stuff
 //   call NBNetBuilder::buildLoaded to save results
@@ -53,6 +52,7 @@
 
 class NBNetBuilder;
 class GNEAdditional;
+class GNEDemandElement;
 class GNEApplicationWindow;
 class GNEAttributeCarrier;
 class GNEConnection;
@@ -78,6 +78,7 @@ class GNENet : public GUIGlObject, public ShapeContainer {
 
     /// @brief declare friend class
     friend class GNEAdditionalHandler;
+    friend class GNERouteHandler;
     friend class GNEChange_Junction;
     friend class GNEChange_Edge;
     friend class GNEChange_Lane;
@@ -85,8 +86,27 @@ class GNENet : public GUIGlObject, public ShapeContainer {
     friend class GNEChange_Shape;
     friend class GNEChange_CalibratorItem;
     friend class GNEChange_Additional;
+    friend class GNEChange_DemandElement;
 
 public:
+    /// @brief struct used for saving all attribute carriers of net, in different formats
+    struct AttributeCarriers {
+        /// @brief map with the name and pointer to junctions of net
+        std::map<std::string, GNEJunction*> junctions;
+
+        /// @brief map with the name and pointer to edges of net
+        std::map<std::string, GNEEdge*> edges;
+
+        /// @brief map with the name and pointer to additional elements of net
+        std::map<SumoXMLTag, std::map<std::string, GNEAdditional*> > additionals;
+
+        /// @brief map with the name and pointer to demand elements of net
+        std::map<SumoXMLTag, std::map<std::string, GNEDemandElement*> > demandElements;
+
+        /// @brief special map used for saving Demand Elements of type "Vehicle" (Vehicles, routeFlows, etc.) sorted by depart time
+        std::map<std::string, GNEDemandElement*> vehicleDepartures;
+    };
+
     /**@brief Constructor
      * @param[in] netbuilder the netbuilder which may already have been filled
      * GNENet becomes responsible for cleaning this up
@@ -151,7 +171,7 @@ public:
     * @return whether the polygon could be added
     */
     bool addPolygon(const std::string& id, const std::string& type, const RGBColor& color, double layer,
-                    double angle, const std::string& imgFile, bool relativePath, const PositionVector& shape, 
+                    double angle, const std::string& imgFile, bool relativePath, const PositionVector& shape,
                     bool geo, bool fill, double lineWidth, bool ignorePruning = false);
 
     /**@brief Builds a POI using the given values and adds it to the container
@@ -255,6 +275,18 @@ public:
      */
     void deleteShape(GNEShape* shape, GNEUndoList* undoList);
 
+    /**@brief remove additional
+     * @param[in] additional The Shape to be removed
+     * @param[in] undoList The undolist in which to mark changes
+     */
+    void deleteAdditional(GNEAdditional* additional, GNEUndoList* undoList);
+
+    /**@brief remove demand element
+     * @param[in] demandElement The Shape to be removed
+     * @param[in] undoList The undolist in which to mark changes
+     */
+    void deleteDemandElement(GNEDemandElement* demandElement, GNEUndoList* undoList);
+
     /**@brief duplicates lane
      * @param[in] lane The lane to be duplicated
      * @param[in] undoList The undolist in which to mark changes
@@ -273,14 +305,14 @@ public:
      * @param[in] edge The edge in which insert restricted lane
      * @param[in] undoList The undolist in which to mark changes
      */
-    bool addRestrictedLane(SUMOVehicleClass vclass, GNEEdge& edge, int index, GNEUndoList* undoList);
+    bool addRestrictedLane(SUMOVehicleClass vclass, GNEEdge* edge, int index, GNEUndoList* undoList);
 
     /**@brief remove restricted lane
      * @param[in] vclass vehicle class to restrict
      * @param[in] edge the edge in which remove sidewalk
      * @param[in] undoList The undolist in which to mark changes
      */
-    bool removeRestrictedLane(SUMOVehicleClass vclass, GNEEdge& edge, GNEUndoList* undoList);
+    bool removeRestrictedLane(SUMOVehicleClass vclass, GNEEdge* edge, GNEUndoList* undoList);
 
     /**@brief split edge at position by inserting a new junction
      * @param[in] edge The edge to be split
@@ -315,6 +347,9 @@ public:
      */
     void mergeJunctions(GNEJunction* moved, GNEJunction* target, GNEUndoList* undoList);
 
+    /// @brief retrieve all attribute carriers of Net
+    const AttributeCarriers& getAttributeCarriers() const;
+
     /**@brief get junction by id
      * @param[in] id The id of the desired junction
      * @param[in] failHard Whether attempts to retrieve a nonexisting junction should result in an exception
@@ -421,7 +456,7 @@ public:
     std::vector<GNEShape*> retrieveShapes(bool onlySelected = false);
 
     /// @brief inform that net has to be saved
-    void requiereSaveNet(bool value);
+    void requireSaveNet(bool value);
 
     /// @brief return if net has to be saved
     bool isNetSaved() const;
@@ -462,8 +497,8 @@ public:
     /// @brief get view net
     GNEViewNet* getViewNet() const;
 
-    /// @brief get all selected attribute carriers
-    std::vector<GNEAttributeCarrier*> getSelectedAttributeCarriers();
+    /// @brief get all selected attribute carriers (or only relative to current supermode
+    std::vector<GNEAttributeCarrier*> getSelectedAttributeCarriers(bool ignoreCurrentSupermode);
 
     /// @brief returns the tllcont of the underlying netbuilder
     NBTrafficLightLogicCont& getTLLogicCont();
@@ -482,9 +517,14 @@ public:
      * param[in] force Whether to force recomputation even if not needed
      * param[in] volatileOptions enable or disable volatile options
      * param[in] additionalPath path in wich additionals were saved before recomputing with volatile options
-     * param[in] shapePath path in wich shapes were saved before recomputing with volatile options
+     * param[in] demandPath path in wich demand elements were saved before recomputing with volatile options
      */
-    void computeEverything(GNEApplicationWindow* window, bool force = false, bool volatileOptions = false, std::string additionalPath = "", std::string shapePath = "");
+    void computeNetwork(GNEApplicationWindow* window, bool force = false, bool volatileOptions = false, std::string additionalPath = "", std::string demandPath = "");
+
+    /**@brief compute demand elements
+     * param[in] window The window to inform about delay
+     */
+    void computeDemandElements(GNEApplicationWindow* window);
 
     /**@brief join selected junctions
      * @note difference to mergeJunctions:
@@ -500,11 +540,20 @@ public:
     /// @brief removes junctions that have no edges
     void removeSolitaryJunctions(GNEUndoList* undoList);
 
+    /// @brief clean unused routes
+    void cleanUnusedRoutes(GNEUndoList* undoList);
+
+    /// @brief join routes
+    void joinRoutes(GNEUndoList* undoList);
+
+    /// @brief clean invalid demand elements
+    void cleanInvalidDemandElements(GNEUndoList* undoList);
+
     /// @brief replace the selected junction by geometry node(s) and merge the edges
     void replaceJunctionByGeometry(GNEJunction* junction, GNEUndoList* undoList);
 
     /// @brief replace the selected junction by a list of junctions for each unique edge endpoint
-    void splitJunction(GNEJunction* junction, GNEUndoList* undoList);
+    void splitJunction(GNEJunction* junction, bool reconnect, GNEUndoList* undoList);
 
     /// @brief clear junction's connections
     void clearJunctionConnections(GNEJunction* junction, GNEUndoList* undoList);
@@ -550,12 +599,6 @@ public:
      */
     std::vector<GNEAdditional*> retrieveAdditionals(bool onlySelected = false) const;
 
-    /**@brief get map with IDs and pointers to additionals
-     * @param[in] type type of additional to get. SUMO_TAG_NOTHING will get all additionals
-     * @return map with IDs and pointers to additionals.
-     */
-    const std::map<std::string, GNEAdditional*>& getAdditionalByType(SumoXMLTag type) const;
-
     /**@brief Returns the number of additionals of the net
      * @param[in] type type of additional to count. SUMO_TAG_NOTHING will count all additionals
      * @return Number of additionals of the net
@@ -568,18 +611,68 @@ public:
     void updateAdditionalID(const std::string& oldID, GNEAdditional* additional);
 
     /// @brief inform that additionals has to be saved
-    void requiereSaveAdditionals(bool value);
+    void requireSaveAdditionals(bool value);
 
     /**@brief save additional elements of the network
     * @param[in] filename name of the file in wich save additionals
     */
     void saveAdditionals(const std::string& filename);
 
+    /// @brief check if additionals are saved
+    bool isAdditionalsSaved() const;
+
     /// @brief generate additional id
     std::string generateAdditionalID(SumoXMLTag type) const;
 
     /// @}
 
+    /// @name Functions related to DemandElement Items
+    /// @{
+
+    /**@brief Returns the named demand element
+     * @param[in] type tag with the type of demand element
+     * @param[in] id The id of the demand element to return.
+     * @param[in] failHard Whether attempts to retrieve a nonexisting demand element should result in an exception
+     */
+    GNEDemandElement* retrieveDemandElement(SumoXMLTag type, const std::string& id, bool hardFail = true) const;
+
+    /**@brief return all demand elements
+     * @param[in] onlySelected Whether to return only selected demand elements
+     */
+    std::vector<GNEDemandElement*> retrieveDemandElements(bool onlySelected = false) const;
+
+    /**@brief Returns the number of demand elements of the net
+     * @param[in] type type of demand element to count. SUMO_TAG_NOTHING will count all demand elements
+     * @return Number of demand elements of the net
+     */
+    int getNumberOfDemandElements(SumoXMLTag type = SUMO_TAG_NOTHING) const;
+
+    /**@brief update demand element ID in container
+    * @note this function is automatically called when user changes the ID of an demand element
+    */
+    void updateDemandElementID(const std::string& oldID, GNEDemandElement* demandElement);
+
+    /**@brief update demand element begin in container
+    * @note this function is automatically called when user changes the begin/departure of an demand element
+    */
+    void updateDemandElementBegin(const std::string& oldBegin, GNEDemandElement* demandElement);
+
+    /// @brief inform that demand elements has to be saved
+    void requireSaveDemandElements(bool value);
+
+    /**@brief save demand element elements of the network
+    * @param[in] filename name of the file in wich save demand elements
+    */
+    void saveDemandElements(const std::string& filename);
+
+    /// @brief check if demand elements are saved
+    bool isDemandElementsSaved() const;
+
+    /// @brief generate demand element id
+    std::string generateDemandElementID(const std::string& prefix, SumoXMLTag type) const;
+
+    /// @}
+
     /// @name Functions related to Shapes
     /// @{
 
@@ -602,14 +695,6 @@ public:
     /// @brief change Shape ID
     void changeShapeID(GNEShape* s, const std::string& OldID);
 
-    /// @brief inform that shapes has to be saved
-    void requiereSaveShapes(bool value);
-
-    /**@brief save shapes elements of the network
-     * @param[in] filename name of the file in wich save shapes
-     */
-    void saveShapes(const std::string& filename);
-
     /// @brief get number of shapes
     int getNumberOfShapes() const;
     /// @}
@@ -617,7 +702,7 @@ public:
     /// @name Functions related to TLS Programs
     /// @{
     /// @brief inform that TLS Programs has to be saved
-    void requiereSaveTLSPrograms();
+    void requireSaveTLSPrograms();
 
     /**@brief save TLS Programs elements of the network
      * @param[in] filename name of the file in wich save TLS Programs
@@ -628,19 +713,20 @@ public:
     int getNumberOfTLSPrograms() const;
     /// @}
 
+    /// @name Functions related to Enable or disable update geometry of elements after insertio
+    /// @{
+    /// @brief enable update geometry of elements after inserting or removing an element in net
+    void enableUpdateGeometry();
+
+    /// @brief disable update geometry of elements after inserting or removing an element in net
+    void disableUpdateGeometry();
+
+    /// @brief check if update geometry after inserting or removing has to be updated
+    bool isUpdateGeometryEnabled() const;
+
+    /// @}
+
 protected:
-    /// @brief struct used for saving all attribute carriers of net, in different formats
-    struct AttributeCarriers {
-        /// @brief map with the name and pointer to junctions of net
-        std::map<std::string, GNEJunction*> junctions;
-
-        /// @brief map with the name and pointer to edges of net
-        std::map<std::string, GNEEdge*> edges;
-
-        /// @brief map with the name and pointer to additional elements of net
-        std::map<SumoXMLTag, std::map<std::string, GNEAdditional*> > additionals;
-    };
-
     /// @brief the rtree which contains all GUIGlObjects (so named for historical reasons)
     SUMORTree myGrid;
 
@@ -671,17 +757,20 @@ protected:
     /// @brief Flag to check if additionals has to be saved
     bool myAdditionalsSaved;
 
-    /// @brief Flag to check if shapes has to be saved
-    bool myShapesSaved;
-
     /// @brief Flag to check if shapes has to be saved
     bool myTLSProgramsSaved;
 
+    /// @brief Flag to check if demand elements has to be saved
+    bool myDemandElementsSaved;
+
+    /// @brief Flag to enable or disable update geometry of elements after inserting or removing element in net
+    bool myUpdateGeometryEnabled;
+
     /// @name Insertion and erasing of GNEAdditionals items
     /// @{
 
     /// @brief return true if additional exist (use pointer instead ID)
-    bool additionalExist(GNEAdditional* additional);
+    bool additionalExist(GNEAdditional* additional) const;
 
     /**@brief Insert a additional element int GNENet container.
      * @throw processError if route was already inserted
@@ -691,7 +780,25 @@ protected:
     /**@brief delete additional element of GNENet container
      * @throw processError if additional wasn't previously inserted
      */
-    bool deleteAdditional(GNEAdditional* additional);
+    bool deleteAdditional(GNEAdditional* additional, bool updateViewAfterDeleting);
+
+    /// @}
+
+    /// @name Insertion and erasing of GNEDemandElements items
+    /// @{
+
+    /// @brief return true if demand element exist (use pointer instead ID)
+    bool demandElementExist(GNEDemandElement* demandElement) const;
+
+    /**@brief Insert a demand element element int GNENet container.
+     * @throw processError if route was already inserted
+     */
+    void insertDemandElement(GNEDemandElement* demandElement);
+
+    /**@brief delete demand element element of GNENet container
+     * @throw processError if demand element wasn't previously inserted
+     */
+    bool deleteDemandElement(GNEDemandElement* demandElement, bool updateViewAfterDeleting);
 
     /// @}
 
@@ -712,16 +819,16 @@ private:
     GNEEdge* registerEdge(GNEEdge* edge);
 
     /// @brief deletes a single junction
-    void deleteSingleJunction(GNEJunction* junction);
+    void deleteSingleJunction(GNEJunction* junction, bool updateViewAfterDeleting);
 
     /// @brief deletes a single edge
-    void deleteSingleEdge(GNEEdge* edge);
+    void deleteSingleEdge(GNEEdge* edge, bool updateViewAfterDeleting);
 
     /// @brief insert shape
-    void insertShape(GNEShape* shape);
+    void insertShape(GNEShape* shape, bool updateViewAfterDeleting);
 
     /// @brief remove created shape (but NOT delete)
-    void removeShape(GNEShape* shape);
+    void removeShape(GNEShape* shape, bool updateViewAfterDeleting);
 
     /// @brief notify myViewNet
     void update();
@@ -738,6 +845,9 @@ private:
     /// @brief save additionals after confirming invalid objects
     void saveAdditionalsConfirmed(const std::string& filename);
 
+    /// @brief save demand elements after confirming invalid objects
+    void saveDemandElementsConfirmed(const std::string& filename);
+
     static void replaceInListAttribute(GNEAttributeCarrier* ac, SumoXMLAttr key, const std::string& which, const std::string& by, GNEUndoList* undoList);
 
     /// @brief the z boundary (stored in the x-coordinate), values of 0 are ignored
