@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GNEUndoList.cpp
 /// @author  Jakob Erdmann
 /// @date    Mar 2011
-/// @version $Id$
 ///
 // FXUndoList is pretty dandy but some features are missing:
 //   - we cannot find out wether we have currently begun an undo-group and
@@ -65,7 +64,7 @@ FXIMPLEMENT_ABSTRACT(GNEUndoList, FXUndoList, GNEUndoListMap, ARRAYNUMBER(GNEUnd
 
 GNEUndoList::GNEUndoList(GNEApplicationWindow* parent) :
     FXUndoList(),
-    myParent(parent) {
+    myGNEApplicationWindowParent(parent) {
 }
 
 
@@ -112,20 +111,19 @@ GNEUndoList::p_abortLastCommandGroup() {
 
 void
 GNEUndoList::undo() {
-    //std::cout << undoName().text() << "\n";
-    WRITE_DEBUG("Keys Ctrl + Z (Undo) pressed");
+    WRITE_DEBUG("Calling GNEUndoList::undo()");
     FXUndoList::undo();
-    myParent->updateControls();
+    // update specific controls
+    myGNEApplicationWindowParent->updateControls();
 }
 
 
 void
 GNEUndoList::redo() {
-    //std::cout << redoName().text() << "\n";
-    //std::cout << undoName().text() << "\n";
-    WRITE_DEBUG("Keys Ctrl + Y (Redo) pressed");
+    WRITE_DEBUG("Calling GNEUndoList::redo()");
     FXUndoList::redo();
-    myParent->updateControls();
+    // update specific controls
+    myGNEApplicationWindowParent->updateControls();
 }
 
 
@@ -151,17 +149,39 @@ GNEUndoList::currentCommandGroupSize() const {
 
 long
 GNEUndoList::p_onUpdUndo(FXObject* sender, FXSelector, void*) {
-    bool enable = canUndo() && !hasCommandGroup();
-    sender->handle(this, enable ? FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE) : FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
-    FXString caption = undoName();
-    if (hasCommandGroup()) {
-        caption = ("Cannot Undo in the middle of " + myCommandGroups.top()->getDescription()).c_str();
-    } else if (!canUndo()) {
-        caption = "Undo";
+    // first check if Undo Menu command or button has to be disabled
+    bool enable = canUndo() && !hasCommandGroup() && myGNEApplicationWindowParent->isUndoRedoEnabled().empty();
+    // cast button (see #6209)
+    FXButton* button = dynamic_cast<FXButton*>(sender);
+    // enable or disable depending of "enable" flag
+    if (button) {
+        // avoid unnnecesary enables/disables (due flickering)
+        if (enable && !button->isEnabled()) {
+            sender->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE), nullptr);
+            button->update();
+        } else if (!enable && button->isEnabled()) {
+            sender->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
+            button->update();
+        }
+    } else {
+        sender->handle(this, enable ? FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE) : FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
     }
-    // only set caption on menu item
-    if (dynamic_cast<FXMenuCommand*>(sender)) {
-        sender->handle(this, FXSEL(SEL_COMMAND, FXMenuCaption::ID_SETSTRINGVALUE), (void*)&caption);
+    // cast menu command
+    FXMenuCommand* menuCommand = dynamic_cast<FXMenuCommand*>(sender);
+    // only set caption on menu command item
+    if (menuCommand) {
+        // change caption of FXMenuCommand
+        FXString caption = undoName();
+        // set caption of FXmenuCommand edit/undo
+        if (myGNEApplicationWindowParent->isUndoRedoEnabled().size() > 0) {
+            caption = ("Cannot Undo in the middle of " + myGNEApplicationWindowParent->isUndoRedoEnabled()).c_str();
+        } else if (hasCommandGroup()) {
+            caption = ("Cannot Undo in the middle of " + myCommandGroups.top()->getDescription()).c_str();
+        } else if (!canUndo()) {
+            caption = "Undo";
+        }
+        menuCommand->handle(this, FXSEL(SEL_COMMAND, FXMenuCaption::ID_SETSTRINGVALUE), (void*)&caption);
+        menuCommand->update();
     }
     return 1;
 }
@@ -169,17 +189,39 @@ GNEUndoList::p_onUpdUndo(FXObject* sender, FXSelector, void*) {
 
 long
 GNEUndoList::p_onUpdRedo(FXObject* sender, FXSelector, void*) {
-    bool enable = canRedo() && !hasCommandGroup();
-    sender->handle(this, enable ? FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE) : FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
-    FXString caption = redoName();
-    if (hasCommandGroup()) {
-        caption = ("Cannot Redo in the middle of " + myCommandGroups.top()->getDescription()).c_str();
-    } else if (!canRedo()) {
-        caption = "Redo";
+    // first check if Redo Menu command or button has to be disabled
+    bool enable = canRedo() && !hasCommandGroup() && myGNEApplicationWindowParent->isUndoRedoEnabled().empty();
+    // cast button (see #6209)
+    FXButton* button = dynamic_cast<FXButton*>(sender);
+    // enable or disable depending of "enable" flag
+    if (button) {
+        // avoid unnnecesary enables/disables (due flickering)
+        if (enable && !button->isEnabled()) {
+            sender->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE), nullptr);
+            button->update();
+        } else if (!enable && button->isEnabled()) {
+            sender->handle(this, FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
+            button->update();
+        }
+    } else {
+        sender->handle(this, enable ? FXSEL(SEL_COMMAND, FXWindow::ID_ENABLE) : FXSEL(SEL_COMMAND, FXWindow::ID_DISABLE), nullptr);
     }
-    // only set caption on menu item
-    if (dynamic_cast<FXMenuCommand*>(sender)) {
-        sender->handle(this, FXSEL(SEL_COMMAND, FXMenuCaption::ID_SETSTRINGVALUE), (void*)&caption);
+    // cast menu command
+    FXMenuCommand* menuCommand = dynamic_cast<FXMenuCommand*>(sender);
+    // only set caption on menu command item
+    if (menuCommand) {
+        // change caption of FXMenuCommand
+        FXString caption = redoName();
+        // set caption of FXmenuCommand edit/undo
+        if (myGNEApplicationWindowParent->isUndoRedoEnabled().size() > 0) {
+            caption = ("Cannot Redo in the middle of " + myGNEApplicationWindowParent->isUndoRedoEnabled()).c_str();
+        } else if (hasCommandGroup()) {
+            caption = ("Cannot Redo in the middle of " + myCommandGroups.top()->getDescription()).c_str();
+        } else if (!canRedo()) {
+            caption = "Redo";
+        }
+        menuCommand->handle(this, FXSEL(SEL_COMMAND, FXMenuCaption::ID_SETSTRINGVALUE), (void*)&caption);
+        menuCommand->update();
     }
     return 1;
 }
