@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GNEUndoList.h
 /// @author  Jakob Erdmann
 /// @date    Mar 2011
-/// @version $Id$
 ///
 // FXUndoList is pretty dandy but some features are missing:
 //   - we cannot find out wether we have currently begun an undo-group and
@@ -29,8 +28,9 @@
 // ===========================================================================
 #include <config.h>
 
-#include <fx.h>
 #include <stack>
+#include <string>
+#include <fx.h>
 
 
 // ===========================================================================
@@ -122,11 +122,11 @@ private:
         const std::string myDescription;
     };
 
-    // the stack of currently active command groups
+    // @brief the stack of currently active command groups
     std::stack<CommandGroup*> myCommandGroups;
 
-    // the parent application for this undolist
-    GNEApplicationWindow* const myParent;
+    // @brief the parent GNEApplicationWindow for this undolist
+    GNEApplicationWindow* const myGNEApplicationWindowParent;
 };
 
 
