@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GNEViewParent.h
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id$
 ///
 // A single child window which contains a view of the edited network (adapted
 // from GUISUMOViewParent)
@@ -32,20 +31,29 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class GNEDialogACChooser;
-class GNENet;
-class GNEUndoList;
-class GNEApplicationWindow;
-class GNEInspectorFrame;
-class GNESelectorFrame;
-class GNEConnectorFrame;
-class GNETLSEditorFrame;
 class GNEAdditionalFrame;
+class GNEApplicationWindow;
+class GNEConnectorFrame;
+class GNECreateEdgeFrame;
 class GNECrossingFrame;
-class GNETAZFrame;
 class GNEDeleteFrame;
+class GNEDialogACChooser;
+class GNEInspectorFrame;
+class GNENet;
+class GNEFrame;
 class GNEPolygonFrame;
 class GNEProhibitionFrame;
+class GNERouteFrame;
+class GNESelectorFrame;
+class GNETAZFrame;
+class GNETLSEditorFrame;
+class GNEUndoList;
+class GNEVehicleFrame;
+class GNEVehicleTypeFrame;
+class GNEStopFrame;
+class GNEPersonTypeFrame;
+class GNEPersonFrame;
+class GNEPersonPlanFrame;
 
 // ===========================================================================
 // class declarations
@@ -92,36 +100,63 @@ public:
     /// @brief hide all frames
     void hideAllFrames();
 
-    /// @brief get frame for GNE_MODE_INSPECT
+    /// @brief get current frame (note: it can be null)
+    GNEFrame* getCurrentShownFrame() const;
+
+    /// @brief get frame for GNE_NMODE_INSPECT
     GNEInspectorFrame* getInspectorFrame() const;
 
-    /// @brief get frame for GNE_MODE_SELECT
+    /// @brief get frame for GNE_NMODE_SELECT
     GNESelectorFrame* getSelectorFrame() const;
 
-    /// @brief get frame for GNE_MODE_CONNECT
+    /// @brief get frame for GNE_NMODE_CONNECT
     GNEConnectorFrame* getConnectorFrame() const;
 
-    /// @brief get frame for GNE_MODE_TLS
+    /// @brief get frame for GNE_NMODE_TLS
     GNETLSEditorFrame* getTLSEditorFrame() const;
 
-    /// @brief get frame for GNE_MODE_ADDITIONAL
+    /// @brief get frame for GNE_NMODE_ADDITIONAL
     GNEAdditionalFrame* getAdditionalFrame() const;
 
-    /// @brief get frame for GNE_MODE_CROSSING
+    /// @brief get frame for GNE_NMODE_CROSSING
     GNECrossingFrame* getCrossingFrame() const;
 
-    /// @brief get frame for GNE_MODE_TAZ
+    /// @brief get frame for GNE_NMODE_TAZ
     GNETAZFrame* getTAZFrame() const;
 
-    /// @brief get frame for GNE_MODE_DELETE
+    /// @brief get frame for GNE_NMODE_DELETE
     GNEDeleteFrame* getDeleteFrame() const;
 
-    /// @brief get frame for GNE_MODE_POLYGON
+    /// @brief get frame for GNE_NMODE_POLYGON
     GNEPolygonFrame* getPolygonFrame() const;
 
-    /// @brief get frame for GNE_MODE_PROHIBITION
+    /// @brief get frame for GNE_NMODE_PROHIBITION
     GNEProhibitionFrame* getProhibitionFrame() const;
 
+    /// @brief get frame for GNE_NMODE_CREATEEDGE
+    GNECreateEdgeFrame* getCreateEdgeFrame() const;
+
+    /// @brief get frame for GNE_DMODE_ROUTE
+    GNERouteFrame* getRouteFrame() const;
+
+    /// @brief get frame for GNE_DMODE_VEHICLE
+    GNEVehicleFrame* getVehicleFrame() const;
+
+    /// @brief get frame for GNE_DMODE_VEHICLETYPE
+    GNEVehicleTypeFrame* getVehicleTypeFrame() const;
+
+    /// @brief get frame for GNE_DMODE_STOP
+    GNEStopFrame* getStopFrame() const;
+
+    /// @brief get frame for GNE_DMODE_PERSONTYPE
+    GNEPersonTypeFrame* getPersonTypeFrame() const;
+
+    /// @brief get frame for GNE_DMODE_PERSON
+    GNEPersonFrame* getPersonFrame() const;
+
+    /// @brief get frame for GNE_DMODE_PERSONFRAME
+    GNEPersonPlanFrame* getPersonPlanFrame() const;
+
     /// @brief show frames area if at least a GNEFrame is showed
     /// @note this function is called in GNEFrame::Show();
     void showFramesArea();
@@ -139,6 +174,9 @@ public:
     /// @brief remove created chooser dialog
     void eraseACChooserDialog(GNEDialogACChooser* chooserDialog);
 
+    /// @brief update toolbar undo/redo buttons (called when user press Ctrl+Z/Y)
+    void updateUndoRedoButtons();
+
     /// @name FOX-callbacks
     /// @{
     /// @brief Called if the user wants to make a snapshot (screenshot)
@@ -160,12 +198,8 @@ public:
     long onCmdUpdateFrameAreaWidth(FXObject*, FXSelector, void*);
     /// @}
 
-    /// @brief true if the object is selected (may include extra logic besides calling gSelected)
-    bool isSelected(GUIGlObject* o) const;
-
 protected:
-    /// @brief FOX needs this
-    GNEViewParent() {}
+    FOX_CONSTRUCTOR(GNEViewParent)
 
 private:
     /// @brief struct for Frames
@@ -182,35 +216,62 @@ private:
         /// @brief return true if at least there is a frame shown
         bool isFrameShown() const;
 
-        /// @brief frame for GNE_MODE_INSPECT
+        /// @brief get current frame show
+        GNEFrame* getCurrentShownFrame() const;
+
+        /// @brief frame for GNE_NMODE_INSPECT
         GNEInspectorFrame* inspectorFrame;
 
-        /// @brief frame for GNE_MODE_SELECT
+        /// @brief frame for GNE_NMODE_SELECT
         GNESelectorFrame* selectorFrame;
 
-        /// @brief frame for GNE_MODE_CONNECT
+        /// @brief frame for GNE_NMODE_CONNECT
         GNEConnectorFrame* connectorFrame;
 
-        /// @brief frame for GNE_MODE_TLS
+        /// @brief frame for GNE_NMODE_TLS
         GNETLSEditorFrame* TLSEditorFrame;
 
-        /// @brief frame for GNE_MODE_ADDITIONAL
+        /// @brief frame for GNE_NMODE_ADDITIONAL
         GNEAdditionalFrame* additionalFrame;
 
-        /// @brief frame for GNE_MODE_CROSSING
+        /// @brief frame for GNE_NMODE_CROSSING
         GNECrossingFrame* crossingFrame;
 
-        /// @brief frame for GNE_MODE_TAZ
+        /// @brief frame for GNE_NMODE_TAZ
         GNETAZFrame* TAZFrame;
 
-        /// @brief frame for GNE_MODE_DELETE
+        /// @brief frame for GNE_NMODE_DELETE
         GNEDeleteFrame* deleteFrame;
 
-        /// @brief frame for GNE_MODE_POLYGON
+        /// @brief frame for GNE_NMODE_POLYGON
         GNEPolygonFrame* polygonFrame;
 
-        /// @brief frame for GNE_MODE_PROHIBITION
+        /// @brief frame for GNE_NMODE_PROHIBITION
         GNEProhibitionFrame* prohibitionFrame;
+
+        /// @brief frame for GNE_NMODE_CREATEDGE
+        GNECreateEdgeFrame* createEdgeFrame;
+
+        /// @brief frame for GNE_DMODE_ROUTE
+        GNERouteFrame* routeFrame;
+
+        /// @brief frame for GNE_DMODE_VEHICLE
+        GNEVehicleFrame* vehicleFrame;
+
+        /// @brief frame for GNE_DMODE_VEHICLETYPE
+        GNEVehicleTypeFrame* vehicleTypeFrame;
+
+        /// @brief frame for GNE_DMODE_STOP
+        GNEStopFrame* stopFrame;
+
+        /// @brief frame for GNE_DMODE_PERSON
+        GNEPersonFrame* personFrame;
+
+        /// @brief frame for GNE_DMODE_PERSONTYPE
+        GNEPersonTypeFrame* personTypeFrame;
+
+        /// @brief frame for GNE_DMODE_PERSONPLAN
+        GNEPersonPlanFrame* personPlanFrame;
     };
 
     /// @brief struct for ACChoosers dialog
@@ -221,25 +282,37 @@ private:
         /// @brief destructor
         ~ACChoosers();
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate junctions
         GNEDialogACChooser* ACChooserJunction;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate edges
         GNEDialogACChooser* ACChooserEdges;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate vehicles
+        GNEDialogACChooser* ACChooserVehicles;
+
+        /// @brief pointer to ACChooser dialog used for locate persons
+        GNEDialogACChooser* ACChooserPersons;
+
+        /// @brief pointer to ACChooser dialog used for locate routes
+        GNEDialogACChooser* ACChooserRoutes;
+
+        /// @brief pointer to ACChooser dialog used for locate stops
+        GNEDialogACChooser* ACChooserStops;
+
+        /// @brief pointer to ACChooser dialog used for locate TLSs
         GNEDialogACChooser* ACChooserTLS;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate additional
         GNEDialogACChooser* ACChooserAdditional;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate POIs
         GNEDialogACChooser* ACChooserPOI;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate Polygons
         GNEDialogACChooser* ACChooserPolygon;
 
-        /// @brief pointer to ACChooser dialog
+        /// @brief pointer to ACChooser dialog used for locate Prohibitions
         GNEDialogACChooser* ACChooserProhibition;
     };
 
@@ -252,6 +325,12 @@ private:
     /// @brief frame to hold GNEFrames
     FXHorizontalFrame* myFramesArea;
 
+    /// @brief toolbar undo button
+    FXButton* myUndoButton;
+
+    /// @brief toolbar redo button
+    FXButton* myRedoButton;
+
     /// @brief Splitter to divide ViewNet und GNEFrames
     FXSplitter* myFramesSplitter;
 
