@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GNEDetectorE2.cpp
 /// @author  Pablo Alvarez Lopez
 /// @date    Nov 2015
-/// @version $Id$
 ///
 //
 /****************************************************************************/
@@ -38,23 +37,24 @@
 // member method definitions
 // ===========================================================================
 
-GNEDetectorE2::GNEDetectorE2(const std::string& id, GNELane* lane, GNEViewNet* viewNet, double pos, double length, double freq, const std::string& filename, const std::string& vehicleTypes,
-                             const std::string& name, const double timeThreshold, double speedThreshold, double jamThreshold, bool friendlyPos, bool blockMovement) :
-    GNEDetector(id, viewNet, GLO_E2DETECTOR, SUMO_TAG_E2DETECTOR, pos, freq, filename, vehicleTypes, name, friendlyPos, blockMovement),
-    myLanes({lane}),
-    myLength(length),
-    myEndPositionOverLane(0.),
-    myTimeThreshold(timeThreshold),
-    mySpeedThreshold(speedThreshold),
-    myJamThreshold(jamThreshold),
-    myE2valid(true) {
+GNEDetectorE2::GNEDetectorE2(const std::string& id, GNELane* lane, GNEViewNet* viewNet, double pos, double length, SUMOTime freq, const std::string& filename, const std::string& vehicleTypes,
+                             const std::string& name, SUMOTime timeThreshold, double speedThreshold, double jamThreshold, bool friendlyPos, bool blockMovement) :
+    GNEDetector(id, viewNet, GLO_E2DETECTOR, SUMO_TAG_E2DETECTOR, pos, freq, filename, vehicleTypes, name, friendlyPos, blockMovement, {
+    lane
+}),
+myLength(length),
+myEndPositionOverLane(0.),
+myTimeThreshold(timeThreshold),
+mySpeedThreshold(speedThreshold),
+myJamThreshold(jamThreshold),
+myE2valid(true) {
 }
 
 
-GNEDetectorE2::GNEDetectorE2(const std::string& id, std::vector<GNELane*> lanes, GNEViewNet* viewNet, double pos, double endPos, double freq, const std::string& filename, const std::string& vehicleTypes,
-                             const std::string& name, const double timeThreshold, double speedThreshold, double jamThreshold, bool friendlyPos, bool blockMovement) :
-    GNEDetector(id, viewNet, GLO_E2DETECTOR, SUMO_TAG_E2DETECTOR_MULTILANE, pos, freq, filename, vehicleTypes, name, friendlyPos, blockMovement),
-    myLanes(lanes),
+GNEDetectorE2::GNEDetectorE2(const std::string& id, std::vector<GNELane*> lanes, GNEViewNet* viewNet, double pos, double endPos, SUMOTime freq, const std::string& filename, const std::string& vehicleTypes,
+                             const std::string& name, SUMOTime timeThreshold, double speedThreshold, double jamThreshold, bool friendlyPos, bool blockMovement) :
+    GNEDetector(id, viewNet, GLO_E2DETECTOR, SUMO_TAG_E2DETECTOR_MULTILANE, pos, freq, filename, vehicleTypes, name, friendlyPos, blockMovement, lanes),
+    myLength(0),
     myEndPositionOverLane(endPos),
     myTimeThreshold(timeThreshold),
     mySpeedThreshold(speedThreshold),
@@ -67,14 +67,14 @@ GNEDetectorE2::~GNEDetectorE2() {
 }
 
 
-bool 
+bool
 GNEDetectorE2::isAdditionalValid() const {
-    if(myLanes.size() == 1) {
+    if (getParentLanes().size() == 1) {
         // with friendly position enabled position are "always fixed"
         if (myFriendlyPosition) {
             return true;
         } else {
-            return (myPositionOverLane >= 0) && ((myPositionOverLane + myLength) <= myLanes.front()->getParentEdge().getNBEdge()->getFinalLength());
+            return (myPositionOverLane >= 0) && ((myPositionOverLane + myLength) <= getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength());
         }
     } else {
         // first check if there is connection between all consecutive lanes
@@ -83,8 +83,8 @@ GNEDetectorE2::isAdditionalValid() const {
             if (myFriendlyPosition) {
                 return true;
             } else {
-                return (myPositionOverLane >= 0) && ((myPositionOverLane) <= myLanes.back()->getParentEdge().getNBEdge()->getFinalLength() &&
-                        myEndPositionOverLane >= 0) && ((myEndPositionOverLane) <= myLanes.back()->getParentEdge().getNBEdge()->getFinalLength());
+                return (myPositionOverLane >= 0) && ((myPositionOverLane) <= getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength() &&
+                                                     myEndPositionOverLane >= 0) && ((myEndPositionOverLane) <= getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength());
             }
         } else {
             return false;
@@ -93,19 +93,19 @@ GNEDetectorE2::isAdditionalValid() const {
 }
 
 
-std::string 
+std::string
 GNEDetectorE2::getAdditionalProblem() const {
-    // declare variable for error position 
+    // declare variable for error position
     std::string errorFirstLanePosition, separator, errorLastLanePosition;
-    if(myLanes.size() == 1) {
+    if (getParentLanes().size() == 1) {
         // check positions over lane
         if (myPositionOverLane < 0) {
             errorFirstLanePosition = (toString(SUMO_ATTR_POSITION) + " < 0");
         }
-        if (myPositionOverLane > myLanes.front()->getParentEdge().getNBEdge()->getFinalLength()) {
+        if (myPositionOverLane > getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength()) {
             errorFirstLanePosition = (toString(SUMO_ATTR_POSITION) + " > lanes's length");
         }
-        if ((myPositionOverLane + myLength) > myLanes.front()->getParentEdge().getNBEdge()->getFinalLength()) {
+        if ((myPositionOverLane + myLength) > getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength()) {
             errorFirstLanePosition = (toString(SUMO_ATTR_POSITION) + " + " + toString(SUMO_ATTR_LENGTH) + " > lanes's length");
         }
     } else {
@@ -114,14 +114,14 @@ GNEDetectorE2::getAdditionalProblem() const {
             if (myPositionOverLane < 0) {
                 errorFirstLanePosition = (toString(SUMO_ATTR_POSITION) + " < 0");
             }
-            if (myPositionOverLane > myLanes.front()->getParentEdge().getNBEdge()->getFinalLength()) {
+            if (myPositionOverLane > getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength()) {
                 errorFirstLanePosition = (toString(SUMO_ATTR_POSITION) + " > lanes's length");
             }
             // check positions over last lane
             if (myEndPositionOverLane < 0) {
                 errorLastLanePosition = (toString(SUMO_ATTR_ENDPOS) + " < 0");
             }
-            if (myEndPositionOverLane > myLanes.back()->getParentEdge().getNBEdge()->getFinalLength()) {
+            if (myEndPositionOverLane > getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength()) {
                 errorLastLanePosition = (toString(SUMO_ATTR_ENDPOS) + " > lanes's length");
             }
         } else {
@@ -137,38 +137,38 @@ GNEDetectorE2::getAdditionalProblem() const {
 }
 
 
-void 
+void
 GNEDetectorE2::fixAdditionalProblem() {
-    if(myLanes.size() == 1) {
-        // obtain position and lenght
+    if (getParentLanes().size() == 1) {
+        // obtain position and length
         double newPositionOverLane = myPositionOverLane;
         double newLength = myLength;
-        // fix pos and lenght using fixE2DetectorPosition
-        GNEAdditionalHandler::fixE2DetectorPosition(newPositionOverLane, newLength, myLanes.at(0)->getParentEdge().getNBEdge()->getFinalLength(), true);
+        // fix pos and length using fixE2DetectorPosition
+        GNEAdditionalHandler::fixE2DetectorPosition(newPositionOverLane, newLength, getParentLanes().at(0)->getParentEdge()->getNBEdge()->getFinalLength(), true);
         // set new position and length
         setAttribute(SUMO_ATTR_POSITION, toString(newPositionOverLane), myViewNet->getUndoList());
         setAttribute(SUMO_ATTR_LENGTH, toString(myLength), myViewNet->getUndoList());
     } else {
-        if(!myE2valid) {
+        if (!myE2valid) {
             // build connections between all consecutive lanes
             bool foundConnection = true;
             int i = 0;
             // iterate over all lanes, and stop if myE2valid is false
-            while (i < ((int)myLanes.size()-1)) {
+            while (i < ((int)getParentLanes().size() - 1)) {
                 // change foundConnection to false
                 foundConnection = false;
                 // if a connection betwen "from" lane and "to" lane of connection is found, change myE2valid to true again
-                for (auto j : myLanes.at(i)->getParentEdge().getGNEConnections()) {
-                    if(j->getLaneFrom() == myLanes.at(i) && j->getLaneTo() == myLanes.at(i+1)) {
+                for (auto j : getParentLanes().at(i)->getParentEdge()->getGNEConnections()) {
+                    if (j->getLaneFrom() == getParentLanes().at(i) && j->getLaneTo() == getParentLanes().at(i + 1)) {
                         foundConnection = true;
                     }
                 }
                 // if connection wasn't found
-                if(!foundConnection) {
+                if (!foundConnection) {
                     // create new connection manually
-                    NBEdge::Connection newCon(myLanes.at(i)->getIndex(), myLanes.at(i+1)->getParentEdge().getNBEdge(), myLanes.at(i+1)->getIndex());
+                    NBEdge::Connection newCon(getParentLanes().at(i)->getIndex(), getParentLanes().at(i + 1)->getParentEdge()->getNBEdge(), getParentLanes().at(i + 1)->getIndex());
                     // allow to undo creation of new lane
-                    myViewNet->getUndoList()->add(new GNEChange_Connection(&myLanes.at(i)->getParentEdge(), newCon, false, true), true);
+                    myViewNet->getUndoList()->add(new GNEChange_Connection(getParentLanes().at(i)->getParentEdge(), newCon, false, true), true);
                 }
                 // update lane iterator
                 i++;
@@ -176,14 +176,14 @@ GNEDetectorE2::fixAdditionalProblem() {
         } else {
             // declare new position
             double newPositionOverLane = myPositionOverLane;
-            // fix pos and lenght  checkAndFixDetectorPosition
-            GNEAdditionalHandler::checkAndFixDetectorPosition(newPositionOverLane, myLanes.front()->getParentEdge().getNBEdge()->getFinalLength(), true);
+            // fix pos and length  checkAndFixDetectorPosition
+            GNEAdditionalHandler::checkAndFixDetectorPosition(newPositionOverLane, getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength(), true);
             // set new position
             setAttribute(SUMO_ATTR_POSITION, toString(newPositionOverLane), myViewNet->getUndoList());
             // declare new end position
             double newEndPositionOverLane = myEndPositionOverLane;
-            // fix pos and lenght  checkAndFixDetectorPosition
-            GNEAdditionalHandler::checkAndFixDetectorPosition(newEndPositionOverLane, myLanes.back()->getParentEdge().getNBEdge()->getFinalLength(), true);
+            // fix pos and length  checkAndFixDetectorPosition
+            GNEAdditionalHandler::checkAndFixDetectorPosition(newEndPositionOverLane, getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength(), true);
             // set new position
             setAttribute(SUMO_ATTR_ENDPOS, toString(newEndPositionOverLane), myViewNet->getUndoList());
         }
@@ -198,66 +198,66 @@ GNEDetectorE2::moveGeometry(const Position& offset) {
     newPosition.add(offset);
     // filtern position using snap to active grid
     newPosition = myViewNet->snapToActiveGrid(newPosition);
-    double offsetLane = myLanes.front()->getShape().nearest_offset_to_point2D(newPosition, false) - myLanes.front()->getShape().nearest_offset_to_point2D(myMove.originalViewPosition, false);
+    double offsetLane = getParentLanes().front()->getLaneShape().nearest_offset_to_point2D(newPosition, false) - getParentLanes().front()->getLaneShape().nearest_offset_to_point2D(myMove.originalViewPosition, false);
     // move geometry depending of number of lanes
-    if(myLanes.size() == 1) {
-        myPositionOverLane = parse<double>(myMove.firstOriginalLanePosition) + offsetLane;
+    if (getParentLanes().size() == 1) {
+        // calculate new position over lane
+        double newPositionOverLane = parse<double>(myMove.firstOriginalLanePosition) + offsetLane;
+        // obtain lane length
+        double laneLength = getParentLanes().front()->getParentEdge()->getNBEdge()->getFinalLength() * getLane()->getLengthGeometryFactor();
+        if (newPositionOverLane < 0) {
+            myPositionOverLane = 0;
+        } else if (newPositionOverLane + myLength > laneLength) {
+            myPositionOverLane = laneLength - myLength;
+        } else {
+            myPositionOverLane = newPositionOverLane;
+        }
     } else {
         // calculate new start and end positions
         double newStartPosition = parse<double>(myMove.firstOriginalLanePosition) + offsetLane;
         double newEndPosition = parse<double>(myMove.secondOriginalPosition) + offsetLane;
         // change start and end position of E2 detector ONLY if both extremes aren't overpassed
-        if((newStartPosition >= 0) && (newStartPosition <= myLanes.front()->getLaneShapeLength()) &&
-           (newEndPosition >= 0) && (newEndPosition <= myLanes.back()->getLaneShapeLength())) {
+        if ((newStartPosition >= 0) && (newStartPosition <= getParentLanes().front()->getLaneShapeLength()) &&
+                (newEndPosition >= 0) && (newEndPosition <= getParentLanes().back()->getLaneShapeLength())) {
             myPositionOverLane = newStartPosition;
             myEndPositionOverLane = newEndPosition;
         }
     }
     // Update geometry
-    updateGeometry(false);
+    updateGeometry();
 }
 
 
 void
 GNEDetectorE2::commitGeometryMoving(GNEUndoList* undoList) {
     // commit geometry moving depending of number of lanes
-    if(myLanes.size() == 1) {
+    if (getParentLanes().size() == 1) {
         // commit new position allowing undo/redo
         undoList->p_begin("position of " + getTagStr());
-        undoList->p_add(new GNEChange_Attribute(this, SUMO_ATTR_POSITION, toString(myPositionOverLane), true, myMove.firstOriginalLanePosition));
+        undoList->p_add(new GNEChange_Attribute(this, myViewNet->getNet(), SUMO_ATTR_POSITION, toString(myPositionOverLane), true, myMove.firstOriginalLanePosition));
         undoList->p_end();
     } else {
         undoList->p_begin("position of " + getTagStr());
-        undoList->p_add(new GNEChange_Attribute(this, SUMO_ATTR_POSITION, toString(myPositionOverLane), true, myMove.firstOriginalLanePosition));
-        undoList->p_add(new GNEChange_Attribute(this, SUMO_ATTR_ENDPOS, toString(myEndPositionOverLane), true, myMove.secondOriginalPosition));
+        undoList->p_add(new GNEChange_Attribute(this, myViewNet->getNet(), SUMO_ATTR_POSITION, toString(myPositionOverLane), true, myMove.firstOriginalLanePosition));
+        undoList->p_add(new GNEChange_Attribute(this, myViewNet->getNet(), SUMO_ATTR_ENDPOS, toString(myEndPositionOverLane), true, myMove.secondOriginalPosition));
         undoList->p_end();
     }
 }
 
 
 void
-GNEDetectorE2::updateGeometry(bool updateGrid) {
-    // first check if object has to be removed from grid (SUMOTree)
-    if (updateGrid) {
-        myViewNet->getNet()->removeGLObjectFromGrid(this);
-    }
-
-    // Clear all containers
-    myGeometry.clearGeometry();
-
+GNEDetectorE2::updateGeometry() {
     // declare variables for start and end positions
     double startPosFixed, endPosFixed;
 
     // calculate start and end positions dependin of number of lanes
-    if (myLanes.size() == 1) {
-        // set shape lane as detector shape
-        myGeometry.shape = myLanes.front()->getShape();
+    if (getParentLanes().size() == 1) {
 
         // set start position
         if (myPositionOverLane < 0) {
             startPosFixed = 0;
-        } else if (myPositionOverLane > myLanes.back()->getParentEdge().getNBEdge()->getFinalLength()) {
-            startPosFixed = myLanes.back()->getParentEdge().getNBEdge()->getFinalLength();
+        } else if (myPositionOverLane > getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength()) {
+            startPosFixed = getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength();
         } else {
             startPosFixed = myPositionOverLane;
         }
@@ -265,99 +265,34 @@ GNEDetectorE2::updateGeometry(bool updateGrid) {
         // set end position
         if ((myPositionOverLane + myLength) < 0) {
             endPosFixed = 0;
-        } else if ((myPositionOverLane + myLength) > myLanes.back()->getParentEdge().getNBEdge()->getFinalLength()) {
-            endPosFixed = myLanes.back()->getParentEdge().getNBEdge()->getFinalLength();
+        } else if ((myPositionOverLane + myLength) > getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength()) {
+            endPosFixed = getParentLanes().back()->getParentEdge()->getNBEdge()->getFinalLength();
         } else {
             endPosFixed = (myPositionOverLane + myLength);
         }
-        
+
         // Cut shape using as delimitators fixed start position and fixed end position
-        myGeometry.shape = myGeometry.shape.getSubpart(startPosFixed * myLanes.front()->getLengthGeometryFactor(), endPosFixed * myLanes.back()->getLengthGeometryFactor());
-    
-        // Get calculate lenghts and rotations
-        myGeometry.calculateShapeRotationsAndLengths();
+        myAdditionalGeometry.updateGeometryShape(getParentLanes().front()->getLaneShape(), startPosFixed * getParentLanes().front()->getLengthGeometryFactor(), endPosFixed * getParentLanes().back()->getLengthGeometryFactor());
 
         // Set block icon position
-        myBlockIcon.position = myGeometry.shape.getLineCenter();
-
-    } else if (myLanes.size() > 1) {
-        // start with the first lane shape
-        myGeometry.multiShape.push_back(myLanes.front()->getShape());
-       
-        // set start position
-        if (myPositionOverLane < 0) {
-            startPosFixed = 0;
-        } else if (myPositionOverLane > myLanes.front()->getParentEdge().getNBEdge()->getFinalLength()) {
-            startPosFixed = myLanes.front()->getParentEdge().getNBEdge()->getFinalLength();
-        } else {
-            startPosFixed = myPositionOverLane;
-        }
-        // Cut shape using as delimitators fixed start position and fixed end position
-        myGeometry.multiShape[0] = myGeometry.multiShape[0].getSubpart(startPosFixed * myLanes.front()->getLengthGeometryFactor(), myLanes.front()->getParentEdge().getNBEdge()->getFinalLength());
-       
-        // declare last shape
-        PositionVector lastShape = myLanes.back()->getShape();
-
-        // set end position
-        if (myEndPositionOverLane < 0) {
-            endPosFixed = 0;
-        } else if (myEndPositionOverLane > myLanes.back()->getParentEdge().getNBEdge()->getFinalLength()) {
-            endPosFixed = myLanes.back()->getParentEdge().getNBEdge()->getFinalLength();
-        } else {
-            endPosFixed = myEndPositionOverLane;
-        }
-
-        // Cut shape using as delimitators fixed start position and fixed end position
-        lastShape = lastShape.getSubpart(0, endPosFixed * myLanes.back()->getLengthGeometryFactor());
-
-        // add first shape connection (if exist, in other case leave it empty)
-        myGeometry.multiShape.push_back(PositionVector{myLanes.at(0)->getShape().back(), myLanes.at(1)->getShape().front()});
-        for (auto j : myLanes.at(0)->getParentEdge().getGNEConnections()) {
-            if (j->getLaneTo() == myLanes.at(1)) {
-                myGeometry.multiShape.back() = j->getShape();
-            }
-        }
-
-        // append shapes of intermediate lanes AND connections (if exist)
-        for (int i = 1; i < ((int)myLanes.size() - 1); i++) {
-            // add lane shape
-            myGeometry.multiShape.push_back(myLanes.at(i)->getShape());
-            // add empty shape for connection
-            myGeometry.multiShape.push_back(PositionVector{myLanes.at(i)->getShape().back(), myLanes.at(i+1)->getShape().front()});
-            // set connection shape (if exist). In other case, insert an empty shape
-            for (auto j : myLanes.at(i)->getParentEdge().getGNEConnections()) {
-                if (j->getLaneTo() == myLanes.at(i+1)) {
-                    myGeometry.multiShape.back() = j->getShape();
-                }
-            }
-        }
-
-        // append last shape
-        myGeometry.multiShape.push_back(lastShape);
-
-        // calculate multi shape rotation and lengths
-        myGeometry.calculateMultiShapeRotationsAndLengths();
-
-        // calculate unified shape
-        myGeometry.calculateMultiShapeUnified();
+        myBlockIcon.position = myAdditionalGeometry.getShape().getLineCenter();
 
+    } else if (getParentLanes().size() > 1) {
+        GNEGeometry::calculateLaneGeometricPath(this, mySegmentGeometry, getParentLanes(), myPositionOverLane, myEndPositionOverLane);
+        /*
         // Set block icon position
-        myBlockIcon.position = myGeometry.multiShape.front().getLineCenter();
+        myBlockIcon.position = myAdditionalGeometry.multiShape.front().getLineCenter();
 
         // check integrity
         checkE2MultilaneIntegrity();
+        */
     }
 
     // Set offset of the block icon
     myBlockIcon.offset = Position(-0.75, 0);
 
     // Set block icon rotation, and using their rotation for draw logo
-    myBlockIcon.setRotation(myLanes.front());
-
-    // last step is to check if object has to be added into grid (SUMOTree) again
-    if (updateGrid) {
-        myViewNet->getNet()->addGLObjectIntoGrid(this);
-    }
+    myBlockIcon.setRotation(getParentLanes().front());
 }
 
 
@@ -367,18 +302,18 @@ GNEDetectorE2::getLength() const {
 }
 
 
-void 
+void
 GNEDetectorE2::checkE2MultilaneIntegrity() {
     // we assume that E2 is valid
     myE2valid = true;
     int i = 0;
     // iterate over all lanes, and stop if myE2valid is false
-    while (i < ((int)myLanes.size()-1) && myE2valid) {
+    while (i < ((int)getParentLanes().size() - 1) && myE2valid) {
         // set myE2valid to false
         myE2valid = false;
         // if a connection betwen "from" lane and "to" lane of connection is found, change myE2valid to true again
-        for (auto j : myLanes.at(i)->getParentEdge().getGNEConnections()) {
-            if(j->getLaneFrom() == myLanes.at(i) && j->getLaneTo() == myLanes.at(i+1)) {
+        for (auto j : getParentLanes().at(i)->getParentEdge()->getGNEConnections()) {
+            if (j->getLaneFrom() == getParentLanes().at(i) && j->getLaneTo() == getParentLanes().at(i + 1)) {
                 myE2valid = true;
             }
         }
@@ -388,119 +323,115 @@ GNEDetectorE2::checkE2MultilaneIntegrity() {
 }
 
 
-GNELane*
-GNEDetectorE2::getLane() const {
-    return myLanes.front();
-}
-
-
 void
 GNEDetectorE2::drawGL(const GUIVisualizationSettings& s) const {
-    // Start drawing adding an gl identificator
-    glPushName(getGlID());
-
-    // Add a draw matrix
-    glPushMatrix();
-
-    // Start with the drawing of the area traslating matrix to origin
-    glTranslated(0, 0, getType());
-
-    // Set color of the base
-    if (isAttributeCarrierSelected()) {
-        GLHelper::setColor(s.selectedAdditionalColor);
-    } else {
-        // set color depending if is or isn't valid
-        if(myE2valid) {
-            GLHelper::setColor(s.SUMO_color_E2);
-        } else {
-            GLHelper::setColor(RGBColor::RED);
-        }
-    }
-
     // Obtain exaggeration of the draw
     const double exaggeration = s.addSize.getExaggeration(s, this);
-
-    // check if we have to drawn a E2 single lane or a E2 multiLane
-    if(myGeometry.shape.size() > 0) {
-        // Draw the area using shape, shapeRotations, shapeLengths and value of exaggeration
-        GLHelper::drawBoxLines(myGeometry.shape, myGeometry.shapeRotations, myGeometry.shapeLengths, exaggeration);
-    } else {
-        // iterate over multishapes
-        for (int i = 0; i < (int)myGeometry.multiShape.size(); i++) {
-            // don't draw shapes over connections if "show connections" is enabled
-            if (!myViewNet->showConnections() || (i%2==0)) {
-                GLHelper::drawBoxLines(myGeometry.multiShape.at(i), myGeometry.multiShapeRotations.at(i), myGeometry.multiShapeLengths.at(i), exaggeration);
-            }
-        }
-    }
-
-    // Pop last matrix
-    glPopMatrix();
-
-    // Check if the distance is enougth to draw details and isn't being drawn for selecting
-    if ((s.scale * exaggeration >= 10) && !s.drawForSelecting) {
-        // draw logo depending if this is an Multilane E2 detector
-        if(myTagProperty.getTag() == SUMO_TAG_E2DETECTOR) {
-            // Push matrix
-            glPushMatrix();
-            // Traslate to center of detector
-            glTranslated(myGeometry.shape.getLineCenter().x(), myGeometry.shape.getLineCenter().y(), getType() + 0.1);
-            // Rotate depending of myBlockIcon.rotation
-            glRotated(myBlockIcon.rotation, 0, 0, -1);
-            //move to logo position
-            glTranslated(-0.75, 0, 0);
-            // draw E2 logo
-            if (isAttributeCarrierSelected()) {
-                GLHelper::drawText("E2", Position(), .1, 1.5, s.selectionColor);
-            } else {
-                GLHelper::drawText("E2", Position(), .1, 1.5, RGBColor::BLACK);
-            }
+    // first check if additional has to be drawn
+    if ((myTagProperty.getTag() == SUMO_TAG_E2DETECTOR) && s.drawAdditionals(exaggeration)) {
+        // Start drawing adding an gl identificator
+        glPushName(getGlID());
+        // Add a draw matrix
+        glPushMatrix();
+        // Start with the drawing of the area traslating matrix to origin
+        glTranslated(0, 0, getType());
+        // Set color of the base
+        if (drawUsingSelectColor()) {
+            GLHelper::setColor(s.colorSettings.selectedAdditionalColor);
         } else {
-            // Push matrix
-            glPushMatrix();
-            // Traslate to center of detector
-            glTranslated(myBlockIcon.position.x(), myBlockIcon.position.y(), getType() + 0.1);
-            // Rotate depending of myBlockIcon.rotation
-            glRotated(myBlockIcon.rotation, 0, 0, -1);
-            //move to logo position
-            glTranslated(-1.5, 0, 0);
-            // draw E2 logo
-            if (isAttributeCarrierSelected()) {
-                GLHelper::drawText("E2", Position(), .1, 1.5, s.selectionColor);
+            // set color depending if is or isn't valid
+            if (myE2valid) {
+                GLHelper::setColor(s.colorSettings.E2);
             } else {
-                GLHelper::drawText("E2", Position(), .1, 1.5, RGBColor::BLACK);
-            }
-            //move to logo position
-            glTranslated(1.2, 0, 0);
-            // Rotate depending of myBlockIcon.rotation
-            glRotated(90, 0, 0, 1);
-            if (isAttributeCarrierSelected()) {
-                GLHelper::drawText("multi", Position(), .1, 0.9, s.selectedAdditionalColor);
-            } else {
-                GLHelper::drawText("multi", Position(), .1, 0.9, RGBColor::BLACK);
+                GLHelper::setColor(RGBColor::RED);
             }
         }
-        // pop matrix
+        // check if we have to drawn a E2 single lane or a E2 multiLane
+        if (myAdditionalGeometry.getShape().size() > 0) {
+            // Draw the area using shape, shapeRotations, shapeLengths and value of exaggeration
+            GNEGeometry::drawGeometry(myViewNet, myAdditionalGeometry, exaggeration);
+        } else {
+            /*
+            // iterate over multishapes
+            for (int i = 0; i < (int)myAdditionalGeometry.multiShape.size(); i++) {
+                // don't draw shapes over connections if "show connections" is enabled
+                if (!myViewNet->getNetworkViewOptions().showConnections() || (i % 2 == 0)) {
+                    GLHelper::drawBoxLines(myAdditionalGeometry.multiShape.at(i), myAdditionalGeometry.multiShapeRotations.at(i), myAdditionalGeometry.multiShapeLengths.at(i), exaggeration);
+                }
+            }
+            */
+        }
+        // Pop last matrix
         glPopMatrix();
-
-        // Show Lock icon depending of the Edit mode
-        myBlockIcon.draw();
-    }
-
-    // Draw name if isn't being drawn for selecting
-    if (!s.drawForSelecting) {
-        drawName(getCenteringBoundary().getCenter(), s.scale, s.addName);
-    }
-    // check if dotted contour has to be drawn
-    if (!s.drawForSelecting && (myViewNet->getDottedAC() == this)) {
-        if(myGeometry.shape.size() > 0) {
-            GLHelper::drawShapeDottedContour(getType(), myGeometry.shape, exaggeration);
-        } else {
-            GLHelper::drawShapeDottedContour(getType(), myGeometry.multiShapeUnified, exaggeration);
+        // Check if the distance is enougth to draw details and isn't being drawn for selecting
+        if ((s.drawDetail(s.detailSettings.detectorDetails, exaggeration)) && !s.drawForRectangleSelection) {
+            // draw logo depending if this is an Multilane E2 detector
+            if (myTagProperty.getTag() == SUMO_TAG_E2DETECTOR) {
+                // Push matrix
+                glPushMatrix();
+                // Traslate to center of detector
+                glTranslated(myAdditionalGeometry.getShape().getLineCenter().x(), myAdditionalGeometry.getShape().getLineCenter().y(), getType() + 0.1);
+                // Rotate depending of myBlockIcon.rotation
+                glRotated(myBlockIcon.rotation, 0, 0, -1);
+                // move to logo position
+                glTranslated(-0.75, 0, 0);
+                // scale text
+                glScaled(exaggeration, exaggeration, 1);
+                // draw E2 logo
+                if (drawUsingSelectColor()) {
+                    GLHelper::drawText("E2", Position(), .1, 1.5, s.colorSettings.selectionColor);
+                } else {
+                    GLHelper::drawText("E2", Position(), .1, 1.5, RGBColor::BLACK);
+                }
+            } else {
+                // Push matrix
+                glPushMatrix();
+                // Traslate to center of detector
+                glTranslated(myBlockIcon.position.x(), myBlockIcon.position.y(), getType() + 0.1);
+                // Rotate depending of myBlockIcon.rotation
+                glRotated(myBlockIcon.rotation, 0, 0, -1);
+                //move to logo position
+                glTranslated(-1.5, 0, 0);
+                // scale text
+                glScaled(exaggeration, exaggeration, 1);
+                // draw E2 logo
+                if (drawUsingSelectColor()) {
+                    GLHelper::drawText("E2", Position(), .1, 1.5, s.colorSettings.selectionColor);
+                } else {
+                    GLHelper::drawText("E2", Position(), .1, 1.5, RGBColor::BLACK);
+                }
+                //move to logo position
+                glTranslated(1.2, 0, 0);
+                // Rotate depending of myBlockIcon.rotation
+                glRotated(90, 0, 0, 1);
+                if (drawUsingSelectColor()) {
+                    GLHelper::drawText("multi", Position(), .1, 0.9, s.colorSettings.selectedAdditionalColor);
+                } else {
+                    GLHelper::drawText("multi", Position(), .1, 0.9, RGBColor::BLACK);
+                }
+            }
+            // pop matrix
+            glPopMatrix();
+            // Show Lock icon depending of the Edit mode
+            myBlockIcon.drawIcon(s, exaggeration);
         }
+        // Draw name if isn't being drawn for selecting
+        if (!s.drawForRectangleSelection) {
+            drawName(getPositionInView(), s.scale, s.addName);
+        }
+        // check if dotted contour has to be drawn
+        if (myViewNet->getDottedAC() == this) {
+            if (myAdditionalGeometry.getShape().size() > 0) {
+                GLHelper::drawShapeDottedContourAroundShape(s, getType(), myAdditionalGeometry.getShape(), exaggeration);
+            } else {
+                /*
+                GLHelper::drawShapeDottedContourAroundShape(s, getType(), myAdditionalGeometry.multiShapeUnified, exaggeration);
+                */
+            }
+        }
+        // Pop name
+        glPopName();
     }
-    // Pop name
-    glPopName();
 }
 
 
@@ -511,13 +442,13 @@ GNEDetectorE2::getAttribute(SumoXMLAttr key) const {
             return getAdditionalID();
         case SUMO_ATTR_LANE:
         case SUMO_ATTR_LANES:
-            return parseIDs(myLanes);
+            return parseIDs(getParentLanes());
         case SUMO_ATTR_POSITION:
             return toString(myPositionOverLane);
         case SUMO_ATTR_ENDPOS:
             return toString(myEndPositionOverLane);
         case SUMO_ATTR_FREQUENCY:
-            return toString(myFreq);
+            return time2string(myFreq);
         case SUMO_ATTR_LENGTH:
             return toString(myLength);
         case SUMO_ATTR_NAME:
@@ -527,7 +458,7 @@ GNEDetectorE2::getAttribute(SumoXMLAttr key) const {
         case SUMO_ATTR_VTYPES:
             return myVehicleTypes;
         case SUMO_ATTR_HALTING_TIME_THRESHOLD:
-            return toString(myTimeThreshold);
+            return time2string(myTimeThreshold);
         case SUMO_ATTR_HALTING_SPEED_THRESHOLD:
             return toString(mySpeedThreshold);
         case SUMO_ATTR_JAM_DIST_THRESHOLD:
@@ -538,8 +469,8 @@ GNEDetectorE2::getAttribute(SumoXMLAttr key) const {
             return toString(myBlockMovement);
         case GNE_ATTR_SELECTED:
             return toString(isAttributeCarrierSelected());
-        case GNE_ATTR_GENERIC:
-            return getGenericParametersStr();
+        case GNE_ATTR_PARAMETERS:
+            return getParametersStr();
         default:
             throw InvalidArgument(getTagStr() + " doesn't have an attribute of type '" + toString(key) + "'");
     }
@@ -568,8 +499,8 @@ GNEDetectorE2::setAttribute(SumoXMLAttr key, const std::string& value, GNEUndoLi
         case SUMO_ATTR_FRIENDLY_POS:
         case GNE_ATTR_BLOCK_MOVEMENT:
         case GNE_ATTR_SELECTED:
-        case GNE_ATTR_GENERIC:
-            undoList->p_add(new GNEChange_Attribute(this, key, value));
+        case GNE_ATTR_PARAMETERS:
+            undoList->p_add(new GNEChange_Attribute(this, myViewNet->getNet(), key, value));
             break;
         default:
             throw InvalidArgument(getTagStr() + " doesn't have an attribute of type '" + toString(key) + "'");
@@ -616,7 +547,7 @@ GNEDetectorE2::isValid(SumoXMLAttr key, const std::string& value) {
                 return SUMOXMLDefinitions::isValidListOfTypeID(value);
             }
         case SUMO_ATTR_HALTING_TIME_THRESHOLD:
-            return (canParse<double>(value) && (parse<double>(value) >= 0));
+            return canParse<SUMOTime>(value);
         case SUMO_ATTR_HALTING_SPEED_THRESHOLD:
             return (canParse<double>(value) && (parse<double>(value) >= 0));
         case SUMO_ATTR_JAM_DIST_THRESHOLD:
@@ -627,13 +558,19 @@ GNEDetectorE2::isValid(SumoXMLAttr key, const std::string& value) {
             return canParse<bool>(value);
         case GNE_ATTR_SELECTED:
             return canParse<bool>(value);
-        case GNE_ATTR_GENERIC:
-            return isGenericParametersValid(value);
+        case GNE_ATTR_PARAMETERS:
+            return Parameterised::areParametersValid(value);
         default:
             throw InvalidArgument(getTagStr() + " doesn't have an attribute of type '" + toString(key) + "'");
     }
 }
 
+
+bool
+GNEDetectorE2::isAttributeEnabled(SumoXMLAttr /* key */) const {
+    return true;
+}
+
 // ===========================================================================
 // private
 // ===========================================================================
@@ -646,7 +583,7 @@ GNEDetectorE2::setAttribute(SumoXMLAttr key, const std::string& value) {
             break;
         case SUMO_ATTR_LANE:
         case SUMO_ATTR_LANES:
-            myLanes = parse<std::vector<GNELane*> >(myViewNet->getNet(), value);
+            replaceParentLanes(this, value);
             checkE2MultilaneIntegrity();
             break;
         case SUMO_ATTR_POSITION:
@@ -656,7 +593,7 @@ GNEDetectorE2::setAttribute(SumoXMLAttr key, const std::string& value) {
             myEndPositionOverLane = parse<double>(value);
             break;
         case SUMO_ATTR_FREQUENCY:
-            myFreq = parse<double>(value);
+            myFreq = parse<SUMOTime>(value);
             break;
         case SUMO_ATTR_LENGTH:
             myLength = parse<double>(value);
@@ -671,7 +608,7 @@ GNEDetectorE2::setAttribute(SumoXMLAttr key, const std::string& value) {
             myVehicleTypes = value;
             break;
         case SUMO_ATTR_HALTING_TIME_THRESHOLD:
-            myTimeThreshold = parse<double>(value);
+            myTimeThreshold = parse<SUMOTime>(value);
             break;
         case SUMO_ATTR_HALTING_SPEED_THRESHOLD:
             mySpeedThreshold = parse<double>(value);
@@ -692,16 +629,12 @@ GNEDetectorE2::setAttribute(SumoXMLAttr key, const std::string& value) {
                 unselectAttributeCarrier();
             }
             break;
-        case GNE_ATTR_GENERIC:
-            setGenericParametersStr(value);
+        case GNE_ATTR_PARAMETERS:
+            setParametersStr(value);
             break;
         default:
             throw InvalidArgument(getTagStr() + " doesn't have an attribute of type '" + toString(key) + "'");
     }
-    // Update Geometry after setting a new attribute (but avoided for certain attributes)
-    if((key != SUMO_ATTR_ID) && (key != GNE_ATTR_GENERIC) && (key != GNE_ATTR_SELECTED)) {
-        updateGeometry(true);
-    }
 }
 
 /****************************************************************************/
