@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
 /// @file    GNEChange_Additional.h
 /// @author  Pablo Alvarez Lopez
 /// @date    Nov 2015
-/// @version $Id$
 ///
 // A network change in which a additional element is created or deleted
 /****************************************************************************/
@@ -27,13 +26,13 @@
 // ===========================================================================
 // class declarations
 // ===========================================================================
-class GNEAdditional;
+
 class GNEViewNet;
-class GNEDetectorEntry;
-class GNEDetectorExit;
-class GNEDetectorE3;
-class GNELane;
 class GNEEdge;
+class GNELane;
+class GNEShape;
+class GNEAdditional;
+class GNEDemandElement;
 
 // ===========================================================================
 // class definitions
@@ -76,23 +75,35 @@ private:
      */
     GNEAdditional* myAdditional;
 
-    /// @brief pointer to lane parents (used by additionals with lane parent)
-    std::vector<GNELane*> myLaneParents;
+    /// @brief reference to vector of parent edges
+    const std::vector<GNEEdge*>& myParentEdges;
 
-    /// @brief pointer to edge parents (used by additionals with edge parents)
-    std::vector<GNEEdge*> myEdgeParents;
+    /// @brief reference to vector of parent lanes
+    const std::vector<GNELane*>& myParentLanes;
 
-    /// @brief pointer to first additional parent (used by additional with parents, for example Entry/exits)
-    GNEAdditional* myFirstAdditionalParent;
+    /// @brief reference to vector of parent shapes
+    const std::vector<GNEShape*>& myParentShapes;
 
-    /// @brief pointer to second additional parent (used by additional with parents, for example Entry/exits)
-    GNEAdditional* mySecondAdditionalParent;
+    /// @brief reference to vector of parent additionals
+    const std::vector<GNEAdditional*>& myParentAdditionals;
 
-    /// @brief list of Edge childs (used by Rerouters)
-    std::vector<GNEEdge*> myEdgeChilds;
+    /// @brief reference to vector of parent demand elements
+    const std::vector<GNEDemandElement*>& myParentDemandElements;
 
-    /// @brief list of Edge childs (used by VSS)
-    std::vector<GNELane*> myLaneChilds;
+    /// @brief reference to vector of child edges
+    const std::vector<GNEEdge*>& myChildEdges;
+
+    /// @brief reference to vector of child lanes
+    const std::vector<GNELane*>& myChildLanes;
+
+    /// @brief reference to vector of child shapes
+    const std::vector<GNEShape*>& myChildShapes;
+
+    /// @brief reference to vector of child additional
+    const std::vector<GNEAdditional*>& myChildAdditionals;
+
+    /// @brief reference to vector of child demand elements
+    const std::vector<GNEDemandElement*>& myChildDemandElements;
 };
 
 #endif
